/* Copyright 2021 Kevin Ryde

   This file is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This file is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this file.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

#define main  A337114_free_orbit_sizes_main
#include "A337114-free-orbit-sizes.c"
#undef main
#include <stdlib.h>

const int numbpart[] = {  /* A000041 */
  1, 1, 2, 3, 5, 7, 11, 15, 22, 30, 42, 56, 77, 101, 135, 176, 231, 297,
  385, 490, 627, 792, 1002, 1255, 1575, 1958, 2436, 3010, 3718, 4565,
  5604, 6842, 8349, 10143, 12310, 14883, 17977, 21637, 26015, 31185,
  37338, 44583, 53174, 63261, 75175, 89134, 105558, 124754, 147273,
  173525
};
const int cumulative_numbpart[] = {  /* A026905 */
  1, 1, 3, 6, 11, 18, 29, 44, 66, 96, 138, 194, 271, 372, 507, 683, 914,
  1211, 1596, 2086, 2713, 3505, 4507, 5762, 7337, 9295, 11731, 14741,
  18459, 23024, 28628, 35470, 43819,
  53962,    /* 33 */
  66272,    /* 34 */
  81155, 99132, 120769, 146784, 177969, 215307
};

int
my_gentreeg_main (int argc, char **argv)
{
  abort();
}

int
random_partition (int *part, int n)
{
  assert (n >= 0);
  assert (n <= MAXN);
  int len = 0;
  while (n > 0) {
    int p = (random() % n) + 1;
    part[len++] = p;
    n -= p;
  }
  qsort (part, len, sizeof(part[0]), compare_int_descending);
#if 0
  { int i; for(i=0;i<len;i++) printf(" %d",part[i]); printf(" len %d\n", len); }
#endif
  return len;
}

void
add_every_partition (int n, int rem, int *part, int len)
{
  assert (n >= 0);
  assert (n <= MAXN);
  assert (rem >= 0);
  assert (rem <= n);
  assert (len >= 0);
  assert (len <= n);
  if (rem == 0) {
    /* { int i; for(i=0;i<len;i++) printf(" %d",part[i]); printf(" len %d\n", len); } */
    num_seen += add_seen(n,part,len);
  } else {
    int p_max = rem;
    if (len > 0) p_max = MIN(p_max, part[len-1]);  /* descending */
    int p;
    for (p = 1; p <= p_max; p++) {
      part[len] = p;
      add_every_partition (n, rem-p, part, len+1);
    }
  }
}

void
test_add_seen (void)
{
  int  n;
  for (n = 1; n <= 32; n++) {
    init_seen(n);
    num_seen = 0;
    int i;
    for (i=0; i<2; i++) {
      int part[MAXN];
      int len = random_partition(part, n);
      num_seen += add_seen(n, part,len);
    }
    printf("n=%d random seen[] size %d num_seen %d\n", n, seen_end-seen, num_seen);
    int part[MAXN];
    add_every_partition (n, n, part, 0);
    printf("  all seen[] size %d num_seen %d\n", seen_end-seen, num_seen);

    assert (num_seen == numbpart[n]);
    assert (seen_end-seen == cumulative_numbpart[n]);
  }
}
int
main (int argc, char **argv)
{
  setbuf(stdout,NULL);
  test_add_seen();
  printf ("end\n");
  return 0;
}
