\\ Copyright 2017, 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("../examples/Huffman-1-to-n.gp");

\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=1;
progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}


\\-----------------------------------------------------------------------------
\\ where duplicates

{
  \\ 1 2 3 4
  \\     3  , 7, 11, 15, 19, 23, 27, 31, 35, 39, 43, 47, 51, 55, 59, 63, 67
  my(l=List([1..20]));
  while(l[1]<20,
        my(w=l[1]+l[2]);
        print(w"  "Vec(l));
        listpop(l,1); listpop(l,1);
        listput(l,w);
        l=vecsort(l));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Huffman 1..n Indnum

\\ (6m+3)*4^k
\\ 3*(2m+1)*4^k
\\ == 0 mod 3 and even low 0s

\\ (2m+1)*4^k
\\ 1, 3, 4, 5, 7, 9, 11, 12, 13, 15, 16, 17, 19, 20, 21, 23
\\ A003159
\\ = cumulative A026465 which is Thue-Morse run lengths
\\ = A056196 / 8

limit = 1500;
l=List([]);
for(k=0,limit,for(m=0,limit,my(a=(6*m+3)/3*4^k);if(a>limit,break); listput(l,a)));
l=vecsort(Vec(l));
l[1..30]
{
  for(n=1,#l,
     print(n" "test_to_base4(n)"   "l[n]"  "valuation(l[n],4)));
}

check_equal(131 - 3, 128);

\\ first 4^3
\\ at n = 16, 43, 70
check_equal(43-16, 27);
check_equal(70-43, 27);
quit


\\ test whether a occurs as some a(n) in A084088
A084088_pred(a) = a>=5 && a%3==2 && valuation(a,2)%2==0;
{
  check_equal(select(A084088_pred, [0..A084088_samples[#A084088_samples]]),
              A084088_samples,
              "A084088_pred() vs A084088_samples");
}

{
  \\ indnum
  \\ not in OEIS: 3,4, 6,7,8,9,10,11, 13,14,15,16,17,18, 20,21,22, 24,25,26, 28,29,30,31,32,33,35,36

  \\ indnum - n
  \\ not in OEIS: 3,3,3,4,4,4,5,5,5,5,5,5,6,6,6,6,6,6,7,7,7,7,7,7,8,8,8,9,9,9,10,10,10,10,10,10,11,11,11,11,11,11,12,12,12,12,12,12,13,13,13,14,14,14,

  \\ count
  \\ 2,3,1,2,2,5,7,11,2,3,3,14,22,37,9,15,24,6,12,16,6,10,10,30,40,60,10,20,20,70,110,160,40,60,60,230,340,552,
  for(n=3,70,
     my(vpar=make_Huffman_n(n),
        ic=vpar_indnum_and_count(vpar));
     \\ print1(vpar_indnum(vpar)-n, ",");
     print1(ic[2], ",");
  );
  print();
}


\\   4^k = 4 mod 6  except k=0  1
\\ 2*4^k = 2 mod 6
\\ 3*4^k = 0 mod 6              3

check_equal(digits(20,4), [1,1,0])
check_equal(4%6, 4);
check_equal(16%6, 4);
check_equal(20%4, 0);
check_equal(20/4, 5);

check_equal(fromdigits([3,2,3,0],4), 236);
check_equal(vector(20,k,k--; 4^k % 6), \
            vector(20,k,k--; if(k==0,1, 4)));
check_equal(vector(20,k,k--; (2*4^k) % 6), \
            vector(20,k,k--; 2));
check_equal(vector(20,k,k--; (3*4^k) % 6), \
            vector(20,k,k--; if(k==0,3, 0)));

\\ step
\\ A002450
\\ vector(20,k, A084088_step(k))


for(n=1,15, print(A084088(n)))

\\ 0 <= n < 4^k
vector(8,k, sum(n=0,4^k-1,A084088_pred(n)))
\\ A145766  cumulative (2/3)*(4^n-1)
\\ A020988  (2/3)*(4^n-1)

\\ 4^k <= n < 4^(k+1)
vector(8,k, sum(n=4^k,4^(k+1)-1,A084088_pred(n)))
\\ [2, 10, 42, 170, 682, 2730, 10922, 43690]
\\ A020988  (2/3)*(4^n-1).

\\ for(k=0,10, for(n=4^k,4^k+100, if(A084088_pred(n),print(n,",");break)))
\\ next at or after 4^k
\\ 5, 17, 65, 257, 1025, 4097, 16385, 65537, 262145, 1048577,
\\ A052539 4^n + 1.

\\ for(k=0,10, for(n=3*4^k,3*4^k+100, if(A084088_pred(n),print(test_to_base4(n),",");break)))
\\ next at or after 3*4^k is 300011 base4
\\ not in OEIS: 5, 17, 53, 197, 773, 3077, 12293, 49157, 196613, 786437, 3145733,

\\ for(i=1,#A084088_samples, print(digits(A084088_samples[i],4)));

\\ for(n=0,127, print1(A084088_pred(n))); print();
\\ for(n=0,127, print1(A084088_pred(n+256))); print();

{
  \\ indnum increment vs A084088_pred()
  my(prev=0);
  for(n=1,100000,
     if(n%10==0,print1("n=",n,Strchr(13)));
     my(indnum=vpar_indnum(make_Huffman_n(n)),
        incr = indnum - prev,
        by_tree=if(incr==1,0, incr==2,1, error()),
        by_func=A084088_pred(n));
     if(by_tree != by_func,
        print("wrong at n="n));
     prev=indnum;
  );
  quit;
}

{
  \\ indnum increment by >1, is always 1 or 2 apparently
  \\ 5,11,17,20,23,29,35,41,44,47,53,59,65,68,71,77,80,83,89,92,95,101,107,113,116,119,125,131,137,140,
  \\ A084088 == 2 mod 3 and even low 0-bits
  \\   (6n+5)*4^k
  my(prev=0);
  for(n=3,80,
     my(vpar=make_Huffman_n(n),
        indnum=vpar_indnum(vpar));
     if(prev && indnum-prev>1, print1(n,","));
     prev=indnum;
  );
  print();

  check_equal(select(n-> vpar_indnum(make_Huffman_n(n))
                        - vpar_indnum(make_Huffman_n(n-1)) > 1,
                     [3..A084088_samples[#A084088_samples]]),
              A084088_samples);
  quit;
}
{
  \\ indnum increment
  \\ not in OEIS: 1,2,1,1,1,1,1,2,1,1,1,1,1,2,1,1,2,1,1,2,1,1,1,1,1,2,1,1,1,1,1,2,1,1,1,1,1
  \\ not A085006 middle match
  \\ not A204697 partial match only
  \\
  \\
  my(prev=0);
  for(n=3,140,
     my(vpar=make_Huffman_n(n),
        indnum=vpar_indnum(vpar));
     if(prev, print1(indnum-prev-1,","));
     prev=indnum;
  );
  quit;
}

\\-----------------------------------------------------------------------------
\\ Tree n and 4*n

n=3
\\ vpar_view(make_Huffman_n(n))
\\ vpar_view(make_Huffman_n(4*n))
vpar_view(make_Huffman_n(12))
vpar_view(make_Huffman_n(13))
vpar_view(make_Huffman_n(14))
vpar_view(make_Huffman_n(15))

vpar_indnum(make_Huffman_n(n))   == 3
vpar_indnum(make_Huffman_n(4*n)) == 14
quit

\\-----------------------------------------------------------------------------
\\ Huffman

\\ num symbols of the minimum codeword length
\\ not in OEIS: 1,1,3,3,2,2,1,1,6,6,5,5,4,4,3,3,2,2,1,1,12,12,11,11,10,10,9,9,8,8,7,7,6,6,5,5,4,4,3,3,2,2,1,1,24,24,23,23,
\\              3 4   6        11                       23                                                 46 47 48 49 50

vpar_view(make_Huffman_n(23))
{
  print("Symbols where longer codeword length A126014, and A126237 many.");
  print("This is v = n..1 where depth increases.");
  for(n=3,50,
     my(vpar=make_Huffman_n(n),
        depths=vpar_depths_vector(vpar),
        height=vecmax(depths),
        protnum=vecmin(vecextract(depths,vpar_childless_vertices(vpar))));
     \\ print1(vpar_height_num_vertices(vpar)",");
     print1(sum(v=1,n,depths[v]==protnum), ",");
  );
  print("");
  quit;
}



\\------------------------------------------------------------------------------
\\ Unused or wrong bits of A084088

digits_negabase5(n) =
{
  my(l=List([]));
  while(n,
        my(d=n%5); if(d>=3,d-=5);
        listput(l,d);
        n=(n-d)/5);
  Vecrev(l);
}

{
  my(m=9);
  for(n=0,200, A084088_pred(n)||next;
     print(valuation(n,4)"  "digits_negabase5(m));
     m++);
  quit
}
{
  for(n=0,2500, A084088_pred(n)||next; print1(valuation(n,4),","));
  quit
}
{
  \\ 4,9,14,       steps 5
  \\ 17,38,59,     steps 21
  \\ 70,155,240,   steps 85
  \\ 283,624,965,  steps 341
  my(m=0,p=4);
  for(n=0,10000, A084088_pred(n)||next; m++; n%4^p==0||next; n%4^(p+1)!=0||break;  print1(m,","));
  quit
}
{
  my(m=0);
  for(n=0,1500,
     A084088_pred(n)||next;
     my(v=valuation(n,4),
        r=n/4^v);
     printf("%4d  %*s%d = %d * 4^%d\n",m,4*v,"",n,r,v);
     m++);
  quit;
}


\\------------------------------------------------------------------------------
