\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(parisizemax,500*10^6);
default(primelimit,2*10^9);
default(recover,0);
default(strictargs,1);

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("../tools/my-oeis.gp");  \\ work-in-progress



\\-----------------------------------------------------------------------------

read("a061773-Matula-sorted.gp");

v=concat(A061773_rows(8));
select(n->vpar_is_asymmetric(vpar_from_primecode(n)), v)
quit


\\-----------------------------------------------------------------------------
\\ Unsorted, Bigness Order

num_trees_vector=vpar_count_unlabelled_vector;

unsorted_A061773_rows(maxn) =
{
  my(num_trees=num_trees_vector(maxn),
     rows=vector(maxn,n, if(n==1,[1], vector(num_trees[n]))),
     bigs=apply(Vecsmall,rows),
     bigidx=1);
  for(n=2,maxn,
     my(upto=0);  \\ where to store in rows[n] vector

     \\ non-planted trees
     for(p=2,n-1,       \\ number of vertices planted part
        my(s=n-p+1);    \\ number of vertices other part, p+s == n+1
        for(i=num_trees[p]-num_trees[p-1]+1, num_trees[p], \\ planted range
           my(P=rows[p][i],       \\ planted tree P
              P_big=bigs[p][i]);  \\ bigness of its Z subtree
           for(j=1,#rows[s],
              P_big >= bigs[s][j] || break;
              rows[n][upto++] = P*rows[s][j];
              bigs[n][upto]   = P_big)));

     \\ planted trees
     for(i=1,num_trees[n-1],
        rows[n][upto++] = prime(rows[n-1][i]);
        bigs[n][upto]   = bigidx++));

  rows;
}

unsorted_A061773_rows(5)

\\ concat(unsorted_A061773_rows(6)[3..6])
\\ not in OEIS: 4, 3, 8, 6, 7, 5, 16, 12, 9, 14, 10, 19, 13, 17, 11, 32, 24, 18, 28, 21, 20, 15, 38, 26, 34, 22, 53, 37, 23, 43, 29, 67, 41, 59, 31

root_subtree_sizes(vpar) =
{
  vecsort(vecextract(vpar_subtree_sizes_vector(vpar),
                     vpar_children_of_vertex(vpar,vpar_root(vpar))),,4);
}
children_of_root(vpar) = vpar_children_of_vertex(vpar,vpar_root(vpar));
check_equal(root_subtree_sizes([0,1,1,1]), [1,1,1]);
\\ children_of_root_by_sizes(vpar) =
\\ {
\\   my(vec=children_of_root(vpar),
\\      subtree_sizes=vpar_subtree_sizes_vector(vpar));
\\   vecsort(vec, (x,y)->subtree_sizes
\\   
\\   vecsort(
\\   vpar_children_of_vertex(vpar,vpar_root(vpar));
\\ }
lex_by_bigness(vpar1,vpar2) =
{
  my(c=lex(#vpar1,#vpar2));
  if(c,return(c));
  my(u=vpar_minmax_child_of_vertex(vpar1,vpar_root(vpar1),min),
     v=vpar_minmax_child_of_vertex(vpar2,vpar_root(vpar2),min));
  while(u && v,
        c=lex_by_bigness(vpar_subtree(vpar1,u),vpar_subtree(vpar2,v));
        if(c,return(c));
        u=vpar_next_sibling_of_vertex(vpar1,u);
        v=vpar_next_sibling_of_vertex(vpar2,v));
  lex(u,v);
}
lex_primecode_by_bigness(x,y) = lex_by_bigness(vpar_from_primecode(x),vpar_from_primecode(y))

vec=unsorted_A061773_rows(5);
vec=vec[#vec]
vecsort(vec,lex_primecode_by_bigness)
quit


\\-----------------------------------------------------------------------------
\\ primelimit

\\ /so/pari/pari-git/src/basemath/prime.c
\\ prime(long N)
\\ prime_table_find_n(ulong N)
\\ /so/pari/pari-git/src/language/forprime.c
\\ pari_init_primes(ulong maxprime)

default(primelimit)
my(c=0); forprime(p=10*10^9, 10*10^9 + 1000000, c++); c
quit


\\-----------------------------------------------------------------------------
\\ vpar_count_unlabelled_vector(14)

A061773_row(n) =
{
  my(vpar=vpar_unlabelled_first(n),
     l=List([]));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,  \\ trees
        listput(l,vpar_to_primecode(vpar)));
  vecsort(Vec(l));
}
A061773_row(3)


greatest_prime_factor(n) = my(f=factor(n)[,1]); f[#f];

A061773_rows_by_gpfs(limit) =
{
  my(rows=Vec([[1]],limit),
     gpfs=rows,
     plants=vector(limit-1));
  for(n=2,limit,
     \\ print("n="n);
     \\ print("  prev ",rows[n-1]);
     \\ print("  plant ",apply(prime,rows[n-1]));
     plants[n-1] = apply(prime,rows[n-1]);
     my(newrow=Vec(plants[n-1], vpar_count_unlabelled(n)),
        newgpf=newrow,
        upto=#rows[n-1]);
     for(s=2,n-1,
        my(o=n-s+1);
        \\ print("  newrow ",newrow);
        \\ print("  s="s" o="o"   s range ",#rows[s]-#rows[s-1]+1" to "#rows[s]," upto="upto);
        \\ my(have_primes=rows[s][#rows[s]-#rows[s-1]+1 .. #rows[s]],
        \\    sel_primes = select(isprime,rows[s]));
        \\ if(have_primes != sel_primes,
        \\    print("s="s" sel_primes at ",select(isprime,rows[s],1));
        \\    print("row s ",rows[s]);
        \\    print("gpf s ",gpfs[s]);
        \\    my(at=select(i->rows[s][i]==gpfs[s][i], [1..#rows[s]]));
        \\    print(vecextract(rows[s],at));
        \\    print(vecextract(gpfs[s],at));
        \\    print(rows[s][26..#rows[s]]);
        \\    print(gpfs[s][26..#gpfs[s]]);
        \\    error(n));
        for(i=1,#rows[s],
           my(t=rows[s][i]);
           if(t!=gpfs[s][i],next);
           isprime(t) || error("not prime");
           for(j=1,#rows[o],
              if(rows[s][i] < gpfs[o][j], break);
              my(u = t*rows[o][j]);
              \\ print("  s="s" i="i" prime "rows[s][i]"   o="o" j="j" code "rows[o][j]" its gpf="gpfs[o][j]" to u="u);
              \\ if(setsearch(Set(newrow),u),
              \\    \\ print(newrow);
              \\    error("duplicate t=",t));
              newrow[upto++]  = u;
              newgpf[upto]    = rows[s][i];
           )));
     \\ print("newrow ",newrow"\n    "newgpf);
     if(upto!=#newrow,
        \\ print("new  "new);
        error("n="n" upto "upto" want "#newrow));

     my(seq=vecsort(newgpf,,1));
     rows[n]=vecextract(newrow,seq);
     gpfs[n]=vecextract(newgpf,seq);
     \\ print("now "rows" "gpfs);
     gpfs[n]== vecsort(gpfs[n]) || error(n);
     gpfs[n]== apply(greatest_prime_factor,rows[n]) || error(n);
     apply(isprime,gpfs[n]) == vector(#gpfs[n],i,1) || error("gpfs not prime ",n);
  );
  apply(vecsort,rows);
}
A061773_rows_by_gpfs(6)
apply(length,A061773_rows_by_gpfs(6))
vpar_count_unlabelled_vector(6)
{
  my(limit=14);
  check_equal(vector(limit,n, A061773_row(n)),
              A061773_rows_by_gpfs(limit));
}
quit

\\-------
A061773_recurse(c,rem, maxn,maxi) =
{
  print("recurse c="c" rem="rem" max "maxn","maxi);
  rem>=0 || error();
  if(rem==0,
     print("  push ",c);
     listput(new,c);
     return);
  for(n=2,maxn,
     for(i=1,if(n==maxn,maxi,#rows[n]),
        for(rep=1,oo,
           my(new_rem=rem-(n-1)*rep);
           print("  rep=",rep" for "c*rows[n][i]^rep"  new_rem "new_rem);
           if(new_rem<0,break);
           A061773_recurse(c*rows[n][i]^rep, new_rem, n,i-1))));
}
A061773_rows_by_recurse(limit) =
{
  rows=Vec([ [1] ],limit);
  plants=rows;
  for(n=2,limit,
     print("n="n);
     plants[n]=apply(prime,rows[n-1]);
     new=List([]);
     A061773_recurse(1,n-1, n-1,#rows[n-1]);
     rows[n]=vecsort(concat(plants[n],Vec(new)));
     print("  "rows[n]);
  );
  rows;
}
A061773_rows_by_recurse(4)
quit

\\ wrong, want multinomial choice among equal parts
A061773_rows_by_partitions(limit) =
{
  rows=vector(limit);
  rowprimes=vector(limit);
  rows[1] =[1];
  for(n=2,limit,
     rowprimes[n-1] = apply(prime,rows[n-1]);
     my(new=vector(vpar_count_unlabelled(n)),upto=0);
     forpart(part=n-1,
        forvec(s=apply(t->[1,#rows[t]],Vec(part)),
           print(s);
           new[upto++] = prod(i=1,#s, rowprimes[part[i]][s[i]]),
           1));
     upto==#new || error(n" upto "upto" want "#new);
     rows[n]=vecsort(new));
  rows;
}

\\        *          prime(s)*prime(m)
\\      / |
\\    s   m          s+m+1 vertices
\\
\\        *          prime(s)*prime(m)*prime(big)
\\      / | \
\\    s   m  big     
\\
\\  n=2
\\    2  *
\\    |  | \
\\    1  *  *
\\ vpar_to_primecode([0,1,1,1]) == 8
\\ vpar_to_primecode([0,1,1,1,1]) == 16

A061773_rows_by_gpfs(limit) =
{
  my(rows=vector(limit),
     biggest=rows);
  rows[1] =[1];
  biggest[1] =[1];
  for(n=2,limit,
     \\ print("n="n);
     my(new=vector(vpar_count_unlabelled(n)),
        new_biggest=new,
        upto=0);
     for(s=2,n-1,
        my(o=n-s+1);
        \\ print("  s="s" o="o"   s range ",#rows[s]-#rows[s-1]+1" to "#rows[s],);
        for(i=#rows[s]-#rows[s-1]+1,#rows[s],
           isprime(rows[s][i]) || error("not prime");
           for(j=1,#rows[o],
              \\ print("  s="s" o="o"  i="i" j="j"  "biggest);
              if(rows[s][i] <= biggest[o][j],
                 new[upto++]   = rows[s][i] * rows[o][j];
                 new_biggest[upto] = biggest[o][j];

                 if(upto>=2 && new_biggest[upto-1] > new_biggest[upto],
                    print("bad extra child not biggest at upto="upto);
                    print(rows[s][i]" * "rows[o][j]" = "new[upto]);
                    error();
                   )
                 ))));
     my(sorted=vecsort(rows[n-1]));
     for(i=1,#sorted,
        \\ print("  plant "prime(rows[n-1][i]));
        new[upto++] = new_biggest[upto] = prime(sorted[i]);
        if(upto>=2 && new_biggest[upto-1] > new_biggest[upto],
           print("bad plant not biggest at upto="upto);
           error();
          )
     );
     print("new ",new"\n    "new_biggest);
     if(upto!=#new,
        \\ print("new  "new);
        error("n="n" upto "upto" want "#new));

     for(i=2,#new_biggest,
        if(new_biggest[i-1] > new_biggest[i],
           print("bad at i="i);
           print(new_biggest[i-8..i+8])
          ));
     new_biggest== vecsort(new_biggest) || error(n);
     rows[n]=new;
     biggest[n]=new_biggest;
     \\ print("now "rows" "biggest);
  );
  apply(vecsort,rows);
}
A061773_rows_by_gpfs(6)
apply(length,A061773_rows_by_gpfs(6))
vpar_count_unlabelled_vector(6)
{
  my(limit=12);
  check_equal(vector(limit,n, A061773_row(n)),
              A061773_rows_by_gpfs(limit));
}
quit
