\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);

read("a354970-CP-smallest.gp");

read("memoize.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
OEIS_check_verbose = 1;
read("../vpar.gp");
read("../test-helpers.gp");
option_verbose = 0;

\\-----------------------------------------------------------------------------
\\ A354970 CP smallest
\\   1, 2, 3, 4, 6, 7, 10, 11, 20, 22, 28

\\ A354970 recursive
c(n) =
{
  if(n<1,error(),
     n==1,1,
     n%2==0, c(n/2)*(c(n/2)-1)/2 + c(n/2) + 1,
     c((n+1)/2)*(c((n+1)/2)-1)/2 + c((n-1)/2) + 1);
}
c=memoize(c);

{
  OEIS_check_func("A354970","c");
  OEIS_check_func("A354970","a");

  check_equal(vector(1000,n, a(n)),
              vector(1000,n, c(n)),
              "a_by_pair()");

  \\ a(0) by odd formula
  check_equal([a(2*0+1), binomial(a(0+1),2), a0, 1],
              [   1,           0,            a0, 1]);
  check_equal(binomial(1,2), 0);

  \\ 7     -> 14
  \\ 7,8   -> 15   2n-1
  \\   8   -> 16   2n     2n
  \\   8,9 -> 17          2n+1   2n+1
  \\     9 -> 18                 2n+2
  check_equal(x*(x+1)/2 + 1,
              1/2*(x^2 + x + 2));
  check_equal(vector(100,n, a(2*n)),
              vector(100,n,
                a(n)*(a(n)+1)/2 + 1));
  check_equal(vector(100,n,n++; a(2*n-1)),
              vector(100,n,n++;
                a(n)*(a(n)-1)/2 + a(n-1) + 1));

  check_equal(vector(100,n, a(2*n)),
              vector(100,n,
                a(n)*(a(n)+1)/2 + 1));
  check_equal(vector(100,n, a(2*n+1)),
              vector(100,n,
                a(n+1)*(a(n+1)-1)/2 + a(n) + 1));
  \\ a(1) would be n=0
  \\ a(1) = a(1)*(a(1)-1)/2 + a(0) + 1
  \\ 0 = a(0)

  check_equal(vector(100,n,n++; a(2*n+1)),
              vector(100,n,n++;
                a(n+1)*(a(n+1)-1)/2 + a(n) + 1));
  check_equal(vector(100,n, a(2*n+2)),
              vector(100,n,
                a(n+1)*(a(n+1)+1)/2 + 1));

  check_equal(vector(100,n, a(2*n+1) - a(2*n)),
              vector(100,n,
                a(n+1)*(a(n+1)-1)/2
              - a(n)  *(a(n)  -1)/2 ));
}

descents(n) =
{
  if(n<2,return([]));
  my(v=if(n%2==0, [n/2], [(n+1)/2, (n-1)/2]));
  v=concat(v,concat(apply(descents,v)););
  Set(v);
}

\\ for(n=256,512,print(descents(n)" "n));quit


\\  n     -> 2n
\\  n,n+1 -> 2n+1
\\
\\  n,n+1 -> 2n+1
\\    n+1 -> 2n+2
\\
b2(n)     = (sqr(n)-n)>>1;
b2plus(n) = (sqr(n)+n)>>1;
a_by_pair(n) =
{
  my(g=0,    \\ a(n)
     f=1,    \\ a(n+1)
     v=valuation(n,2));
  forstep(i=logint(n,2),v+1,-1,
     my(t = b2(f) + 1);
     if(bittest(n,i),
        g+=t;  \\ a(2n+1)
        f+=t;  \\ a(2n+2)
        ,
        f=t+g;              \\ a(2n+1)
        g=b2plus(g) + 1));  \\ a(2n)
  
  g += b2(f) + 1;
  for(i=1,v,
     g = b2plus(g) + 1);
  g;
}

{
  check_equal(vector(100,n, binomial(n,2)),
              vector(100,n, (sqr(n)-n)>>1),
              "binomial 2");
  if(0,
     \\ speed of binomial() vs sqr()
     \\ 272 / 188. = 1.4
     my(len=10 000 000,
        n=random(2^len), u,v);
     gettime(); u=binomial(n+1,2); print(gettime());
     gettime(); v=b2(-n);          print(gettime());
     u==v || error();
     quit);
}

\\ logint(a(2^16),2)
\\ plot(x=1,2^20, logint(a(floor(x)), 2))

\\-----------------------------------------------------------------------------
\\ 2^k Childless Vertices,
\\ complete binary tree

\\ A006894 Number of planted 3-trees of height < n.
\\ growth power 1.11625303268733048
\\
\\ 

A006894(n) =
{
  \\ cf Michael Somos in A006894 similar but recursiv e and binomial()
  n>=1 || error();
  my(ret=0);
  for(i=1,n, ret = (sqr(ret)+ret)>>1 + 1);
  ret;
}
{
  OEIS_check_func("A006894");
  check_equal(vector(20,k,k--; A006894(k+1)),
              vector(20,k,k--; a(2^k)),
              "A006894 as smallest 2^k");
}
CP_join(L,R) =
{
  (L>=1 && R>=1) || error();
  my(m=max(L,R));
  (sqr(m)-m)>>1 + min(L,R) + 1;
}
{
  check_equal(vector(20,k, A006894(k+1)),
              vector(20,k, my(c=A006894(k));
                     CP_join(c,c)),
              "A006894 same child subtrees");
}

\\-----------------------------------------------------------------------------
\\ claimed a(m) values checked

a_allowing_0(n) = if(n==0,0, a(n));

{
  my(b2(n)     = (sqr(n)-n)>>1);   \\ binomial(n,2)
  my(b2plus(n) = (sqr(n)+n)>>1);   \\ binomial(n+1,2)

a_with_checks(n) =
  n>=1 || error();
  my(x=0,    \\ a(m), staring at m=0
     y=1,    \\ a(m+1)
     v=valuation(n,2));
  forstep(i=logint(n,2),v+1,-1,  \\ bits high to low
     my(m=n>>(i+1));
     x == a_allowing_0(m)  || error();
     y == a(m+1)           || error();

     my(t = b2(y) + 1);
     if(bittest(n,i),
        x+=t;               \\ a(2m+1)
        y+=t;               \\ a(2m+2)
        x == a(2*m+1)  || error();
        y == a(2*m+2)  || error();
        ,
        y=t+x;              \\ a(2m+1)
        x=b2plus(x) + 1;    \\ a(2m)
        y == a(2*m+1)  || error();
        x == a(2*m)    || error();
        ));

  my(m=n>>(v+1));
  x == a_allowing_0(m)  || error();
  y == a(m+1)           || error();

  bittest(n,v) == 1  || error();
  x += b2(y) + 1;         \\ a(2m+1), the 1-bit at v

  my(m=n>>v);
  x == a(m)    || error();

  for(i=1,v,
     x = b2plus(x) + 1;  \\ a(2m) final doublings

     my(m=n>>(v-i));
     x == a(m)    || error());

  x;
}

{
  check_equal(vector(1000,n, a(n)),
              vector(1000,n, a_with_checks(n)),
              "a_with_checks()");
}

\\-----------------------------------------------------------------------------
\\ How Fast Growing
\\   2^8 vertices not until 5695183504492614029263279

{
  if(1,
     print(vector(8,n,a(2^n)));
     quit);
}






\\-----------------------------------------------------------------------------
print("end");