\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
default(parisizemax,100*10^6);

read("a357701-CP-depths-vector.gp");

read("OEIS-data.gp");
read("OEIS-data-wip.gp");
OEIS_check_verbose = 1;
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("various-CP.gp");
option_verbose = 0;


\\-----------------------------------------------------------------------------

got = vector(1000,n,row(n));
{
  check_equal(got,
              vector(#got,n,vpar_depths_vector(vpar_from_CPcode(n))),
              "row() vs vpar_depths_vector()");
}

\\-----------------------------------------------------------------------------
\\ row()
{
  check_equal(row(2),
              [0,1,1],
              "row() 2");
  check_equal(vector(#got-1,n, lex(got[n],got[n+1])),
              vector(#got-1,n, -1),
              "CP depths vectors lex order");

  \\ DATA
  my(want=[0, 0, 1, 1, 0, 1, 2, 2, 1, 0, 1, 2, 2, 1, 2, 2, 0, 1, 2, 3, 3, 2, 1, 0, 1, 2, 3, 3, 2, 1, 2, 2, 0, 1, 2, 3, 3, 2, 1, 2, 3, 3, 2, 0, 1, 2, 3, 3, 2, 3, 3, 1, 0, 1, 2, 3, 3, 2, 3, 3, 1, 2, 2, 0, 1, 2, 3, 3, 2, 3, 3, 1, 2, 3, 3, 2, 0, 1, 2, 3, 3, 2, 3, 3, 1, 2, 3, 3, 2, 3, 3]);
  check_equal(concat(vector(11,n, row(n))), want);
  if(0,
     my(v=concat(vector(11,n,row(n))));
     for(n=1,#v,
        print1(v[n],",");
        if(n==31||n==62,print()));
     print();
     quit);

  \\ example triangle
  my(want=[
    [  0 ],
    [  0, 1, 1 ],
    [  0, 1, 2, 2, 1 ],
    [  0, 1, 2, 2, 1, 2, 2 ],
    [  0, 1, 2, 3, 3, 2, 1 ],
    [  0, 1, 2, 3, 3, 2, 1, 2, 2 ],
    [  0, 1, 2, 3, 3, 2, 1, 2, 3, 3, 2 ],
    [  0, 1, 2, 3, 3, 2, 3, 3, 1 ],
    [  0, 1, 2, 3, 3, 2, 3, 3, 1, 2, 2 ] ]);
  check_equal(vector(#want,n, row(n)), want);

  if(0,
     for(n=1,9,
        my(v=row(n));
        printf("%%e   n=%d: ",n);
        for(i=1,#v, printf(" %d,",v[i]));
        print());
     quit);

  \\ example picture
  my(n=6,
     depths=[0,1,2,3,3,2,1,2,2]);
  check_equal(vpar_is_depths_vector(depths),1,
              "is depths");
  my(vpar=vpar_make_preorder_depths(depths));
  vpar_validate(vpar);
  my(n = vpar_to_CPcode(vpar));
  check_equal(n, 6);
  check_equal(row(n), depths,
              "example depths");

}

\\-----------------------------------------------------------------------------
\\ by call-by-reference list

CP_depths_vector_by_list_store(~l,d,n) =
{
  listput(l,d);
  if(n>1,
     d++;
     foreach(CP_children_z(n),c, self()(~l,d,c)));
}
CP_depths_vector_by_list(n) =
{
  my(l=List([]));
  CP_depths_vector_by_list_store(~l,0,n);
  Vec(l);
}
{
  check_equal(got,
              vector(#got,n,
                     CP_depths_vector_by_list(n)));
}

\\-----------------------------------------------------------------------------
\\ by recursion, like the row() code

{
  \\ children(n), for n>=2, returns a vector [x,y] of the child subtrees of n,
  \\ x = A002024(n-1) and y = A002260(n-1)
  my(CP_children_z(n) =
    my(r, s=sqrtint((n-1)<<1,&r));
    if(r>s,  r-=s;s++, r+=s);
    [s,r>>1]);

CP_depths_vector_by_recursion(n,d) =
  if(n==1, [d],
     my(c=CP_children_z(n));
     concat([d,
             CP_depths_vector_by_recursion(c[1],d+1),
             CP_depths_vector_by_recursion(c[2],d+1)]));
}

{
  check_equal(CP_depths_vector_by_recursion(2,0),
              [0,1,1],
              "CP_depths_vector_by_recursion() 2");

  check_equal(got,
              vector(#got,n, CP_depths_vector_by_recursion(n,0)));
}

\\-----------------------------------------------------------------------------
\\ by stack of pending work

CP_depths_vector_by_stack(n) =
{
  \\ print("CP_depths_vector() n="n);
  my(stack=List([]),
     ret=List([]),
     d=0, y);
  while(1,
        \\ print("n="n" d="d"  "Vec(stack));
        if(n,
           listput(ret,d);
           d++;
           if(([n,y]=CP_children_z(n)),
              listput(stack,0);
              listput(stack,y));
           ,
           #stack || return(Vec(ret));
           d--;
           n=stack[#stack]; listpop(stack)));
}

{
  check_equal(CP_depths_vector_by_stack(2),
              [0,1,1],
              "CP_depths_vector() 2");
 check_equal(got,
              vector(#got,n, CP_depths_vector_by_stack(n)));
}


\\-----------------------------------------------------------------------------
\\ Speed

{
  my(no_concat(n,d) =
     if(n==1, d,
        my(c=CP_children_z(n));
        [d,
         self()(c[1],d+1),
         self()(c[2],d+1)];
        d;
       ));
  my(global_store_traverse(n,d) =
     global_store__v[global_store__i++] = d;
     if(n>1,
        d++;
        foreach(CP_children_z(n),c,
           self()(c,d))));
  global_store__v = vector(1000000);
  my(global_store(n) =
     global_store__i = 0;
     global_store_traverse(n,0);
     Vec(global_store__v,global_store__i));

  if(0,
     my(nmax=10000);

     gettime();
     for(n=1,nmax, CP_num_vertices(n));;
     print("num verts    ",gettime());

     gettime();
     for(n=1,nmax, no_concat(n,0));;
     print("no concat    ",gettime());

     gettime();
     for(n=1,nmax, global_store(n));;
     print("global       ",gettime());

     gettime();
     for(n=1,nmax, CP_depths_vector_by_list(n));;
     print("by list      ",gettime());

     gettime();
     for(n=1,nmax, row(n));;
     print("row          ",gettime());

     gettime();
     for(n=1,nmax, CP_depths_vector_by_recursion(n,0));;
     print("by recursion ",gettime());

     quit);

  if(0,
     my(n=random(1<<20000),
       reps=10,
        x,y,z,num_vertices);

     gettime();
     for(i=1,reps, num_vertices=CP_num_vertices(n));;
     print("num verts    ",gettime());

     gettime();
     for(i=1,reps, no_concat(n,0));;
     print("no concat    ",gettime());

     gettime();
     for(i=1,reps, z=global_store(n));;
     print("global       ",gettime());

     gettime();
     for(i=1,reps, x=CP_depths_vector_by_recursion(n,0));;
     print("by recursion ",gettime());
     check_equal(x,z);

     gettime();
     for(i=1,reps, x=CP_depths_vector_by_list(n));
     print("by listref   ",gettime());
     check_equal(x,z);

     gettime();
     for(i=1,reps, x=row(n));
     print("by row       ",gettime());
     check_equal(x,z);

     print("num vertices = ",#x," = ",num_vertices);
     quit);
}

\\-----------------------------------------------------------------------------
check_equal(c,'c);
check_equal(k,'k);
check_equal(n,'n);
check_equal(v,'v);
check_equal(x,'x);
check_equal(y,'y);
check_equal(vpar,'vpar);
print("end, ",check_count," checks");
