\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../test-helpers.gp");

fbin_to_footprint(fbin) =
{
  fbin >= 0 || error();
  if(fbin==0, [],
     concat(binary(fbin)[^1], [0]));
}
A209859(n) =
{
  \\ using a list for the bits of the return
  if(n==0,0,
     my(l=List([1]));
     forstep(i=logint(max(n,1),2)-1, 0, -1,
        if(bittest(n,i),
           listput(l, bittest(n,i+1))));
     fromdigits(Vec(l),2));
}
balanced_binary_to_footprint(bal) =
{
  my(footprint=vector(hammingweight(bal)),  \\ hammingweight = num vertices
     upto=0);
  forstep(i=logint(max(bal,1),2),1,-1, \\ bit pos of highest 1-bit, and down
     if(bittest(bal,i),
        footprint[upto++] = bittest(bal,i-1))); \\ bit below each 1-bit
  footprint;
}



\\-----------------------------------------------------------------------------
  \\ footprint of each balanced binary
  \\ not in OEIS: 1,2,3,4,5,6,6,7,8,9,10,10,11,12,13,12,12,13,14,14,14,15


{
  print("balanced binary last of footprint");
  print1("  ");
  for(fbin=0,15,
     my(footprint=fbin_to_footprint(fbin));
     print1(footprint_to_balanced_binary_last(footprint)","));
  print("     (A071162)");
  print1("  binary ");
  for(fbin=1,7,
     my(footprint=fbin_to_footprint(fbin));
     print1(test_to_binary(footprint_to_balanced_binary_last(footprint))","));
  print();
}


\\  footprint   balanced binary
\\                 last
\\    [0,0]     1010 only
\\    [1,0]     1100 only
\\    [0,0,0]   10 10 10
\\    [1,0,0]   110 10 0





     \\ print("fbin="fbin" "footprint
     \\       "  "test_to_binary(found_last)
     \\       "  "test_to_binary(footprint_to_balanced_binary_last(footprint)));

\\ {
\\   for(n=0,4,
\\      forvec(footprint=vector(n,i, [0,i!=n]),
\\            my(b=footprint_to_balanced_binary_last(footprint));
\\            vpar_is_balanced_binary(b) || error();
\\            print1(b",")));
\\   print();
\\ }
     \\ print("fbin="fbin" "footprint
     \\       "  "test_to_binary(found_b)
     \\       "  "test_to_binary(footprint_to_balanced_binary_first(footprint)));


quit
\\-----------------------------------------------------------------------------

\\ In terms of balanced binary of a forest, the 1-bit in the replacement
\\ rule is a vertex v which is not a child of its preceding v-1.  That means
\\ v-1 is childless and so footprint 0 for it.

vector(#A209859_samples,n,n--; A209859(n)) == A209859_samples   || error();


quit


\\ A209859 keep the high 1-bit then bit below each 1-bit
vector(20,n,n++; my(v=balanced_binary_to_footprint(n)); fromdigits(concat(1,v),2))
\\ cf A071162 balanced binary of degenerate binary trees
\\    A209642 leftmost

above_each_0(b) =
{
  if(b==0, return([]));
  my(p=logint(b,2),
     footprint=List([]));
  forstep(i=0,p,1,
     if(bittest(b,i),
        listput(footprint, bittest(b,i+1))));
  Vec(footprint);
}
\\ middle match
vector(20,n,n++; my(v=above_each_0(n)); fromdigits(concat([1],v),2))
vector(20,n,n++; my(v=above_each_0(n)); fromdigits(concat([],v),2))
quit
{
  for(n=1,4,
     print("checking n="n);
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(footprint=vpar_is_childful_vector(vpar),
              b=vpar_to_balanced_binary(vpar),
              got=balanced_binary_to_footprint(b));
           print(footprint" "got));
     );
  quit;
}

\\-----------------------------------------------------------------------------
\\ 98LodayRonco(PBT).pdf

\\ Jean-Louis Loday and Maria O. Ronco, "Hopf Algebra of the Planar Binary
\\ Trees", Advances in Mathematics, volume 139, 1998, article number
\\ AI981759, pages 293-309.
\\
\\ Section 4.4 "Hypercube and Trees".
\\ Q[n] = set of n-1 vertices +1,-1 of the hypercube
\\ Map phi Y[n] tree of n vertices -> Q[n] hypercube of n-1 vertices
\\ phi(y elem of Y[n]) = vector [ e[1],...,e[n-1] ]
\\ e[i] = +1 when external vertex i is right child
\\      = -1 when external vertex i is left child


\\ Wenjie Fang, Louis-Francois Preville-Ratelle, "The Enumeration of
\\ Generalized Tamari Intervals"
\\ Diagram of footprint intervals.


\\-----------------------------------------------------------------------------
\\ Baxter
\\   pairs of twin binary trees   Dulucq, Guibert, 1994
\\   pair of twin binary trees = two binary trees with complementary canopy
\\   bijection  Dulucq + Guibert



\\-----------------------------------------------------------------------------
\\ A920000

a(n) = {
  my(clen=hammingweight(n)); n<<=1;
  my(q=valuation(n,2)); n>>=q;                  /* num low 0s */
  my(p=valuation(n+1,2)); n>>=p; if(n==0,p--);  /* num low 1s */
  clen -= p;
  my(counts=vector(clen,d,d--; binomial(d+p + q-1, q-1)));
  while(n,
        q=valuation(n,2); n>>=q;
        p=valuation(n+1,2); n>>=p; if(n==0,p--);
        my(bins=vector(clen,t,t--; binomial(t + q-1, q-1)));
        clen -= p;
        counts=vector(clen,d,d--; sum(t=0,d+p, bins[1+t] * counts[1+d+p-t])));
  counts[1];  }

\\ samples
for(n=1,100,print1(a(n),",");if(n==34||n==67,print()))
vector(20,n,n+=8; a(n))

A054429(n) = my(v=binary(n)); for(i=2,#v,v[i]=1-v[i]); fromdigits(v,2);
vector(20,n, A054429(n))
my(v=OEIS_samples("A054429")); check_equal(vector(#v,n, A054429(n)), v);
for(k=1,10, for(n=2^k,2*2^k-1, A054429(n) == 3*2^k-1-n || error()));

check_equal(a(13), 2, "%e 1010 a(13)");
check_equal(a(6), 2, "%e a(6) not used");
{

  my(foot= 10101101111110000101010001011000,
     v=digits(foot));
  check_equal(v[#v],0);
  my(v_mangled=concat(1,v[^#v]),
     n=fromdigits(v_mangled,2));
  check_equal(n,   3606850092, "fromdigits");
  check_equal(a(n), 114044694, "a(n) 30(b)");

  my(pq=[1,0, 0,0, 1,0, 5,3, 0,0, 0,0, 0,2, 0,0, 1,2]);
  pq[1]--;
  my(l=List([]));
  forstep(i=1,#pq,2,
     for(j=0,pq[i], listput(l,1));
     for(j=0,pq[i+1], listput(l,0)));
  l=Vec(l);
  check_equal(l,v, "l=v");
}

A001181(n) =
{
  n>=0 || error();
  if(n==0,0,
     sum(k=1,n, binomial(n+1,k-1)*binomial(n+1,k)*binomial(n+1,k+1))
     / (binomial(n+1,1)*binomial(n+1,2)));
}
{
  my(v=OEIS_samples("A001181"));
  check_equal(vector(#v,n,n--; A001181(n)), v,
              "A001181");  \\ OFFSET=0
}
print("A001181 ",vector(10,n,n--; A001181(n)))

compls_both(k) = sum(n=2^k,2*2^k-1, a(n)*a(A054429(n)));
check_equal(vector(14,k,k--; compls_both(k)), \
            vector(14,k,k--; A001181(k+1)));

A046996(n) =
{
  n>=2 || error();
  sum(k=1,n, binomial(n+1,k-1)*binomial(n+1,k)*binomial(n+1,k+1))
  / (2*binomial(n+1,1)*binomial(n+1,2));
}
{
  my(v=OEIS_samples("A046996"));
  check_equal(vector(#v,n,n++; A046996(n)), v,
             "A046996");  \\ OFFSET=2
}
print("A046996 ",vector(10,n,n++; A046996(n)))

compls_one(k) = sum(n=2^k,(3/2)*2^k-1, a(n)*a(A054429(n)));
check_equal(vector(14,k, compls_one(k)), \
            vector(14,k, A046996(k+1)), \
            "compls_one()")

\\ Catalans
A000108(n) = binomial(n<<1,n) / (n+1);
{
  my(v=OEIS_samples("A000108"));
  check_equal(vector(#v,n,n--; A000108(n)), v,
             "A000108");  \\ OFFSET=0
}
{
  check_equal(vector(14,k,k--; sum(n=2^k,2*2^k-1, a(n))),
              vector(14,k,k--; A000108(k+1)),
              "total forests")
}


\\----------
system("rm -f /tmp/b920000.txt");
for(n=1,4095, write("/tmp/b920000.txt", Str(n" "a(n))));
system("ls -l /tmp/b920000.txt");
system("wc -l /tmp/b920000.txt");
system("tail -1 /tmp/b920000.txt");

s = OEIS_bfile_gf("A920000");
F = sum(n=1,4095, a(n)*x^n);
check_equal(F, s, "b920000.txt vs F");

quit


\\-----------------------------------------------------------------------------

\\ 8 upwards
\\ 1,1,2,1,3,2,3,1,1,1,2,1,3,2,3,1,4,3,5,2,6,3,4,1,

vector(30,n,n+=7; count_footprint_using_upwards_of_n(n))


count_footprint_using_gfs_of_n(n) =
{
  my(clen=hammingweight(n), cpoly=1); n<<=1;
  while(n,
        my(q=valuation(n,2)); n>>=q;
        my(p=valuation(n+1,2)); n>>=p; if(n==0,p--);
        cpoly *= Ser(vector(clen,i,i--; binomial(i+q-1, q-1)));
        cpoly = serchop(cpoly,p)/x^p;
        clen -= p);
  polcoeff(cpoly,0);
}

vector(30,n,n+=7; count_footprint_using_gfs_of_n(n))
quit


footprint_to_pqvecs(footprint) =
{
  my(plist=List([]),
     qlist=List([]),
     i=1);
  while(i<=#footprint,
        my(p=0); while(i<=#footprint && footprint[i]==1, p++;i++);
        my(q=0); while(i<=#footprint && footprint[i]==0, q++;i++);
        listput(plist,p);
        listput(qlist,q));
  [Vec(plist), Vec(qlist)];
}

\\------------------

\\ num preorder forests of footprint = S(1,0) where recurrence
\\   S(k,0) = 1 and S(k,>=1) = 0
\\   S(i,d) = sum(t=0,d+p[i], binomial(t+q[i]-1, q[i]-1) * S(i+1,d+p[i]-t));


S(i,d) =
{
  if(i==k+1, return(d==0));
  sum(t=0, d+p[i],
      binomial(t+q[i]-1, q[i]-1) * S(i+1, d+p[i]-t))
}

\\ vpar is a preorder labelled forest.  Return its childful footprint.
\\ This is equivalent to vector(#vpar,v, vpar_is_childful(vpar,v)),
\\ but here calculated from all num_children at once.
\\ (Perhaps this could be a vpar_is_childful_vector() func in the future.)
\\
footprint_of_vpar(vpar) = \
  apply(c->c!=0, vpar_num_children_vector(vpar));

{
  for(n=1,8,
     print("checking n="n);

     my(footprints=List([]));     \\ all possible footprints of n vertices
     forvec(footprint=vector(n,i, [0,i!=n]),
           listput(footprints,footprint));
     footprints=Set(footprints);  \\ ready for setsearch()
     #footprints == 2^(n-1) || error();

     my(counts=vector(#footprints));  \\ of each footprint
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(footprint=footprint_of_vpar(vpar),
              pos=setsearch(footprints,footprint));
           pos || error("oops, footprint not found");
           counts[pos]++);

     for(i=1,#footprints,
        [p,q]=footprint_to_pqvecs(footprints[i]);
        k=#p;
        S(1,0) == counts[i] || error(footprints[i])));
}


[p,q]=footprint_to_pqvecs([1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0])
k=#p
S(1,0)

[p,q]=footprint_to_pqvecs([1,1,0,0,0])
k=#p
S(1,0)

\\------------------

\\ S(i,d) constrain Q depths >= d because next i+1 will be top at d
\\ i top at c, constrain i-1 to >= c
\\ i bottom at c+p[i]
\\ Qs from i bottom up to d
\\ t = c+p[i]-d
\\ c = t+d-p[i]
\\ 0 <= t <= sum p[1..i] - d

\\ c+p[i] >= d or no way to put the Qs as deep as d
\\ c+p[i] bottom of this
\\ t = c+p[i]-d range for Qs
\\ p[i]-d <= t <= p[1..i-1] + p[i]-d
\\ p[i]-d <= t <= p[1..i]-d
\\ c = d+t-p[i]

S(i,d) =
{
  if(d<0,return(0));
  if(i==0,
     return(d==0));
  sum(t=0, vecsum(p[1..i])-d,
      print("i="i" d="d" c="d+t-p[i]" recurse to "i-1" "d+t-p[i]);
      binomial(t+q[i]-1, q[i]-1) * S(i-1, d+t-p[i]));
}
read("memoize.gp");
S=memoize(S);

[p,q]=footprint_to_pqvecs([1,0, 1,0, 1,1,0, 1,1,1,1,1,1,0,0,0,0, 1,0, 1,0, 1,0,0,0, 1,0, 1,1,0,0,0])
[p,q]=footprint_to_pqvecs([1,1,0,0,0])
[p,q]=footprint_to_pqvecs([1,0, 1,0])
k=#p
S(k,0)

\\ d takes i-1 S t+d-p[i] times binomial t
\\ 0 <= t <= sum p[1..i] - d
\\      S d-p[i] .. sum p[1..i-1]

binomial_places_gf(s,limit) = \
  Pol(Ser(vector(limit,i,i--; binomial(i+s, s)),,limit));
binomial_places_gf(6,6)

{
  cpoly = 1;
  for(i=1,k,
     my(b=binomial_places_gf(q[i]-1, vecsum(p[1..i])+1));
     b=polrecip(b);
     print("cpoly "cpoly"    ptotal "vecsum(p[1..i])"  b = "b);
     cpoly *= b;
     print("  unchopped "cpoly);
     \\ for(j=1,p[i], cpoly-=polcoeff(cpoly,0); cpoly/=x);
  );
  print("final "cpoly);
}









\\ vec = vpar_to_RNLB(vpar)
\\ Return vec[v] = RNLB type for vertex v.
\\ RNLB type 0 = N    no children
\\           1 = R    right child only
\\           2 = L    left child only
\\           3 = B    left and right child (sum of R,L)
RNLB_of_vpar(vpar) =
{
  my(left_child=vpar_next_sibling_vector(vpar,1),
     right_child=vpar_minmax_child_vector(vpar,max));
  vector(#vpar,v, if(left_child[v],2) + if(right_child[v],1));
}

\\ low bit 1 = binary has right child = forest vertex is childful
RNLB_to_footprint(vec) = vec % 2;

\\ RNLB vec is RRR N LLL B RRR N LLL ... B RRR N LLL
\\ Return [pvec,qvec], with p
RNLB_to_pqvecs(vec) =
{
  my(plist=List([]),
     qlist=List([]),
     i=1,
     p=0,      \\ num R and B
     q=0);     \\ num L and N
  while(i<=#vec,
        while(i<=#vec && vec[i]==1, p++; i++);        \\ count R
        (i<=#vec && vec[i]==0) || error(); q++; i++;  \\ must N
        while(i<=#vec && vec[i]==2, q++; i++);        \\ count L
        listput(plist,p);
        listput(qlist,q);  p=0; q=0;
        if(i>#vec, break);                            \\ either end of vec
        vec[i]==3 || error();                         \\ or B
        i++; p++);
  [Vec(plist),Vec(qlist)];
}
{
  for(n=0,8,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(vec=RNLB_of_vpar(vpar),
              footprint=footprint_of_vpar(vpar));
           footprint == RNLB_to_footprint(vec) || error();
           RNLB_to_pqvecs(vec) == footprint_to_pqvecs(footprint) || error()));
}