\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
setrand(getwalltime()%2^32);

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

\\ WRONG, maybe
test_vector_uniq_by_func(v,func) =
{
  if(#v>=2,
     my(ret=Vec(v),
        pos=1);
     for(i=2,#v,
        for(j=1,pos,
           if(func(v[i],v[pos]), next(2)));  \\ v[i] is a duplicate, skip it
        v[pos++] = v[i]);  \\ pre-increment store
     ret[1..pos];
     ,
     v);
}
 
\\------------------------------------------------------------------------------
\\ halves cross connected at possibly different vertex

\\ n=32 none

try(half_vpar) =
{
  for(v=1,#half_vpar,
     my(half_rerooted=vpar_reroot(half_vpar,v));
     for(u=1,#half_vpar,
        my(vpar=vpar_join(half_vpar,u,half_rerooted));
        \\ print(half_vpar" "half_rerooted" "vpar);
        \\ vpar_validate(vpar);
        \\ vpar_is_tree(vpar) || error();
        my(c = vpar_minimal_domsets_count(vpar));
        if(c>target,
           print("found count "c);
           print(" "vpar);
           listput(l,vpar));
        m=max(c,m);
     ));
}
{
  for(h=1,17,
     my(n=2*h);
     l=List([]);
     m=0;
     target=2^h;
     read(Str("/tmp/trees-"h".gp"));
     print("n="n" max "m"  "target"=pow ",if(m<target,"smaller",m==target,"equal","bigger"));
     my(full_count=#l);
     l = test_vector_uniq_by_func(l,vpar_is_free_isomorphic);
     print("  full_count "full_count" to isomorphic "#l);

     if(#l<10, for(i=1,#l, print("  ",l[i])));
  );
  quit;
}



\\------------------------------------------------------------------------------
\\ join halves at same vertex

\\ n=22 1816  2048=pow smaller
\\   count 2 to iso 1
\\   [0,1,2,3,2,5,1,7,8,7,10,13,2,13,14,13,16,12,18,19,18,21]
\\   subdivided bistar 3,3 connected at middle
\\
\\ n=26 7744  8192=pow smaller
\\   count 1 to iso 1
\\   [0,1,2,3,2,5,2,7,1,9,10,9,12,22,14,15,16,15,18,15,20,9,22,23,22,25]
\\   subdivided bistar 3,4 connected at middle of star
\\
\\ n=30 32368  32768=pow smaller
\\   count 2 to iso 1
\\   [0,1,2,3,2,5,2,7,1,9,10,9,12,9,14,17,2,17,18,17,20,17,22,16,24,25,24,27,24,29]
\\   subdivided bistar 4,4 connected at middle of star
\\
\\ n=34 134432  131072=pow bigger
\\   count 1 to iso 1
\\   [0,1,2,3,2,5,2,7,2,9,1,11,12,11,14,11,16,28,18,19,20,19,22,19,24,19,26,11,28,29,28,31,28,33]


try(half_vpar) =
{
  for(v=1,#half_vpar,
     my(vpar=vpar_join(half_vpar,v,vpar_reroot(half_vpar,v)),
        c = vpar_minimal_domsets_count(vpar));
     if(c>target, listput(l,vpar));
     m=max(c,m);
  );
}
{
  for(h=1,17,
     my(n=2*h);
     l=List([]);
     m=0;
     target=2^h;
     read(Str("/tmp/trees-"h".gp"));
     print("n="n" "m"  "target"=pow ",if(m<target,"smaller",m==target,"equal","bigger"));
     my(count=#l);
     for(i=1,#l,
        my(j=i+1);
        while(j<=#l,
              if(vpar_is_free_isomorphic(l[i],l[j]), listpop(l,j), j++)));
     print("  count "count" to iso "#l);

     if(#l<10, for(i=1,#l, print("  ",l[i])));
  );
  quit;
}

{
  for(h=1,16,
     my(n=2*h,
        depths=vpar_preorder_depths_first(h),
        seen=Map(),
        m=0);
     until((depths=vpar_preorder_depths_next(depths))=='none,
           my(half_vpar=vpar_make_preorder_depths(depths));
           for(v=1,#half_vpar,
              my(vpar=vpar_join(half_vpar,v,vpar_reroot(half_vpar,v)),
                 c = vpar_minimal_domsets_count(vpar));
              if(c>m, m=c);
              ));
     print("n="n" "m"  "2^h"=pow");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ minimum domsets vertex in common

\\ free trees with its minimum domsets having no vertex common
\\              0, 1, 0, 1, 1, 1, 3, 4, 7, 14, 23, 49, 88, 185, 376, 760,
\\ not in OEIS:                   3, 4, 7, 14, 23, 49, 88, 185, 376, 760,

\\ free trees with its minimum domsets having =1 vertex common
\\              1, 0, 1, 1, 2, 3, 5, 11, 18, 39, 77, 155, 345, 707, 1564, 3423,
\\ not in OEIS:                3, 5, 11, 18, 39, 77, 155, 345, 707, 1564, 3423,

\\ free tree most vertices common to its minimum domsets
\\          n = 1              6        9          12           15
\\              1, 0, 1, 1, 1, 2, 2, 2, 3,  3,  3,  4,  4,   4,  5,   5,
\\ trees with this many
\\              1, 1, 1, 2, 2, 2, 3, 9, 6, 16, 41, 15, 65, 245, 58, 357, 1578,
\\ not in OEIS:                   3, 9, 6, 16, 41, 15, 65, 245, 58, 357, 1578,

try(vpar) =
{
  my(sets=vpar_domnum_sets(vpar));
  \\ if(#vpar <= 6,
  \\    print(vpar"  "sets));
  my(c=#test_setintersect_sets(sets));
  if(c>max_common, max_common=c; max_common_reps++,
     c==max_common, max_common_reps++);
  if(#sets>1,
     if(c<min_common, min_common=c; min_common_reps++,
        c==min_common, min_common_reps++));
  if(c==0, no_common_reps++);
  if(c==1, one_common_reps++);
}
{
  for(n=1,30,
     max_common = 0;
     max_common_reps = 0;
     min_common = n+1;
     min_common_reps = 0;
     no_common_reps = 0;
     one_common_reps = 0;
     read(Str("/tmp/trees-"n".gp"));
     if(min_common==n+1, min_common='none);


     \\ print("n="n" max_common "max_common" reps "max_common_reps"   min_common "min_common" reps "min_common_reps);
     \\ print("  one_common reps "one_common_reps);

     \\ print(one_common_reps",");
     \\ print(no_common_reps",");
      print(max_common_reps",");
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ minimal domsets never a vertex in common maybe

try(vpar) =
{
  my(sets=vpar_domsets(vpar));
  sets=select(set->vpar_is_minimal_domset(vpar,set),sets);
  if(#vpar <= 6,
     print(vpar"  "sets));
  my(c=#test_setintersect_sets(sets));
  if(c>max_common, max_common=c; max_common_reps++,
     c==max_common, max_common_reps++);
}
{
  for(n=1,30,
     max_common = 0;
     max_common_reps = 0;
     read(Str("/tmp/trees-"n".gp"));
     print("n="n" max_common "max_common" reps "max_common_reps);
  );
  quit;
}


\\------------------------------------------------------------------------------
\\ n=34  >= 2^(n/2)

\\ 6,4 and 4,4
vpar1=[0, 1, 2, 3, 2, 5, 2, 7, 2, 9, 1, 11, 12, 11, 14, 11, 16, 19, 2, 19, 20, 19, 22, 19, 24, 19, 26, 18, 28, 29, 28, 31, 28, 33];
check_exact(vpar_minimal_domsets_count(vpar1), 132400);

vpar2=[0, 1, 2, 3, 2, 5, 2, 7, 2, 9, 1, 11, 12, 11, 14, 11, 16, 28, 18, 19, 20, 19, 22, 19, 24, 19, 26, 11, 28, 29, 28, 31, 28, 33];
check_exact(vpar_minimal_domsets_count(vpar2), 134432);

{
  vpar_is_free_isomorphic(vpar1,
                          vpar_join(vpar_subdivide(vpar_make_bistar(5,4)),1,
                                    vpar_subdivide(vpar_make_bistar(5,4))))
}
{
  vpar_is_free_isomorphic(vpar2,
                          vpar_join(vpar_subdivide(vpar_make_bistar(4,5)),1,
                                    vpar_subdivide(vpar_make_bistar(4,5))))
}

quit


\\------------------------------------------------------------------------------
\\ bistars

{
  my(size=9);
  my(target_n=2*(2*size-1));
  my(target=2^(target_n/2));
  print("target_n "target_n" target count "target);
  my(seen=List([]));
  for(a=1,size-1,
     my(vpar_a = vpar_make_bistar(a,size-a));
     #vpar_a == size || error();
     vpar_a = vpar_subdivide(vpar_a);
     vpar_degree_of_vertex(vpar_a,1)==a || error();
     vpar_degree_of_vertex(vpar_a,1+a)==size-a || error();

     for(b=1,size-1,
        my(vpar_b = vpar_make_bistar(b,size-b));
        #vpar_b == size || error();
        vpar_b = vpar_subdivide(vpar_b);
        vpar_degree_of_vertex(vpar_b,1)==b || error();
        vpar_degree_of_vertex(vpar_b,1+b)==size-b || error();

        for(aflag=0,0,
           my(va=1 + aflag*a);
           for(bflag=0,0,
              my(vb=1 + bflag*b);
              my(vpar=vpar_join(vpar_a,va,vpar_reroot(vpar_b,vb)),
                 c = vpar_minimal_domsets_count(vpar));
              if(c<target, next());
              for(i=1,#seen, if(vpar_is_isomorphic(vpar,seen[i]),next(2)));
              listput(seen,vpar);

              if(a==4 && b==4 && va==1 && vb==1,
                 print("size "#vpar);
                         \\ vpar_view(vpar);
                );
                      \\ print(vpar);
              print(a","size-a" and "b","size-b" at "va","vb" count "c,
                    if(c>target," bigger",""));
              \\ print("  "vpar);
           ))));
  print("distinct "#seen);
  quit;
}


\\------------------------------------------------------------------------------
\\ even n=34 minimal domsets count > 2^(n/2)
\\ 134432
\\ 131072

\\ n=34
\\          r ------ r
\\       / /\ \ 
\\     1  2  3  4
\\     |  |  |  |
\\     1  2  3  4
\\   //\\
\\  **  **
\\  ||  ||
\\  **  **

1+8+8 == 17
vpar=[0, 8, 10, 11, 12, 13, 14, 1, 15, 8, 9, 9, 9, 9, 8, 1, 2, 25, 25, 27, 28, 29, 30, 31, 8, 32, 25, 26, 26, 26, 26, 25, 18, 19];
#vpar
\\ vpar_view(vpar);
vpar_minimal_domsets_count(vpar)
2^(#vpar/2)
quit;








