#!/usr/bin/perl -w

# Copyright 2018, 2019, 2020, 2021 Kevin Ryde

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use strict;
use FindBin;
use List::Util 'max','min';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

{
  # Matulas untested
  my %got;
  require File::Slurp;
  require Math::OEIS::Names;
  @got{File::Slurp::read_file("$FindBin::Bin/test-Matula.gp") =~ /\bA\d{6,7}/g} = ();
  print "got ",scalar(keys %got),"\n";
  my $count = 0;
  while (defined (my $line = readline(Math::OEIS::Names->fh))) {
    my ($anum,$name) = Math::OEIS::Names->line_split($line) or next;
    next if $name =~ /GF2X-Matula|A014486/;
    if ($name =~ /Matula/) {
      $count++;
      if (! exists $got{$anum}) {
        print "$anum $name\n";
      }
    }
  }
  print "of $count total\n";
  exit 0;
}
__END__
{
  # A161820 sort run lengths descending
  sub A161820 { 
    # oct('0b'.join('0', sort {length($a)<=>length($b)}
    #                                split /0+/, sprintf '%b', @_));
              }
  unshift @INC, "$FindBin::Bin/../../shared";
  require MyOEIS;

  MyOEIS::compare_values
    (anum => 'A161820', # OFFSET=1
     func => sub {
       my ($count) = @_;
       my @got;
       foreach my $n (1 .. $count) {
         push @got, A161820($n);
       }
       return \@got;
     });
  exit 0;
}


{
  require Math::OEIS::Grep;
  require Math::NumSeq::PlanePathN;
  my $bigclass = Math::NumSeq::PlanePathN::_bigint();

  foreach my $elem
    (
     # not in OEIS: 2,8,47,365,3570,42796,616610
     [20,sub { my($n) = @_; $n->copy->bpow($n-2) }, "n^(n-2) MaxAncestor"],

     # A111538 column 2 of triangle A111536
     [20,sub { my($n) = @_; $n->copy->bfac }, "n!"],

     # A104981 Column 1 of triangle A104980
     [20,sub { my($n) = @_; $n--; $n->copy->bfac }, "(n-1)! MinDescendant"],

     # not in OEIS: 2,6,25,130,806,5790,47260,432018,4373144,48571678
     [20,sub { my($n) = @_; $n-=min(2,$n); $n->copy->bfac }, "(n-2)!"],

     # A000165 Double factorial of evens: (2n)!! = 2^n*n!
     [20,sub { my($n) = @_; 2**$n }, "2^n"],

     # A001710 a(n) = numerator(n!/2) = n!/2 for n>=2
     [30,sub { my($n) = @_; 2**($n-1) }, "2^(n-1)"],

     # not in OEIS: 5,23,134,943,7701,71329,737336,8403713,104602565
     [20,sub { my($n) = @_; $n**2 }, "n^2"],

     # 2*A106174 = 2*n*a(n-1)-a(n-2).
     [20,sub { my($n) = @_; 2*$n }, "2*n"],

     # not in OEIS: 1,-1,4,-13,67,-385,2662,-20977,186769,-1850761,20205460
     [20,sub { my($n) = @_; $n-2 }, "n-2"],

     # A005425 a(n) = 2*a(n-1) + (n-1)*a(n-2)
     # switchboard problem
     # = binomial transform of A000085 = 1+A001189(n)
     #   A001189 = a(n-1)+(1+a(n-2))*(n-1) degree-n permutations order=2
     [20,sub { my($n) = @_; $n-1 }, "n-1"],

     # not in OEIS: 7,38,283,2678,30721,414026,6409369,112056434,2183444359
     [20,sub { my($n) = @_; $n+1 }, "n+1"],

     # A058799 Column 2 of A007754.
     # a(n) = Sum_{k=0..floor(n/2)}
     #         (-1)^k * (n-2*k)! * binomial(n-k,k) * binomial(n-k+2,k+2)
     [20,sub { my($n) = @_; $n }, "n"],

     # superseeker:
     # LISTTOREC
     # [{(-3 n - 7) a(n) + a(n + 1), a(0) = 12}, ogf]
     # [{(-3 n - 7) a(n) + (n + 1) a(n + 1), a(0) = 12}, egf]
     # [{-3 a(n) + a(n + 1), a(0) = 7}, lgdegf]
     # RATE
     # (4*3^n*Gamma[4/3 + n])/Gamma[7/3]
     # vector(10,n,n--; 4*3^n * gamma(4/3 + n)/gamma(7/3) )
     # vector(10,n,n--; 3^(n+1) * gamma(1/3 + n)/gamma(1/3)
     #
     # divide 12 to 1, 7, 70, 910, 14560, 276640, 6086080, 152152000
     # 12 * A051604 (3*n+4)!!!/4!!!
     # vector(8,n,n--; 3*triplefactorial(3*n-2))
     # 84 * A051607 (3*n+7)!!!/7!!!
     #   n!!  = n(n-2)(n-4)...
     #   n!!! = n(n-3)(n-6)...
     # GP-DEFINE  triplefactorial(n) = prod(i=0,n, max(n-3*i,1));
     # vector(10,n,n--; triplefactorial(n))
     # vector(10,n,n--; triplefactorial(3*n+7) / triplefactorial(7)) * 84
     #
     # not in OEIS: 12,84,840,10920,174720,3319680,73032960,1825824000
     [20,sub { 3 }, "constant 3"],

     # A097801 (2*n)! / (n! * 2^(n-1)) = 2*(2*n-1)!!
     # double factorial
     [20,sub { 2 }, "constant 2"],

     # factorials
     [20,sub { 1 }, "constant 1"],

    ) {
    my ($len, $f, $name) = @$elem;
    my @f_values = map { $f->($bigclass->new($_)) } 0 .. $len-1;
    my @a_values;

    foreach my $n (1 .. $len-1) {
      my $a = $f_values[$n];
      foreach my $p (1 .. $n-1) {
        ### at: "p=$p  n-p=".($n-$p)
        $a += $bigclass->new($p) * $a_values[$p] * $f_values[$n-$p];
      }
      $a_values[$n] = $a;
    }
    my @values = @a_values[2..$len-1];
    Math::OEIS::Grep->search(array => \@values, verbose=>1,
                             name => $name);
  }
  exit 0;
}
