\\ Copyright 2018, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Clemens Heuberger and Stephan Wagner, "The Number of Maximum Matchings In
\\ a Tree", Discrete Mathematics, volume 311, issue 21, November 2011, pages
\\ 2512-2542.
\\ https://arxiv.org/abs/1011.6554
\\
\\ https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3226351/
\\ full text html


default(recover,0)
default(strictargs,1)
read("../examples/most-maximum-matchings.gp");
read("a333347-most-maximum-matchings.gp");

read("pol-pfrac.gp");
\\ read("../vpar-wip.gp");
read("../test-helpers.gp");

print("===============================================================\n\n");

option_verbose = 0;
option_progress = 1;

\\ most maximum matchings of n vertices
count(n) = test_gf_term(gf,n);


\\-----------------------------------------------------------------------------
progress("A190872 = G(k)");

A190872(k) = simplify(G(k));
gA190872(x) = gG(x);
{
  check_equal(vector(#A190872_samples,n,n--; A190872(n)),
                      A190872_samples,   \\ OFFSET=0
              "A190872() vs samples");
  check_equal(test_gf_terms(gA190872(x), #A190872_samples),
                                          A190872_samples,
              "gA190872() vs samples");   \\ OFFSET=0

  \\ definition of A190872
  check_equal(vector(100,n,n++; A190872(n)),
              vector(100,n,n++; 11*A190872(n-1) - 9*A190872(n-2)));
  check_equal(A190872(0), 0);
  check_equal(A190872(1), 1);

  \\ formula, by T. D. Noe is it?
  check_equal(vector(100,n,n--; A190872(n)),
              vector(100,n,n--; simplify( ((11+sqrt85)^n - (11-sqrt85)^n)
                                         /(2^n*sqrt85) )),
              "A190872 powers formula");

  \\ Philippe Deleham in A190872
  check_equal(gA190872(x),
              x/(1-11*x+9*x^2));

  \\ G. C. Greubel in A190872
  my(len=100);
  check_equal(test_func_to_egf(G,len),
              2/sqrt85 * exp(11/2*x + O(x^len)) * sinh(sqrt85/2*x + O(x^len)));

  \\ Altug Alkan in A190872
  check_equal(concat(0, Vec(x/(1-11*x+9*x^2) + O(x^#A190872_samples))),
                                                    A190872_samples);
}

{
  \\ without the x numerator
  check_equal(test_gf_terms(1/(1-11*x+9*x^2), 10),
              vector(10,k,k--; simplify((l^(k+1) - h^(k+1)) / (l-h))),
              "G but 1/(...)");
}

A190872_with_negatives(n) = simplify((l^n - h^n) / (l-h));
check_equal(A190872_with_negatives(-1), -1/9);

{
  \\ PARI for A190872
  my(a = (n) -> polcoeff(lift(Mod('x,'x^2-11*'x+9)^n), 1)
    );
  check_equal(vector(100,n,n--; A190872(n)),
              vector(100,n,n--; a(n)));

  \\ the poly 0 coeff
  \\ not in OEIS: 9, 99, 1008, 10197, 103095, 1042272, 10537137, 106528059
  \\ check_equal(vector(10,n,n--; polcoeff(lift(Mod('x,'x^2-11*'x+9)^n), 0)),
  \\             vector(10,n,n--; 0));
}


\\-----------------------------------------------------------------------------
progress("a333347-most-maximum-matchings.gp");

check_equal(simplify(sqrt85^2), 85);

check_equal(L, l);
check_equal(gf, gcount(x));
{
  for(i=1,#gf_vector,
     my(g=gf_vector[i],
        d=denominator(g),
        m=factor(d));
     check_equal(poldegree(d)>0, i>1);
     if(poldegree(d)>0,
        check_equal(matsize(m),[1,2])));
}

{
  \\ n = 7k most maximum matchings
  check_equal(vector(100,k, my(n=7*k); count(n)),
              vector(100,k, 8*A190872(k) - 7*A190872(k-1)),
              "n = 0 mod 7 by A190872");
  \\ not at k=0, is 7/9 but want 1
  my(k=0);
  check_equal(8*A190872(k) - 7*A190872_with_negatives(k-1), 7/9);
}
{
  \\ n = 7k+1 most maximum matchings
  check_equal(vector(10,k,k--; my(n=7*k+1); count(n)),
              vector(10,k,k--; A190872(k+1)),
              "n = 1 mod 7 by A190872");

  \\ n = 7k+4 most maximum matchings
  check_equal(vector(10,k,k--; my(n=7*k+4); count(n)),
              vector(10,k,k--; 3*(A190872(k+1) - A190872(k))),
              "n = 4 mod 7 by A190872");
}

A333344(n) = simplify(G(n+1) - G(n));
gA333344(x) = (1 - x)/(1 - 11*x + 9*x^2);

\\ with n<0 allowed
A333344_allowing_negatives(n) = polcoeff(lift(('x-1)*Mod('x,'x^2-11*'x+9)^n),1);

{
  \\ A333344 diffs of G
  check_equal(vector(#A333344_samples,n,n--; A333344(n)),
              A333344_samples,  \\ OFFSET=0
              "A333344() vs A333344_samples");

  check_equal(vector(10,n,n--; A333344(n)),
            vector(10,n,n--; polcoeff(lift(('x-1)*Mod('x,'x^2-11*'x+9)^n), 1)));

  check_equal(vector(100,n,n++; A333344(n)),
              vector(100,n,n++; 11*A333344(n-1) - 9*A333344(n-2)),
              "A333344 recurrence");
  check_equal(A333344(0), 1,    "A333344 starting 1,10");
  check_equal(A333344(1), 10);

  check_equal(vector(100,n,n--; A333344(n)),
              vector(100,n,n--; A333344_allowing_negatives(n)));
  check_equal(vector(100,n,n-=10; A333344_allowing_negatives(n)),
              vector(100,n,n-=10; 11*A333344_allowing_negatives(n-1)
                                 - 9*A333344_allowing_negatives(n-2)),
              "A333344 recurrence");

  check_equal(vector(100,n,n--; A333344(n)),
              vector(100,n,n--; A190872(n+1) - A190872(n)),
              "A333344 first diffs of A190872, formula");

  check_equal(vector(100,n,n--; A333344(n)),
              vector(100,n,n--; simplify( ((l-1)*l^n - (h-1)*h^n) / (l-h)) ));
  check_equal(vector(100,n,n--; A333344(n)),
              vector(100,n,n--; round((l-1.0)/(l-h)*l^n)));

  \\ some l and h cross relations
  check_equal((l-1)/(l-h), (1/2 + 9/170*sqrt85));
  check_equal((l-1)/(l-h), (l-1)/(l-9/l));
  check_equal(1/l, 11/18 - 1/18*sqrt85);
  check_equal((l-1)/(l-h), (l-1)/(l-9/l));
  check_equal((l-1)/(l-h), (1-1/l)/(1-9/l^2));

  check_equal(test_gf_terms(gA333344(x), 100),
              vector(100,n,n--; A333344(n)),
              "gA333344() vs A333344()");
}

\\ vector(8,k, G(k+1) - 3*G(k))   \\ m0(C^k L)
\\ not in OEIS: 79, 797, 8056, 81443, 823369, 8324072, 84154471

\\ vector(10,k, 3*(G(k+1) - G(k)))   \\ for m(C^k F)
\\ not in OEIS: 30966, 313059, 3164955, 31996974, 323482119, 3270330543

{
  \\ M eigenvalues
  check_equal(M,[8,3;5,3]);
  my(v=mateigen(M,1)[1]);
  check_equal(abs(v[1]-h) < 1e-10, 1, "h eigenvalue of M");
  check_equal(abs(v[2]-l) < 1e-10, 1, "h eigenvalue of M");
}

\\ gf
{
  check_equal(gf,
              gcount(x),
              "gf vs gcount()");
}

\\ gf_vector
{
  my(p=pol_partial_fractions(gcount(x)),
     seen=vector(#gf_vector));
  for(i=1,#gf_vector,
     for(j=1,#p,
        if(gf_vector[i]==p[j],
           seen[i]++)));
  check_equal(seen, vector(#gf_vector,i, 1),
              "gf_vector[] vs pol_partial_fractions of gcount()");
}

\\ values[]
{
  check_equal(#values, 700);
  for(i=1,#values,
     my(n=i-1);
     check_equal(values[i],
                 vpar_matchnum_and_count(make(n))[2]));
}

\\ num numerator
check_equal(num, lk_numerator);


\\-----------------------------------------------------------------------------
progress("A147841  (1-2x)/(1-11x+9*x^2)");

A147841(n) = polcoeff(lift(('x-2)*Mod('x,'x^2-11*'x+9)^n), 1);

\\ Philippe Deleham in A147841
gA147841(x) = (1 - 2*x)/(1 - 11*x + 9*x^2);

{
  check_equal(vector(#A147841_samples,n,n--; A147841(n)),
                      A147841_samples,   \\ OFFSET=0
              "A147841() vs samples");
  check_equal(test_gf_terms(gA147841(x), 100),
              vector(100,n,n--; A147841(n)),
              "gA147841() vs A147841()");

  \\ A147841 in terms of A190872
  check_equal(vector(100,n,n--; A147841(n)),
              vector(100,n,n--; A190872(n+1) - 2*A190872(n)),
              "A147841() diff two");

  \\ A147841 in terms of combination
  check_equal(vector(100,n,n--; A147841(n)),
              vector(100,n,n--; A333344(n) - A190872(n)),
              "A147841() combination diff");

  \\ A147841 in terms of A333344 diff
  check_equal(vector(100,n,n--; A147841(n)),
              vector(100,n,n--; 11*A333344(n) - A333344(n+1)),
              "A147841() diff 11");

  check_equal(vector(100,n, A147841(n)),
              vector(100,n, 9*A333344(n-1)),
              "A147841() as 9*A333344");
  check_equal(vector(100,n,n--; A147841(n)),
              vector(100,n,n--; 9*A333344_allowing_negatives(n-1)),
              "A147841() as 9*A333344 including n=0");


  check_equal(vector(100,n,n--; A333344(n)),
              vector(100,n,n--; A190872(n) + A147841(n)),
              "A333344 formula in A190872(n) and A147841");
}

\\ gf_shift_down(g,n) = {
\\   my(x = variable(g));
\\   for(i=1,n, g -= test_gf_term(g,0); g /= x);
\\   g;
\\ }
\\ gf_in_terms_of(g,h) = {        \\ g in terms of h
\\   my(gdenv = [denominator(g)]);
\\   for(len=1,50,
\\       \\ print("len="len);
\\       my(hv = vector(len,i, gf_shift_down(h,i-1)),
\\          d = lcm(concat(gdenv,apply(denominator,hv))),
\\          hvnums = hv*d,
\\          gnum = g*d);
\\       \\ for(i=1,#hv,pol_ascending_print(hv[i])); print(d);
\\       poldegree(gnum) < len || next();
\\       vecmax(apply(poldegree,hvnums)) < len || next();
\\       my(m = matrix(len,len,i,j, polcoeff(hvnums[j],i-1)),
\\          v = vectorv(len,i, polcoeff(gnum,i-1)),
\\          f = iferr(matsolve(m,v),e,[]));
\\       if(f!=[],
\\          \\ print(sum(i=1,#f, f[i]*hv[i])); print(g);
\\          while(#f>0 && f[#f]==0, f=f[1..-2]);
\\          sum(i=1,#f, f[i]*hv[i]) == g || error("oops wrong sum");
\\          return(f)));
\\   error("no solution found");
\\ }
\\
\\ gf_in_terms_of(gA147841(x),gA333344(x))
\\ gf_in_terms_of(x*gA147841(x),gA333344(x))
\\ gf_in_terms_of(gA147841(x),x*gA333344(x))

\\ {
\\   lindep([vector(20,n,n+=2; A147841(n)),
\\           vector(20,n,n+=2; A333344(n)),
\\           vector(20,n,n+=2; A333344(n-1))])
\\ }
\\ {
\\   lindep([vector(20,n,n+=2; A147841(n)),
\\           vector(20,n,n+=2; A333344(n+1)),
\\           vector(20,n,n+=2; A333344(n))])
\\ }
\\ {
\\   lindep([vector(20,n,n+=2; A147841(n)),
\\           vector(20,n,n+=2; A333344(n+2)),
\\           vector(20,n,n+=2; A333344(n+1))])
\\ }
\\ quit


\\-----------------------------------------------------------------------------
progress("T6 and T34");

{
  \\ n=6 make is a star T_{6,1}
  my(vpar=make(6));
  check_equal(vpar_matchnum_and_count(vpar), [1,5],
              "n=6");
  check_equal(vpar_is_free_isomorphic(vpar,vpar_make_star(6)), 1);

  \\ n=6 other is a path and hanging
  my(six_other_vpar=make(6.5));
  check_equal(vpar_matchnum_and_count(six_other_vpar), [2,5],
              "n=6 other");
  check_equal(#six_other_vpar, 6);
  check_equal(vpar_is_free_isomorphic(vpar,six_other_vpar), 0);

  vpar=make(6.5);
  check_equal(#vpar, 6);
  check_equal(vpar_matchnum_and_count(vpar), [2,5]);
  check_equal(vpar_is_free_isomorphic(vpar,six_other_vpar), 1);
}


{
  \\ n=34 and 34.5
  my(vpar=make(34));
  my(other=make(34.5));
  check_equal(vpar_matchnum_and_count(vpar), [10, 59049],
             "n=34");
  check_equal(vpar_matchnum_and_count(other), [10, 59049],
             "n=34.5");
  check_equal(vpar_is_free_isomorphic(vpar,other), 0);
}



\\------------------------------------------------------------------------------
progress("tree isomorphism and count vs nex/devel/maximum-matchings.c");

\\ counts
\\ not in OEIS: 1, 2, 3, 4, 5, 8, 11, 15, 21, 30, 41, 56, 81, 112, 153, 216, 303, 418, 571, 819, 1133, 1560, 2187, 3063, 4235, 5832, 8280

{
  \\ Count of how many maximum matchings in each tree, as found by
  \\ maximum-matchings.c search and nautyextra_matchnum_and_count_vpar() there.
  \\ These should equal the corresponding vpar_matchnum_and_count() here.
  \\
  my(want_count=[
0,      \\ n=1
1,
2,
3,
4,
5,
8,
11,
15,
21,
30,
41,
56,
81,
112,
153,
216,
303,
418,
571,
819,
1133,
1560,
2187,
3063,
4235,
5832,
8280
]);
  for(n=1,#want_count,
     if(n==1,next());
     my(vpar=make(n),
        got=vpar_matchnum_and_count(vpar));
     check_equal(got[2], want_count[n]);
  );
}

{
  \\ Trees found by maximum-matchings.c search, in the labelling of
  \\ gentreeg.
  \\ These should be isomorphic to what make() creates.
  \\
  my(want_vec
     =[
[0],        \\ n=1
[0,1],
[0,1,1],
[0,1,1,1],
[0,1,1,1,1],  \\ n=5 star
\\ [0,1,2,2,1,5],    \\ n=6 other
[0,1,1,1,1,1],
[0,1,2,2,1,5,5],
[0,1,2,2,2,1,6,6],
[0,1,2,2,2,1,6,6,6],
[0,1,2,3,3,1,6,7,7,1],
[0,1,2,3,3,1,6,7,7,1,1],
[0,1,2,3,3,3,1,7,8,8,1,1],
[0,1,2,3,3,3,1,7,8,8,8,1,1],
[0,1,2,3,3,1,6,7,7,1,10,11,11,1],
[0,1,2,3,4,4,2,2,1,9,10,11,11,9,9],
[0,1,2,3,4,4,4,2,2,1,10,11,12,12,10,10],
[0,1,2,3,3,1,6,7,7,1,10,11,11,1,14,15,15],
[0,1,2,3,4,4,2,7,8,8,2,1,12,13,14,14,12,12],
[0,1,2,3,4,5,5,3,3,1,10,11,12,13,13,11,11,1,1],
[0,1,2,3,4,5,5,5,3,3,1,11,12,13,14,14,12,12,1,1],
[0,1,2,3,4,4,2,7,8,8,2,1,12,13,14,14,12,17,18,18,12],
[0,1,2,3,4,5,5,3,3,1,10,11,12,13,13,11,11,1,18,19,19,1],
[0,1,2,3,4,5,6,6,4,4,2,2,1,13,14,15,16,17,17,15,15,13,13],
[0,1,2,3,4,4,2,7,8,8,2,11,12,12,1,15,16,17,17,15,20,21,21,15],
[0,1,2,3,4,5,5,3,8,9,9,3,1,13,14,15,16,16,14,14,1,21,22,22,1],
[0,1,2,3,4,5,5,3,3,1,10,11,12,13,13,11,11,1,18,19,20,21,21,19,19,1],
[0,1,2,3,4,4,2,7,8,8,2,11,12,12,1,15,16,17,17,15,20,21,21,15,24,25,25],
[0,1,2,3,4,5,5,3,8,9,9,3,1,13,14,15,16,16,14,19,20,20,14,1,24,25,25,1],
[]
       ]
);
  for(i=1,#want_vec,
     my(want=want_vec[i],
        got=make(#want),
        is_iso=vpar_is_free_isomorphic(got,want));
     if(!is_iso,
        print("want "want);
        print("got  "got);
        );
     check_equal(is_iso, 1, "vs gentreeg search");
  );
}

\\-----------------------------------------------------------------------------
progress("Centroids");

\\ unicentroidal except n=2,8,16, it seems
check_equal(vector(200,n,n--; #vpar_centroids(make(n))), \
            vector(200,n,n--; if(n<=2,n, n==8,2, n==16,2, 1)));

\\ unicentral except n=2 path-2 is bicentral
\\ this is per the type A,B bipartition
check_equal(vector(50,n,n--; #vpar_centres(make(n))), \
            vector(50,n,n--; if(n<=2,n, 1)));


\\----------------------------------------------------------------------------
progress("matchnum()");

matchnum_formula(n) = floor((2*n+2)/7);
matchnum(n) = if(n==2,1, n==6,1, n==13,3, n==20,5, floor((2*n+2)/7));
{
  \\ odds 3 times, evens 4 times
  \\ 1 mod 7    2*n
  \\ 2 mod 7    2*n
  \\ 3 mod 7    2*n+1
  \\ 4 mod 7    2*n+1
  \\ 5 mod 7    2*n+1
  \\ 6 mod 7    2*n+2
  \\ 0 mod 7    2*n
  \\ recurrence_guess(vector(100,i,i--; my(n=7*i, vpar=make(n));
  \\                         vpar_matchnum(vpar)));
  \\ print(vector(20,i,i--; my(n=35+i, vpar=make(n));
  \\                         vpar_matchnum(vpar)));
  \\ print(vector(20,i,i--; my(n=35+i); floor((2*n+2)/7)));

  check_equal(vector(150,n,n--; my(vpar=make(n)); vpar_matchnum(vpar)),
              vector(150,n,n--; matchnum(n)),
              "matchnum()");
  check_equal(matchnum(0), 0);
  check_equal(matchnum(1), 0);
  check_equal(matchnum(2), 1);
  check_equal(matchnum_formula(0), 0);
  check_equal(matchnum_formula(1), 0);
  check_equal(matchnum_formula(2), 0);    \\ whereas want 1

  forstep(n=6,20,7,
     check_equal(matchnum(n), matchnum_formula(n) - 1));
  check_equal(matchnum(6), 1);
  check_equal(matchnum_formula(6), 2);   \\ whereas want 1
}


\\-----------------------------------------------------------------------------
progress("2 mod 7");
{
  my(n=37);
  my(kfunc = (j)->if(j==0,  max(0, floor((n-37)/35)),
                     n>=37, floor((n-2+7*j)/35),
                            floor((n-9+7*j)/35)));
  check_equal(vector(5,j,j--; kfunc(j)), [0, 1, 1, 1, 1]);
  check_equal(7*4+4 + 3 + 2, 37);
}


\\-----------------------------------------------------------------------------
progress("A333345 lambda  (11 + sqrt(85))/2");

{
  my(s=fromdigits(A333345_samples)/10^(#A333345_samples-2));
  check_equal(P(s) < 0, 1);
  s=(fromdigits(A333345_samples)+1)/10^(#A333345_samples-2);
  check_equal(P(s) > 0, 1);
}

\\-----------------------------------------------------------------------------
progress("A333346 lambda seventh root  ((11 + sqrt(85))/2)^(1/7)");

{
  my(s=fromdigits(A333346_samples)/10^(#A333346_samples-1));
  check_equal(P(s^7) < 0, 1);
  s=(fromdigits(A333346_samples)+1)/10^(#A333346_samples-1);
  check_equal(P(s^7) > 0, 1);
}

\\-----------------------------------------------------------------------------
progress("A333347 count maximum matchings");

A333347(n) =
{
  n>=0 || error();
  vpar_matchnum_and_count(make(n))[2];
}
{
  for(i=1,#A333347_samples,  \\ OFFSET=0
     my(n=i-1);
     check_equal(A333347(n), A333347_samples[i]));

  \\ formula 0 mod 7
  forstep(n=7,500,7,
     my(k=n/7);
     check_equal(k,floor(k), "k integer");
     check_equal(k>=1, 1, "k big enough");
     check_equal(A333347(n),
                 8*A190872(k) - 7*A190872(k-1),
                 "0 mod 7"));

  \\ formula 1 mod 7
  forstep(n=1,500,7,
     my(k=(n-1)/7);
     check_equal(k,floor(k), "k integer");
     check_equal(k>=0, 1, "k big enough");
     check_equal(A333347(n),
                 A190872(k+1),
                 "1 mod 7"));

  \\ formula 4 mod 7
  forstep(n=4,500,7,
     my(k=(n-4)/7);
     check_equal(k,floor(k), "k integer");
     check_equal(k>=0, 1, "k big enough");
     check_equal(A333347(n),
                 3*A333344(k),
                 "1 mod 7"));
}

\\-----------------------------------------------------------------------------
progress("A333348 matchnum");

A333348(n) =
{
  n>=0 || error();
  vpar_matchnum_and_count(make(n))[1];
}
A333348_formula(n) = floor((2*n+2)/7);
{
  for(i=1,#samples,
     my(n=i-1);  \\ OFFSET=0
     check_equal(A333348(n), A333348_samples[i]));

  \\ formula 0 mod 7
  for(n=0,200,
     if(n==2, check_equal(A333348(n), 1, "A333348(2)"),
        n==6, check_equal(A333348(n), 1, "A333348(6)"),
        n==13, check_equal(A333348(n), 3, "A333348(3)"),
        n==20, check_equal(A333348(n), 5, "A333348(20)"),
        check_equal(A333348(n), floor((2*n+2)/7))));

  check_equal(A333348_formula(2), 0, "not general at 2");
  check_equal(A333348_formula(6), 2, "not general at 6");
  check_equal(A333348_formula(13), 4, "not general at 13");
  check_equal(A333348_formula(20), 6, "not general at 20");
}


\\-----------------------------------------------------------------------------
print("total "check_count" tests");
progress("end");
