\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");


\\ Alister Justin Marcon, thesis "Semitotal Domination in Graphs".
\\
\\ Suspect the "else if" exclusives shown in the algorithm under
\\ i==provisional are not right.  Believe if i==provisional and 1 required
\\ child and all children required and the required child has no required
\\ children or grandchildren, then continue to try parent==provisional.
\\ Believe need that to make the example v=10 required rather than
\\ provisional.
\\
\\ Also, not sure about the final loop on provisionals.  The rerooted path-8
\\ comes out with single v=1 provisional due to v=3 undominated.  How does
\\ it (or a child of it) become required so as to dominate v=3?
\\
test_semitotdomnum_one_set_by_bound(vpar) =
{
  print("test_semitotdomnum_one_set_by_bound() "vpar);

  vpar_roots(vpar) == [1] || error();
  vpar_degree_of_vertex(vpar,1) > 1 || error();
  vpar_is_downwards(vpar) || error();

  my(state=vector(#vpar,i, 'bound),
     children=vpar_children_vector(vpar),
     grandchildren=apply(vec->if(#vec, concat(apply(c->children[c], vec)), []),
                        children),
     ret=List([]));

  forstep(i=#vpar,2,-1,
         print(" i="i" state "state[i]);
         if(state[i]=='bound,
            print("  set parent "vpar[i]" required");
            state[vpar[i]]='required;
            state[i]='free;
            print("  leave i="i" state = "state[i]);
            next());

         if(state[i]=='required,
            print("  add to set");
            listput(ret,i);
            if(state[vpar[i]]=='bound,
               print("  set parent "vpar[i]" provisional");
               state[vpar[i]]='provisional);
            next());
         
         if(state[i]=='free,
            next());

         state[i]=='provisional || error();
         my(num_required=0,
            num_free=0);
         for(j=1,#children[i],
            my(c=children[i][j]);
            num_required += (state[c]=='required);
            num_free     += (state[c]=='free));
         print("  num_children="#children[i]" num_required="num_required" num_free="num_free);
         if(num_required >= 2 && num_required+num_free==#children[i],
            print("   >=2 children required and all children required or free");
            state[i]='free;
            print("  leave i="i" state = "state[i]);
            next());

         if(num_required==1 && num_required+num_free==#children[i],
            print("   =1 child required and all children required or free");
            my(k);
            for(j=1,#children[i],
               k = children[i][j];
               if(state[k]=='required, break()));
            print("   k="k" child required");
            my(num_required=sum(j=1,#children[k],
                                state[children[k][j]]=='required)
                            + sum(j=1,#grandchildren[k],
                                  state[grandchildren[k][j]]=='required));
            print("   k children+grandchildren num_required="num_required);
            if(num_required >= 1,
               print("   set free");
               print("  leave i="i" state = "state[i]);
               next()));

         my(num_provisional=sum(j=1,#children[i],
                                state[children[i][j]]=='provisional));
         print("  children num_provisional="num_provisional);
         if(state[vpar[i]]=='provisional && num_provisional >= 1,
            print("   add to set");
            state[i] = 'required;
            listput(ret,i);
            for(j=1,#children[i],
               my(c=children[i][j]);
               if(state[c]=='provisional,
                  print("   child "c" provisional -> free");
                  state[c]='free));
            next());

         if(state[vpar[i]]=='bound || state[vpar[i]]=='free,
            print("   parent is "state[vpar[i]]);
            if(num_provisional >= 1,
               print("   and num_provisional>=1, add to set");
               state[i] = 'required;
               listput(ret,i);
               state[vpar[i]] = 'free;
               for(j=1,#children[i],
                  my(c=children[i][j]);
                  if(state[c]=='provisional,
                     print("   child "c" provisional -> free");
                     state[c]='free));
               ,
               state[vpar[i]] = 'provisional));

         print("  leaving i="i" state = "state[i]);
  );

  print(" .");
  print(" root state = "state[1]);
  if(state[1]=='bound || state[1]=='required,
     print("  add to set");
     state[1]='required;
     listput(ret,1));

  print(" states now "state);
  print(" set now    "Set(ret));
  print(" provisionals "Vec(select(s->s=='provisional, state, 1)));

  print(" .");
  for(i=1,#vpar,
     if(state[i]=='provisional,
        print(" i="i" provisional");
        my(num_provisional=sum(j=1,#children[i],
                               state[children[i][j]]=='provisional));
        print("  children num_provisional = "num_provisional);
        if(num_provisional >= 1,
           print("  set required");
           state[i] = 'required;
           listput(ret,i);
           for(j=1,#children[i],
              my(c=children[i][j]);
              if(state[c]=='provisional,
                 print("   child "c" provisional -> free");
                 state[c]='free));
           ,
           print("  set free");
           state[i]='free)));

  print(" .");
  print(" root grandchildren="grandchildren[1]);
  my(num_required=vecsum(apply(v->state[v]=='required,
                              concat(children[1], grandchildren[1]))));
  print("  required children+grandchildren = "num_required);
  if(num_required == 0,
     my(c=children[1][1]);   \\ first child of root
     print("  add root first child "c" (was state "state[c]")");
     state[children[1][1]] = 'required;
     listput(ret,c));

  print(" final states "state);
  Set(ret) == vecsort(Vec(ret)) || error();
  ret = Set(ret);
  print(" final set "Set(ret));

  my(ret_by_states=Vec(select(s->s=='required, state, 1)));
  ret == ret_by_states
        || error("oops, ret doesn't match states: ",ret_by_states);
  ret;
}

{
  \\ Alister Justin Marcon, thesis "Semitotal Domination in Graphs"
  \\ page 32 example
  \\           1
  \\         / | \ 
  \\       2   3   4
  \\     / |   |   |  \
  \\   5   6   7   8   9              first pass
  \\   |   |   |   |   |                 3=prov
  \\  10  11  12  13  14                15=prov ?
  \\   |    
  \\  15 
  \\   |    
  \\  16
  \\   |    
  \\  17
  my(vpar = [0,1,1,1, 2,2, 3, 4,4, 5,6,7,8,9, 10,15,16]);
  check_equal(#vpar, 17);
  check_equal(vpar_is_downwards(vpar), 1);
  my(set=test_semitotdomnum_one_set_by_bound(vpar));
  check_equal(vpar_is_semitotdomset(vpar,set), 1);
  check_equal(set, [1,6,7,8,9,10,16]);
}

{
  \\       1
  \\     /  \ 
  \\    2    4
  \\    |    |
  \\    3    6
  \\    |    
  \\    5
  \\    |    
  \\    7    
  my(vpar = [0, 1, 2, 3, 1, 4, 5]);
  \\ set  = [5, 4, 1]
  my(set=test_semitotdomnum_one_set_by_bound(vpar));
  check_equal(set, [2,4,5]);
  check_equal(vpar_is_semitotdomset(vpar,set), 1,
              "vpar_is_semitotdomset() 7");
}

{
  \\       1 P
  \\     /  \ 
  \\  F 2    3 *
  \\    |    |
  \\  * 4    5 F
  \\    |    
  \\  F 6
  \\    |    
  \\  * 7
  \\    |    
  \\  F 8    
  my(vpar = [0, 1,1, 2,3, 4,6,7]);
  check_equal(#vpar, 8);
  \\ set  = [3, 4, 7]
  my(set=test_semitotdomnum_one_set_by_bound(vpar));
  check_equal(set, [3, 5, 6]);
  check_equal(vpar_is_semitotdomset(vpar,set), 1,
              "vpar_is_semitotdomset() 8");
}

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);

     #vpar >= 1 || next();
     vpar=vpar_relabel_seq(vpar,Vecrev(vpar_seq_upqueue(vpar)));
     vpar_degree_of_vertex(vpar,1)>=2 || next();
     vpar_height(vpar)>=2 || next();

     my(set=test_semitotdomnum_one_set_by_bound(vpar));
     print("vpar = "vpar);
     print("set  = "set);
     check_equal(vpar_is_semitotdomset(vpar,set), 1,
       "test_semitotdomnum_one_set_by_bound() vs vpar_is_semitotdomset()");
     );
}


\\-----------------------------------------------------------------------------