\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,1000*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("various-CP.gp");
read("memoize.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

option_verbose=0;
progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}

\\ C. Colijn and G. Plazzotta,
\\ "A Metric On Phylogenetic Tree Shapes",
\\ Systematic Biology, volume 67, number 1,
\\ January 2018, pages 113-126, pages 113-126.
\\ https://doi.org/10.1093/sysbio/syx046

\\ A173282  cumul A001190=num 0,2 rooted binary


\\-----------------------------------------------------------------------------
\\ vpar_count_unlabelled_binary_of_height_vector()
\\ Wedderburn-Etherington numbers

\\ including a(0)=0 not in vpar_count_unlabelled_binary_vector()
A001190_vector(len) =
{
  if(len==0, [],
     concat(0, vpar_count_unlabelled_binary_vector(len-1)));
}
A001190(n) =
{
  n>=0 || error();
  A001190_vector(n+1)[n+1];
}

{
  check_equal(A001190_vector(#A001190_samples),
              A001190_samples);

  \\ A(x) = x + (1/2)*(A(x)^2 + A(x^2))
  my(g=Ser(A001190_vector(50)));
  check_equal(g == x + (1/2)*(g^2 + subst(g,x,x^2)),
             1);
}

\\--------------
\\ A173282 num rooted binary of <= n chlidless vertices

A173282_vector(len) =
{
  if(len==0,[],
     my(v=concat(0,vpar_count_unlabelled_binary_vector(len-1)));
     for(i=3,#v,v[i]+=v[i-1]);
     v);
}
A173282(n) =
{
  n>=0 || error();
  vecsum(vpar_count_unlabelled_binary_vector(n));
}
{
  OEIS_check_func("A173282");
  OEIS_check_vecfunc("A173282_vector");
}


\\-----------------------------------------------------------------------------
\\ CP_child_R()

{
  check_equal(CP_child_R(1), 0);

  check_equal(vector(1000,n,CP_child_R(n)),
              vector(1000,n,CP_children_z(n)[2]),
              "CP_child_R() vs CP_children_z()");

  check_equal(vector(1000,n, CP_child_R(n)),
              vector(1000,n, if(n==1,0, A002260_runs_1_to_k(n-1))),
              "CP_child_R() = A002260(n-1)");
}
\\-----------------------------------------------------------------------------
\\ CP_child_L()

{
  check_equal(CP_child_L(1), 0);
  check_equal(vector(1000,n,CP_child_L(n)),
              vector(1000,n,CP_children_z(n)[1]),
              "CP_child_L() vs CP_children()");

  check_equal(vector(1000,n, CP_child_L(n)),
              vector(1000,n, if(n==1,0, A002024_k_rep_k(n-1))),
              "CP_child_L() = A002024(n-1)");
}

\\-----------------------------------------------------------------------------
\\ A002260_runs_1_to_k()

OEIS_check_func("A002260_runs_1_to_k");


\\-----------------------------------------------------------------------------
\\ A002024_k_rep_k()
\\           1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5,
\\       n = 1  2  3  4

OEIS_check_func("A002024_k_rep_k");



\\-----------------------------------------------------------------------------
\\ Total number of vertices in 0,2 trees height <= n

TotalSame_by_height_vector(len) =
{
  my(S=vector(len),
     C=vector(len,i,1),
     Stotal=S,
     Ctotal=C);
  for(n=2,len-1,
     C[n+1] = if(n>1, C[n] * Ctotal[n-1])
             + (C[n]*C[n] + C[n])/2;
     Ctotal[n+1] = Ctotal[n] + C[n+1];
  );
  C;
}
\\ TotalSame_by_height_vector(10)


\\-----------------------------------------------------------------------------
\\ INVERT Transform

{
  \\ b(3)  1*1*1
  \\       1*2
  \\       2*1
  \\ b(4)  1*1*1*1
  \\       1*1*2  *3
  my(v=[1,2,5,7,13,17],
     w=[1,3,10,28, 83, 240],
     L=O(x^(#v+1)),
     g=Polrev(v)*x + L,
     gw=Polrev(w)*x + L);
  check_equal(1/(1-g) + L,  1 + gw);
  my(a(n) = n>=0 || error(); if(n==0,0, v[n]));
  my(b(n) = n>=0 || error(); if(n==0,1, w[n]));
  check_equal(vector(6,n, b(n)),
              vector(6,n, sum(i=1,n, a(i)*b(n-i))));
  \\ gw = 1/(1-g)
  \\ g*gw = gw - 1
  \\ b(0) = 1 empty list, otherwise convolution
}


\\-----------------------------------------------------------------------------
\\ Recursion

{
  my(a(n) = if(n==0,return('x)); self()(n-1)+1);
  check_equal(a(50), 'x+50);
}



\\------------------------------------------------------------------------------
\\ Rooted binary tree
\\ 0 or 2 children
\\ p*(p-1)/2 + q + 1    p >= q >= 1
\\ a_n smallest of n leaves
\\ b_n biggest of n leaves

\\ L and R are CP tree numbers.
\\ Return the tree number with them child subtrees of the root.
CP_join(L,R) =
{
  (L>=1 && R>=1) || error();
  my(m=max(L,R));
  (sqr(m)-m)>>1 + min(L,R) + 1;
}
{
  check_equal(CP_join(1,1), 2, "CP_join() 1,1");
  check_equal(CP_join(2,1), 3);
  check_equal(CP_join(2,2), 4)
}

\\-----------------------------------------------------------------------------
\\ CP_children()

{
  check_equal(CP_children_z(1), [0,0]);
}
{
  \\ with a multiply by 8
  my(CP_children_z_by_8(n) =
     n>=1 || error();
     my(R, L=sqrtint((n-1)<<3,&R));
     R>>=1;
     if(L%2==0, R+=L-1);
     L=(L+1)>>1;
     R=(R+1)>>2;
     [L, R]);
  check_equal(vector(2000,n, CP_children_z(n)),
              vector(2000,n, CP_children_z_by_8(n)));
  if(0,
     my(prev=0);
     for(n=1,500,
        my(v=CP_children_z_by_8(n));
        print(v"   "v[2]-prev);
        prev=v[2]);
     quit);
}

\\ Return a vector of the two children of tree n.
\\ n=1 has no children and the return is empty [].
CP_children(n) =
{
  n>=1 || error();
  if(n==1, [], CP_children_z(n));

     \\ my(R, L=sqrtint(n<<1,&R));
     \\ if(R>L,  R-=L;L++, R+=L);
     \\ [L,R>>1],
     \\ []);

  \\ if(n--,
  \\    my(L = (sqrtint(n<<3) + 1)>>1);
  \\    [L, n - (sqr(L) - L)>>1],
  \\    []);
}
{
  \\ print(vector(29,n, CP_children(n)));
  check_equal(CP_children(2), [1,1],
              "CP_children(2)");
  check_equal(CP_children(4), [2,2]);
  check_equal(CP_children(11), [4,4]);
  check_equal(CP_children(67), [11,11]);

  check_equal(CP_children(3), [2,1]);
  check_equal(CP_children(5), [3,1]);
  check_equal(CP_children(12), [5,1]);

  for(n=2,1000,
     my(v=CP_children(n));
     CP_join(v[1],v[2]) == n || error();
     CP_join(v[2],v[1]) == n || error());
}

CP_children_vector(len) =
{
  my(L=1,R=0);
  vector(len,i,
         if(i==1,[],
            if(R++ > L, L++;R=1);
            [L,R]));
}
{
  check_equal(CP_children_vector(7),
              [[], [1,1],
               [2,1], [2,2],
               [3,1], [3,2], [3,3]]);
  check_equal(CP_children_vector(500),
              vector(500,n,CP_children(n)),
             "CP_children_vector()");
}


\\-----------------------------------------------------------------------------
\\ A064002_CP_num_vertices()
\\    1, 3, 5, 7, 7, 9, 11, 9, 11, 13

{
  OEIS_check_func("A064002_CP_num_vertices");

  my(got=vector(2000,n,CP_num_vertices(n)));
  check_equal(got,
              vector(2000,n,#vpar_from_CPcode(n)),
              "CP_num_vertices() vs vpar");
  OEIS_check_func("A064002",CP_num_vertices);

  my(a(n)=if(n<=1,n, vecsum([self()(c) |c<-CP_children_z(n)]) + 1));
  check_equal(vector(1000,n,a(n)),
              vector(1000,n,CP_num_vertices(n)));

  \\ stack vs recrusive speed
  if(0,
     my(len=1000);
     gettime(); for(n=1,len,CP_num_vertices(n)); print("stack     ",gettime());
     gettime(); for(n=1,len,a(n)); print("recursive ",gettime());

     my(len=100000,
        n=random(2^len)+1);
     gettime(); CP_num_vertices(n); print("stack     ",gettime());
     gettime(); a(n); print("recursive ",gettime());
     quit);

  \\ num vertices new highs
  \\   occurs at A354970 smallest of n childless
  \\   each odd number of vertices
  if(0,
     my(m=0,nlist=List([]),tlist=List([]));
     for(n=1,100,
        my(t=CP_num_vertices(n));
        if(t>m, 
           m=t;
           listput(nlist,n);
           listput(tlist,t)));
     print(Vec(nlist));
     print(Vec(tlist));
     quit);
}

\\----------------------------------
\\ a064002-CP-num-vertices.gp checks

check_equal(a,'a);
check_equal(a_vector,'a_vector);
print("==============");
print("read a064002-CP-num-vertices.gp");
read("a064002-CP-num-vertices.gp");
my_A064002_CP_num_vertices = a;
my_A064002_CP_num_vertices_vector = a_vector;
kill(a);
kill(a_vector);
print("==============");
{
  my(want=vector(5000,n,CP_num_vertices(n)));
  check_equal(my_A064002_CP_num_vertices_vector(5000),
             want,
             "my_A064002_CP_num_vertices_vector()");
  check_equal(vector(5000,n, my_A064002_CP_num_vertices(n)),
             want,
             "my_A064002_CP_num_vertices()");

  \\ from a064002-CP-num-vertices.gp
  my(pair(n) =
     my(r, s=sqrtint((n-1)<<1,&r));
     if(r>s,  r-=s;s++, r+=s);
     [r>>1, s]);

  \\ NAME in A064002
  check_equal(vector(7,n,n++; pair(n)),
              [[1,1],
               [1,2], [2,2],
               [1,3], [2,3], [3,3],
               [1,4]]);
  check_equal(pair(2), [1,1]);

  check_equal(vector(100,n,n++; pair(n)),
              vector(100,n,n++; [A002260_runs_1_to_k(n-1),
                                 A002024_k_rep_k(n-1)]));
  check_equal(vector(100,n,n++; my(i,j); [i,j]=pair(n);
                     1 <= i && i <= j),
              vector(100,n,n++; 1));

  if(0,
     system("rm -f /tmp/new/b064002.txt");
     for(n=1,10000,
        write("/tmp/new/b064002.txt", Str(n," ",A064002_CP_num_vertices(n))));
     system("ls -l /tmp/new/b064002.txt");
     quit);
  OEIS_check_func("A064002_CP_num_vertices");
  if(0,
     OEIS_check_func("A064002_CP_num_vertices",,'bfile);
     check_equal(my_A064002_CP_num_vertices_vector(10000),
                 OEIS_data("A064002",'bfile)));
}

\\-----------------------------------------------------------------------------
\\ CP_num_childless()
\\ A064064 = 1, 2, 3, 4, 4, 5, 6, 5, 6, 7, 8, 5,

{
  check_equal(vector(1000,n,CP_num_childless(n)),
              vector(1000,n,vpar_num_childless(vpar_from_CPcode(n))),
              "CP_num_childless() vs vpar");

  \\ Noah Rosenberg table
  my(want=[1, 2, 3, 4, 4, 5, 6, 5, 6, 7, 8, 5, 6, 7, 8, 8, 6, 7, 8, 9, 9, 10, 7, 8, 9, 10, 10, 11, 12, 6, 7, 8, 9, 9, 10, 11, 10]);
  check_equal(vector(#want,n, CP_num_childless(n)),
              want,
              "Rosenberg example");
}
{
  \\     0, 1, 1, 1, 2, 3, 6, 11, 23, 46, 98,
  \\   n=0  2  3  4  5  6  7

  \\ my(m=vector(10));
  \\ for(n=1,10000,
  \\    my(c=CP_num_childless(n));
  \\    if(c<=#m,m[c]=n));
  \\ print(m);
  \\ my(v=vpar_count_unlabelled_binary_of_height_vector(33));
  \\ check_equal(apply(t-> CP_num_childless(t-1),
  \\                  [4..#v]),
  \\             vector(#v-3,n, n));
}

\\-----------------------------------------------------------------------------
\\ A064064_num_childless_offset()

{
  my(a=A064064_num_childless_offset);
  OEIS_check_func("A064064_num_childless_offset");

  \\ childless vertices vs total vertices
  check_equal(vector(1000,n, 2*a(n-1) - 1),
              vector(1000,n, A064002_CP_num_vertices(n)));
  check_equal(vector(1000,n, a(n)),
              vector(1000,n, (A064002_CP_num_vertices(n+1) + 1)/2));
}


\\-----------------------------------------------------------------------------
\\ CP_height()

{
  check_equal(vector(256,n,CP_height(n)),
              vector(256,n,vpar_height(vpar_from_CPcode(n))),
              "CP_height() vs vpar_height()");

  if(0,
     \\ number of trees by height
     \\ 1, 2, 7, 56, 2212, 2595782
     \\ A002658 a(0) = a(1) = 1
     \\ a(n+1) = a(n)*(a(0) + ... + a(n-1))
     \\        + a(n)*(a(n) + 1)/2.

     my(count=vector(20));
     for(n=2,3000000,
        count[CP_height(n)]++);
     while(count[#count]==0, count=count[^#count]);
     count=count[^#count];
     print(count);
     quit);


  if(0,
     my(v=vpar_count_unlabelled_binary_of_height_vector(33));
     print(vector(#v,n, CP_height(v[n])));
     print(vector(#v,n, CP_height(v[n]+1)));
     \\ not in OEIS: 0, 0, 0, 1, 2, 3, 3, 4, 4, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7
     \\ not in OEIS: 1, 1, 1, 2, 2, 3, 4, 4, 4, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7
     quit);
}
{
  \\ ascending height, so runs of num trees
  if(0,
     print(vector(100,n, CP_height(n)-1));
     quit);
}
\\ for(n=1,10, print(vpar_from_CPcode(n)));

CP_metric_d(n) =
{
  n>=1 || error();
  if(n==1,0,
     my(v=CP_children(n));
     abs(v[1]-v[2]));
}
\\ vector(50,n, CP_metric_d(n))

{
  my(want=[1, 2, 3, 4, 4, 5, 6, 5, 6, 7, 8, 5, 6, 7, 8, 8, 6, 7, 8, 9, 9, 10, 7, 8, 9, 10, 10, 11, 12, 6, 7, 8, 9, 9, 10, 11, 10]);
  check_equal(vector(#want,n, CP_num_childless(n)),
              want);
}


\\ a_n first with n childless
smallest_vector(len) =
{
  my(ret=vector(len));
  for(n=1,oo,
     my(t=CP_num_childless(n));
     if(!ret[t],
        ret[t] = n;
        if(hammingweight(ret)==#ret, return(ret))));
}
{
  my(want=[1, 2, 3, 4, 6, 7, 10, 11, 20, 22, 28, 29, 53, 56, 66, 67, 202, 211, 252, 254, 401, 407, 435, 436, 1408, 1432, 1594]);
  check_equal(smallest_vector(#want), want);

  for(n=2,#want,
     if(n%2==0,
        \\ even two same halves
        check_equal(want[n],
                    CP_join(want[n/2], want[n/2])));

     if(n%2==1,
        \\ odd one bigger by one vertex
        check_equal(want[n],
                    CP_join(want[ceil(n/2)],
                            want[floor(n/2)])));
  );
}


\\------------------------------------------------------------------------------
\\ Depths Vector
\\ a357701-CP-depths-vector--test.gp

check_equal(row,'row);
print("==============");
print("read a357701-CP-depths-vector.gp");
read("a357701-CP-depths-vector.gp");
CP_depths_vector = row;
A357701_CP_depths_vector = row;
kill(row);
print("==============");

{
  my(row=A357701_CP_depths_vector);
  my(got=vector(1000,n, row(n)));
  check_equal(got,
              vector(#got,n,vpar_depths_vector(vpar_from_CPcode(n))),
              "CP_depths_vector() vs vpar_depths_vector()");
}


\\------------------------------------------------------------------------------
\\ Depths Sum = Total Path Length

check_equal(a,'a);
check_equal(a_vector,'a_vector);
print("==============");
print("read a357702-CP-depths-total.gp");
read("a357702-CP-depths-total.gp");
A357702_CP_depths_total = a;
A357702_CP_depths_total_vector = a_vector;
kill(a);
kill(a_vector);
print("==============");

{
  my(a=A357702_CP_depths_total);
  check_equal(a(1), 0);
  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++; 
    vecsum(A357701_CP_depths_vector(n))));
}


\\-----------------------------------------------------------------------------
\\ Shorten by Cherry to Singleton
\\ delete childless siblings

vpar_shorten_cherry(vpar) =
{
  my(num_children=vpar_num_children_vector(vpar),
     sets=vpar_siblings_sets(vpar),
     flags=vector(#vpar));
  sets = select(set->vecmax(vecextract(num_children,set))>0, sets);
  sets = setunion(sets, [vpar_roots(vpar)]);
  vpar_keep_vertices(vpar,Set(concat(sets)));
}
{
  check_equal(vpar_shorten_cherry([0,1,1]), [0]);
}

CP_shorten_cherry(n) =
{
  n>=1 || error();
  if(n<=2,1,
     my(v=vecsort(apply(self(),CP_children_z(n))));
     (sqr(v[2])-v[2])>>1 + v[1] + 1);
}
{
  check_equal(vpar_from_CPcode(2), [0,1,1]);
  check_equal(CP_shorten_cherry(2), 1);

  check_equal(vpar_from_CPcode(5),
              [0,1,2,3,3, 2, 1]);
  check_equal(vpar_shorten_cherry([0,1,2,3,3, 2, 1]),
              [0,1,2, 2, 1]);
  check_equal(vpar_to_CPcode([0,1,2,2, 1]), 3);
  check_equal(CP_shorten_cherry(5), 3,
              "CP_shorten_cherry() 5");

  check_equal(vector(1000,n, CP_shorten_cherry(n)),
              vector(1000,n,
                     vpar_to_CPcode(vpar_shorten_cherry(vpar_from_CPcode(n)))),
              "CP_shorten_cherry() vs vpar");

  check_equal(CP_shorten_cherry(6), 3);
  if(0,
     print(vector(50,n,n+=3;CP_shorten_cherry(n)));
     \\ not in OEIS: 2, 3, 3, 4, 3, 3, 4, 4, 5, 5, 6, 6, 7, 5, 5
     quit);
}



\\-----------------------------------------------------------------------------
\\ CP_num_orbits()

{
  my(a=CP_num_orbits);
  check_equal(vector(1024,n, a(n)),
              vector(1024,n,vpar_num_orbits(vpar_from_CPcode(n))),
              "CP_num_orbits() vs vpar");
}

test_is_binary_tree(vpar) = \
  test_set_is_subset(Set(vpar_num_children_vector(vpar)), [0,2]);

{
  \\ total orbits in unordered binary trees
  \\ 1, 2, 4, 9, 20, 46, 106, 248, 582, 1376
  \\ A085748 =  gf 1/(1-A001190(x))
  \\ where A001190 = Wedderburn-Etherington counts
  \\
  \\ 1/(1 - (x + x^2 + x^3 + 2*x^4 + 3*x^5 + O(x^6)))
  \\ x * A(x)^2 = B(x*A(x^2))
  \\ B(x) = x/(1 - 2*x)
  \\
  \\ cf all unlabelled A000107
  \\ G.f.: A000081(x)/(1-A000081(x))
  \\ INVERT transform, list of unlabelled things

  if(0,
     my(C=List([]),
        T=List([]));
     for(n=1,10,
        \\ only ever odd num vertices for binary tree
        my(vpar=vpar_unlabelled_first(2*n-1),
           total=0,
           count=0);
        until((vpar=vpar_unlabelled_next(vpar)) == 'none,  \\ trees
              \\ test_is_binary_tree(vpar) || next;
              total += vpar_num_orbits(vpar);
              count++);
        \\ print1(total", ");
        print("n="n" count "count" total orbits "total);
        listput(C,count); listput(T,total);
     );
     print();
     print(Vec(C));
     print(Vec(T));
     quit);
}


\\-----------------------------------------------------------------------------
\\ Complete Binary Tree Sans 1 Childless Pair
\\ A072638, precedes complete binary

A072638(n) =
{
  \\ (x+1)*(x+2)/2 - 1 == (x^2 + 3*x)/2
  n>=0 || error();
  my(ret=0);
  for(i=1,n, ret+=2; ret = (sqr(ret)-ret)>>1);
  ret;
}
{
  OEIS_check_func("A072638");
  check_equal(A072638(0), 0);
  check_equal(A072638(1), 1);

  \\      3
  \\     / \
  \\    2   1
  \\   / \
  \\  1   1

  check_equal(A072638(2), 3);  \\ cf complete is 4

  \\ with n levels
  check_equal(vector(5,n,n++;A072638(n-1)),
              vector(5,n,n++;
    my(vpar=vpar_make_complete_tree(2,n-1));
    vpar_num_children_of_vertex(vpar,#vpar) == 0 || error();
    vpar_num_children_of_vertex(vpar,#vpar-1) == 0 || error();
    vpar=vpar[1..#vpar-2];  \\ sans last 2
    vpar_to_CPcode(vpar)));

}

\\-----------------------------------------------------------------------------
\\ Complete Binary Tree
\\   1, 2, 4, 11, 67, 2279, 2598061, 3374961778892
\\ A006894 Number of rooted binary trees height < n

A006894_CP_compelte_binary(n) =
{
  n>=1 || error();
  my(ret=0);
  for(i=1,n, ret = (sqr(ret)+ret)>>1 + 1);
  ret;
}
{
  my(a=A006894_CP_compelte_binary);
  OEIS_check_func("A006894_CP_compelte_binary");

  my(by_binomial(n) =
     n>=0 || error();
     my(ret=0);
     for(i=1,n, ret = binomial(ret+1,2) + 1);
     ret);
  check_equal(vector(10,n, a(n)),
              vector(10,n, by_binomial(n)));
  if(0,
     \\ about 1.4x speedup with sqr()
     my(n=28);
     gettime(); print(logint(a(n),2)); print(gettime());
     gettime(); print(logint(by_binomial(n),2)); print(gettime());
     quit);

  \\       4
  \\     /  \
  \\    2    2
  \\   / \  / \
  \\  1  1  1  1

  check_equal(a(3), 4);  \\ cf complete is 4

  \\ with n levels
  check_equal(vector(5,n,a(n)),
              vector(5,n,
    vpar_to_CPcode(vpar_make_complete_tree(2,n-1))));

}
{
  if(0,
     \\
     print(vector(8,n, vpar_to_CPcode(vpar_make_complete_tree(2,n-1))));

     for(n=1,8,
        print(vpar_from_CPcode(1+vpar_to_CPcode(vpar_make_complete_tree(2,n-1)))));
     quit);
}
{
  \\ 7, 56, 2212, 2595782, 3374959180831
  \\ A103410
  \\ cf A002658

  if(0,
     my(upto=2,v=[1]);
     for(n=1,6,
        my(len=vecsum(v));
        v=[upto..upto+len-1];
        upto+=len;
        \\ print(v);
        print(vecsum(v));
     );
     quit);
}



\\-----------------------------------------------------------------------------
\\ Vpar

{
  check_equal(vpar_from_CPcode(1), [0], "singleton");
  check_equal(vpar_from_CPcode(2), [0,1,1], "2 children");


  \\           12
  \\          / \
  \\         5   1
  \\        / \
  \\       3   1
  \\      / \
  \\     2   1
  \\    / \
  \\   1  1
  check_equal(vpar_from_CPcode(12),
              [0,1,2,3,4,4, 3,2,1]);

  \\        4              1     [0,1,2,2, 1,5,5]
  \\      /   \          /   \
  \\     2     2        2     5
  \\    / \   / \      / \   / \
  \\   1  1   1  1    3  4   6  7
  check_equal(vpar_from_CPcode(4), [0,1,2,2,1,5,5]);

  \\               11
  \\            /      \
  \\        4             4
  \\      /   \         /   \
  \\     2     2       2     2
  \\    / \   / \     / \   / \
  \\   1  1   1  1   1  1   1  1
  check_equal(vpar_from_CPcode(11),
    [0,1,2,3,3, 2,6,6, 1,9,10,10, 9,13,13],
              "complete height=3");
}
\\ vpar_from_CPcode(67)
\\ quit

\\ Return 1 if every vertex of vpar has 0 or 2 children.
vpar_is_binary(vpar) =
{
  my(count=vectorsmall(#vpar),
     histogram=vector(3),
     p);
  for(v=1,#vpar,
     if((p=vpar[v]),
        if(count[p], histogram[count[p]]--);
        histogram[count[p]++]++;
        if(histogram[3], return(0))));
  !histogram[1];
}
{
  check_equal(vpar_is_binary([0]), 1,
              "vpar_is_binary() singleton");
  check_equal(vpar_is_binary([0,1]), 0);
  check_equal(vpar_is_binary([0,1,1]), 1);
  check_equal(vpar_is_binary([0,1,1,2]), 0);
  check_equal(vpar_is_binary([0,1,1,2,2]), 1);
}

{
  check_equal(vpar_to_CPcode_vector([0]), [1]);
  check_equal(vpar_to_CPcode_vector([0,1,1]), [2,1,1]);
  check_equal(vpar_to_CPcode_vector([0,1,1,2,2]),
              [3, 2, 1, 1, 1]);
}

{
  check_equal(vpar_to_CPcode([0]), 1);
  check_equal(vpar_to_CPcode([0,1,1]), 2);
  check_equal(vpar_to_CPcode([0,1,1,2,2]), 3,
              "vpar_to_CPcode() for 3");
  for(n=1,1000,
     my(vpar=vpar_from_CPcode(n));
     vpar_validate(vpar);
     vpar_is_binary(vpar)  || error();
     vpar_to_CPcode(vpar) == n || error());

  \\                 84
  \\             /        \
  \\          13             5
  \\        /    \          / \
  \\       5      2        3   1
  \\      / \    / \      / \
  \\     3   1  1   1    2   1
  \\    / \             / \
  \\   2   1           1   1
  \\  / \
  \\ 1   1
  my(depths=[0,1,2,3,4,5,5,4,3, 2,3,3,
             1,2,3,4,4,3,2],
     vpar=vpar_make_preorder_depths(depths));
  check_equal(vpar_depths_vector(vpar), depths,
              "84 depths");
  vpar_validate(vpar);
  \\ vpar_view(vpar);
  check_equal(vpar_num_children_vector(vpar),
    [2,2,2,2,2,0,0,0,0,2,0,0,2,2,2,0,0,0,0],
             "84 num children");
  check_equal(vpar_is_binary(vpar), 1,
              "84 is binary");
  check_equal(vpar_to_CPcode_vector(vpar),
    [84,13,5,3,2,1,1,1,1, 2,1,1, 5,3,2,1,1,1,1],
              "84 sample");
  check_equal(vpar_to_CPcode(vpar), 84,
              "CP sample 84");
}


\\-----------------------------------------------------------------------------
\\ A356121 Matula of CP

check_equal(a,'a);
print("==============");
print("read a356121-Matula-of-CP.gp");
read("a356121-Matula-of-CP.gp");
A356121_Matula_of_CP = a;
kill(a);
print("==============");

A356121_Matula_of_CP_vector(len) =
{
  my(v=vector(len,i,1),
     pr=vector(len),     \\ pr[i] = prime(v[i])
     x=1,
     y=1);  \\ x >= y >= 1
  for(n=2,len,
     if(!pr[x], pr[x] = prime(v[x]));
     v[n] = pr[x] * pr[y];
     if(y++ > x, x++;y=1));
  v;
}


{
  my(a=A356121_Matula_of_CP);
  OEIS_check_func("A356121_Matula_of_CP");

  check_equal(vector(50,n, a(n)),
              vector(50,n, vpar_to_primecode(vpar_from_CPcode(n))),
              "A356121_Matula_of_CP() vs vpar");

  \\ A356121_Matula_of_CP_vector()
  check_equal(A356121_Matula_of_CP_vector(50),
              vector(50,n, a(n)),
              "A356121_Matula_of_CP_vector()");
  check_equal(vector(10,len,len--; #A356121_Matula_of_CP_vector(len)),
              vector(10,len,len--; len),
              "A356121_Matula_of_CP_vector() lengths");

  \\ code caterpillar
  \\ peak prime 8455786
  check_equal(vpar_from_CPcode(3374961778893),
              [0,1,2,3,4,5,6,7, 8,8, 7,6,5,4,3,2,1]);
  check_equal(a(3374961778893),
              300427382);
  check_equal(vpar_from_CPcode(408),
              [0,1,2,3,4,5,5, 4,3, 9,10,10, 9,
               2,14,15,16,16, 15,14, 20,21,21,20, 1]);
  check_equal(CP_children(407), [28, 28]);
  check_equal(a(28), 316 42 861);
  check_equal(CP_children(408), [29, 1]);
  check_equal(a(29), 252 079 129);
  \\ my(n=252 079 129);  /* lower bound */  \
  \\ n*(log(n) + log(log(n)) - 3/2) + 1
  \\The 252,079,129th prime is 5,383,070,131.

  my(children(n) =
     my(r, s=sqrtint((n-1)<<1,&r));
     if(r>s,  r-=s;s++, r+=s);
     [s, r>>1]);
  check_equal(vector(100,n,n++; children(n)),
              vector(100,n,n++; [A002024_k_rep_k(n-1),
                                 A002260_runs_1_to_k(n-1)]));

  \\ make DATA
  if(0,
     print(vector(15,n, a(n)));
     for(n=1,37,
        print1(a(n),",");
        if(n==17||n==28,print()));
     print(); quit);

  \\ A111299  Matula binary trees (0 or 2 children)
  if(0,
     print(select(n->setminus(Set(vpar_num_children_vector(vpar_from_primecode(n))),[0,2])==[], [1..100]));
     quit);

  if(0,
     \\ How long for too big for Matula.
     \\ Needs bigger primes table for more than short file.
     my(total=0);
     for(n=1,oo,
        my(got=a(n));
        total += length(Str(n" "got"\n"));
        print(n" "got"  "vpar_from_CPcode(n));
        print("   b-file length ",total));
     quit);
  if(0,
     \\ CP of caterpillar lasts longer
     my(caterpillars=OEIS_data("A108225")[^1]);
     for(i=1,#caterpillars,
        my(n=caterpillars[i]);
        print("i="i" "n" "CP_to_Matula(n)"  "vpar_from_CPcode(n))));
}


  \\ a(n) = if(n==1,1,
  \\           my(x,y); [x,y]=children(n);
  \\           x=self()(x);
  \\           y=self()(y);
  \\           print("prime of x=",x);
  \\           x=prime(x);
  \\           print("prime of y=",y);
  \\           y=prime(y);
  \\           x*y);



\\-----------------------------------------------------------------------------
\\ Num Childful

CP_num_childful(n) =
{
  if(n==1,0,
     my(x,y); [x,y]=CP_children_z(n);
     1 + self()(x) + self()(y));
}
{
  check_equal(vector(1000,n,CP_num_childful(n)),
              vector(1000,n,vpar_num_childful(vpar_from_CPcode(n))),
              "CP_num_childful() vs vpar");

  if(0,
     print(vector(30,n,n+=3;CP_num_childful(n)));
     \\ not in OEIS: 3, 3, 4, 5, 4, 5, 6, 7, 4, 5, 6, 7, 7, 5, 6, 7, 8, 8, 9, 6, 7, 8, 9, 9, 10, 11, 5, 6, 7, 8
     \\ not A120244= 3, 3, 4, 5, 4, 5, 6, 7, 4, 5, 6, 7, 8 <-- different
     quit);
}


\\-----------------------------------------------------------------------------
\\ A108225  Largest CP of n Childless
\\          Smallest CP of height
\\ caterpillar
\\ 0, 2, 3, 5, 12, 68, 2280, 2598062, 3374961778893,
\\
\\ starting x=0 which becomes x=2
\\ (1/2)*x^2 - (1/2)*x + 2

CP_caterpillar(n) =
{
  n>=
  my(ret=1);
  for(i=2,n,
     ret=CP_join(ret,1));
  ret;
}
{
  check_equal(vector(8,n,CP_num_childless(CP_caterpillar(n))),
              vector(8,n,n),
              "CP_caterpillar() num childless");
}
A108225(n) =
{
  n>=0 || error();
  my(t=0); for(i=1,n, t=(sqr(t)-t)>>1 + 2); t;
}
{
  my(a=A108225);
  OEIS_check_func("A108225");
  check_equal(vector(8,n, if(n==1,1, A108225(n-1))),
              vector(8,n, CP_caterpillar(n)));
  check_equal(vector(16,n, valuation(A108225(n),2)),
              [1, 0, 0, 2, 2, 3, 1, 0,
               4, 1, 0, 0, 0, 5, 1, 0]);
  \\ not in OEIS: 2, 2, 3, 1, 0,               4, 1, 0, 0, 0, 5, 1, 0

  check_equal(#digits(A108225(14)),  783);
  check_equal(#digits(A108225(15)), 1566);

  \\ A072638
  check_equal(CP_caterpillar(5), 12);
  check_equal(vpar_from_CPcode(CP_caterpillar(5)),
              [0, 1, 2, 3, 4, 4, 3, 2, 1],
              "CP_caterpillar(5) to vpar");
  
  \\ n = A108225(h) is smallest n with height h
  check_equal(vector(10,n, CP_height(A108225(n))),
              vector(10,n, n));
  check_equal(vector(10,n, CP_height(A108225(n)-1)),
              vector(10,n, n-1));
  my(got=vector(6));
  for(n=2,oo,
     if(hammingweight(got)==#got,break);
     my(h=CP_height(n));
     if(!got[h], got[h]=n));
  check_equal(got,
              vector(#got,h, A108225(h)));
}

{
  check_equal(x,'x);
  if(0,
     for(n=1,10, print(n"  "to_binary(CP_caterpillar(n))));
     my(p=(1/2)*'x^2 - (1/2)*'x + 2,
        t=p);
     for(n=1,3,
        t=subst(p,'x,t);
        my(c=denominator(polcoeff(t,poldegree(t))));
        print(n"  "t*c));
     quit);
}
{
  \\ height 30  85182138 bits
  \\            10,647,767   10 mbytes
  if(0,
     print(vector(15,n, logint(CP_caterpillar(n),2)));
     \\ not in OEIS: 6, 11, 21, 41, 82, 163, 325, 650, 1300
     my(t=1);
     for(n=0,50,
        if(1<=n && n<=10, t==A108225(n) || error());
        print(n,"  ",logint(t,256));
        t=CP_join(t,1));
     quit);
}

\\-----------------------------------------------------------------------------
CP_subtrees_preorder(n) =
{
  if(n==1,[1],
     concat(concat(n,apply(self(),CP_children(n)))));
}

\\-----------------------------------------------------------------------------
\\ A356917  Subtree List L_n
\\ All subtrees ocurring, ascending order, with multiplicity.

print("=========\nread a356917-CP-subtrees-L.gp");
read("a356917-CP-subtrees-L.gp");
print("=========\n");
A356917_CP_subtrees_L = row;
kill(row);


A356917_CP_subtrees(n) = vecsort(CP_subtrees_preorder(n))
{
  my(row=A356917_CP_subtrees_L);
  check_equal(row(1), [1], "tree 1");
  check_equal(row(2), [1,1,2], "tree 2");
  check_equal(row(3), [1,1,1,2,3]);
  check_equal(row(4), [1,1,1,1,2,2,4]);
  check_equal(row(11), [1,1,1,1,1,1,1,1,2,2,2,2,4,4,11]);
  check_equal(row(12), [1,1,1,1,1,2,3,5,12]);

  \\ DATA
  OEIS_check_func("A356917_CP_subtrees");
  my(want=[1, 1, 1, 2, 1, 1, 1, 2, 3, 1, 1, 1, 1, 2, 2, 4, 1, 1, 1, 1, 2, 3, 5, 1, 1, 1, 1, 1, 2, 2, 3, 6, 1, 1, 1, 1, 1, 1, 2, 2, 3, 3, 7, 1, 1, 1, 1, 1, 2, 2, 4, 8, 1, 1, 1, 1, 1, 1, 2, 2, 2, 4, 9, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3, 4, 10, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 4, 4, 11]);
  check_equal(concat(vector(11,n,row(n))), want);
  if(0,
     my(v=concat(vector(11,n,row(n))));
     for(n=1,#v,
        print1(v[n],",");
        if(n==29||n==58,print()));
     print();
     quit);

  \\ bfile
  if(0, OEIS_check_func("A356917_CP_subtrees",,'bfile));
  if(0,
     system("mkdir -p /tmp/new");
     system("rm -f /tmp/new/b356917.txt");
     my(n=1);
     for(r=1,500,
        my(v=row(r));
        for(i=1,#v,
           write("/tmp/new/b356917.txt", Str(n," ",v[i]));
           n++));
     system("ls -l /tmp/new/b356917.txt");
     quit);

  \\ example triangle
  my(want=[[  1 ],
           [  1, 1, 2 ],
           [  1, 1, 1, 2, 3 ],
           [  1, 1, 1, 1, 2, 2, 4 ],
           [  1, 1, 1, 1, 2, 3, 5 ],
           [  1, 1, 1, 1, 1, 2, 2, 3, 6 ],
           [  1, 1, 1, 1, 1, 1, 2, 2, 3, 3, 7 ],
           [  1, 1, 1, 1, 1, 2, 2, 4, 8 ] ]);
  check_equal(vector(8,n,row(n)), want);
  if(0,
     for(n=1,8,
        my(v=row(n));
        print1("%e   n=",n,": ");
        for(i=1,#v, print1(" ",v[i],","));
        print());
     quit);

  check_equal(vector(500,n, #row(n)),
              vector(500,n, CP_num_vertices(n)));

  check_equal(vector(500,n, row(n)),
              vector(500,n, A356917_CP_subtrees_L(n)));

  \\ example n=6
  \\          6                1
  \\        /   \            /   \
  \\      3       2        2       7
  \\     / \     / \      / \     / \
  \\    2   1   1   1    3   6   8   9
  \\   / \              / \
  \\  1   1            4   5
  \\       1 2 3 4 5  6  7 8 9
  my(vpar=[0,1,2,3,3, 2, 1,7,7]);
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar),1);
  check_equal(vpar_to_CPcode_vector(vpar),
              [6, 3, 2, 1, 1, 1, 2, 1, 1]);

  if(0,
     \\ sum of subtree numbers, not very meaningful
     print(vector(30,n,n++; vecsum(row(n))));
     \\ not in OEIS: 4, 8, 12, 14, 18, 23, 21, 25, 30, 35, 27, 31, 36, 41
     quit);
}

{
  my(row=A356917_CP_subtrees_L);
  my(row_unsorted(n) =
     if(n==1, [1],
        concat(concat(apply(self(), CP_children(n))), n)));
  check_equal(vector(100,n, row(n)),
              vector(100,n, vecsort(row_unsorted(n))));

  check_equal(vector(10,n, row_unsorted(n)),
              vector(10,n,
    my(vpar=vpar_from_CPcode(n),
       codes=vpar_to_CPcode_vector(vpar),
       seq=vpar_seq_postorder(vpar));
    vecextract(codes,seq)));

  \\ speed sorted or unsorted
  if(0,
     my(len=10000);
     gettime(); for(n=1,len,row(n)); print("row      ",gettime());
     gettime(); for(n=1,len,row_unsorted(n)); print("unsorted ",gettime());

     my(len=100000,
        n=random(2^len)+1);
     print("num vertices ",CP_num_vertices(n));
     gettime(); row(n); print("row      ",gettime());
     gettime(); row_unsorted(n); print("unsorted ",gettime());
     quit);

}


\\-----------------------------------------------------------------------------
\\ Distinct Subtree Numbers

CP_distinct_subtrees(n) =
{
  my(r);
  concat(if(n<2,[],fold(setunion,apply(self(),CP_children(n)))),n);
}


\\-----------------------------------------------------------------------------
\\ multiset_symdiff_size()

\\ multiset_symdiff_size(u,v) takes u and v vectors of numbers in ascending
\\ order.  Return the number of terms in their symmetric difference,
\\
\\         sum    abs( count(u,t) - count(v,t) )
\\    t term in u or v
\\
\\    count(vec,elem) = number of times elem occurs in vector vec
\\
\\ The effect is an exclusive-OR where elements common to u and v are
\\ discarded, and only the excess elemnts in one over the other are counted.
\\
multiset_symdiff_size(u,v) =
{
  \\ start by assuming all of u,v are distinct,
  \\ then drop by 2 for each element common to both
  my(ret=#u+#v, i=1,j=1);
  while(i<=#u && j<=#v,
        if(u[i]==v[j], ret-=2; i++; j++,
           u[i]<v[j], i++, j++));
  ret;
}
{
  check_equal(multiset_symdiff_size([1,1,1],
                                    [1,1]),  1);
  check_equal(multiset_symdiff_size([1,1,1,2],
                                    [1,1,2]),  1);
  check_equal(multiset_symdiff_size([1,1,1,2,2,2],
                                    [1,1,2]),  3);
}


\\-----------------------------------------------------------------------------
\\ d_1(u,v) Symdiff / Manhattan Distance

print("=========\nread a356918-CP-distance-d1.gp");
read("a356918-CP-distance-d1.gp");
print("=========\n");
A356918_CP_distance_d1_T_of_upload = T;
kill(T);
A356918_CP_distance_d1_T(n,k) =
{
  (1 <= k && k <= n) || error();
  A356918_CP_distance_d1_T_of_upload(n,k);
}
A356918_CP_distance_d1_row(n) =
{
  n>=1 || error();
  vector(n,k, A356918_CP_distance_d1_T(n,k));
}

{
  my(T = A356918_CP_distance_d1_T,
     row = A356918_CP_distance_d1_row);
  my(CP_d1_distance_by_symdiff(n,k) =
    (n>=1 && k>=1) || error();
    multiset_symdiff_size(A356917_CP_subtrees_L(n),
                          A356917_CP_subtrees_L(k)));
  check_equal(vector(#want,n, row(n)),
              vector(#want,n, vector(n,k,
                CP_d1_distance_by_symdiff(n,k))));

  \\ T versus d1.gp generated from d1-table.R
  if(0,
     read("/tmp/d1.gp");
     check_equal(vector(#want,n,n++; vector(n,k,
                   if(k==1,'S,T(n,k)))),
                 want);
     quit);

  \\ column k=1
  \\ T(n,1) = num vertices - 1
  check_equal(vector(100,n, T(n,1)),
              vector(100,n,
                A064002_CP_num_vertices(n) - 1),
              "d1(n,1) = num vertices - 1");
  if(0,
     print(vector(30,n,n++; CP_num_vertices(n) - 1));
     \\ not in OEIS: 2, 4, 6, 6, 8, 10, 8, 10, 12, 14, 8, 10, 12, 14, 14, 10, 12, 14, 16, 16, 18, 12, 14, 16, 18, 18, 20, 22, 10, 12
     \\ not A120244 after initial terms
     quit);

  \\ diagonal T(n,n)
  check_equal(vector(100,n, T(n,n)),
              vector(100,n, 0));

  my(coords(n) = [A002024_k_rep_k(n),
                  A002260_runs_1_to_k(n)]);
  check_equal(vector(7,n,coords(n)),
              [[1, 1],
               [2, 1], [2, 2],
               [3, 1], [3, 2], [3, 3],
               [4, 1]]);
  my(a(n) = my(k); [n,k]=coords(n); T(n,k));
  check_equal(concat(vector(4,n,row(n))),
              vector(10,n,a(n)));

  \\ a(m) flat sequence
  \\ n,k = 1,1   is flat m=2
  check_equal(vector(10,m,m++; a(m)),
              vector(10,m,m++;
    my(x,y); [x,y]=CP_children(m+1); T(x,y)),
             "flat sequence subtrees");

  \\ DATA
  \\ OEIS_check_func("A356918",row,['offset,1]);
  my(want=[0, 2, 0, 4, 2, 0, 6, 4, 4, 0, 6, 4, 2, 4, 0, 8, 6, 4, 4, 4, 0, 10, 8, 6, 6, 6, 4, 0, 8, 6, 6, 2, 6, 4, 6, 0, 10, 8, 8, 4, 8, 6, 6, 4, 0, 12, 10, 8, 6, 8, 6, 6, 6, 4, 0, 14, 12, 12, 8, 12, 10, 10, 8, 6, 6, 0, 8, 6, 4, 6, 2, 4, 6, 6, 8, 8, 12, 0]);
  check_equal(concat(vector(12,n,row(n))),want);
  if(0,
     my(v=concat(vector(12,n,row(n))));
     for(n=1,#v,
        print1(v[n],",");
        if(n==28||n==55,print()));
     print();
     quit);

  \\ bfile
  if(0, OEIS_check_func("A356918",row,'bfile));
  if(0,
     system("mkdir -p /tmp/new");
     system("rm -f /tmp/new/b356918.txt");
     my(n=1);
     for(r=1,150,
        my(v=row(r));
        for(i=1,#v,
           write("/tmp/new/b356918.txt", Str(n," ",v[i]));
           n++));
     system("ls -l /tmp/new/b356918.txt");
     quit);

  \\ example triangle
  my(want=[[  0 ],    
           [  2, 0 ],
           [  4, 2, 0 ],
           [  6, 4, 4, 0 ],
           [  6, 4, 2, 4, 0 ],
           [  8, 6, 4, 4, 4, 0 ],
           [ 10, 8, 6, 6, 6, 4, 0 ],
           [  8, 6, 6, 2, 6, 4, 6, 0]]);
  check_equal(vector(8,n,row(n)), want);
  if(0,
     my(num_rows=8);
     printf("%%e Triangle begins:\n%%e        k=");
     for(k=1,num_rows, printf("%-3d",k));
     print();
     for(n=1,num_rows,
        printf("%%e   n=%d:  ", n);
        for(k=1,n, printf("%2d,",T(n,k)));
        print());
     quit);

  \\ example 68,4
  my(n=68,k=4);
  check_equal(A356917_CP_subtrees_L(n),
          [ 1,1,1,1,1,1, 2,   3,    5,12,68 ],
             "row 68");
  check_equal(A356917_CP_subtrees_L(k),
          [ 1,1,1,1,     2,2,    4 ],
              "row 4");
  my(d=[            1,1,   2, 3, 4, 5,12,68 ]);
  check_equal(#d,      8, "d len");
  check_equal(T(n,k),  8, "T(68,4)");
  check_equal(T(68,4), 8, "T(68,4)");
}

my(children=CP_children_z;); \
CP_subtrees_to_map(~m,n,add) =
{
  my(count=0); mapisdefined(m,n,&count); mapput(m,n,count+add);
  if(n>1,
     foreach(children(n),child, self()(~m,child,add)));
}
{
  my(m=Map());
  CP_subtrees_to_map(~m,5,1);
  check_equal(Mat(m),
              [1, 4;
               2, 1;
               3, 1;
               5, 1],
             "CP_subtrees_to_map() 5");
}
my(children=CP_children_z;); \
CP_d1_distance_by_map(n,k) =
{
  my(m=Map());
  forstep(add=-1,1,2,
     my(stack=List([n]));
     while(#stack,
           n=stack[#stack];
           my(net=0); mapisdefined(m,n,&net);
           mapput(m,n,net+add);
           if(n==1, listpop(stack),
              my(x,y); [x,y]=children(n);
              stack[#stack] = x;
              listput(stack,y)));
     n = k);
  vecsum(abs(Mat(m)[,2]));
}
{
  check_equal(matrix(10,10,n,k,
    if(k>n,'none, A356918_CP_distance_d1_T(n,k))),
              matrix(10,10,n,k,
    if(k>n,'none, CP_d1_distance_by_map(n,k))));
}
{
  \\ some rows for picking an example
  if(0,
     my(show(n) =
        my(v=A356917_CP_subtrees_L(n));
        printf("%2d  ",n);
        for(i=1,#v,
           if(i>1 && v[i]!=v[i-1], print1(" "));
           print1(v[i],","));
        print());
     for(n=1,10000,
        CP_num_vertices(n) <= 12 || next;
        show(n));
     quit);
}


\\-----------------------------------------------------------------------------
\\ CP_num_cherries()

{
  check_equal(CP_children(2), [1,1]);
  check_equal(vector(100,n,CP_num_cherries(n)),
              vector(100,n,
                     my(v=CP_subtrees_preorder(n));
                     sum(i=1,#v, v[i]==2)));
  if(0,
     print(vector(50,n,n+=3; CP_num_cherries(n)));
     \\ not in OEIS: 2, 1, 2, 2, 2, 3, 3, 4, 1, 2, 2, 3, 2, 2, 3, 3, 4
     quit);
}


\\-----------------------------------------------------------------------------
\\ Num Same Subtree Siblings
\\ 2^s Automorphisms

CP_num_same_siblings(n) =
{
  n>=1 || error();
  my(x,y);
  if([x,y]=CP_children_z(n),
     if(x==y, 2*self()(x) + 1,
              self()(x) + self()(y)),
     0);
}
{
  my(a=CP_num_same_siblings);

  check_equal(vector(256,n,2^a(n)),
              vector(256,n,vpar_num_automorphisms(vpar_from_CPcode(n))),
              "CP_num_same_siblings() for num automorphisms");

  check_equal(vector(256,n,a(n)),
              vector(256,n,
    my(vpar=vpar_from_CPcode(n),
       ids=vpar_automorphism_ids_vector(vpar),
       siblings=vpar_siblings_sets(vpar));
    sum(i=1,#siblings,
        my(s=siblings[i]);
        s==[1] || #s==2 || error();
        #s==2 && ids[s[1]]==ids[s[2]])),
              "CP_num_same_siblings() vs vpar_automorphism_ids_vector()");

  my(v=select(n->CP_num_same_siblings(n)==1,
              [1..10000]));
  check_equal(v, vector(#v,n,A108225(n)));
}

{
  \\ total by height
  \\ not in OEIS: 1, 4, 24, 330, 24120
  \\ not in OEIS: 1, 5, 29, 359, 24479
  if(0,
     my(v=vector(5,n,
       sum(i=A006894_CP_compelte_binary(n)+1,
           A006894_CP_compelte_binary(n+1),
           CP_num_same_siblings(i)
       )));
     print(v);
     print(vector(#v,n,sum(i=1,n,v[i])));
     quit);

  \\           root
  \\       /          \
  \\  height n-1     height <= n-1
  \\
  \\  S(n-1) * Ctotal(n-1
  \\   Sum i=1,n-1  C(i) * S(n-1)
  \\   Sum i=1,n-1, S(i) * C(n-1)   \\ counts twice

  \\   *     *         *         *
  \\        / \       / \      /   \   1 + 3 = 4
  \\       *   *     *   *    *     *
  \\   0     1      / \      / \   / \
  \\               *   *    *   * *   *

  if(0,
     \\ 1, 1, 4, 24, 330, 24120
     my(C=OEIS_data("A002658"));
     check_equal(C[1..4], [1, 1, 2, 7]);
     my(S(n) =
        n>=1 || error();
        if(n==1, return(0));
        if(n==2, return(1));
     sum(i=1,n-2, self()(n-1) * C[i]   \\ in first
                + C[n-1] * self()(i))  \\ in second
     + self()(n-1) * C[n-1]  \\ h equal one way
     + self()(n-1)
     + C[n-1]  \\ root over x=y subtrees
       );
     my(n=3);
     check_equal(S(2), 1);
     check_equal(C[1], 1);
     check_equal(C[2], 1);
     print(vector(8,n,S(n)));
     \\ 0, 1, 4, 24, 330, 24120, 55789920, 145009215878400
     quit);
}

\\-----------------------------------------------------------------------------
\\ Num Automorphisms

CP_num_automorphisms(n) =
{
  n>=1 || error();
  my(x,y);
  if([x,y]=CP_children_z(n),
     if(x==y, 2*CP_num_automorphisms(x)^2,
        CP_num_automorphisms(x)
        * CP_num_automorphisms(y));
     ,
     1);
}
{
  my(a=CP_num_automorphisms);
  check_equal(vector(256,n,a(n)),
              vector(256,n,vpar_num_automorphisms(vpar_from_CPcode(n))),
              "CP_num_automorphisms() vs vpar_num_automorphisms()");

  \\ formula
  check_equal(vector(256,n,n++; a(n)),
              vector(256,n,n++;
                     my(x=A002024_k_rep_k(n-1),
                        y=A002260_runs_1_to_k(n-1));
                     if(x==y, 2*a(x)^2, a(x)*a(y))),
              "CP_num_automorphisms() formula");
  check_equal(vector(256,n,n++; a(n)),
              vector(256,n,n++;
                     my(x=A002024_k_rep_k(n-1),
                        y=A002260_runs_1_to_k(n-1));
                     a(x)*a(y) * if(x==y,2,1)),
              "CP_num_automorphisms() formula");

  \\ formula
  check_equal(vector(256,n, a(n)),
              vector(256,n, 1<<CP_num_same_siblings(n)),
              "CP_num_automorphisms() = CP_num_same_siblings");
}
{
  if(0,
     print(vector(15,n,n++; vpar_num_automorphisms(vpar_from_CPcode(n))));
     \\ not in OEIS: 2, 2, 8, 2, 4, 8, 8, 16, 16, 128, 2, 4, 4, 16, 8
     print(vector(15,n,n++; logint(vpar_num_automorphisms(vpar_from_CPcode(n)),2)));
     \\ not in OEIS: 1, 1, 3, 1, 2, 3, 3, 4, 4, 7, 1, 2, 2, 4, 3
    quit);
}


\\-----------------------------------------------------------------------------
\\ Descent Peak Stack

CP_descent_peak_stack(n,swap=0) =
{
  n>=1 || error();
  my(want_num_vertices = CP_num_vertices(n));
  my(stack=List([]), num_vertices=0, m=0);
  while(1,
        while(n,
              num_vertices++;
              my(r); [n,r]=CP_children_z(n);
              if(swap,[n,r]=[r,n]);
              listput(stack,r);
              m=max(m,#stack));
        if(#stack==0,
           num_vertices == want_num_vertices || error();
           return(m));
        n=stack[#stack]; listpop(stack));
}
{
  if(0,
     print(vector(50,n,CP_descent_peak_stack(n)));
     print(vector(50,n,CP_descent_peak_stack(n,1)));
     \\ not in OEIS: 1, 2, 2, 3, 2, 3, 3, 3, 3, 3, 4, 2, 3, 3, 4, 3, 3, 3, 3, 4, 3, 4, 3, 3, 3, 4, 3, 4, 4, 3, 3, 3, 4, 3, 4, 4, 4, 3, 3, 3, 4, 3, 4, 4, 4, 4, 3, 3, 3, 4
     quit);
}
CP_right_height(n) =
{
  my(L,R); [L,R]=CP_children_z(n);
  max(if(L, CP_right_height(L)),
      if(R, CP_right_height(R)+1));
}
{
  check_equal(vector(1000,n, CP_descent_peak_stack(n,1)),
              vector(1000,n, CP_right_height(n) + 1));
}



\\------------------------------------------------------------------------------
check_equal(c,'c);
check_equal(k,'k);
check_equal(n,'n);
check_equal(row,'row);
check_equal(v,'v);
check_equal(x,'x);
check_equal(y,'y);
check_equal(vpar,'vpar);
print("end, total ",check_count," tests");
