# Copyright 2018, 2019 Kevin Ryde

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Read output from dreadnaut "t" command and write GP code setting
# dreadnaut_neighbours = [ [v1,v2,...], [v3,v4,...] ];


use 5.005;
use strict;
use FindBin;
use File::Spec;
use File::Slurp;
use File::Temp;
use IPC::Run;

# uncomment this to run the ### lines
# use Smart::Comments;

my @neighbours;
while (defined (my $line = readline)) {
  ### $line
  unless ($line =~ s/^ *(\d+) *://) {
    print("error()");
    die;
  }
  my $v = $1;
  $neighbours[$v] = [];
  while ($line =~ /\d+/g) {
    push @{$neighbours[$v]}, $&;
  }
}

### @neighbours

print "dreadnaut_neighbours = {[\n";
foreach my $v (1 .. $#neighbours) {
  my $aref = $neighbours[$v];
  print "  [", join(',', @$aref), "]",
    ($v==$#neighbours ? '' : ','),
     "\n";
}
print "]};\n";
exit 0;
