\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <path-lengths-matrix.gp
\\
\\ As noted in the addhelp of vpar_path_lengths_matrix(), Graham and Pollak
\\ considered the determinant of the path lengths matrix (distance matrix),
\\
\\     R.L. Graham and H.O. Pollak, "On the Addressing Problem for Loop
\\     Switching", Bell Systems Tech J, volume 50, 1971, pages 2495-2519.
\\     http://www.math.ucsd.edu/~ronspubs/71_05_loop_switching.pdf
\\
\\ and showed for a tree it is depends only on the number of vertices n, not
\\ on the tree structure,
\\
\\     det = (-1)^(n-1) * (n-1) * 2^(n-2)
\\
\\ Bapat and Sivasubramanian consider the determinant of path length matrix
\\ with each entry squared (as if a distance "norm" maybe), and show the
\\ result goes as the degrees of the vertices (code below).
\\
\\     R. B. Bapat and S. Sivasubramanian, "Product Distance Matrix of a
\\     Graph and Squared Distance Matrix of a Tree", Applicable Analysis and
\\     Discrete Mathematics, volume 7, number 2, October 2013, pages
\\     285-301.  http://www.jstor.org/stable/43660717
\\

default(recover,0)
default(strictargs,1);
default(output,3);         \\ compact matrix printing
read("../vpar.gp");

setrand(gettime());

n = random(4) + 4;      \\ num vertices, >= 1

\\ As noted by Bapat and Sivasubramanian, if there are 2 or more degree=2
\\ vertices then the determinant of the path lengths squared matrix is 0.
\\ Find a vpar with <2 such vertices in the interests of seeing some non-0
\\ numbers.  (Cf vpar_free_branch_reduce() which can delete all degree=2.)
{
  until(sum(v=1,#degrees, degrees[v]==2)<2,
        vpar = vpar_make_random(n);
        degrees=vpar_degrees_vector(vpar));
}
print("vpar = "vpar);
print();

\\-----------------------------------------------------------------------------
print("path lengths matrix");
m=vpar_path_lengths_matrix(vpar)

f = (-1)^(n-1) * (n-1) * 2^(n-2);    \\ Graham and Pollak
print("matdet      "matdet(m));
print("by formula  "f);
print("\n------");

\\-----------------------------------------------------------------------------
print("path lengths matrix, entries squared");
m=vpar_path_lengths_matrix(vpar);
m = apply(sqr,m)      \\ squared entries

t=apply(d->2-d, degrees);      \\ t = 2 - degree
print("degrees "degrees);

{
  \\ Bapat and Sivasubramanian
  f = (-1)^n
     * 4^(n-2)
     * ( (2*n-1) * prod(i=1,n, t[i])
        - 2* sum(i=1,n, prod(j=1,n, if(i==j,1, t[j]))));
}
print("matdet      "matdet(m));
print("by formula  "f);

\\-----------------------------------------------------------------------------
