\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <tree-counts.gp
\\
\\ Print some tables of how many labelled/unlabelled/ordered,
\\ rooted/unrooted, trees and forests, using the vpar count functions.
\\
\\ There are several other counts in vpar.gp, some more or less specific,
\\ which are not illustrated here.  For example vpar_count_asymmetric() and
\\ its unlabelled etc variants.
\\

print("Counts of trees using vpar_count() etc.");
print("          labelled   ordered  unlabelled   labelled   unlabelled");
print("            and        and       but         but      & unrooted");
print("           rooted     rooted    rooted     unrooted    = free");
print("         (A000169)  (A000108)  (A000081)   (A000272)   (A000055)");
{
  for(n=0,11,
     printf("n=%2d   %11d %8d %8d %13d %9d\n", n,
            vpar_count(n),
            vpar_count_ordered(n),
            vpar_count_unlabelled(n),
            vpar_count_unrooted(n),
            vpar_count_free(n)));
}
print();

print("Counts of forests using vpar_count() etc with \"forest\" parameter 1.");
print("          labelled   ordered  unlabelled   labelled   unlabelled");
print("            and        and       but         but      & unrooted");
print("           rooted     rooted    rooted     unrooted    = free");
print("         (A000272)  (A000108)  (A000081)   (A001858)  (A005195)");
{
  for(n=0,10,
     printf("n=%2d   %11d %8d %8d %13d %9d\n", n,
            vpar_count(n,1),
            vpar_count_ordered(n,1),
            vpar_count_unlabelled(n,1),
            vpar_count_unrooted(n,1),
            vpar_count_free(n,1)));
}
