\\ Copyright 2017, 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_randomize=1;
option_progress=1;

read("vpar.gp");
read("vpar-wip.gp");  \\ multiple reads ok to the various install() or alias()
read("vpar-wip.gp");
read("vpar-wip.gp");
read("test-helpers.gp");

\\ informational diagnostics
print("default(secure) is "default(secure));
print("vpar_INTERNAL_identity_perm type ", \
      type(vpar_INTERNAL_identity_perm)," ",vpar_INTERNAL_identity_perm);
\\ print("test_identity_perm          type ", \
\\       type(test_identity_perm), " ",test_identity_perm);
print("vpar_INTERNAL_gnot          type ", \
      type(vpar_INTERNAL_gnot),      " ",vpar_INTERNAL_gnot);
print("vpar_INTERNAL_fromdigits    type ", \
      type(vpar_INTERNAL_fromdigits),   " ",vpar_INTERNAL_fromdigits);
print("test_permorder              type ", \
      type(test_permorder),     " ",test_permorder);
print("test_perm_commute           type ", \
      type(test_perm_commute),  " ",test_perm_commute);
print("test_perm_is_even           ",test_perm_is_even);
print("have_test_is_gnil = "have_test_is_gnil);
print("test_is_gnil                ",test_is_gnil);


\\-----------------------------------------------------------------------------
\\ A002658 = C(n)   num 0,2 children height = n
\\     1, 1, 2,  7, 56, 2212, 2595782,
\\   n=0  1  2   3   4     5        6
\\ a(n+1) = a(n)*(a(0) + ... + a(n-1))
\\        + a(n)*(a(n) + 1)/2
\\ size n pair with any one of size <= n-1
\\ size n pair with same size n but one way around

\\ A(x) = x + (1/2)*(A(x)^2 + A(x^2))
\\ growth 1.2460208329836

A002658_vector(len) =
{
  if(len==0,[],
     concat(1,vpar_count_unlabelled_binary_of_height_vector(len-1)));
}
{
  check_equal(vector(10,h,h--; vpar_count_unlabelled_binary_of_height(h)),
              A002658_samples,  \\ OFFSET=0
              "A002658 path irredundant sets count");

  check_equal(A002658_vector(#A002658_samples),
              A002658_samples,
              "A002658_vector() vs DATA");
  my(a(n) = n>=0 || error();
     A002658_vector(n+1)[n+1]);

  \\ formula from Melzak
  check_equal(vector(8,n, a(n+1)),
              vector(8,n,
    a(n) * (a(n) / a(n-1) + (a(n) + a(n-1)) / 2)),
              "formula");

  \\ left tree and smaller right,
  \\ plus equal sizes not both ways around
  check_equal(vector(8,n, a(n+1)),
              vector(8,n,
                     a(n)*sum(i=0,n-1,a(i))
                   + binomial(a(n)+1,2)),
              "left and right");

  check_equal(vector(8,n, a(n)),
              vector(8,n,
      a(n-1)*((a(n-1)+1)/2 + sum(i=0,n-2,a(i)))),
             "mangled");

  my(n=1); check_equal(sum(i=0,n-2,a(i)),  0);
  my(n=1); check_equal(a(n-1), 1);

  my(n=3); check_equal(sum(i=0,n-1,a(i)),  4);
  my(n=3); check_equal(a(n-1), 2);
  my(n=3); check_equal((a(n-1) - 1)/2, 1/2);
  my(n=3); check_equal(2*( 4 - (2-1)/2), 7);

  check_equal(vector(5,h,h--; vpar_count_unlabelled_binary_of_height(h)), 
              [1, 1, 2, 7, 56]);
  check_equal(vpar_count_unlabelled_binary_of_height_vector(4),
                 [1, 2, 7, 56],
              "vpar_count_unlabelled_binary_of_height_vector() 4");
}


\\ A072638 = Ctotal(n)  num 0,2 of height <= n
\\     0, 1, 3, 10, 66, 2278, 2598060,
\\   n=0  1  2   3   4     5        6
\\   unary-binary height <= n
\\   every vertex degree <= 3
\\   a(n) = 1 + a(n-1)*(a(n-1)+3)/2


\\-----------------------------------------------------------------------------
\\ vpar_edgecover_num()

test_edgecover_num_by_best(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     covered_size = vector(#vpar,i,-1),
     any_size     = vector(#vpar),
     ret = 0);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(p,
        print("at v="v" "covered_size[v]","any_size[v]," to p="p" "covered_size[p]","any_size[p]);
        covered_size[p] = min(
           any_size[v] + any_size[p] + 1,
           if(covered_size[v]<0 || covered_size[p]<0,
              oo,
              covered_size[v] + covered_size[p]));
        any_size[p] = min(covered_size[p],
           if(covered_size[v] < 0, oo,
           covered_size[v] + any_size[p]));
        ,
        if(covered_size[v] < 0, return(-1));
        ret += covered_size[v]));
  ret;
}
{
  check_equal(test_edgecover_num_by_best([0,1,2]),2,
              "test_edgecover_num_by_best() path-3");
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_edgecover_num(vpar),
        by_best=test_edgecover_num_by_best(vpar));
     \\ print(vpar);
     check_equal(got,by_best,
                 "vpar_edgecover_num() bs by_best"));
}

\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_CP_children()");

{
  check_equal(vector(10,n,n++; vpar_INTERNAL_CP_children(n)),
              [[1,1],
               [2,1],[2,2],
               [3,1],[3,2],[3,3],
               [4,1],[4,2],[4,3],[4,4]]);
}

\\-----------------------------------------------------------------------------
progress("vpar_subfree_poly()");

{
  my(vpar=[0]);
  check_equal(vpar_subfree_poly(vpar),
              1*x^1 + 1,
              "vpar_subfree_poly() singleton");
  check_equal(vpar_subfree_poly(vpar,,1),
              1*x^1,
              "vpar_subfree_poly() singleton with");

  \\ path-2
  \\ with: 0, 0--1
  my(vpar=[0,1]);
  check_equal(vpar_subfree_poly(vpar),
              1*x^2 + 2*x^1 + 1,
              "vpar_subfree_poly() path-2");
  check_equal(vpar_subfree_poly(vpar,,1),
              1*x^2 + 1*x^1,
              "vpar_subfree_poly() path-2 with");

  \\ two singletons 
  check_equal(vpar_subfree_poly([0,0]),
              2*x + 1,
              "vpar_subfree_poly() 2 signletons");

  my(vpar=[0,1,2]);
  check_equal(vpar_subfree_poly(vpar),
              1*x^3 + 2*x^2 + 3*x^1 + 1,
              "path-3");
  \\ [], 0, 0-1, 0-1-2
  check_equal(vpar_subfree_poly(vpar,,1),
              1*x^3 + 1*x^2 + 1*x^1,
              "path-3 with root");

  my(vpar=[0,1,1]);
  check_equal(vpar_subfree_poly(vpar),
              1*x^3 + 2*x^2 + 3*x^1 + 1,
              "vpar_subfree_poly() star-3");
  check_equal(vpar_subfree_poly(vpar,,1),
              1*x^3 + 2*x^2 + 1*x^1,
              "vpar_subfree_poly() star-3 with root");

  my(vpar=[0, 1,1,1,1,1]);  \\ 5 leaves
  vpar_validate(vpar);
  check_equal(#vpar, 6);
  check_equal(vpar_subfree_poly(vpar,,1),
              x^6       \\ all vertices
              + 5*x^5   \\ 1 omitted
              + 10*x^4
              + 10*x^3
              + 5*x^2   \\ 2 vertices centre and leaf
              + x       \\ 1 vertex root
              ,         \\ none empty
    "vpar_subfree_poly() star-5 with root");



}
test_num_subfree_poly_by_flags(vpar,var='x) =
{
  my(ret=vector(#vpar+1));
  forvec(flags=vector(#vpar,i,[0,1]),
     my(sub=vpar_keep_flags(vpar,flags));
     if(vpar_is_tree(sub),
        \\ print(vpar" flags "flags);
        ret[vecsum(flags)+1]++));
  \\ print("final "ret);
  Polrev(ret,var);
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_subfree_poly(vpar));
     \\ print(vpar);
     check_equal(got,
                 test_num_subfree_poly_by_flags(vpar),
                 "vpar_subfree_poly() vs flags"));
}

\\-----------------------------------------------------------------------------
\\ vpar_is_irredset()

test_is_irredset_by_neighbours(vpar,set) =
{
  my(neighbours=vpar_neighbours_vector(vpar),
     counts=vectorsmall(#vpar));
  neighbours=vector(#vpar,v, concat(v,neighbours[v]));
  for(i=1,#set,
     my(v=set[i]);
     for(i=1,#neighbours[v], counts[neighbours[v][i]]++));
  \\ print(vpar" "set" "counts);
  for(i=1,#set,
     my(v=set[i]);
     if(vecmin(vecextract(counts,neighbours[v])) >= 2, return(0)));
  \\ print("  yes");
  1;
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(rep=1,#vpar+1,
        my(flags=vpar_random_flags(vpar),
           set=test_flags_to_Set(flags),
           want=test_is_irredset_by_neighbours(vpar,set));
        check_equal(vpar_is_irredset(vpar,set),
                    want,
                    "vpar_is_irredset() vs test_is_irredset_flags_by_neighbours()");
        \\ check_equal(vpar_is_irredset_flags(vpar,flags),
        \\             want,
        \\             "vpar_is_irredset_flags() vs test_is_irredset_flags_by_neighbours()");
     ));
}

test_irredsets_count(vpar) =
{
  my(ret=0);
  forvec(flags=vector(#vpar,i,[0,1]), ret+=vpar_is_irredset_flags(vpar,flags));
  ret;
}
{
  \\ A286887 number of irredundant sets in the path graph P_n.
  check_equal(vector(10,n, test_irredsets_count(vpar_make_path(n))),
              A286887_samples[1..10],
              "A286887 path irredundant sets count");
}
{
  check_equal(vector(10,n,n--; test_irredsets_count(vpar_make_star(n))),
              vector(10,n,n--; if(n==0,1, 2^(n-1) + 1)),
              "star irredundant sets count");
}

\\-----------------------------------------------------------------------------
progress("vpar_preorder_all()");

{
  for(forest=0,1,
     for(n=1, test_limit_trees_search - forest,
        my(vpars=vpar_preorder_all(n,forest));
        check_equal(setisset(Vecrev(vpars)), 1,
                    "vpar_preorder_all() reverse lex order");
        for(i=1,#vpars,
           check_equal(vpar_is_preorder(vpars[i]), 1,
                      "vpar_preorder_all() is preorder"))));
}

\\ concat(apply(vpar->vpar_depths_vector(vpar),concat(vector(3,n,n++;vpar_preorder_all(n)))))
\\ not in OEIS: 0, 1, 0, 1, 2, 0, 1, 1, 0, 1, 2, 3, 0, 1, 2, 2, 0, 1, 2, 1, 0, 1, 1, 2, 0, 1, 1, 1
\\ apply(n->n+1,concat(apply(vpar->vpar_depths_vector(vpar),concat(vector(3,n,n++;vpar_preorder_all(n))))))
\\ not in OEIS: 1, 2, 1, 2, 3, 1, 2, 2, 1, 2, 3, 4, 1, 2, 3, 3, 1, 2, 3, 2, 1, 2, 2, 3, 1, 2, 2, 2
\\ 
\\ concat(apply(vpar->vpar_depths_vector(vpar),concat(vector(3,n,n++;Vecrev(vpar_preorder_all(n))))))
\\ not in OEIS: 0, 1, 0, 1, 1, 0, 1, 2, 0, 1, 1, 1, 0, 1, 1, 2, 0, 1, 2, 1, 0, 1, 2, 2, 0, 1, 2, 3
\\ apply(n->n+1,concat(apply(vpar->vpar_depths_vector(vpar),concat(vector(3,n,n++;Vecrev(vpar_preorder_all(n)))))))
\\ not in OEIS: 1, 2, 1, 2, 2, 1, 2, 3, 1, 2, 2, 2, 1, 2, 2, 3, 1, 2, 3, 2, 1, 2, 3, 3, 1, 2, 3, 4
\\ 
\\ my(b=2); concat(vector(10,n, b=vpar_balanced_binary_next(b); vpar_from_balanced_binary(b)))
\\ not in OEIS: 0, 0, 0, 1, 0, 0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 1, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 2, 0
\\ my(b=2); concat(vector(10,n, b=vpar_balanced_binary_next(b); vpar_depths_vector(vpar_from_balanced_binary(b))))
\\ not in OEIS: 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0
{
  \\ balanced binary depths to preorder depth levels 1
  \\ my(want=OEIS_data("A120698"),
  \\    b=2,
  \\    got=[]);
  \\ while(#got<#want,
  \\       got=concat(got, apply(n->n+1,vpar_depths_vector(vpar_from_balanced_binary(b))));
  \\       b=vpar_balanced_binary_next(b));
  \\ for(i=1,#got,if(got[i]!=want[i],print(i);break));
  \\ check_equal(got,want, "A120698");

  \\ not in OEIS: 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 2, 1, 2, 1, 1, 2, 2, 1, 2, 3, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 1, 1, 1, 2, 2, 1, 1, 2, 3, 1, 2, 1, 1, 1, 2, 1, 2, 1, 2, 2, 1, 1, 2, 2, 2, 1, 2, 2, 3, 1, 2, 3, 1, 1, 2, 3, 2, 1, 2, 3, 3, 1, 2, 3, 4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 1, 1, 1, 2, 3, 1, 1, 2, 1, 1
  \\ not A120698, differs at n=51
}

\\-----------------------------------------------------------------------------
\\ Number of Subcons (connected induced subgraphs)

{
  \\ star-4
  \\ 4 singles
  \\ 3 pairs
  \\ 3 triples
  \\ 1 all
  check_equal(4+3+3+1, 11);
  check_equal(vpar_num_subcons(vpar_make_star(4)), 11);
  check_equal(vpar_num_subcons([0,1]), 3);
}

test_num_subcons_by_flags(vpar) =
{
  my(count=0);
  forvec(flags=vector(#vpar,i,[0,1]),
        count += flags && vpar_is_tree(vpar_keep_flags(vpar,flags)));
  count;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_num_subcons(vpar));
     if(#vpar <= test_limit_flags_search,
        check_equal(got,
                    test_num_subcons_by_flags(vpar),
                    "vpar_num_subcons() vs test_num_subcons_by_flags()"));
     check_equal(got,
                 vpar_span_count(vpar) - 1,
                 "vpar_num_subcons() same as vpar_span_count() - 1");
     );
}
{
  \\ sole tree with 48 subcons
  my(vpar=[0,1,2,3, 1,5, 1,7]); 
  check_equal(vpar_num_subcons(vpar), 48);
  check_equal(test_num_subcons_by_flags(vpar), 48);
}

\\-----------------------------------------------------------------------------
progress("vpar_span_count()");

check_equal(vpar_span_count([]), 1, "vpar_span_count() empty");
check_equal(vpar_span_count([0]), 2, "vpar_span_count() singleton");
check_equal(vpar_span_count([0,1]), 4, "vpar_span_count() path-2");


\\-----------------------------------------------------------------------------
\\ Make Degrees Vector

\\ non-singleton trees each 2*c-2
\\ so if all non-singletons then n - vecsum/2
\\ singletons + (n-singletons) - vecsum/2
test_degrees_num_components(degrees) =
{
  #degrees - vecsum(degrees)/2;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        degrees=vpar_degrees_vector(vpar),
        got=test_degrees_num_components(degrees),
        want=vpar_num_components(vpar));
     \\ print(vpar);
     check_equal(got,want,
                 "test_degrees_num_components()"));
}

test_degrees_num_nonsingleton_components(degrees) =
{
  hammingweight(degrees) - vecsum(degrees)/2;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        degrees=vpar_degrees_vector(vpar),
        got=test_degrees_num_nonsingleton_components(degrees),
        want=vpar_num_components(vpar) - vpar_num_singletons(vpar));
     check_equal(got,want,
                 "test_degrees_num_nonsingleton_components()"));
}

check_equal(vpar_make_degrees([0]), [0]);
check_equal(vpar_make_degrees([1,1]), [2,0]);
check_equal(vpar_make_degrees([1,2,1]), [2,0,2]);
check_equal(vpar_degrees_vector([2,0,2]), [1,2,1]);
check_equal(vpar_make_degrees([0,0]), [0,0], \
            "vpar_make_degrees() two singletons");
check_equal(vpar_make_degrees([1,1, 1,1]), [2,0,4,0], \
            "vpar_make_degrees() two path-2s");
check_equal(vpar_make_degrees([0,1,1]), [0,3,0], \
            "vpar_make_degrees() singleton and path-2");

test_make_degrees_by_num_children(degrees) =
{
  my(num_components = #degrees - vecsum(degrees)/2,
     seq=vpar_INTERNAL_seq_bucket_sort(degrees));
  for(i=1, #degrees - num_components,
     degrees[seq[i]]--);
  vpar_make_num_children(degrees);
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        degrees=vpar_degrees_vector(vpar),
        made=vpar_make_degrees(degrees));
     \\ print(vpar" "degrees" "made);
     check_equal(vpar_degrees_vector(made), degrees,
                 "vpar_make_degrees() degrees"));
}

  \\ vpar_is_degrees_vector(degrees)
  \\   || error("oops, not degreees of tree: ",degrees);
  \\ my(root=0);
  \\ if(#degrees, vecmax(degrees,&root));
  \\ for(v=1,#degrees,
  \\    if(v!=root && degrees[v],
  \\       degrees[v]--;
  \\       decr++));
  \\ vpar_is_num_children_vector(degrees)
  \\   || error("oops, not num children of tree: ",degrees);
  \\ vpar_make_num_children(degrees);

  \\ print("vpar_make_degrees() ",degrees);
  \\ vpar_is_degrees_vector(degrees,1)
  \\   || error("oops, not degreees of forest: ",degrees);
  \\ my(num_decr = test_degrees_num_nonsingleton_components(degrees),
  \\    seq=vpar_INTERNAL_seq_bucket_sort(degrees));
  \\ num_decr = hammingweight(degrees) - num_decr;
  \\ print("  num_decr ",num_decr);
  \\ forstep(i=1,#seq,1,
  \\    if(num_decr<=0,break);
  \\    if(degrees[seq[i]],
  \\       degrees[seq[i]]--;
  \\       num_decr--;
  \\       print("  decrease v="seq[i]" to "degrees[seq[i]]);
  \\      ));
  \\ vpar_is_num_children_vector(degrees,1)
  \\   || error("oops, not num children of forest: ",degrees);
  \\ vpar_make_num_children(degrees);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        degrees=vpar_degrees_vector(vpar),
        made=vpar_make_degrees(degrees));
     \\ print(vpar" "degrees" "made);
     check_equal(vpar_degrees_vector(made), degrees,
                 "vpar_make_degrees() degrees"));
}

{
  for(n=2,5,
     my(comps=test_compositions(2*n-2,[0,n-1],[n,n]),
        count_upwards=0,
        count_downwards=0,
        count_nonewards=0,
        count_degrees_increasing=0,
        count_degrees_decreasing=0);
     for(i=1,#comps,
        my(degrees=Vec(comps[i]));
        \\ print("degrees "degrees);
        check_equal(vpar_is_degrees_vector(degrees), 1, 
                    "vpar_is_degrees_vector()");

        my(vpar=vpar_make_degrees(degrees));
        \\ print(" vpar "vpar);
        vpar_validate(vpar);
        check_equal(vpar_degrees_vector(vpar), degrees,
                    "vpar_make_degrees() gives degrees");

        my(is_upwards  =vpar_is_upwards(vpar),
           is_downwards=vpar_is_downwards(vpar));
        count_upwards   += is_upwards;
        count_downwards += is_downwards;
        count_nonewards += (!is_upwards && !is_downwards);
        \\ print(" is_upwards "is_upwards" is_downwards "is_downwards);

        my(degrees_is_increasing=test_vector_is_increasing(degrees),
           degrees_is_decreasing=test_vector_is_decreasing(degrees));
        count_degrees_increasing += degrees_is_increasing;
        count_degrees_decreasing += degrees_is_decreasing;

        \\ some non-monotonic degrees can also give vpar_is_upwards()
        if(degrees_is_increasing,
           check_equal(is_upwards, 1, "vpar_make_degrees() is_upwards"));

        \\ if(degrees_is_decreasing,
        \\    check_equal(is_downwards, 1, "vpar_make_degrees() is_downwards"));
        );

     \\ print(n);
     \\ print(count_degrees_increasing);
     \\ print(count_degrees_decreasing);
     \\ print(count_upwards);
     \\ print(count_downwards);

     check_equal(count_upwards  >=1, 1, "vpar_make_degrees() count_upwards");
     check_equal(count_downwards>=1, if(n>=3,1,0),
           "vpar_make_degrees() count_downwards");
     check_equal(count_nonewards>=1, if(n>=3,1,0),
           "vpar_make_degrees() count_nonewards");

     check_equal(count_degrees_increasing >=1, 1, "count_degrees_increasing");
     check_equal(count_degrees_decreasing >=1, 1, "count_degrees_decreasing");
  );
}

\\-----------------------------------------------------------------------------
progress("vpar_bipartition_flags()");

check_equal(vpar_bipartition_flags([]), [], "vpar_bipartition_flags() empty");
check_equal(vpar_bipartition_flags([0]), [0]);
check_equal(vpar_bipartition_flags([0,1]), [0,1]);
check_equal(vpar_bipartition_flags([0,1,2]), [0,1,0]);
{
  check_equal(vpar_bipartition_flags([3,1,2]), [-1,-1,-1],
              "vpar_bipartition_flags() odd cycle");

  check_equal(vpar_bipartition_flags([3,1,2, 1,4,5, 0]), [-1,-1,-1, -1,-1,-1, 0],
              "vpar_bipartition_flags() odd cycle and tail");
  check_equal(vpar_bipartition_flags([3,1,2, 1,4,5, 0, 2,8,9, 0, 4]), [-1,-1,-1,
                                                                       -1,-1,-1,
                                                                       0,
                                                                       -1,-1,-1,
                                                                       0,
                                                                       -1],
              "vpar_bipartition_flags() odd cycle and tails");
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got = vpar_bipartition_flags(vpar));
     for(v=1,#vpar,
        if(vpar[v],
           check_equal(got[v] == !got[vpar[v]], 1,
                       "vpar_bipartition_flags() v to parent different halves")));

     my(comp=vector(#got,i,1-got[i]));  \\ set complement
     \\ print(vpar" "got" "comp);
     check_equal(vpar_is_indset_flags(vpar,got), 1,
                 "vpar_bipartition_flags() is indset");
     check_equal(vpar_is_indset_flags(vpar,comp), 1,
                 "vpar_bipartition_flags() is indset");

     check_equal(vpar_is_domset_flags(vpar,comp), 1,
                 "vpar_bipartition_flags() complement is domset");
     check_equal(vpar_is_indomset_flags(vpar,comp), 1,
                 "vpar_bipartition_flags() complement is indomset");
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_bipartition()");

check_equal(vpar_bipartition([]), [[],[]], "vpar_bipartition() empty");
check_equal(vpar_bipartition([0]), [[1],[]], "vpar_bipartition() singleton");
check_equal(vpar_bipartition([0,1]), [[1],[2]]);
check_equal(vpar_bipartition([0,0]), [[1,2],[]]);
check_equal(vpar_bipartition([0,0,2]), [[1,2],[3]]);
check_equal(vpar_bipartition([4,1,2,3]), [[1,3],[2,4]]);
{
  check_equal(vpar_bipartition([3,1,2]), 'none,
              "vpar_bipartition() odd cycle");
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got = vpar_bipartition(vpar));
     check_equal(#got[1] + #got[2], #vpar, "vpar_bipartition() all present");
     for(s=1,2,
        check_equal(Set(got[s]), got[s], "vpar_bipartition() is set"));
     check_equal(#Set(concat(got)), #vpar, "vpar_bipartition() disjoint");
     my(neighbours=vpar_neighbours_vector(vpar));
     for(v=1,#vpar,
        for(s=1,2,
           if(setsearch(got[s],v),
              check_equal(setintersect(got[s],neighbours[v]),[]));
           next(2));
        error("oops, v not in set"));
     );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_free()");

print("  not working yet ...");

\\ {
\\   check_equal(vpar_is_free([]),  1, "vpar_is_free() empty");
\\   check_equal(vpar_is_free([0]), 1, "vpar_is_free() singleton");
\\ }
\\ {
\\   \\   1
\\   \\   |  \
\\   \\   2    6
\\   \\   |    |
\\   \\   3    7
\\   \\   |    |
\\   \\   4    8
\\   \\   |
\\   \\   5
\\   my(vpar=[0,1,2,3,4, 1,6,7]);
\\   check_equal(vpar, vpar_free_first(8), "vpar_free_first() 8");
\\   check_equal(vpar_is_free(vpar), 1,
\\              "vpar_is_free() path 8");
\\ }
\\ {
\\   \\   1
\\   \\   |  \
\\   \\   2    5
\\   \\   |    |
\\   \\   3    6
\\   \\   |    |
\\   \\   4    7
\\   check_equal(vpar_is_free([0,1,2,3, 1,5,6]), 1);
\\ }
\\ 
\\ test_is_free_by_relabel(vpar) =
\\ {
\\   !vpar_is_cyclic(vpar)
\\   && vpar_is_tree(vpar)
\\   && vpar_to_free(vpar)==vpar;
\\ }
\\ test_is_free_by_depths(vpar) =
\\ {
\\   !vpar_is_cyclic(vpar)
\\   && vpar_is_tree(vpar)
\\   && vpar_is_preorder(vpar)
\\   && test_depths_is_free_primary_by_vectors(vpar_depths_vector(vpar));
\\ }
\\ 
\\ {
\\   my(samples=concat([[[0, 1, 1, 2],
\\                       [0,1,1,3,0,5,6],
\\                       [0, 1, 2, 1, 4],
\\ 
\\                       \\ vpar_to_free([0, 1,2,3,3, 1,6, 1])
\\                       \\   1       
\\                       \\   |   \    \    bicentral, L1 lex bigger
\\                       \\   2     6   8
\\                       \\   |     |
\\                       \\   3     7
\\                       \\   |\
\\                       \\   4 5
\\                       [0, 1,2,3,3, 1,6, 1],
\\ 
\\                       \\   1       5   not lex descending
\\                       \\   | \     |
\\                       \\   2   3   6
\\                       \\       |   |
\\                       \\       4   7
\\                       [0,1,1,3, 0,5,6],
\\ 
\\                       \\   1       
\\                       \\   |   \       bicentral, L1 bigger
\\                       \\   2      6
\\                       \\   |\ \  
\\                       \\   3 4 5 
\\                       [0,1, 2,2,2, 1],
\\ 
\\                       \\ vpar_to_free([0,1,2,1,1])
\\                       \\   1       
\\                       \\   |  \ \
\\                       \\   2   4 5
\\                       \\   |\ 
\\                       \\   3 4
\\                       [0, 1, 2, 1, 1],
\\ 
\\                       \\   1       
\\                       \\   |  \ 
\\                       \\   2   3
\\                       \\       |
\\                       \\       4
\\                       [0, 1, 1,3],
\\ 
\\                       \\   1       
\\                       \\   | \         heights 3, 1 bad
\\                       \\   2   5
\\                       \\   |
\\                       \\   3
\\                       \\   |
\\                       \\   4
\\                       [0, 1,2,3, 1],
\\ 
\\                       \\   1       
\\                       \\   |   \       unicentral, ok
\\                       \\   2     5
\\                       \\   |\    |\
\\                       \\   3 4   6 7
\\                       \\  
\\                       \\  
\\                       [0, 1,2,2, 1,5,5]
\\                      ],
\\                      test_sample_forests,
\\                      test_sample_preorder_forests,
\\                      test_sample_cyclics
\\                     ]));
\\   for(i=1,#samples,
\\      my(vpar=samples[i],
\\         got=vpar_is_free(vpar));
\\      \\ print(vpar" seq "Vec(vpar_seq_premax(vpar)));
\\      check_equal(got,
\\                  test_is_free_by_relabel(vpar),
\\                  "vpar_is_free() vs test_is_free_by_relabel() of sample forest");
\\ 
\\      \\ suspect this one not right
\\      \\ check_equal(got,
\\      \\             test_is_free_by_depths(vpar),
\\      \\             "vpar_is_free() vs test_is_free_by_depths() of sample forest");
\\   );
\\ }
\\ 
\\ {
\\   my(forest=0);
\\   for(n=0, 3+test_limit_trees_search - forest,
\\      my(count_free_trees=0);
\\      forvec(vec=vpar_forvec_from(n,forest), \\ forest
\\            my(vpar=vpar_from_upascend(vec),
\\               got=vpar_is_free(vpar),
\\               by_relabel=test_is_free_by_relabel(vpar));
\\            if(got != by_relabel,
\\               print(vpar);
\\              );
\\            check_equal(got, by_relabel,
\\                        "vpar_is_free() vs test_is_free_by_relabel() of exhaustive");
\\            count_free_trees += got;
\\      );
\\ 
\\      check_equal(count_free_trees, vpar_count_free(n),
\\                  "vpar_is_free() count vs vpar_count_free()");
\\   );
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_free_next_ff()");

{
  my(vpar=vpar_free_first(5));
  check_equal(vpar_free_next_ff(vpar,0), [0, 1, 2, 1, 1]);
  check_equal(vpar_free_next_ff(vpar,1), [0, 1, 2, 1, 1]);
  check_equal(iferr(vpar_free_next_ff(vpar,'forest);0, e, 1),
              1,
              "vpar_free_next_ff() error on forest=symbol (instead of 0,1)");
}
{
  my(vpar=vpar_free_last(5));
  check_equal(vpar_free_next_ff(vpar,0), 'none);
  check_equal(vpar_free_next_ff(vpar,1), [0, 1, 2, 1, 0]);
  check_equal(iferr(vpar_free_next_ff(vpar,'forest);0, e, 1),
              1,
              "vpar_free_next_ff() error on forest=symbol (instead of 0,1)");
}
\\ {
\\   for(singletons=0,30,
\\      for(t=2,10,
\\         my(first=vpar_free_last(t),
\\            n=#first + singletons,
\\            vpar=Vec(first,n),
\\            want=vpar_free_first(t-1));
\\         while(#want+t-1 <= n,
\\               want=vpar_concat_forest(want,vpar_free_first(t-1)));
\\         want=vpar_concat_forest(want,vpar_free_first(n-#want));
\\ 
\\         my(got=vpar_free_next_ff(vpar,1));
\\         check_equal(got,want,
\\                     "vpar_free_next_ff() last component last")));
\\ }

test_free_next_forest_by_components(vpar) =
{
  my(n=#vpar,
     components=vpar_component_trees(vpar));
  while(#components,
        if(#components[#components]>1,
           break);
        components = components[^#components]); \\ drop singleton
  #components || return('none);  \\ all singletons, no next

  my(cnext=vpar_free_next(components[#components]));
  if(cnext=='none,
     cnext = vpar_free_first(#components[#components]-1));
  components[#components] = cnext;

  my(next_vpar=vpar_concat_forests(components));
  while(#next_vpar + #cnext <= #vpar,    \\ whole copies of last
        next_vpar=vpar_concat_forest(next_vpar,cnext));

  \\ final partial biggest
  next_vpar=vpar_concat_forest(next_vpar,vpar_free_first(#vpar-#next_vpar));
  #next_vpar==#vpar || error("oops length want ",#vpar," got ",#next_vpar);
  next_vpar;
}

{
  \\   1        9
  \\   | \      | \   \                L1 height 2 size 4
  \\   2  6    10  14  16              L2 height 2 size 4
  \\   |  |     |   |\
  \\   3  7    11  15 16  can raise 16
  \\   |  |     |\
  \\   4  8    12 13
  \\   |    
  \\   5    

  \\                        9            14    16
  my(vpar=[0,1,2,3,4,1,6,7, 0,9,10,11,11, 9,14,14]);
  my(want=test_free_next_forest_by_components(vpar),
     got=vpar_free_next_ff(vpar,1));  \\ forest
  check_equal(got,want,
              "vpar_free_next_ff() vs test_free_next_forest_by_components() 6,6");
}
{
  \\   1   
  \\   | \ 
  \\   2  5           L1 height 2 size 3
  \\   |  |           L2 height 3 size 4
  \\   3  6
  \\   |  | 
  \\   4  7

  my(vpar=[0,1,2,3, 1,5,6]);
  check_equal(vpar_subtree_height(vpar,2), 2);
  check_equal(vpar_free_next(vpar), [0,1,2,3,1,5,5]);
  my(want=test_free_next_forest_by_components(vpar),
     got=vpar_free_next_ff(vpar,1));  \\ forest
  check_equal(got,want,
              "vpar_free_next_ff() vs test_free_next_forest_by_components() 7");
}

{
  \\   1      7
  \\   | \    | \
  \\   2  5   8  11
  \\   |  |   |   |
  \\   3  6   9  12
  \\   |      |
  \\   4     10

  check_equal(vpar_free_next([0,1,2,3,1,5]), [0,1,2,2,1,5]);

  \\                     7  8  9    11
  my(vpar=[0,1,2,3,1,5,  0, 7, 8, 9, 7, 11]);
  my(want=test_free_next_forest_by_components(vpar),
     got=vpar_free_next_ff(vpar,1));  \\ forest
  check_equal(got,want,
              "vpar_free_next_ff() vs test_free_next_forest_by_components() 6,6");
}

{
  for(n=1,10,
     my(vpar=vpar_free_first(n));
     while(vpar!='none,
           my(want=test_free_next_forest_by_components(vpar),
              got=vpar_free_next_ff(vpar,1));  \\ forest
           \\ print(vpar);
           check_equal(got,want,
                       "vpar_free_next_ff() vs test_free_next_forest_by_components()");
           vpar=want;
          ));
}

\\-----------------------------------------------------------------------------
progress("vpar_is_subtree_sizes_vector()");

check_equal(vpar_is_subtree_sizes_vector([]),   1);
check_equal(vpar_is_subtree_sizes_vector([],1), 1);
check_equal(vpar_is_subtree_sizes_vector([0]),   0);
check_equal(vpar_is_subtree_sizes_vector([0],1), 0);
check_equal(vpar_is_subtree_sizes_vector([1]),   1);
check_equal(vpar_is_subtree_sizes_vector([1],1), 1);

check_equal(vpar_subtree_sizes_vector([2,0]), [1,2]);
check_equal(vpar_is_subtree_sizes_vector([1,2]), 1, \
            "vpar_is_subtree_sizes_vector() path-2 reversed");
check_equal(vpar_is_subtree_sizes_vector([1,2],1), 1);
check_equal(vpar_is_subtree_sizes_vector([1,2],-1), 0, \
            "vpar_is_subtree_sizes_vector() path-2 reversed, root=1");

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i],
        subtree_sizes = vpar_subtree_sizes_vector(vpar));
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes), 1,
                 "vpar_is_subtree_sizes_vector() tree");
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes,1), 1,
                 "vpar_is_subtree_sizes_vector() tree, as forest");

     if(#vpar, vpar=vpar_reroot(vpar,1));
     subtree_sizes = vpar_subtree_sizes_vector(vpar);
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes,-1), 1,
                 "vpar_is_subtree_sizes_vector() tree,root=1");
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes), 1,
                 "vpar_is_subtree_sizes_vector() tree");
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes,1), 1,
                 "vpar_is_subtree_sizes_vector() tree, as forest");
  );
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        subtree_sizes = vpar_subtree_sizes_vector(vpar));
     check_equal(vpar_is_subtree_sizes_vector(subtree_sizes,1), 1,
                 "vpar_is_subtree_sizes_vector() random forest"));
}

{
  for(forest=-1,1,
     for(n=0, test_limit_trees_search - forest - 1,
        my(seen=List([]));  \\ Set() of subtree_sizes vectors seen
        forvec(vec=vpar_forvec_from(n,forest),
              my(vpar=vpar_from_upascend(vec),
                 subtree_sizes=vpar_subtree_sizes_vector(vpar));
              my(pos=setsearch(seen,subtree_sizes,1));
              if(pos, listinsert(seen,subtree_sizes,pos)));
        \\ print("seen "seen);

        forvec(vec=vector(n,i,[-1,n+1]),
              \\ print(" vec "vec);
              check_equal(vpar_is_subtree_sizes_vector(vec,forest),
                          setsearch(seen,vec)!=0,
                          "vpar_is_subtree_sizes_vector() vs setsearch"));
     ));
}


\\-----------------------------------------------------------------------------
progress("vpar_lower_matchnum()");

check_equal(vpar_lower_matchnum([]), 0, \
            "vpar_lower_matchnum() empty");
check_equal(vpar_lower_matchnum([0]), 0, \
            "vpar_lower_matchnum() singleton");
check_equal(vpar_lower_matchnum([0,1]), 1, \
            "vpar_lower_matchnum() path-2");

test_lower_matchnum_by_search(vpar) =
{
  my(m=#vpar);
  forvec(flags=vector(#vpar,i,[0,1]),
        if(vpar_is_maximal_matching_flags(vpar,flags),
           m=min(m,hammingweight(flags))));
  m;
}

\\ check_equal(vpar_lower_matchnum([1]), 0, \
\\             "vpar_lower_matchnum() self-loop");
\\
\\ *==*
check_equal(vpar_lower_matchnum([2,1]), 1, \
            "vpar_lower_matchnum() 2-cycle");
\\
\\ *==* *
check_equal(vpar_lower_matchnum([2,3,1]), 1, \
            "vpar_lower_matchnum() 3-cycle");
\\
\\ *==* *==*
check_equal(vpar_lower_matchnum([2,3,4,1]), 2, \
            "vpar_lower_matchnum() 4-cycle");
\\
\\ *==* * *==*
check_equal(vpar_lower_matchnum([2,3,4,5,1]), 2, \
            "vpar_lower_matchnum() 5-cycle");
\\
\\ *==* * *==* *
check_equal(vpar_lower_matchnum([2,3,4,5,6,1]), 2, \
            "vpar_lower_matchnum() 6-cycle");
\\
check_equal(vpar_lower_matchnum([2,3,4,5,6,1]), 2, \
            "vpar_lower_matchnum() 7-cycle");
{
  \\ FIXME: n=1 too
  for(n=2,test_limit_flags_search,
     my(vpar=test_make_cycle(n),
        want=test_lower_matchnum_by_search(vpar));
     check_equal(want, if(n==1,0, ceil(n/3)),
                 Str("vpar_lower_matchnum() cycle n=",n," formula"));
     for(rep=0,3,
        my(got=vpar_lower_matchnum(vpar));
        check_equal(got, want,
                    Str("vpar_lower_matchnum() cycle n=",n));
        vpar=vpar_relabel_random(vpar)));
}

test_lower_matchnum_by_min(vpar) =
{
  \\ with[v] = num pairs when v is matched
  \\ without_max[v] = num pairs when without v but it is maximal
  \\ without[v] = num pairs when without v, possibly not maximal,
  \\   but with v matched everything below is maximal

  \\ print("vpar_lower_matchnum() "vpar);

  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     with               = vectorsmall(#vpar),
     with_exists        = vectorsmall(#vpar),
     without            = with,
     without_max        = vectorsmall(#vpar),
     without_max_exists = vectorsmall(#vpar,v,1), \\ empty is maximal
     ret = 0);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v],
        v_max = min(if(with_exists[v],        with[v],        #vpar),
                    if(without_max_exists[v], without_max[v], #vpar)));

     \\ print(" v="v" with "with[v]"["with_exists[v]"] without "without[v]" without max["without_max_exists[v]"] "without_max[v]);
     \\ print("  v_max "v_max);

     with_exists[v] || without_max_exists[v] || error();     \\ one or both
     with_exists[v] == (with[v]!=0) || error();

     \\ \\ get arbitrarily large without_max difference here
     \\ my(rel=[if(with_exists[v],with[v]-without[v],'none),
     \\         if(without_max_exists[v],without_max[v]-without[v],'none)]);
     \\ \\ print("  rel "rel);
     \\    rel == ['none, 0]
     \\ || rel == [0, 'none]
     \\ || rel == [1, 'none]
     \\ || rel == [0, 0]
     \\ || rel == [1, 0]
     \\ || rel == [0, 1]
     \\ || rel == [0, 2]
     \\ || rel == [1, 1]
     \\ || rel == [1, 2]
     \\ || error(rel);

     if(with_exists[v],
        with[v] >= without[v] || error();
        with[v] - without[v] <= 1 || error();
       );
     if(without_max_exists[v],
        without_max[v] >= without[v] || error();
        \\ difference without_max[v] - without[v] arbitrarily large
       );
     if(with_exists[v] && without_max_exists[v],
        with[v] - without_max[v] <= 1 || error();
        -2 <= with[v] - without_max[v] || error("with-without_max");
       );
     if(vpar_degree_of_vertex(vpar,v)>=2,
        with_exists[v] || error());

     if(p,
        \\ if(with_exists[p] &&   without[p] + without[v] + 1 < with[p] + v_max, print("  p="p" with+min ",with[p] + v_max," bettered by new "without[p] + without[v] + 1));

        with[p] = min(if(with_exists[p], with[p] + v_max,  #vpar),
                      without[p] + without[v] + 1);  \\ new

        with_exists[p] = 1;
        without[p] += v_max;
        without_max[p] += with[v];
        if(!with_exists[v], without_max_exists[p] = 0);

        \\ print("  set p="p" with "with[p]"["with_exists[p]"] without "without[p]" without max "without_max[p]"["without_max_exists[p]"]");
        ,
        \\ print("  at root");
        ret += v_max));
  ret;
}

{
  \\     v-|    v-|
  \\      1      4
  \\      |               self-loops 1 and 4
  \\      3
  \\      |
  \\      2
  my(vpar=[1,3,1,4],
     got=vpar_lower_matchnum(vpar),
     want=test_lower_matchnum_by_search(vpar));
  check_equal(want, 1,
              "test_lower_matchnum_by_search() self 1,4");
  \\ FIXME:
  \\ check_equal(got, want,
  \\         "vpar_lower_matchnum() vs test_lower_matchnum_by_search() self 1,4");
}
{
  \\     10
  \\      |
  \\      6
  \\     / \
  \\    1   5
  \\    |   |
  \\    8   7
  \\    |   |
  \\    9   4
  \\    |   |
  \\    2   3
  my(vpar=[6, 9, 4, 7, 6, 10, 5, 1, 8, 0]);
  check_equal(vpar_lower_matchnum(vpar),
              test_lower_matchnum_by_search(vpar),
              "vpar_lower_matchnum() vs test_lower_matchnum_by_search() 10");
}
{
  \\       2
  \\     / | \
  \\    4  9  11
  \\   /   |   |
  \\  5    8   7
  \\       |   | \
  \\       6   1  12
  \\       |       |
  \\       3      10
  \\
  my(vpar=[7, 0, 6, 2, 4, 8, 11, 9, 2, 12, 2, 7]);
  check_equal(vpar_lower_matchnum(vpar),
              test_lower_matchnum_by_search(vpar),
              "vpar_lower_matchnum() vs test_lower_matchnum_by_search() 10");
}

\\       g
\\       | \
\\       p  q
\\       |  |\
\\       c  d e
\\
\\ d,e mark q as must match
\\
\\
test_lower_matchnum_by_states(vpar) =
{
  \\ 0 = unmatched
  \\ 1 = unmatched, but must match to be maximal
  \\ 2 = matched
  \\
  \\ Leaf demand its connection be matched later.

  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     state=vectorsmall(#vpar),
     ret=0);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(p,
        if(state[v]==0,
           if(state[p]==0, state[p]=1);
           ,
           state[v]==1,
           ret++;
           state[p]=2);
        ,
        if(state[v]==1, ret++)));
  ret;
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_lower_matchnum(vpar));

     if(#vpar <= test_limit_flags_search,
        check_equal(got,
                    test_lower_matchnum_by_search(vpar),
                  "vpar_lower_matchnum() vs test_lower_matchnum_by_search()"));

     check_equal(got,
                 test_lower_matchnum_by_min(vpar),
                 "vpar_lower_matchnum() vs test_lower_matchnum_by_min()");
     check_equal(got,
                 test_lower_matchnum_by_states(vpar),
                 "vpar_lower_matchnum() vs test_lower_matchnum_by_states()"));
}

\\ FIXME:
\\ {
\\   for(i=1,#test_sample_forests_and_cyclics,
\\      my(vpar=test_sample_forests_and_cyclics[i],
\\         got=vpar_lower_matchnum(vpar));
\\      print(vpar);
\\      if(#vpar <= test_limit_flags_search,
\\         check_equal(got,
\\                     test_lower_matchnum_by_search(vpar),
\\                     "vpar_lower_matchnum() vs test_lower_matchnum_by_search()"));
\\   );
\\ }

\\ test_random_in_range(lo,hi) = random(hi-lo+1)+lo;
\\ {
\\   for(i=1,1000000,
\\      my(vpar=vpar_make_random(test_random_in_range(5,12)));
\\      check_equal(vpar_lower_matchnum(vpar),
\\                  test_lower_matchnum_by_states(vpar),
\\                  "vpar_lower_matchnum() vs test_lower_matchnum_by_states()"));
\\ }


\\------------------------------------------------------------------------------
progress("test_sqrt_as_quad()");

\\ Unused:
\\ sqrt(n) as a suitable t_QUAD
test_sqrt_as_quad(n) =
{
  if(n==0,0,
     my(v=core(n,1));  \\ v = [squarefree, squares]
     v[2] * if(v[1]==1,1, quadgen(4*v[1])));
}
my(s=test_sqrt_as_quad(3)); check_equal(simplify(s^2),3);
my(s=test_sqrt_as_quad(5)); check_equal(simplify(s^2),5);


\\------------------------------------------------------------------------------
progress("vpar_is_bileaves()");

my(t=-1); check_equal(-3-t, -2);
my(t=-2); check_equal(-3-t, -1);

check_equal(vpar_is_bileaves([]), 1, "vpar_is_bileaves() empty");
check_equal(vpar_is_bileaves([0,1,2]), 1, "vpar_is_bileaves() path-3");
check_equal(vpar_is_bileaves([2,3,0]), 1, "vpar_is_bileaves() path-3 reverse");

test_is_bileaves_by_pairwise(vpar) =
{
  \\ all paris of leaves an even distance apart
  my(leaves=vpar_leaves(vpar),
     m=vpar_path_lengths_matrix(vpar));
  for(i=1,#leaves,
     for(j=i+1,#leaves,
        if(m[leaves[i],leaves[j]]>=0 && m[leaves[i],leaves[j]]%2,
           return(0))));
  1;
}

test_is_all_components_unicentral(vpar) =
{
  my(trees=vpar_component_trees(vpar));
  for(i=1,#trees, if(vpar_centres(vpar)==2, return(0)));
  1;
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_is_bileaves(vpar));
     check_equal(got,
                 test_is_bileaves_by_pairwise(vpar),
                 "vpar_is_bileaves() vs test_is_bileaves_by_pairwise()");
     if(got,
        check_equal(test_is_all_components_unicentral(vpar), 1,
                    "vpar_is_bileaves() always unicentral"));
  );
}


\\------------------------------------------------------------------------------
progress("vpar_matchnumsets()");

check_equal(vpar_matchnumsets([]), [[]], "vpar_matchnumsets() empty");
check_equal(vpar_matchnumsets([0]), [[]], "vpar_matchnumsets() singleton");
check_equal(vpar_matchnumsets([0,1]), [[2]], "vpar_matchnumsets() path-2");
check_equal(vpar_matchnumsets([0,1,2]), [[2],[3]], "vpar_matchnumsets() path-3");

{
  \\      4           12 43
  \\    / | \         12 45
  \\   1  3  5
  \\   |
  \\   2
  check_equal(vpar_matchnumsets([4, 1, 4, 0, 4]),
              [[2,3], [2,5]]);
}

test_flagssize_first(n,size) = vectorsmall(n,i,i>n-size);
check_equal(test_flagssize_first(3,1), Vecsmall([0,0,1]));
check_equal(test_flagssize_first(3,3), Vecsmall([1,1,1]));
check_equal(test_flagssize_first(5,2), Vecsmall([0,0,0,1,1]));

\\ [1,0,0,1,1,1,0,0]
\\ [1,0,1,0,0,0,1,1]
test_flagssize_next(flags) =
{
  my(pos=#flags);
  while(1,
        if(pos<=0, return('none));
        if(flags[pos],break);
        pos--);
  \\ print("last 1 at pos="pos);
  flags[pos]=0;   \\ last 1
  my(endpos=#flags);
  while(1,
        pos--; if(pos<=0, return('none));
        if(!flags[pos],break);
        \\ print("pos="pos" endpos="endpos"  "flags);
        flags[pos]=0; flags[endpos]=1; endpos--);
  flags[pos]=1;
  flags;
}
{
  check_equal(test_flagssize_next([1,0,0,1,1,1,0,0]),
                                  [1,0,1,0,0,0,1,1]);
  for(n=0,8,
     for(k=0,n,
        my(flags=test_flagssize_first(n,k),
           count=0,
           l=List([]));
        until((flags=test_flagssize_next(flags))=='none,
              check_equal(#flags, n);
              check_equal(hammingweight(flags), k);
              listput(l,flags));
        check_equal(#l, binomial(n,k));
        check_equal(#Set(l), #l,
                    "test_flagssize_next() all distinct")));
}

test_matchnumsets_by_flags(vpar) =
{
  my(mc=vpar_matchnum_and_count(vpar),
     ret=vector(mc[2]),    \\ count many
     upto=0,
     flags=test_flagssize_first(#vpar,mc[1]));
  until((flags=test_flagssize_next(flags))=='none,
        if(vpar_is_matching_flags(vpar,flags),
           ret[upto++]=test_flags_to_Set(flags)));
  ret;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        mc=vpar_matchnum_and_count(vpar),
        matchnum=mc[1],
        count=mc[2]);
     if(count > test_limit_sets, next);

     my(got=vpar_matchnumsets(vpar));
     check_equal(#got, count, "vpar_matchnumsets() count");
     for(i=1,#got,
        check_equal(setisset(got[i]), 1);
        check_equal(#got[i], matchnum);
        check_equal(vpar_is_matching_set(vpar,got[i]), 1));

     check_equal(vecsort(got),
                 vecsort(test_matchnumsets_by_flags(vpar)));
  );
}


\\------------------------------------------------------------------------------
progress("vpar_is_free_automorphisms_le() Free 2-Group Count");
\\ Free 2-Group Count
\\ A309352

\\ Return a gf of count how many free trees of n vertices whose automorphism
\\ group is a 2-group (order some 2^k).
test_count_free_2group_gf(len,x='x) =
{
  \\ T(x) = rooted unlabelled gf, without its constant term, and x substituted
  my(T=(x)->subst(test_count_unlabelled_2group_gf(len)-1,'x,x));
  polcoeff(T(x),0)==0 || error();
  polcoeff(T(x^2),0)==0 || error();
  1 + T(x) - 1/2*T(x)^2 + 1/2*T(x^2);
}
check_equal(test_count_free_2group_gf(1), 1  + O(x), \
            "test_count_free_2group_gf()");
check_equal(test_count_free_2group_gf(6), \
            1 + x + x^2 + x^3 +x^4 +2*x^5  + O(x^6), \
            "test_count_free_2group_gf()");
check_equal(test_count_free_2group_gf(6,'y^2), \
            1 + y^2 + y^4 + y^6 + y^8 + 2*y^10  + O(y^12), \
            "test_count_free_2group_gf()");
\\
test_count_free_2group(n) = polcoeff(test_count_free_2group_gf(n+1),n);
check_equal(test_count_free_2group(5), 2, "test_count_free_2group()");
\\
test_count_free_2group_vector(len) = Vec(test_count_free_2group_gf(len+1))[^1];
check_equal(vector(10,len,len--; #test_count_free_2group_vector(len)), \
            vector(10,len,len--; len), \
            "test_count_free_2group_vector() length");

{
  my(len=50,
     T = test_count_unlabelled_2group_vector(len));
  check_equal(test_count_free_2group_vector(len),
              vector(len,n, vpar_INTERNAL_rooted_vec_to_free(T,n,1)),
              "test_count_free_2group_vector() vs vpar_INTERNAL_rooted_vec_to_free()");
}

A248869(n) = if(n==0,0, test_count_unlabelled_2group(n));
check_equal(vector(#A248869_samples,n,n--; A248869(n)), \
            A248869_samples, \
            "A248869");
{
  \\ my formula in A309352
  check_equal(vector(50,n, test_count_free_2group(n)),
              vector(50,n,
                     A248869(n)
                     - sum(i=1,floor(n/2), A248869(i)*A248869(n-i))
                     + if(n%2==0, binomial(A248869(n/2)+1,2))
                     ),
              "test_count_free_2group() from A248869()");
}

\\ print(test_count_free_2group_gf(8));
\\ print(test_count_free_2group(0));
\\ print(Vec(test_count_free_2group_gf(#A248869_samples)));

{
  \\ free trees 2-group by iteration
  for(n=1,14,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_free_automorphisms_le(vpar,2));
     \\ print("n="n);
     check_equal(count,
                 test_count_free_2group(n),
                 "vpar_is_free_automorphisms_le() count vs test_count_free_2group()"));
}


\\------------------------------------------------------------------------------
progress("vpar_cordial_labelling_delta()");

check_equal(vpar_cordial_labelling_delta([0],[1]), 0);


\\------------------------------------------------------------------------------
progress("vpar_to_lexmax()");

check_equal(test_to_lexmax_by_search([0,1,2,3]), [4,3,1,0], \
            "test_to_lexmax_by_search()");

test_vector_num_different(vec1,vec2) = \
  sum(i=1,#vec1, vec1[i]!=vec2[i]);
test_vector_is_one_different(vec1,vec2) = \
  test_vector_num_different(vec1,vec2)==1;

{
  \\ n=4 lexmax forests differing by 1 entry
  \\ https://hog.grinvin.org/ViewGraphInfo.action?id=32274
  my(n=4,
     vpars=vpar_unlabelled_all(n,1));
  vpars=apply(test_to_lexmax_by_search,vpars);
  vpars=vecsort(vpars);
  check_equal(#vpars, 9);

  my(m=matrix(#vpars,#vpars,i,j, test_vector_is_one_different(vpars[i],vpars[j])),
     degrees=vector(#vpars,i, vecsum(m[i,])));
  check_equal(degrees, [1, 4, 3, 4, 1, 6, 1, 2, 2]);

  my(leaves=select(i->degrees[i]==1, [1..#vpars]));
  check_equal(#leaves, 3);
  check_equal(leaves, [1, 5, 7]);
  check_equal(vecextract(vpars,leaves),
              [[0,0,0,0],      \\ singletons
               [4,3,1,0],      \\ path-4
               [4,4,0,3]]);    \\ claw

  my(neighbours=vector(#leaves,i,
                       my(f=leaves[i],t); for(j=1,#vpars, if(m[f,j],t=j)); t));
  check_equal(neighbours,[2, 4, 6]);
  check_equal(vecextract(vpars,neighbours),
              [[4,0,0,0],      \\ path-2 and singletons
               [4,3,0,0],      \\ two path-2
               [4,4,0,0]]);    \\ path-3 and singleton
  check_equal(vecextract(degrees,neighbours),
              [4, 4, 6]);

  my(cliques=select(i->my(a=vpars[i]);a[2]=x;a==[4,x,0,0], [1..#vpars]));
  check_equal(cliques, [2, 3, 4, 6]);
  check_equal(vecextract(vpars,cliques),
              [[4,0,0,0],
               [4,1,0,0],
               [4,3,0,0],
               [4,4,0,0]]);
  for(i=1,#cliques,
     for(j=1,#cliques,
        \\ print(i","j"  "cliques[i]","cliques[j]);
        check_equal(m[cliques[i],cliques[j]],  i!=j)));
}

{
  \\ n=5 lexmax forests differing by 1 entry
  \\ https://hog.grinvin.org/ViewGraphInfo.action?id=32276
  my(n=5,
     vpars=vpar_unlabelled_all(n,1));
  vpars=apply(test_to_lexmax_by_search,vpars);
  vpars=vecsort(vpars);
  check_equal(#vpars, 20);

  my(m=matrix(#vpars,#vpars,i,j, test_vector_is_one_different(vpars[i],vpars[j])),
     degrees=vector(#vpars,i, vecsum(m[i,])));
  check_equal(degrees, [1,4,3,5,2,3,3,7,4,6,4,2,6,2,5,1,6,4,2,2]);

  my(leaves=select(i->degrees[i]==1, [1..#vpars]));
  check_equal(#leaves, 2);
  check_equal(leaves, [1, 16]);
  check_equal(vecextract(vpars,leaves),
              [[0,0,0,0,0],      \\ singletons
               [5,5,4,2,0]]);    \\ path-5

  my(neighbours=vector(#leaves,i,
                       my(f=leaves[i],t); for(j=1,#vpars, if(m[f,j],t=j)); t));
  check_equal(neighbours, [2, 13]);
  check_equal(vecextract(vpars,neighbours),
              [[5,0,0,0,0],      \\ path-2 and singletons
               [5,5,4,0,0]]);    \\ path-3 and path-2
  check_equal(vecextract(degrees,neighbours),
              [4, 6]);

  my(neighbours2=select(j->m[2,j], [1..#vpars]));
  check_equal(neighbours2, [1, 3, 4, 8]);
  check_equal(vecextract(vpars,neighbours2),
              [[0,0,0,0,0],
               [5,1,0,0,0],   \\ cf [5,0,0,0,0]
               [5,4,0,0,0],
               [5,5,0,0,0]]);

  my(neighbours13=select(j->m[13,j], [1..#vpars]));
  check_equal(neighbours13, [8, 10, 14, 15, 16, 17]);
  check_equal(vecextract(vpars,neighbours13),
              [[5,5,0,0,0],   \\  \ pos=3
               [5,5,2,0,0],   \\  /              [5,5,4,0,0]
               [5,5,4,0,3],   \\  \ pos=5
               [5,5,4,0,4],   \\  /
               [5,5,4,2,0],   \\  pos=4
               [5,5,5,0,0]]); \\  pos=3 again
}


\\-----------------------------------------------------------------------------
progress("vpar_girth()");

check_equal(vpar_girth([]), 0, "vpar_girth() empty");
check_equal(vpar_girth([0]), 0);
check_equal(vpar_girth([1]), 1);
check_equal(vpar_girth([2,1]), 2);
check_equal(vpar_girth([2,1,1]), 2);
check_equal(vpar_girth([2,1,4,5,3]), 2);
check_equal(vpar_girth([2,3,1]), 3);

test_girth_by_cycle_lengths(vpar) =
{
  \\ smallest among the non-zeros
  my(vec=vpar_component_cycle_lengths(vpar));
  vec=select(n->n, vec);
  if(#vec, vecmin(vec), 0);
}
{
  for(i=1,#test_sample_forests_and_cyclics,
     my(vpar=test_sample_forests_and_cyclics[i],
        got=vpar_girth(vpar));
     check_equal(got,
                 test_girth_by_cycle_lengths(vpar),
                 "vpar_girth() vs test_girth_by_cycle_lengths()");
  );
}


{
  \\ 5 --> 1 --> 2 --> 3       1111 -> 4 -> 13 -> 22
  \\              ^   v                       ^   v
  \\                4                          112
  my(n=4);
  check_equal(test_make_Bulgarian_solitaire(n,1),
              [2, 3, 4, 2, 1]);
}

\\ vector(16,n, vpar_girth(test_make_Bulgarian_solitaire(n,1)))
{
  for(n=0,min(#A277227_samples-1,20),
     my(vpar=test_make_Bulgarian_solitaire(n,1),
        got=vpar_girth(vpar));
     check_equal(got, A277227_samples[n+1],
                 "A277227 Bulgarian solitaire vpar_girth()");

     my(t=test_untriangular_ceil(n),
        r=test_triangular(t)-n);
     check_equal(test_triangular(t) - r,  n);
     check_equal(got, if(t, t/gcd(t,r), 1),
                 "Bulgarian solitaire vpar_girth() by gcd");
  );

  for(n=0,#A277227_samples-1,
     my(t=test_untriangular_ceil(n),
        r=test_triangular(t)-n);
     check_equal(test_triangular(t) - r,  n);
     check_equal(0 <= r && r <= t, 1);
     check_equal(if(t, t/gcd(t,r), 1),
                 A277227_samples[n+1],
                 "A277227 n/gcd(n,k) by untriangular"));

  my(n=1, t=1, k=0);
  while(n<=#A277227_samples,
        check_equal(t/gcd(t,k), A277227_samples[n],
                    "A277227 n/gcd(n,k) by t,k");
        n++; k++; if(k>=t, t++; k=0);
       );
}
{
  for(n=1,#A054531_samples,
     my(t=test_untriangular_ceil(n),
        r=test_triangular(t)-n);
     check_equal(0 <= r && r <= t, 1);
     check_equal(test_triangular(t) - r,  n);
     check_equal(if(t, t/gcd(t,r), 1),  A054531_samples[n],
                 "A054531 n/gcd(n,k) by untriangular"));

  my(n=1, t=1, k=1);   \\ t>=1, 1<=k<=t
  while(n<=#A054531_samples,
        check_equal(t/gcd(t,k), A054531_samples[n],
                    "A054531 n/gcd(n,k) by t,k");
        n++; k++; if(k>t, t++; k=1);
       );
}


\\-----------------------------------------------------------------------------
progress("vpar_preorder_reverse()");

{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i],
        got=vpar_preorder_reverse(vpar));
     check_equal(got,
                 vpar_relabel_preorder(vpar_relabel_reverse(vpar)),
                 "vpar_preorder_reverse() vs vpar_relabel_reverse()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_preorder_reverse_perm()");

{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i],
        got=vpar_preorder_reverse_perm(vpar));
     check_equal(got,
                 vpar_seq_preorder(vpar,1)^-1,
                 "vpar_preorder_reverse_perm() vs vpar_seq_preorder() down=1"));
}



\\-----------------------------------------------------------------------------
progress("vpar_preorder_depths_vector()");

check_equal(vpar_preorder_depths_vector([0,0]), [0,0], \
           "vpar_preorder_depths_vector() 2-singletons");
check_equal(vpar_preorder_depths_vector([0,1]), [0,1], \
           "vpar_preorder_depths_vector() path-2");

{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i],
        got=vpar_preorder_depths_vector(vpar));
     check_equal(got, vpar_depths_vector(vpar),
                 "vpar_preorder_depths_vector() vs vpar_depths_vector()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_preorder_depths_to_sizes()");

check_equal(vpar_preorder_depths_to_sizes([0,0,0,0,0]), [1,1,1,1,1]);
check_equal(vpar_preorder_depths_to_sizes([0,1,0,0,0]), [2,1,1,1,1]);
check_equal(vpar_preorder_depths_to_sizes([0,0,0,0,1]), [1,1,1,2,1]);
{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i],
        depths=vpar_depths_vector(vpar),
        want_sizes=vpar_subtree_sizes_vector(vpar),
        got_sizes=vpar_preorder_depths_to_sizes(depths));
     \\ print(vpar);
     check_equal(got_sizes, want_sizes,
                 "vpar_preorder_depths_to_sizes() vs vpar_subtree_sizes_vector()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_ancestor_cumulative()");

check_equal(vpar_INTERNAL_ancestor_cumulative([],[],0), [], \
            "vpar_INTERNAL_ancestor_cumulative() empty");
check_equal(vpar_INTERNAL_ancestor_cumulative([0],[99],0), [99], \
            "vpar_INTERNAL_ancestor_cumulative() singleton");
check_equal(vpar_INTERNAL_ancestor_cumulative([0],[99],1), [99], \
            "vpar_INTERNAL_ancestor_cumulative() singleton d=1");

check_equal(vpar_INTERNAL_ancestor_cumulative([0,1,2],[10,200,3000],0), \
            [10, 210, 3210], \
            "vpar_INTERNAL_ancestor_cumulative() path-3");
check_equal(vpar_INTERNAL_ancestor_cumulative([2,3,0],[10,200,3000],0), \
            [3210, 3200, 3000], \
            "vpar_INTERNAL_ancestor_cumulative() path-3 down");
check_equal(vpar_INTERNAL_ancestor_cumulative([0,1,2],[10,200,3000],1), \
            [10, 211, 3212], \
            "vpar_INTERNAL_ancestor_cumulative() path-3 d=1");

check_equal(vpar_INTERNAL_ancestor_cumulative([1],[10],0), [10], \
            "vpar_INTERNAL_ancestor_cumulative() self-loop");
check_equal(vpar_INTERNAL_ancestor_cumulative([1],[10],1), [10], \
            "vpar_INTERNAL_ancestor_cumulative() self-loop d=1");

check_equal(vpar_INTERNAL_ancestor_cumulative([3,1,2, 1], [10,200,3000,40000],0), \
            [3210,3210,3210, 43210], \
            "vpar_INTERNAL_ancestor_cumulative() cycle");
check_equal(vpar_INTERNAL_ancestor_cumulative([3,1,2, 1], [10,200,3000,40000],1), \
            [3212,3212,3212, 43213], \
            "vpar_INTERNAL_ancestor_cumulative() cycle d=1");
check_equal(vpar_INTERNAL_ancestor_cumulative([2, 4,2,3], [10,200,3000,40000],0), \
            [43210, 43200,43200,43200], \
            "vpar_INTERNAL_ancestor_cumulative() cycle");
check_equal(vpar_INTERNAL_ancestor_cumulative([2, 4,2,3], [10,200,3000,40000],1), \
            [43213, 43202,43202,43202], \
            "vpar_INTERNAL_ancestor_cumulative() cycle d=1");

test_ancestor_cumulative_of_vertex(vpar,v,vec,d) =
{
  my(seen=vectorsmall(#vpar),
    ret=0);
  while(1,
        ret += vec[v];
        seen[v] = 1;
        v=vpar[v];
        if(!v, break);       \\ root
        if(seen[v], break);  \\ cycle
        ret += d);
  ret;
}

{
  for(i=1,#test_sample_forests_and_cyclics,
     my(vpar=test_sample_forests_and_cyclics[i],
        zeros=vector(#vpar),
        ones=vector(#vpar,i,1));
     check_equal(vpar_INTERNAL_ancestor_cumulative(vpar,zeros,1),
                 vpar_depths_vector(vpar),
                 "vpar_INTERNAL_ancestor_cumulative() zeros for vpar_depths_vector()");

     check_equal(vpar_INTERNAL_ancestor_cumulative(vpar,ones,0),
                 apply(d->d+1, vpar_depths_vector(vpar)),
                 "vpar_INTERNAL_ancestor_cumulative() ones for depths+1");

     my(vec=vector(#vpar,i, random(20)));
     forstep(d=0,16,16,
        check_equal(vpar_INTERNAL_ancestor_cumulative(vpar,vec,d),
                    vector(#vpar,v, test_ancestor_cumulative_of_vertex(vpar,v,vec,d)),
                    "vpar_INTERNAL_ancestor_cumulative() vs test_ancestor_cumulative_of_vertex()"));
     );
}


\\-----------------------------------------------------------------------------
progress("vpar_delete_edge()");

check_equal(vpar_delete_edge([0],1,1), [0]);
check_equal(vpar_delete_edge([1],1,1), [0]);
check_equal(vpar_delete_edge([0,1],1,2), [0,0]);
check_equal(vpar_delete_edge([0,1],2,1), [0,0]);
check_equal(vpar_delete_edge([2,1],1,2), [0,0]);
check_equal(vpar_delete_edge([2,1],2,1), [0,0]);


\\-----------------------------------------------------------------------------
progress("vpar_num_grandchildren_of_vertex()");

check_equal(vpar_num_grandchildren_of_vertex([1],1), 1, \
           "vpar_num_grandchildren_of_vertex() self-loop own grandchild");
check_equal(vpar_num_grandchildren_of_vertex([1],0), 0, \
           "vpar_num_grandchildren_of_vertex() self-loop no roots");

check_equal(vpar_num_grandchildren_of_vertex([2,1],0), 0);
check_equal(vpar_num_grandchildren_of_vertex([2,1],1), 1, \
           "vpar_num_grandchildren_of_vertex() 2-cycle own grandchild");
check_equal(vpar_num_grandchildren_of_vertex([2,1],2), 1);
check_equal(vpar_num_grandchildren_of_vertex([2,3,1],1), 1, \
           "vpar_num_grandchildren_of_vertex() 3-cycle");

test_num_grandchildren_of_vertex_by_children(vpar,v) =
{
  my(vec=vpar_children_of_vertex(vpar,v));
  sum(i=1,#vec, vpar_num_children_of_vertex(vpar,vec[i]));
}

{
  for(i=1,#test_sample_forests_and_cyclics,
     my(vpar=test_sample_forests_and_cyclics[i]);
     check_equal(vpar_num_grandchildren_of_vertex(vpar,0),
                 #vpar_children_of_set(vpar,vpar_roots(vpar)),
                 "vpar_num_grandchildren_of_vertex() v=0 is children of roots, per addhelp");

     for(v=0,#vpar,
        check_equal(vpar_num_grandchildren_of_vertex(vpar,v),
                    test_num_grandchildren_of_vertex_by_children(vpar,v))));
}


\\-----------------------------------------------------------------------------
progress("vpar_diameter_and_count_cyclic()");

check_equal(vpar_diameter_and_count_cyclic([]), [-1,1], \
            "vpar_diameter_and_count_cyclic() empty");
check_equal(vpar_diameter_and_count_cyclic([0]), [0,1], \
            "vpar_diameter_and_count_cyclic() singleton");

check_equal(vpar_diameter_and_count_cyclic([2,1]), [1,1], \
            "vpar_diameter_and_count_cyclic() 2-cycle");
check_equal(vpar_diameter_and_count_cyclic([2,3,1]), [1,3], \
            "vpar_diameter_and_count_cyclic() 3-cycle");
check_equal(vpar_diameter_and_count_cyclic([2,3,4,1]), [2,2], \
            "vpar_diameter_and_count_cyclic() 4-cycle");


{
  for(n=1,20,
     my(vpar=test_make_cycle(n),
        want_diameter=n\2);
     check_equal(vpar_diameter_and_count_cyclic(vpar),
                 [want_diameter,
                  if(n%2==0, n/2, n)],
                 "vpar_diameter_and_count_cyclic() cycle");
  );
}

test_cycle_heights_and_counts_to_diameter_and_count(heights,heights_count) =
{
  \\ print("test_cycle_heights_and_counts_to_diameter_and_count()");
  \\ print("  "heights);
  \\ print("  "heights_count);
  #heights == #heights_count || error();

  my(diameter=0,
     count=0);
  for(i=1,#heights,
     my(prospective= heights[i]);
     if(prospective > diameter,
        diameter = prospective; count=heights_count[i],
        prospective == diameter,
        count += heights_count[i]);

     \\ counts attained by star at end of path, which is length 2
     if(heights_count[i]>=2,
        my(prospective= 2,
           prospective_count=binomial(heights_count[i],2));
        if(prospective > diameter,
           diameter = prospective; count=prospective_count,
           prospective == diameter,
           count += prospective_count));
  );

  for(i=1,#heights,
     for(j=i+1,#heights,
        my(prospective= heights[i] + heights[j]
                       + abs(test_mod_nearest(i-j,#heights)));
        if(prospective > diameter,
           diameter = prospective;
           count=heights_count[i]*heights_count[j];
           \\ print("  "i","j" new high "diameter" count "count);
           ,
           prospective == diameter,
           count += heights_count[i]*heights_count[j];
           \\ print("  "i","j" equal high "diameter" count "count);
          )));
  [diameter,count];
}
{
  check_equal(test_cycle_heights_and_counts_to_diameter_and_count([7],[1]),
              [7,1],
              "test_cycle_heights_and_counts_to_diameter_and_count()");
  check_equal(test_cycle_heights_and_counts_to_diameter_and_count([3,2,1,0,2],
                                                                  [1,1,1,1,1]),
              [6,4],
              "test_cycle_heights_and_counts_to_diameter_and_count()");
  check_equal(test_cycle_heights_and_counts_to_diameter_and_count
              ([0,0,10,10,0,0],
               [9,9, 2, 3,9,9]),
              [21, 6],
              "test_cycle_heights_and_counts_to_diameter_and_count()");
}

\\ Return a vpar vector which is a path length n plus c many additional
\\ vertices hanging off the n'th vertex.
test_make_path_end_count(n,c) =
{
  c >= 1 || error();
  if(n==0, c==1 || error("test_make_path_end_count() n=0 must c=1"));
  vector(n+c,i, min(i-1,n));
}
{
  check_equal(test_make_path_end_count(3,1), [0,1,2,3],
             "test_make_path_end_count() 3,1");
  check_equal(test_make_path_end_count(3,2), [0,1,2, 3,3],
             "test_make_path_end_count() 3,2");

  for(n=1,8,
     for(c=1,5,
        my(vpar=test_make_path_end_count(n,c));
        vpar_validate(vpar);
        check_equal(vpar_eccentricity_of_vertex(vpar,1), n,
                    "test_make_path_end_count() ecc");
        \\ print(vpar);
        for(j=#vpar-c+1,#vpar,
           check_equal(vpar_path_length(vpar,1,j), n,
                    "test_make_path_end_count() path length"))));
}

test_make_cyclic_heights_and_counts(heights,heights_count) =
{
  \\ print("test_make_cyclic_heights_and_counts()");
  \\ print("  "heights);
  \\ print("  "heights_count);
  my(vpars=vector(#heights,i,
                  test_make_path_end_count(heights[i],heights_count[i])),
     vpar=test_concat_forest_vector(vpars));
  \\ print("  vpars "vpars);
  \\ print("  forest "vpar);
  vpar_validate(vpar);
  test_cyclic_roots(vpar);
}
{
  check_equal(test_make_cyclic_heights_and_counts([3],[1]),
              [1,1,2,3],
              "test_make_cyclic_heights_and_counts()");
  check_equal(test_make_cyclic_heights_and_counts([3,3],[1,1]),
              [5,1,2,3, 1,5,6,7],
              "test_make_cyclic_heights_and_counts()");
}

{
  for(rep=1,100,
     my(cycle_length=random(3)+1,
        heights=vector(cycle_length,i, random(5)),
        heights_count=vector(#heights,i,
                             if(heights[i]==0, 1, random(5)+1)),
        vpar=test_make_cyclic_heights_and_counts(heights,heights_count));
     \\ vpar=vpar_relabel_random(vpar);

     \\ print("cycle_length "cycle_length);
     \\ print("heights "heights);
     \\ print("heights_count "heights_count);
     \\ print("vpar  "vpar);

     check_equal(vpar_num_components(vpar), 1,
                 "test_make_cyclic_heights_and_counts() got vpar_num_components()");
     check_equal(vpar_component_cycle_lengths(vpar), [cycle_length],
                 "test_make_cyclic_heights_and_counts() got vpar_component_cycle_lengths()");

     my(want=test_cycle_heights_and_counts_to_diameter_and_count
             (heights,heights_count));

     check_equal(vpar_diameter(vpar), want[1],
                 "vpar_diameter() random vs test_cycle_heights_and_counts_to_diameter_and_count()");

     check_equal(vpar_diameter_and_count_cyclic(vpar), want,
                 "vpar_diameter_and_count_cyclic() random vs test_cycle_heights_and_counts_to_diameter_and_count()");
  );
}

{
  for(n=0,10,
     my(vpar=vpar_make_path(n));
     if(n>=1,
        check_equal(vpar_degree_of_vertex(vpar,1),if(n==1,0,1)));

     \\ PENDING
     check_equal(vpar_diameter_and_count(Vecsmall(vpar)), [n-1,1],
                 Str("vpar_diameter_and_count() path "n)));
}


\\-----------------------------------------------------------------------------
progress("vpar_eccentricities_vector()");

{
  \\   v++
  \\   1-+
  \\   |
  \\   2
  \\   |
  \\   3
  \\   |
  \\   4
  \\  /|\ \
  \\ 5 6 7 8
  my(vpar=[1, 1,2,3, 4,4,4,4]);
  check_equal(vpar_eccentricity_of_vertex(vpar,1), 4);
  check_equal(vpar_eccentricities_vector(vpar),
              [4,3,2,3, 4,4,4,4]);
}


{
  for(rep=1,100,
     my(cycle_length=random(3)+1,
        heights=vector(cycle_length,i, random(5)),
        heights_count=vector(#heights,i,
                             if(heights[i]==0, 1, random(5)+1)),
        vpar=test_make_cyclic_heights_and_counts(heights,heights_count));
     \\ vpar=vpar_relabel_random(vpar);

     \\ print("cycle_length "cycle_length);
     \\ print("heights "heights);
     \\ print("heights_count "heights_count);
     \\ print("vpar  "vpar);

     check_equal(vpar_num_components(vpar), 1,
                 "test_make_cyclic_heights_and_counts() num components");
     check_equal(vpar_component_cycle_lengths(vpar), [cycle_length],
                 "test_make_cyclic_heights_and_counts() cycle length");

     check_equal(vpar_eccentricities_vector(vpar),
                 apply(vecmax,Vec(vpar_path_lengths_matrix(vpar))),
                 "vpar_eccentricities_vector() vs max vpar_path_lengths_matrix()");
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_childful_vector() count");
\\
\\ This all better in examples/footprints-preorder.gp but can have some
\\ checks here too.

test_count_preorder_childful_vector_by_search(vec,forest=0) =
{
  my(vpar=vpar_preorder_first(#vec),
     count=0);
  until((vpar=vpar_preorder_next(vpar,forest))=='none,
        if(vpar_is_childful_vector(vpar)==vec, count++));
  count;
}

check_equal(test_count_preorder_childful_vector_by_search([0]),1, \
           "test_count_preorder_childful_vector_by_search() singleton");

check_equal(test_count_preorder_childful_vector_by_search([0,0]),  0);
check_equal(test_count_preorder_childful_vector_by_search([0,0],1),1);

check_equal(test_count_preorder_childful_vector_by_search([1,0]),  1);
check_equal(test_count_preorder_childful_vector_by_search([1,0],1),1);

check_equal(test_count_preorder_childful_vector_by_search([1,0,0]),  1);
check_equal(test_count_preorder_childful_vector_by_search([1,0,0],1),2);

check_equal(test_count_preorder_childful_vector_by_search([1,1,1,0],1),1);

test_count_preorder_childful_vector_by_recursion_Ccount(vec,must=0,k=1) =
{
  \\ print("Ccount "vec" must="must" k="k);
  if(#vec==0,
     return(if(must, 0, 1)));

  my(rest=if(#vec>=2,vec[2..#vec],[]));  \\ no empty range in gp 2.7
  if(vec[1],
     if(must, test_count_preorder_childful_vector_by_recursion_Ccount(rest,1,k+1),
        sum(i=1,k, test_count_preorder_childful_vector_by_recursion_Ccount(rest,1,i+1)));
     ,
     \\ if(must, Ccount(rest,0,k),
     \\    sum(i=1,k, Ccount(rest,0,i)));

     my(i=1,zeros=0); while(i<=#vec && vec[i]==0, zeros++;i++);
     rest=if(i<=#vec,vec[i..#vec],[]);  \\ no empty range in gp 2.7
     \\ print("zeros "zeros" rest "rest);
     if(must,
        zeros--;
        \\ print("decr for must to zeros "zeros);
       );
     \\ print("zeros "zeros);
     if(zeros==0,
        return(test_count_preorder_childful_vector_by_recursion_Ccount(rest,0,k)));
     sum(i=1,k,  \\ i many left open, k-i closed
         \\ print(" bin "k-i+zeros-1," ",zeros-1" to "rest);
         binomial(k-i+zeros-1,zeros-1)
         * test_count_preorder_childful_vector_by_recursion_Ccount(rest,0,i));
    );


  \\ not working
  \\ error();
  \\ if(#vec==0, return(1));
  \\ if(vec[#vec], return(0));
  \\ if(#vec==1, return(1));
  \\ if(!forest,
  \\    if(!vec[1], return(0));
  \\    vec=vec[^1]);
  \\ vec=concat(1,vec);
  \\ my(runs=test_vector_run_lengths(vec),
  \\    factors=vector(#vec));
  \\ factors[1]=1;
  \\ #runs%2==0 || error();
  \\ forstep(i=1,#runs-3,2,
  \\    my(p=runs[i], q=runs[i+1],
  \\       new_factors=vector(#factors));
  \\    for(t=0,p,
  \\       new_factors[t+1] += binomial(p+q,q));
  \\    );
  \\ vecsum(factors);
}
test_count_preorder_childful_vector_by_recursion(vec,forest=0) = \
  test_count_preorder_childful_vector_by_recursion_Ccount(vec);
\\
\\ speedup:
\\ read("memoize.gp");
\\ test_count_preorder_childful_vector_by_recursion_Ccount = \
\\   memoize(test_count_preorder_childful_vector_by_recursion_Ccount);



test_count_preorder_childful_vector_by_upward(vec) =
{
  \\ print("test_count_preorder_childful_vector_by_upward() "vec);
  if(#vec==0 || vec[#vec], return(0));
  vec=test_vector_run_lengths(vec);
  #vec%2==0 || vec=concat(0,vec);
  \\ print("  runs "vec);
  my(P=sum(i=1,#vec/2, vec[2*i-1]),
     counts=vector(P+1));     \\ counts[k+1] = with k headroom
  counts[1]=1;

  my(p=vec[#vec-1],q=vec[#vec],
     new_P=P-p);
  counts=vector(new_P+1,k,k--;
                my(places=k+p+1);
                \\ print("    k="k" places="places" binomial "places-1+q-1" "q-1);
                binomial(places-1 + q-1, q-1));
  P=new_P;

  forstep(i=#vec-3,1,-2,
     my(p=vec[i], q=vec[i+1],
        new_P=P-p,
        b=vector(P+1,places, binomial(places-1 + q-1, q-1)));
     \\ print("  i="i" p="p" q="q"  P="P" new_P="new_P"  counts "counts"   b="b);
     my(bpoly=Polrev(b),
        cpoly=Polrev(counts),
        prodpoly=bpoly*cpoly);
     \\ print("    b      ",bpoly+O(x^(P+2)));
     \\ print("    counts ",cpoly+O(x^(P+2)));
     \\ print("    prod   ",prodpoly+O(x^(P+2)));
     counts=vector(new_P+1,k,k--;
                   \\ print("   k="k" headroom");
                   my(s=sum(j=0,k+p,
                            my(places=k+p+1-j,
                               num=places-1 + q-1);
                            num = k+p-j + q-1;
                            \\ print("    j="j" places="places" binomial("num","q-1") = b["k+p-j+1"]="binomial(num,q-1)" * counts["j+1"]="counts[j+1]);
                            \\ binomial(num,q-1)
                            \\ b[places]
                            b[k+p-j +1]          \\ reverse cross products
                            * counts[j +1]
                        ));
                   s;
                   \\ print("    s="s);
                   my(c=polcoeff(prodpoly, k+p));
                   \\ print("    coeff="c);
                   c;
                  );
     P=new_P;
     );
  #counts==1 || error();
  counts[1];
}

\\ g = test_binomial_places_gf(k,limit)
\\ Return a generating function t_SER where c = polcoeff(g,i) is the number
\\ of ways to put k many things into i+1 many buckets.  The things are
\\ undistinguished, the buckets are distinguished.
\\ The binomial is i+k,k, being a list of i-1 separators and k things,
\\ choose where are the things.  Or binomial(i+k,i) the same, choosing where
\\ the separators.  g has "limit" many terms.
\\
test_binomial_places_gf(k,limit) = \
  Ser(vector(limit,i,i--; binomial(i+k, k)),,limit);
\\
check_equal(test_binomial_places_gf(0,5), 1 + x + x^2 + x^3 + x^4 + O(x^5), \
            "test_binomial_places_gf() 0");
check_equal(test_binomial_places_gf(1,5), 1 + 2*x + 3*x^2 + 4*x^3 + 5*x^4 + O(x^5), \
            "test_binomial_places_gf() 1");
check_equal(test_binomial_places_gf(2,5), 1 + 3*x + 6*x^2 + 10*x^3 + 15*x^4 + O(x^5), \
            "test_binomial_places_gf() 2");


test_count_preorder_childful_vector_by_gf(vec) =
{
  \\ print("test_count_preorder_childful_vector_by_gf() "vec);
  if(#vec==0 || vec[#vec], return(0));
  vec=test_vector_run_lengths(vec);
  #vec%2==0 || vec=concat(0,vec);
  \\ print("  runs "vec);
  my(P=sum(i=1,#vec/2, vec[2*i-1]));
  my(T=P);

  my(cpoly=1);
  forstep(i=#vec-1,1,-2,
     my(p=vec[i], q=vec[i+1]);
     \\ print("  i="i" p="p" q="q"  P="P"  cpoly "cpoly);
     cpoly *= test_binomial_places_gf(q-1,P+1);

     \\ cpoly = serchop(cpoly,p)/x^p;     \\ serchop() new in 2.11
     for(i=1,p, cpoly = (cpoly-polcoeff(cpoly,0))/x);

     P -= p);
  \\ print("  final T="T" cpoly "cpoly);
  polcoeff(cpoly,0);
}

\\ places=k+p+1-j ranges 1 at j=k+p to k+p=new_P+p = P at j=0

\\ k things in m buckets
\\ m-1 separators
\\ binomial(m+k-1,m-1)
{
  my(m=1,k=1);
  check_equal(m+k-1, 1);
  check_equal(binomial(m+k-1,m-1), 1);
  check_equal(binomial(m+k-1,k),   1);
}
{
  my(m=2,k=1);
  check_equal(m+k-1, 2);
  check_equal(binomial(m+k-1,m-1), 2);
  check_equal(binomial(m+k-1,k),   2);
}
{
  my(m=2,k=2);
  check_equal(m+k-1, 3);
  check_equal(binomial(m+k-1,m-1), 3);
  check_equal(binomial(m+k-1,k),   3);
}

test_count_preorder_childful_vector_tvec_is_good(pqvec,tvec) =
{
  \\ each partial sum ps >= ts
  \\ sum(i=1,k, p[i]) >= sum(i=1,k, t[i])
  \\ and whole sum(p) == sum(t)
  \\
  \\          p +--+ t
  \\            |  |
  \\         +--+  |        non-zero steps t
  \\         |  |  |        staying below p steps
  \\         |  |  |        final t going up to p end
  \\      +--+  |  |
  \\      |  |  |  |
  \\      |--|--|  |
  \\  p+--+  |  |  |
  \\   |  |  |  |  |
  \\  t|--|  |  |  |
  \\   |  |  |  |  |
  \\   +--+--+--+--+

  \\ print("test_count_preorder_childful_vector_tvec_is_good() "pqvec" "tvec);
  my(total=0);
  for(i=1,#tvec,
     total += pqvec[2*i-1]; \\ p
     total -= tvec[i];      \\ t
     if(total<0, return(0)));
  total==0;
}
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1], [3]), 0);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1], [4]), 1);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1], [5]), 0);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1,5,1], [0,9]), 1);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1,5,1], [0,10]), 0);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1,5,1], [4,5]), 1);
check_equal(test_count_preorder_childful_vector_tvec_is_good([4,1,5,1], [4,6]), 0);

test_count_preorder_childful_vector_tvec_product(pqvec,tvec) =
{
  \\ print("test_count_preorder_childful_vector_tvec_product() "pqvec" "tvec);
  prod(i=1,#tvec,
       my(t=tvec[i], q=pqvec[2*i]);
       binomial(tvec[i]+q-1, q-1));
}
\\
check_equal(test_count_preorder_childful_vector_tvec_product([4,1], [0]), 1);
check_equal(test_count_preorder_childful_vector_tvec_product([4,1], [4]), 1);
check_equal(test_count_preorder_childful_vector_tvec_product([4,2], [0]), 1);
check_equal(test_count_preorder_childful_vector_tvec_product([4,2], [4]), 5);

test_count_preorder_childful_vector_tvec_first(pqvec) =
{
  my(tvec=vector(#pqvec/2));
  if(#tvec,
     tvec[#tvec] = sum(i=1,#tvec, pqvec[2*i-1]));
  tvec;
}
check_equal(test_count_preorder_childful_vector_tvec_first([4,1,5,1]), [0,9]);

test_count_preorder_childful_vector_tvec_next(pqvec,tvec) =
{
  \\ print("test_count_preorder_childful_vector_tvec_next() "pqvec" "tvec);
  #pqvec==2*#tvec || error();
  forstep(i=#tvec-1,1,-1,
     tvec[i]++;
     tvec[#tvec]--;
     \\ print("  try "tvec);
     if(test_count_preorder_childful_vector_tvec_is_good(pqvec,tvec),
        test_count_preorder_childful_vector_tvec_is_good(pqvec,tvec) || error();
        return(tvec));
     tvec[#tvec] += tvec[i];
     tvec[i]=0);
  return('none);
}
check_equal(test_count_preorder_childful_vector_tvec_next([4,1,5,1],[0,9]), [1,8], \
           "test_count_preorder_childful_vector_tvec_next()");
check_equal(test_count_preorder_childful_vector_tvec_next([4,1,5,1],[4,5]), 'none);
check_equal(test_count_preorder_childful_vector_tvec_first([4,1,5,1,1,1]), [0,0,10]);
check_equal(test_count_preorder_childful_vector_tvec_next([4,1,5,1,1,1],[0,0,10]), [0,1,9]);
check_equal(test_count_preorder_childful_vector_tvec_next([4,1,5,1,1,1],[0,9,1]), [1,0,9]);
check_equal(test_vector_run_lengths([0]), [1]);

test_count_preorder_childful_vector_by_binomials(vec) =
{
  \\ print("test_count_preorder_childful_vector_by_binomials() "vec);
  if(#vec==0, return(1));
  if(vec[#vec], return(0));
  my(pqvec=test_vector_run_lengths(vec));
  if(#pqvec%2, pqvec=concat(0,pqvec));
  \\ print("  pqvec "pqvec);
  my(tvec=test_count_preorder_childful_vector_tvec_first(pqvec),
     total=0);
  until((tvec=test_count_preorder_childful_vector_tvec_next(pqvec,tvec))=='none,
        my(p=test_count_preorder_childful_vector_tvec_product(pqvec,tvec));
        \\ print("  "tvec" product "p);
        total += p;
       );
  total;
}

\\  1  3     1
\\  |  |     | \
\\  2  4     2  3
\\              |
\\              4
check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0]), 1, \
            "test_count_preorder_childful_vector_by_search() tree");
check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0],1), 2, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,1,0]), 2, \
            "test_count_preorder_childful_vector_by_upward() 1010");
check_equal(test_count_preorder_childful_vector_by_binomials([1,0,1,0]), 2, \
            "test_count_preorder_childful_vector_by_binomials() 1010");

\\      1     5  6     1            1              1  3     6
\\      | \      |     | \          | \            |  | \  \
\\      2  3  5  7     2  3  5  6   2  3           2  4  5  6
\\         |              |     |      | \  \               |
\\         4              4  5  7      4  5  6              7
\\                                           |
\\                                           7
\\       mid k=1         mid k=1      mid k=1       mid k=0
\\           j=0             j=1          j=2         j=1 1in1
\\                                                    j=0 1in2
check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0,0,1,0],1), 9, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,1,0,0,1,0]), 9, \
            "test_count_preorder_childful_vector_by_upward() 1010010");
check_equal(test_count_preorder_childful_vector_by_binomials([1,0,1,0,0,1,0]), 9, \
            "test_count_preorder_childful_vector_by_binomials() 1010010");

\\  1   *
\\  | \    4,5
\\  2
\\  | \
\\  3
check_equal(test_count_preorder_childful_vector_by_search([1,1,0,0,0],1), 6, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,1,0,0,0]), 6, \
            "test_count_preorder_childful_vector_by_upward() 11000");

check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0,1,0],1), 5, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,1,0,1,0]), 5, \
            "test_count_preorder_childful_vector_by_upward() 1000");

check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0,1,0,0],1), 14, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,1,0,1,0,0]), 14, \
            "test_count_preorder_childful_vector_by_upward() 1000");

check_equal(test_count_preorder_childful_vector_by_search([1,0,0,1,0,1,0],1), 7, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,0,1,0,1,0]), 7, \
            "test_count_preorder_childful_vector_by_upward() 1000");

\\    P=1 new_P=0
\\    j=0         j=0        j=1
\\  1     4     1  3  4    1
\\  | \   |     |     |    | \  \            q-1 leaves
\\  2  3  5     2     5    2  3  4           P-j places
\\                               |
\\                               5
check_equal(test_count_preorder_childful_vector_by_search([1,0,0,1,0],1), 3, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,0,1,0]), 3, \
            "test_count_preorder_childful_vector_by_upward() 100 10");

check_equal(test_count_preorder_childful_vector_by_search([1,0,0,0,1,0],1), 4, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,0,0,1,0]), 4, \
            "test_count_preorder_childful_vector_by_upward() 1000 10");

\\   j=0       j=0          j=1      j=1      j=1
\\  1  3     1  3  5     1        1        1    5         end j+p places
\\  |  |\    |  |        | \      | \  \   | \                q leaves
\\  2  4 5   2  4        2  3     2  3  5  2  3
\\                          | \      |        |
\\                          4  5     4        4
check_equal(test_count_preorder_childful_vector_by_search([1,0,1,0,0],1), 5, \
            "test_count_preorder_childful_vector_by_search() forest");
check_equal(test_count_preorder_childful_vector_by_recursion([1,0,1,0,0]), 5, \
            "test_count_preorder_childful_vector_by_recursion() 10 100");
check_equal(test_count_preorder_childful_vector_by_upward([1,0,1,0,0]), 5, \
            "test_count_preorder_childful_vector_by_upward() 10 100");

check_equal(test_count_preorder_childful_vector_by_search([1,0]), 1, \
            "test_count_preorder_childful_vector_by_search()");
check_equal(test_count_preorder_childful_vector_by_upward([1,0]), 1, \
            "test_count_preorder_childful_vector_by_upward()");

{
  for(n=2,10,
     \\ print("n="n);
     my(vpar=vpar_preorder_first(n),
        vecs=vector(2^n,i,
                    my(vec=binary(i-1));
                    while(#vec<n, vec=concat(0,vec));
                    vec),
        counts=vector(2^n));
     \\ print(vecs);
     until((vpar=vpar_preorder_next(vpar,1))=='none,
           my(vec=vpar_is_childful_vector(vpar),
              pos=setsearch(vecs,vec));
           if(!pos, error("not found n="n" vpar="vpar" vec="vec));
           counts[pos]++);

     for(i=1,#vecs,
        \\ print(vecs[i]);
        check_equal(test_count_preorder_childful_vector_by_recursion(vecs[i]),
                    counts[i],
                    "test_count_preorder_childful_vector_by_recursion() vs vpar_is_childful_vector() counts");
        check_equal(test_count_preorder_childful_vector_by_upward(vecs[i]),
                    counts[i],
                    "test_count_preorder_childful_vector_by_upward() vs vpar_is_childful_vector() counts"));
     );
}
{
  \\ print("start");
  for(n=2,2^10,
     \\ print("n="n);
     my(vec=binary(n)[^1],
        by_upward=test_count_preorder_childful_vector_by_upward(vec));
     \\ print(vec);
     check_equal(by_upward,
                 test_count_preorder_childful_vector_by_recursion(vec),
                 "test_count_preorder_childful_vector_by_upward() vs test_count_preorder_childful_vector_by_recursion() vs vpar_is_childful_vector()");
     check_equal(test_count_preorder_childful_vector_by_gf(vec),
                 by_upward,
                 "test_count_preorder_childful_vector_by_gf() vs test_count_preorder_childful_vector_by_upward()");
     check_equal(test_count_preorder_childful_vector_by_binomials(vec),
                 by_upward,
                 "test_count_preorder_childful_vector_by_binomials() vs test_count_preorder_childful_vector_by_upward()");
     );
  \\ print("end");
}

{
  \\ two blocks
  for(p1=0,4,
     for(q1=1,4,
        for(p2=1,4,
           for(q2=1,4,
              my(vec=concat([vector(p1,i,1), vector(q1),
                             vector(p2,i,1), vector(q2)]));
              check_equal(test_count_preorder_childful_vector_by_upward(vec),
                          test_count_preorder_childful_vector_by_recursion(vec),
                          "test_count_preorder_childful_vector_by_upward() vs test_count_preorder_childful_vector_by_recursion()");
           ))));
}

{
  for(k=2,10,
     my(total=0);
     forvec(vec=vector(k,i,[0,1]),
           total += test_count_preorder_childful_vector_by_upward(vec));
     check_equal(total,
                 vpar_count_ordered(k,1),
                 "test_count_preorder_childful_vector_by_upward() total vs vpar_count_ordered() forests"));
}


test_footprint_runs(vec) =
{
  vec[#vec]==0 || error();
  my(l=List([]),
     i=1);
  while(i<=#vec,
        my(p=-1);
        while(i<=#vec && vec[i]==1, p++; i++);
        listput(l,p);
        my(q=-1);
        while(i<=#vec && vec[i]==0, q++; i++);
        listput(l,q);
        );
  l=Vec(l);
  if(#l, l[1]++);
  l;
}
test_footprint_count(vec) =
{
  \\ Knuth's formula
  my(runs=test_footprint_runs(vec),
     k=#runs/2);
  if(k,
     prod(i=1,k,
          my(p=runs[2*i-1], q=runs[2*i]);
          binomial(p+q, q))
     * vpar_INTERNAL_Catalan_number(k-1));
}

\\ {
\\   for(k=2,7,
\\      my(total=0);
\\      print1("k="k"  ");
\\      forvec(vec=vector(k,i,[0,1]),
\\            vec[#vec]==0 || next;
\\            print1(test_footprint_count(vec)","));
\\      print();
\\      print1("     ");
\\      forvec(vec=vector(k,i,[0,1]),
\\            vec[#vec]==0 || next;
\\            print1(test_count_preorder_childful_vector_by_upward(vec)","));
\\      print();
\\      );
\\ }

\\ {
\\   \\ not always a multiple of Catalan(k-1)
\\   for(n=2,7,
\\      my(total=0);
\\      print1("n="n"  ");
\\      forvec(vec=vector(n,i,if(i==n,[0,0],[0,1])),
\\            my(count=test_count_preorder_childful_vector_by_upward(vec),
\\               pqvec=test_vector_run_lengths(vec));
\\            if(#pqvec%2, pqvec=concat(0,pqvec));
\\            \\ print(vec" "pqvec);
\\            #pqvec%2 == 0 || error();
\\            my(k = #pqvec/2,
\\               C=vpar_INTERNAL_Catalan_number(k-1));
\\            print(vec" k="k" count "count" rem "count%C));
\\      );
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_seq_diagpost()");

\\ preorder to diagpost is not unique, in general
check_equal(vpar_seq_diagpost([0,0]), Vecsmall([2,1]));
check_equal(vpar_seq_diagpost([0,1]), Vecsmall([2,1]));

check_equal(vpar_seq_diagpost([0,0,2]), Vecsmall([3,2,1]));
check_equal(vpar_horizpos_vector([0,0,2]), [0,1,1]);
{
  my(vpar=[0,1,0]);
  check_equal(vpar_seq_diagpost(vpar), Vecsmall([2,3,1]));
}

test_seq_diagpost_by_recurse(vpar,v=vpar_root(vpar)) =
{
  if(!v, Vecsmall([]),
     my(c=vpar_minmax_child_of_vertex(vpar,v,min),
        s=vpar_next_sibling_of_vertex(vpar,v));
     concat([test_seq_diagpost_by_recurse(vpar,c),
             test_seq_diagpost_by_recurse(vpar,s),
             Vecsmall([v])]));
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_seq_diagpost(vpar));
     check_equal(got,
                 test_seq_diagpost_by_recurse(vpar),
                 "vpar_seq_diagpost() vs test_seq_diagpost_by_recurse()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_preorder_is_parsed()");

{
  for(i=1,3,
     for(j=1,3,
        my(o=vpar_INTERNAL_other[i,j]);
        check_equal(o!=i && o!=j, 1);
        check_equal(o!=0, i!=j)));
}
check_equal(vpar_preorder_is_parsed([0], [1,2]), 1, \
           "vpar_preorder_is_parsed() singleton");
check_equal(vpar_preorder_is_parsed([0,1], [2,2,3]), 1, \
           "vpar_preorder_is_parsed() path-2");

\\  1 ... 3          ... 2.3
\\  |              |     1
\\  2              3.2
\\                 1
check_equal(vpar_preorder_is_parsed([0,1,0], [1,2,1,3]), 1, \
           "vpar_preorder_is_parsed() three");


\\-----------------------------------------------------------------------------
progress("Normalized Laplacian");

\\ NOT RIGHT
\\ test_sqrt_exact_if_possible(n) = \
\\   if(issquare(n), sqrtint(n), sqrt(n));
\\ test_normalized_Laplacian(vpar) =
\\ {
\\   my(degrees=vpar_degrees_vector(vpar),
\\     m=matid(#vpar));
\\   for(v=1,#vpar,
\\      if(vpar[v],
\\         m[v,vpar[v]] = m[vpar[v],v]
\\           = test_sqrt_exact_if_possible(degrees[v]*degrees[vpar[v]])));
\\   m
\\ }
\\
\\ test_num_components_ge2_vertices(vpar) =
\\ {
\\   my(sizes=vpar_component_sizes(vpar));
\\   sum(i=1,#sizes, sizes[i]>=2);
\\ }
\\ test_poly_multiplicity_of_2(poly) =
\\ {
\\   \\  my(ret=0, x=variable(poly));
\\   \\  print(poly"  "x);
\\   \\  while(poly % (x-2) == 0,
\\   \\        poly /= (x-2);
\\   \\ ret++;
\\   \\        print(ret"  "poly);
\\   \\       );
\\   \\  ret;
\\
\\   \\ my(ret=0,
\\   \\    f=factor(poly));
\\   \\ for(i=1,matsize(f)[1],
\\   \\    if(f[i,1]==x-2, ret=f[i,2]));
\\   \\ ret;
\\
\\   my(roots=polroots(poly));
\\   sum(i=1,#roots, abs(roots[i]-2) < 1e-8);
\\ }
\\ {
\\   for(i=1,#test_sample_trees_and_forests,
\\      my(vpar=test_sample_trees_and_forests[i],
\\         m=test_normalized_Laplacian(vpar),
\\         poly=charpoly(m));
\\      \\ print(m" "poly);
\\      \\ print(test_poly_multiplicity_of_2(poly), " ", test_num_components_ge2_vertices(vpar)" "vpar_component_sizes(vpar));
\\
\\      print(test_poly_multiplicity_of_2(poly), " ",
\\            test_num_components_ge2_vertices(vpar)
\\            " "vpar_component_sizes(vpar)
\\            "  "factor(poly));
\\        );
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_path_lengths_poly()");

\\ cf Graham and Lovasz, "Distance Matrix Polynomials of Trees", Bell
\\ Systems Technical Journal.

check_equal(vpar_path_lengths_poly([]), Pol(1), \
            "vpar_path_lengths_poly() empty");
check_equal(vpar_path_lengths_poly([0]), x);
check_equal(vpar_path_lengths_poly([0,1]), x^2-1);
check_equal(vpar_path_lengths_poly([0,1,2]), x^3 - 6*x - 4);
check_equal(vpar_path_lengths_poly([0,1,2,3]), x^4 - 20*x^2 - 32*x - 12);

{
  for(i=1,#test_sample_trees_and_forests,
     my(vpar=test_sample_trees_and_forests[i],
        m = vpar_path_lengths_matrix(vpar));

     my(poly = vpar_path_lengths_poly(vpar));
     check_equal(type(poly), "t_POL",
                 "vpar_Laplacian_poly() type");
     check_equal(poly, charpoly(m),
                 "vpar_Laplacian_poly() vs charpoly(vpar_Laplacian_matrix())");

     check_equal(poly, Pol(matdet('x*matid(#vpar) - m)),
                 "vpar_Laplacian_poly() by matdet");
     check_equal((-1)^#vpar * poly,
                 Pol(matdet(m - 'x*matid(#vpar))),
                 "vpar_Laplacian_poly() matdet D-xI");

     check_equal(polcoeff(poly,#vpar), 1,
                 "vpar_path_lengths_poly() high coeff #vpar = 1");
     my(n=#vpar);
     if(#vpar >= 1 && vpar_is_tree(vpar),
        check_equal(polcoeff(poly,0), -(n-1)*2^(n-2),
                    "vpar_path_lengths_poly() low coeff same all trees"));

     \\ print(polcoeff(poly,0)"  ",-(n-1)*2^(n-2),"  "#vpar" "vpar_num_components(vpar));

     check_equal(test_poly_is_all_real_roots(poly), 1,
               "vpar_path_lengths_poly() all roots real, as matrix symmetric");
  );
}

{
  for(n=0,10,
     my(vpar=vector(n));
     check_equal(matdet(vpar_path_lengths_matrix(vpar)), 1-n,
                 "vpar_path_lengths_matrix() matdet of singletons"));
}

\\------
\\ formula per http://mathworld.wolfram.com/DistancePolynomial.html
test_path_lengths_poly_of_star(n) = \
  if(n==0,Pol(1), (x+2)^(n-2) * (1+x*(x+4)-n*(1+2*x)));
{
  for(n=0,6,
     check_equal(vpar_path_lengths_poly(vpar_make_star(n)),
                 test_path_lengths_poly_of_star(n)));
}
{
  \\ recurrence per http://mathworld.wolfram.com/DistancePolynomial.html
  \\ p_n(x) = (x+2)^2 (-p_(n-2)(x)) - 2(x+2)p_(n-1)(x)
  for(n=3,12,
     check_equal(vpar_path_lengths_poly(vpar_make_star(n)),
                 2*(x+2) * test_path_lengths_poly_of_star(n-1)
                 -(x+2)^2 * test_path_lengths_poly_of_star(n-2)));
}{
  for(n=0,12,
     my(p=vpar_path_lengths_poly(vpar_make_star(n)));
     check_equal(p, test_path_lengths_poly_of_star(n)));
}
\\ recurrence_guess(vector(50,n,n--; charpoly(vpar_path_lengths_matrix(vpar_make_star(n)),'y)))

\\------
\\ per http://mathworld.wolfram.com/DistancePolynomial.html
\\ distance polynomial = charpoly of path lengths matrix
test_path_lengths_poly_of_path(n) =
{
  \\ FIXME: power form
  \\ my(x = x+O(x^(n+10)));
  \\ (-1 + x + sqrt(1+2*x))^n
  \\ + n * ( (-1-x+sqrt(2*x))^n - (-1-x-sqrt(1+2*x))^n )
  \\   / (sqrt(1+2*x));

  if(n<5, return(charpoly(vpar_path_lengths_matrix(vpar_make_path(n)))));

    x^5 * test_path_lengths_poly_of_path(n-5)
  - x^3 * (5*x+4) * test_path_lengths_poly_of_path(n-4)
  + 2*x*(5*x^2 + 6*x + 2) * test_path_lengths_poly_of_path(n-3)
  - 2*  (5*x^2 + 6*x + 2) * test_path_lengths_poly_of_path(n-2)
  + (5*x+4) * test_path_lengths_poly_of_path(n-1);
}
{
  for(n=0,12,
     check_equal(test_path_lengths_poly_of_path(n),
                 charpoly(vpar_path_lengths_matrix(vpar_make_path(n)))));
}
\\ recurrence_guess(vector(50,n,n--; vpar_path_lengths_poly(vpar_make_path(n),'y)))
\\ y^5
\\ - 5*y^4 - 4*y^3
\\ + 10*y^3 + 12*y^2 + 4*y
\\ - 10*y^2 - 12*y - 4
\\ + 5*y + 4]


\\-----------------------------------------------------------------------------
progress("vpar_preorder_depths_to_postorder()");

check_equal(vpar_preorder_depths_to_postorder([]), []);
check_equal(vpar_preorder_depths_to_postorder([0]), [0]);
check_equal(vpar_preorder_depths_to_postorder([0,0]), [0,0]);
check_equal(vpar_preorder_depths_to_postorder([0,1]), [1,0]);

{
  \\  1  2       1  3
  \\     |          |
  \\     3          2
  my(vpar=[0,0,2],
     depths=[0,0,1]);
  check_equal(vpar_preorder_depths_to_postorder(depths), [0,1,0]);
}

{
  for(n=0,8,
     my(preorder_vpar=vpar_preorder_first(n));
     until((preorder_vpar=vpar_preorder_next(preorder_vpar,1))=='none,  \\ forests
           my(preorder_depths=vpar_depths_vector(preorder_vpar),
              postorder_vpar=vpar_relabel_postorder(preorder_vpar),
              want_postorder_depths=vpar_depths_vector(postorder_vpar),
              got_postorder_depths=vpar_preorder_depths_to_postorder(preorder_depths));
           \\ print(preorder_vpar" depths "preorder_depths" got "got_postorder_depths);
           check_equal(got_postorder_depths, want_postorder_depths,
                      "vpar_preorder_depths_to_postorder()")));
}


\\-----------------------------------------------------------------------------
progress("vpar_make_preorder_siblings_sets()");

{
  for(n=0,8,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar,1))=='none,  \\ forests
           my(siblings_sets=vpar_siblings_sets(vpar),
              vpar_again=vpar_make_preorder_siblings_sets(siblings_sets));
           vpar_validate(vpar_again);
           check_equal(vpar_is_preorder(vpar), 1,
                      "vpar_make_preorder_siblings_sets() is preorder");
           check_equal(vpar_again,vpar,
                      "vpar_make_preorder_siblings_sets() back again")));
}


\\-----------------------------------------------------------------------------

test_is_siblings_split_by_sets(vpar,split_vpar) =
{
  my(sets=vpar_siblings_sets(split_vpar));
  for(i=1,#sets,
     for(j=1,#sets[i]-1,
        if(vpar[sets[i][j]] != vpar[sets[i][j+1]],
           return(0))));
  1;
}

{
  my(vpar=[0,0,0,0],
     split_vpar=[0,1,1,0]);
  check_equal(vpar_is_siblings_split(vpar,split_vpar), 1);
  check_equal(test_is_siblings_split_by_sets(vpar,split_vpar), 1);
}

{
  for(i=1,#test_sample_forests-1,
     my(vpar=test_sample_forests[i],
        split_vpar=test_sample_forests[i+1]);
     if(#vpar<#split_vpar,
        my(t=vpar); vpar=split_vpar; split_vpar=t);

     check_equal(vpar_is_siblings_split(vpar,vpar), 1,
                 "vpar_is_siblings_split() of self");

     check_equal(vpar_is_siblings_split(vpar,split_vpar),
                 test_is_siblings_split_by_sets(vpar,split_vpar),
                 "vpar_is_siblings_split() vs test_is_siblings_split_by_sets()");

     if(#vpar==#split_vpar,
        my(t=vpar); vpar=split_vpar; split_vpar=t;
        check_equal(vpar_is_siblings_split(vpar,split_vpar),
                    test_is_siblings_split_by_sets(vpar,split_vpar),
                    "vpar_is_siblings_split() vs test_is_siblings_split_by_sets()")));
}


\\-----------------------------------------------------------------------------
progress("vpar_is_preorder_sizes_vector()");

{
  for(forest=0,1,
     for(n=1, test_limit_trees_search - forest,
        my(vpar=vpar_preorder_first(n),
           l=List([]));
        until((vpar=vpar_preorder_next(vpar,forest))=='none,
              my(sizes=vpar_subtree_sizes_vector(vpar));
              listput(l, sizes));
        l=Set(l);
        \\ print(l);

        forvec(vec=vector(n,i,[-1,n+1]),
              \\ print("vec="vec);
              check_equal(vpar_is_preorder_sizes_vector(vec,forest),
                          setsearch(l,vec)!=0,
                          "vpar_is_preorder_sizes_vector() vs subtree sizes occurring");
              check_equal(vpar_is_postorder_sizes_vector(Vecrev(vec),forest),
                          setsearch(l,vec)!=0,
                          "vpar_is_preorder_sizes_vector() vs subtree sizes occurring"))));
}


\\-----------------------------------------------------------------------------
progress("vpar_Seidel_adjacency_matrix()");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_Seidel_adjacency_matrix(vpar),
                 matrix(#vpar,#vpar,i,j, 1) - matid(#vpar)
                 - 2*vpar_adjacency_matrix(vpar),
                 "vpar_Seidel_adjacency_matrix() vs vpar_adjacency_matrix()"));
}

{
  my(vpar1=[0, 1, 2, 3, 4, 4, 3, 1, 8, 9, 10, 1],
     vpar2=[0, 1, 2, 3, 4, 4, 1, 7, 8, 9, 7, 1]);
  vpar_validate(vpar1);
  vpar_validate(vpar2);
  my(m1=vpar_Seidel_adjacency_matrix(vpar1),
     m2=vpar_Seidel_adjacency_matrix(vpar2));
  check_equal(m1==m2, 0);
  check_equal(charpoly(m1),charpoly(m2));

  check_equal(vpar_centres(vpar1),[1]);
  check_equal(vpar_centres(vpar2),[1]);

  check_equal(vpar_centroids(vpar1),[1,2]);
  check_equal(vpar_centroids(vpar2),[1]);
}

{
  \\ count of how many vpar_Seidel_adjacency_matrix() occur, per the addhelp
  for(forest=0,1,
     for(n=0, test_limit_trees_search - forest,
        my(l=List([])); \\ distinct matrices
        forvec(vec=vpar_forvec_from(n,forest), \\ forests
              my(vpar=vpar_from_upascend(vec),
                 m=vpar_Seidel_adjacency_matrix(vpar));
              my(pos=setsearch(l,m,1));
              if(pos, listinsert(l,m,pos)));
        check_equal(#l, vpar_count_unrooted(n,forest),
                    "vpar_Seidel_adjacency_matrix() count distinct")));
}


\\-----------------------------------------------------------------------------
progress("vpar_Seidel_adjacency_poly()");

{
  for(n=1,10,
     my(poly=vpar_Seidel_adjacency_poly(vpar_make_star(n)));
     check_equal(polcoeff(poly,0), 1-n);
     check_equal(polcoeff(poly,1), 1-(n-1)^2);
     check_equal(polcoeff(poly,1), n*(2-n));
     check_equal(polcoeff(poly,2), if(n<=3,!(n%2), -A005564_samples[n-3]),
                 "A005564 star-n Seidel poly x^2 term");
     check_equal(poly, (x-(n-1)) * (x+1)^(n-1));
  );
}

\\ {
\\   for(n=1,10,
\\      my(poly=vpar_Seidel_adjacency_poly(vpar_make_path(n)));
\\      \\ print(factor(poly));
\\      print(Vec(poly));
\\      \\ check_equal(polcoeff(poly,0), 1-n);
\\      \\ check_equal(polcoeff(poly,1), 1-(n-1)^2);
\\      \\ check_equal(polcoeff(poly,1), n*(2-n));
\\      \\ check_equal(polcoeff(poly,2), if(n<=3,!(n%2), -A005564_samples[n-3]),
\\      \\             "A005564 star-n Seidel poly x^2 term");
\\      \\ check_equal(poly, (x-(n-1)) * (x+1)^(n-1));
\\   );
\\ }

{
  my(n=3,
     vpar=vpar_make_path(n),
     m=vpar_Seidel_adjacency_matrix(vpar));
  check_equal(m, [0, -1,  1;
                  -1, 0, -1;
                  1, -1,  0]);
  my(poly=vpar_Seidel_adjacency_poly(vpar));
  check_equal(poly, x^3 - 3*x - 2);
  check_equal(poly, -matdet([-x, -1,  1;
                             -1, -x, -1;
                             1,  -1, -x]));
}

\\-----------------------------------------------------------------------------

\\ E. Makinen, "A Survey on Binary Tree Codings", The Computer Journal,
\\ volume 34, number 5, 1991, pages 438-443.

\\ seq is vpar_seq_preorder() of a vpar tree or forest which was labelled in
\\ postorder.  Return that original postorder vpar.
test_postorder_from_seq_preorder(seq) =
{
  my(vpar=vector(#seq));
  for(i=1,#seq,
     my(v=seq[i]);
     forstep(j=i-1,1,-1, if(seq[j]>v, vpar[v]=seq[j]; break)));
  vpar;
}

test_balance_positions(vpar,bit) = \
  Vec(select(b->b==bit, binary(vpar_to_balanced_binary(vpar)), 1));

{
  \\            6                            6       10
  \\        /        \                       |\      | \
  \\       4          10                     4 5     8  9
  \\      / \        /                       |\      |
  \\     2   5      8    in-order            2 3     7   postorder
  \\    / \        / \                       |
  \\   1   3      7   9                      1

  my(vpar=[2,4,4,6,6,0, 8,10,10,0],
     n=#vpar);
  vpar_validate(vpar);
  check_equal(vpar_is_postorder(vpar), 1);

  \\ Section 3.1 Tree Permutations.
  \\ In-order labelled binary tree (= postorder labelled forest).
  \\ Read vertex numbers pre-order.
  my(seq=vpar_seq_preorder(vpar));
  check_equal(seq, Vecsmall([6,4,2,1,3,5,10,8,7,9]), "tree permutation");
  check_equal(test_postorder_from_seq_preorder(seq), vpar,
              "test_postorder_from_seq_preorder() tree permutation");

  check_equal(vpar_seq_diagqueue(vpar), Vecsmall([6, 4,10, 2,5,8, 1,3,7,9]),
              "vpar_seq_diagqueue() BT rows");

  \\ characterize 1s positions  pos[i] < 2i
  \\ characterize 0s positions  pos[i] >= 2i
  my(ones_positions=test_balance_positions(vpar,1),
     zeros_positions=test_balance_positions(vpar,0));
  check_equal(ones_positions, [1,2,3,4,7,10,13,14,15,18]);
  check_equal(zeros_positions, [5,6,8,9,11,12,16,17,19,20]);

  my(preorder_vpar=vpar_relabel_preorder(vpar));
  check_equal(preorder_vpar, [0,1,2,3,2,1, 0,7,8,7]);

  my(preorder_depths=vpar_depths_vector(preorder_vpar));
  check_equal(preorder_depths, [0,1,2,3,2,1, 0,1,2,1]);
  check_equal(vector(#preorder_depths,i, 2*i-1-preorder_depths[i]),
              ones_positions,
              "ones_positions from preorder depths");

  my(postorder_depths=vpar_depths_vector(vpar));
  check_equal(postorder_depths, [3,2,2,1,1,0, 2,1,1,0]);
  check_equal(vector(#postorder_depths,i, 2*i+postorder_depths[i]),
              zeros_positions,
              "zeros_positions from postorder depths");

  check_equal(vector(#postorder_depths,i, i+postorder_depths[i]),
              [4,4,5,5,6,6,9,9,10,10],
              "P-sequence from postorder depths");

  my(is_first_child=apply(s->!s,vpar_next_sibling_vector(vpar,1)));
  check_equal(is_first_child, [1,1,0, 1,0, 1,1,1,0, 0]);

  my(preorder_sizes=vpar_subtree_sizes_vector(preorder_vpar));
  check_equal(preorder_sizes, [6,4,2,1,1,1, 4,2,1,1]);
  my(preorder_is_first_child=apply(s->!s,vpar_next_sibling_vector(preorder_vpar,1)));
  check_equal(preorder_is_first_child, [1,1,1,1, 0,0,0, 1,1,0]);
  my(l=List([]));
  for(v=1,#vpar,
     if(preorder_is_first_child[v],
        my(size=1, s=v);
        until(!(s=vpar_next_sibling_of_vertex(preorder_vpar,s)),
              size += preorder_sizes[s]);
        listput(l,size));
     my(c=vpar_minmax_child_of_vertex(preorder_vpar,v,min));
     if(!c, listput(l,1)));
  l=Vec(l);
  check_equal(l, [11,6,4,2,1,1,1, 4,2,1,1], "L-sequence");

  my(sizes=vpar_subtree_sizes_vector(vpar));
  check_equal(sizes, [1,2, 1,4, 1,6, 1,2, 1,4], "weight sequence (postorder)");
}

{
  for(n=1,6,
     my(vpar=vpar_preorder_first(n),
        l=List([]));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(vpar=vpar_relabel_reverse(vpar));  \\ postorder
           check_equal(vpar_is_postorder(vpar), 1);
           my(seq=vpar_seq_preorder(vpar));
           \\ print("post vpar="vpar"  seq="seq);
           my(got_vpar = test_postorder_from_seq_preorder(seq));
           check_equal(got_vpar, vpar,
                      "test_postorder_from_seq_preorder()")));
}


\\-----------------------------------------------------------------------------
progress("vpar_num_automorphism_generators()");

check_equal(vpar_num_automorphism_generators([]), 0);
check_equal(vpar_num_automorphism_generators([0]), 0);
check_equal(vpar_num_automorphism_generators([0,0]), 1);
check_equal(vpar_num_automorphism_generators([0,0,0]), 2);
check_equal(vpar_num_automorphism_generators([0,0,0,0]), 2);
check_equal(vpar_num_automorphism_generators([0,1,1]), 1);
check_equal(vpar_num_automorphism_generators([0,1,1,1]), 2);
check_equal(vpar_num_automorphism_generators([0,1,1,1,1]), 2);
check_equal(vpar_num_automorphism_generators([0,1,1,1,1,1,2,3,4]), 3);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        forest=!vpar_is_tree(vpar),
        got=vpar_num_automorphism_generators(vpar));
     check_equal(0 <= got,  1);
     check_equal(got <= test_num_automorphism_generators_max(#vpar,forest), 1,
                 "vpar_num_automorphism_generators() vs test_num_automorphism_generators_max()"));
}
for(n=0,20, check_equal(ceil(n/2), floor((n+1)/2)));


\\-----------------------------------------------------------------------------
progress("vpar_num_free_automorphism_generators()");

check_equal(vpar_num_free_automorphism_generators([]), 0);
check_equal(vpar_num_free_automorphism_generators([0]), 0);
check_equal(vpar_num_free_automorphism_generators([0,0]), 1);
check_equal(vpar_num_free_automorphism_generators([0,0,0]), 2);
check_equal(vpar_num_free_automorphism_generators([0,0,0,0]), 2);
check_equal(vpar_num_free_automorphism_generators([0,1,1]), 1);
check_equal(vpar_num_free_automorphism_generators([0,1,1,1]), 2);
check_equal(vpar_num_free_automorphism_generators([0,1,1,1,1]), 2);
check_equal(vpar_num_free_automorphism_generators([0,1,1,1,1,1,2,3,4]), 3);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        forest=!vpar_is_tree(vpar),
        got=vpar_num_free_automorphism_generators(vpar));
     check_equal(0 <= got,  1);
     check_equal(got <= test_num_free_automorphism_generators_max(#vpar,forest), 1,
                 "vpar_num_free_automorphism_generators() vs test_num_free_automorphism_generators_max()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_postorder_all()");

{
  for(forest=0,1,
     for(n=1, test_limit_trees_search - forest,
        my(vpars=vpar_postorder_all(n,forest));
        \\ check_equal(setisset(vpars), 1);
        for(i=1,#vpars,
           check_equal(vpar_is_postorder(vpars[i]), 1,
                      "vpar_postorder_all() is postorder"))));
}


\\-----------------------------------------------------------------------------
progress("vpar_make_postorder_sizes()");

{
  for(n=1,6,
     my(vpar=vpar_preorder_first(n),
        l=List([]));
     until((vpar=vpar_preorder_next(vpar,1))=='none, \\ forests
           my(vpar=vpar_relabel_reverse(vpar),  \\ postorder
              subtree_sizes=vpar_subtree_sizes_vector(vpar));
           \\ print(vpar" sizes "subtree_sizes);
           my(got_vpar = vpar_make_postorder_sizes(subtree_sizes));
           check_equal(got_vpar, vpar,
                      "vpar_make_postorder_sizes() reversible")));
}


\\-----------------------------------------------------------------------------
progress("vpar_is_postorder_sizes_vector()");

check_equal(vpar_is_postorder_sizes_vector([1,1,2,3,5]), 1);
check_equal(vpar_is_postorder_sizes_vector([1,2,1,2,5]), 1);
check_equal(vpar_is_postorder_sizes_vector([2,2,4]), 0);

check_equal(vpar_is_postorder_sizes_vector([]), 1, \
            "vpar_is_postorder_sizes_vector() empty");
check_equal(vpar_is_postorder_sizes_vector([1]), 1, \
            "vpar_is_postorder_sizes_vector() singleton");
check_equal(vpar_is_postorder_sizes_vector([1,1],1), 1, \
            "vpar_is_postorder_sizes_vector() 2 singletons, forest");
check_equal(vpar_is_postorder_sizes_vector([1,1]), 0, \
            "vpar_is_postorder_sizes_vector() 2 singletons, not tree");
check_equal(vpar_is_postorder_sizes_vector([1,2]), 1);
check_equal(vpar_is_postorder_sizes_vector([1,1,3]), 1);


\\ test_is_postorder_sizes_vector_by_search() is same test as
\\ vpar_is_postorder_sizes_vector(), but direct form of the conditions in
\\ Pallo's theorem 1.  Here write j for i'.
\\
\\     J. M. Pallo, "Enumerating, Ranking and Unranking Binary Trees",
\\     The Computer Journal, volume 29, number 2, 1986, pages 171-175.
\\
\\ j = i-vec[i]+1 through i-1 are the vertices of the subtree under i.
\\ A subtree at j extends j down to j-vec[j]+1 inclusive.
\\ The condition is that those vertices must be in the range implied by i.
\\
\\ This code naively checks each j in the range under i.  The actual code of
\\ vpar_is_postorder_sizes_vector() uses a stack to enforce a tighter
\\ (bigger) limit as descend into further subtrees.
\\
test_is_postorder_sizes_vector_by_search(vec) =
{
  for(i=1,#vec,
     1 <= vec[i] && vec[i] <= i || return(0);
     for(j=i-vec[i]+1, i-1,
        i-vec[i] <= j-vec[j] || return(0)));
  1;
}

{
  \\ Pallo's list of n=5 weights vectors

  my(vpar=vpar_preorder_first(5),
     l=List([]));
  until((vpar=vpar_preorder_next(vpar,1))=='none,
        my(vpar=vpar_relabel_postorder(vpar));
        listput(l,
             vpar_INTERNAL_fromdigits(vpar_subtree_sizes_vector(vpar))));
  l=vecsort(Vec(l));
  check_equal(#l, #Set(l),
             "vpar_subtree_sizes_vector() postorder no duplicates");

  my(want=[11111,    11211,    12111,
           11112,    11212,    12112,
           11113,    11214,    12113,
           11114,    11215,    12115,
           11115,    11231,    12121,
           11121,    11234,    12123,
           11123,    11235,    12125,
           11124,    11241,    12141,
           11125,    11245,    12145,
           11131,    11311,    12311,
           11134,    11312,    12312,
           11135,    11315,    12315,
           11141,    11341,    12341,
           11145,    11345,    12345]);
  check_equal(#want, 42);
  check_equal(l, vecsort(want), "Pallo sample weights vectors");
}

{
  for(forest=0,1,
     for(n=1, test_limit_trees_search - forest,
        my(vpar=vpar_preorder_first(n),
           l=List([]));
        until((vpar=vpar_preorder_next(vpar,forest))=='none,
              my(vpar=vpar_relabel_reverse(vpar));
              listput(l, vpar_subtree_sizes_vector(vpar)));
        l=vecsort(Vec(l));
     \\ print(l);

     forvec(vec=vector(n,i,[-1,n+1]),
           \\ print("vec="vec);
           if(forest,
              check_equal(test_is_postorder_sizes_vector_by_search(vec),
                          setsearch(l,vec)!=0,
                          "test_is_postorder_sizes_vector_by_search() vs subtree sizes occurring"));
           check_equal(vpar_is_postorder_sizes_vector(vec,forest),
                       setsearch(l,vec)!=0,
                       "vpar_is_postorder_sizes_vector() vs subtree sizes occurring"));

     if(forest,
        for(i=1,#l,
           my(count=0);
        \\ print(l[i]"   vpar "vpar_make_postorder_sizes(l[i]));
           for(pos=2,n,
              for(t=l[i][pos]+1,n,
                 my(to=l[i]); to[pos]=t;
                 if(setsearch(l,to),
                 \\ print(" to "to" = "setsearch(l,to)"  incr pos="pos"   vpar "vpar_make_postorder_sizes(to));
                    count++; break));
              forstep(t=l[i][pos]-1,1,-1,
                 my(to=l[i]); to[pos]=t;
                 if(setsearch(l,to),
                 \\ print(" to "to" = "setsearch(l,to)"  decr pos="pos"   vpar "vpar_make_postorder_sizes(to));
                    count++; break)));
           check_equal(count, n-1)));
     ));
}

\\ Return the total siblingpos of v and its ancestors.
test_siblingpos_to_root(vpar,v) =
{
  my(siblingpos=vpar_siblingpos_vector(vpar));
  my(ret=0);
  until(!(v=vpar[v]), ret += siblingpos[v]);
  ret;
}
{
  \\ Joan Lucas, Dominique Roelants van Baronaigien, Frank Ruskey, "On
  \\ Rotations and the Generation of Binary Trees", Journal of Algorithms,
  \\ volume 15, 1993, pages 343-366.
  \\ http://webhome.cs.uvic.ca/~ruskey/Publications/Rotation/Rotation.html
  \\ Figure 6
  \\
  \\            6
  \\         /     \    in-order             6    7   9   10
  \\       2        7                       / \       |
  \\      / \        \                     2   5      8
  \\     1   5        9                    |   |
  \\        /        / \                   1   4       corresponding
  \\       4        8   10                     |         postorder
  \\      /                                    3          forest
  \\     3
  my(vpar=[2,6,4,5,6,0, 0, 9,0, 0]);
  vpar_validate(vpar);
  check_equal(vpar_subtree_sizes_vector(vpar), [1,2,1,2,3,6,1,1,2,1]);

  check_equal(vpar_depths_vector(vpar), [2,1,3,2,1,0,0,1,0,0]);
  my(preorder_vpar=vpar_relabel_preorder(vpar));
  check_equal(vpar_depths_vector(preorder_vpar), [0,1,2,1,2,3,0,0,1,0]);

  \\ "distance" representation by Makinen
  check_equal(vector(#vpar,v, test_siblingpos_to_root(vpar,v)),
                     [0,0,1,1,1,0,1,2,2,3]);
}


\\-----------------------------------------------------------------------------
progress("vpar_disjoint_domsets_count()");

check_equal(vpar_disjoint_domsets_count([0,1]), 1, "vpar_disjoint_domsets_count() path-2");
check_equal(vpar_disjoint_domsets_count([2,0]), 1);
check_equal(vpar_disjoint_domsets_count([0,1,0,3]), 2);

\\ 0 <- 1 <- 2    0,2 inv 1 only
check_equal(vpar_disjoint_domsets_count([0,1,2]), 1);

\\ 0 <- 1 <- 2 <- 3    0,2 inv 1,3
\\                     0,3 inv 1,2
check_equal(vpar_disjoint_domsets_count([0,1,2,3]), 2);

test_domatic_count_by_search(vpar) =
{
  if(#vpar==0,return(1));
  my(count=0);
  forvec(flags=vector(#vpar,i,if(i==1,[0,0],[0,1])),
        if(vpar_is_domset_flags(vpar,flags)
           && vpar_is_domset_flags(vpar, apply(f->!f,flags)),
           count++));
  count;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got = vpar_disjoint_domsets_count(vpar));
     check_equal(got, test_domatic_count_by_search(vpar));

     check_equal(2*got >= vpar_minimal_domsets_count(vpar),
                 vpar_num_singletons(vpar)==0,
     "2*vpar_disjoint_domsets_count > vpar_minimal_domsets_count described in addhelp");
  );
}

for(n=0,10,check_equal(vpar_disjoint_domsets_count(vpar_make_path(n)), fibonacci(n-1)));
for(n=0,10,check_equal(vpar_disjoint_domsets_count(vpar_make_star(n)), n!=1));
for(n=0,10,check_equal(vpar_disjoint_domsets_count(vpar_make_bistar(n,n)), 1+(n>=2)));
for(n=2,10, \
   check_equal(vpar_disjoint_domsets_count(vpar_make_binomial_tree(n)), 2^floor(n/2-1)));
for(n=2,10, \
   check_equal(vpar_disjoint_domsets_count(vpar_corona_singletons(vpar_make_path(n))), \
               2^(n-1)));

{
  \\ vpar_disjoint_domsets_count() of complete binary tree
  \\ u0=1; a0=1; d0=0; [a0,u0,d0]
  \\ a1 = (a0+d0)^2; u1 = d0^2; d1=a1-u1; [a1,u1,d1] == [1,0, 1]
  \\ a2 = (a1+d1)^2; u2 = d1^2; d2=a2-u2; [a2,u2,d2] == [4,1, 3]
  \\ a3 = (a2+d2)^2; u3 = d2^2; d3=a3-u3; [a3,u3,d3] == [49,9, 40]
  \\ a3 = (2*a2-u2)^2; u3 = (a2-u2)^2; [a3,u3,a3-u3] == [49,9, 40]
  \\ a3-u3 == (2*a2-u2)^2 - (a2-u2)^2
  \\ a3-u3 == a2*(3*a2-2*u2)
  \\ d6 = a6-u6 = (a5+d5)^2 - d5^2 = a5*(a5+2*d5)

  \\ not in OEIS: 3, 40, 6321, 162879523, 107217054417536040, 46568079211634997234733246737311281
  my(want=[ 0, 1, 3, 40, 6321, 162879523, 107217054417536040,
           46568079211634997234733246737311281 ]);
  check_equal(vector(#want,h,h--;
                     vpar_disjoint_domsets_count(vpar_make_complete_tree(2,h))),
              want);
}
{
  \\ vpar_disjoint_domsets_count() of complete ternary tree
  \\ not in OEIS: 7, 3032, 235131852375, 110608183184284710797943397140073457
  my(want=[ 0, 1, 7, 3032, 235131852375, 110608183184284710797943397140073457 ]);
  check_equal(vector(#want,h,h--;
                     vpar_disjoint_domsets_count(vpar_make_complete_tree(3,h))),
              want);
}


\\-----------------------------------------------------------------------------
progress("test_is_rooted_semistable_at_vertex()");
\\ test_is_rooted_semistable_at_vertex_by_automorphisms()

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);

     if(vpar_num_automorphisms(vpar) <= test_limit_perms_search,
        for(v=1,#vpar,
           check_equal(test_is_rooted_semistable_at_vertex(vpar,v),
                       test_is_rooted_semistable_at_vertex_by_automorphisms(vpar,v),
                       "test_is_rooted_semistable_at_vertex() vs test_is_rooted_semistable_at_vertex_by_automorphisms()"));
        );
  );
}


\\-----------------------------------------------------------------------------
progress("test_is_free_semistable_at_vertex()");
\\ test_is_free_semistable_at_vertex_by_automorphisms()

{
  \\ D. A. Holton, "Completely Semi-Stable Trees", Bulletin of the Australian
  \\ Mathematical Society, volume 9, 1973, pages 355-362.
  \\ Figure 6 example stable but not completely stable.

  \\      1                  not 1, makes 45,67
  \\     /| \                not 2or3, make 13,67
  \\    2 3  4               not 4, makes 6-7
  \\        / \              5 ok
  \\       5   6             6 ok
  \\            \            not 7, makes bicentral
  \\             7
  my(vpar=[0,1,1,1, 4,4,6],
     want=[0, 0,0, 0, 1,1, 0]);
  vpar_validate(vpar);
  check_equal(vector(#vpar,v,
                test_is_free_semistable_at_vertex(vpar,v)),
              want);
  check_equal(vector(#vpar,v,
                test_is_free_semistable_at_vertex_by_automorphisms(vpar,v)),
              want);
  check_equal(vpar_is_free_stable(vpar), 1);
  check_equal(test_seq_free_stabilizing_by_search(vpar),
              Vecsmall([5, 7, 4, 6, 1, 2, 3]));
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);

     if(vpar_num_automorphisms(vpar) <= test_limit_perms_search,
        for(v=1,#vpar,
           check_equal(test_is_free_semistable_at_vertex(vpar,v),
                       test_is_free_semistable_at_vertex_by_automorphisms(vpar,v),
                       "test_is_free_semistable_at_vertex() vs test_is_free_semistable_at_vertex_by_automorphisms()"));
        );
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_free_stable()");

check_equal(vpar_is_free_stable([]), 1, \
            "vpar_is_free_stable() empty");

test_tree_is_automorphism_stable_by_neighbours(vpar) =
{
  if(#vpar<=2, return(1));
  my(degrees=vpar_degrees_vector(vpar),
     neighbours=vpar_neighbours_vector(vpar));
  for(v=1,#vpar,
     if(sum(i=1,#neighbours[v], degrees[neighbours[v][i]]==1) >= 2,
        return(1)));
  0;
}
test_is_automorphism_stable_by_neighbours(vpar) =
{
  my(trees=vpar_component_trees(vpar));
  for(i=1,#trees,
     test_tree_is_automorphism_stable_by_neighbours(trees[i])
       || return(0));
  1;
}

\\ This depends on vpar_free_automorphism_generators() being a swap and a
\\ cycle at the leaves.
test_has_free_bunch_by_generators(vpar) =
{
  my(gens=vpar_free_automorphism_generators(vpar));
  for(i=1,#gens,
     if(test_perm_is_swap1(gens[i]), return(1)));
  0;
}

{
  \\ path-4 root at second
  my(vpar=[2,0,2,3]);
  check_equal(vpar_free_automorphisms(vpar), [Vecsmall([1,2,3,4]),
                                              Vecsmall([4,3,2,1]) ],
              "vpar_free_automorphisms() path-4 root at second");
  check_equal(vpar_free_automorphisms(vpar_make_path(3)),
              [Vecsmall([1,2,3]), Vecsmall([3,2,1]) ]);
  check_equal(vpar_is_free_stable(vpar), 0,
              "vpar_is_free_stable() path-4");
  check_equal(test_is_automorphism_stable_by_neighbours(vpar), 0,
              "test_is_automorphism_stable_by_neighbours() path-4");
}

{
  \\       1          6         not 2,4,7 makes components same
  \\      / \        /|\        can 1 or 6
  \\     3   5      2 4 7
  my(vpar=[0, 6, 1, 6, 1, 0, 6],
     want_semistable=[1,0,0,0,0,1,0]);
  check_equal(vector(#vpar,v, test_is_free_semistable_at_vertex(vpar,v)),
              want_semistable);
  check_equal(vector(#vpar,v, test_is_free_semistable_at_vertex_by_automorphisms(vpar,v)),
              want_semistable);
  check_equal(test_seq_free_stabilizing_by_search(vpar), Vecsmall([1, 2, 3, 5, 6, 4, 7]));
  check_equal(vpar_is_free_stable(vpar), 1);
}

{
  for(i=1,#test_sample_trees_and_forests,
     my(vpar=test_sample_trees_and_forests[i],
        got=vpar_is_free_stable(vpar));
     check_equal(got,
                 test_is_automorphism_stable_by_neighbours(vpar),
                 "vpar_is_free_stable() vs test_is_automorphism_stable_by_neighbours()");

     if(#vpar >= 2 && vpar_is_tree(vpar),
        check_equal(got, test_has_free_bunch_by_generators(vpar),
                    "vpar_is_free_stable() vs test_has_free_bunch_by_generators()"));

     if(#vpar <= test_limit_perms_search,
        check_equal(got,
                    test_seq_free_stabilizing_by_search(vpar) != 'none,
          "vpar_is_free_stable() vs test_seq_free_stabilizing_by_search()"));
  );
}

\\ Holton table 1
check_equal(vector(8,n, vpar_count_unlabelled(n)), \
            [1,1,2,4,9,20,48,115]);

{
  my(want=[1,1, 1,1,2,4,8,17,37,85]);  \\ Holton table 1
  for(n=3,10,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_free_stable(vpar));
     check_equal(count, want[n]));
}
{
  for(n=11,min(#A003426_samples,14),
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           count += vpar_is_free_stable(vpar));
     check_equal(count, A003426_samples[n],
                 "A003426 number of stable free trees"));
}
{
  \\ stable free forest if and only if all component trees stable, so Euler
  \\ transform of trees counts
  my(T=A003426_samples,      \\ starting n=1
     F=A006544_samples[^1],  \\ starting n=0, skip n=0
     len=min(#T,#F));
  T=T[1..len];
  F=F[1..len];
  check_equal(F, vpar_INTERNAL_Euler_transform(T),
             "A006544 stable free forests from A003426 stable free trees");
}

\\ tree-only
  \\ if(#vpar<=2, 1,
  \\    my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
  \\       bunch=vectorsmall(#vpar));
  \\    for(v=1,#vpar,
  \\       my(p=vpar[v]);
  \\       if(p,
  \\          \\ childless adds to its parent
  \\          if(!num_children[v] && bunch[p]++>=2, return(1));
  \\
  \\          \\ sole child of a root adds to itself
  \\          if(!vpar[p] && num_children[p]==1 && bunch[v]++>=2, return(1))));
  \\
  \\    0);


\\-----------------------------------------------------------------------------
progress("test_free_tree_ceil()");

test_free_tree_ceil_by_search(vpar) =
{
  until((vpar=vpar_unlabelled_next(vpar))=='none,  \\ trees
        if(vpar_is_tree(vpar) && test_unlabelled_is_free_primary(vpar),
           break));
  vpar;
}
check_equal(test_free_tree_ceil_by_search([0,1]), [0,1]);

\\ Probably not quite right.
\\ {
\\   for(i=1,#test_sample_trees,
\\      my(vpar=vpar_to_unlabelled(test_sample_trees[i]));
\\      check_equal(test_free_tree_ceil(vpar),
\\                  test_free_tree_ceil_by_search(vpar),
\\                  "test_free_tree_ceil() vs test_free_tree_ceil_by_search()"));
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_preorder_subtree_height()");

check_equal(vpar_INTERNAL_premax_subtree_height([0],1), 0);
check_equal(vpar_INTERNAL_premax_subtree_height([0,1],1), 1);
check_equal(vpar_INTERNAL_premax_subtree_height([0,1,1],1), 1);
check_equal(vpar_INTERNAL_premax_subtree_height([0,1,2,1],1), 2);

{
  for(i=1,#test_sample_unlabelled_forests,
     my(vpar=test_sample_unlabelled_forests[i]);
     for(v=1,#vpar,
        check_equal(vpar_INTERNAL_premax_subtree_height(vpar,v),
                    vpar_subtree_height(vpar,v),
                    "vpar_INTERNAL_premax_subtree_height() vs vpar_subtree_height()")));
}


\\-----------------------------------------------------------------------------
progress("vpar_free_next_forest()");

check_equal(vpar_free_next_forest([],1), 'none);
check_equal(vpar_free_next_forest([0],1), 'none);
check_equal(vpar_free_next_forest([0,0],1), 'none);
check_equal(vpar_free_next_forest([0,1],1), [0,0]);

{
  \\       1        7  8
  \\      /  \ \
  \\     2    5 6
  \\    / \
  \\   3   4
  my(vpar=[0, 1, 2, 2, 1, 1, 0, 0]);
  vpar_validate(vpar);
  check_equal(test_free_r_m_e(vpar), [1,5,6], "test_free_r_m_e()");
  check_equal(test_free_L1_height(vpar), 1);
  check_equal(test_free_L2_height(vpar), 1);
  check_equal(test_free_L1_size(vpar), 3, "test_free_L1_size()");
  check_equal(test_free_L2_size(vpar), 3);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 1, 4, 1, 6, 1],
              "vpar_free_next_forest() A8");
}
{
  \\        1        8
  \\       /  \ \
  \\      2    5  7
  \\     /     |
  \\    3      6
  \\   /
  \\  4
  my(vpar=[0, 1, 2, 3, 1, 5, 1, 0]);
  vpar_validate(vpar);
  check_equal(test_free_r_m_e(vpar), [1,5,7]);
  check_equal(test_free_L1_height(vpar), 2);
  check_equal(test_free_L2_height(vpar), 2);
  check_equal(test_free_L1_size(vpar), 3);
  check_equal(test_free_L2_size(vpar), 4);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 1, 5, 0, 7],
              "vpar_free_next_forest() B");
}
{
  \\       1      8           1
  \\     /  \               /  \ \ \
  \\    2    6     =>      2    6 7 8
  \\   /|\    \           /|\
  \\  3 4 5    7         3 4 5
  my(vpar=[0, 1, 2, 2, 2, 1, 6, 0]);
  vpar_validate(vpar);
  check_equal(vpar_unlabelled_next_pos(vpar,1), 7);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 2, 2, 1, 1, 1],
              "vpar_free_next_forest() C8");
}
{
  \\       1               1
  \\     /  \            /  \
  \\    2    6          2    6          new L1 same as L2
  \\    |    |   =>     |    |\
  \\    3    7          3    7 8
  \\   / \   |         / \
  \\  4   5  8        4   5
  my(vpar=[0, 1, 2, 3, 3, 1, 6, 7]);
  vpar_validate(vpar);
  check_equal(vpar_unlabelled_next_pos(vpar), 8);
  check_equal(vpar_unlabelled_next_pos(vpar,1), 8);
  check_equal(test_free_r_m_e(vpar), [1,6,8]);
  check_equal(test_free_L1_height(vpar), 2);
  check_equal(test_free_L2_height(vpar), 3);
  check_equal(test_free_L1_size(vpar), 4);
  check_equal(test_free_L2_size(vpar), 4);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 3, 1, 6, 6],
              "vpar_free_next_forest() D");
}
{
  \\       1                1
  \\     /  \ \ \          / \        9 raise to singleton
  \\    2    6  8 9       2   7       drops L2 size to 4 == L1 size
  \\    |    |           / \   \      and violates lex L1 <= L2
  \\    3    7     =>   3   5   8     L1 depths = [0,1,2,2]
  \\   / \              |   |   |     L2 depths = [0,1,2,1,1]
  \\  4   5             4   6   9     must decr L1 by raise p=5
  my(vpar=[0, 1, 2, 3, 3, 1, 6, 1, 1]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 2, 5, 1, 7, 8],
              "vpar_free_next_forest() E");
}
{
  \\       1                   1     9
  \\     /  \  \             /   \        9 raise to singleton
  \\    2    6   9   =>     2     6       drops L2 size to 4 == L1 size
  \\   / \   |\            / \    |\      and have L1 <= L2 ok
  \\  3   5  7 8          3   5   7 8     L1 depths = [0,1,2,1]
  \\  |                   |               L2 depths = [0,1,2,2,1]
  \\  4                   4
  my(vpar=[0, 1, 2, 3, 2, 1, 6, 6, 1]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 2, 1, 6, 6, 0],
              "vpar_free_next_forest() F");
}
{
  \\       1     8 9          1      7
  \\     /  \ \              / \    / \    partial copy within L1 height = 3
  \\    2    5  7     =>    2   5  8   9
  \\   /     |             /    |
  \\  3      6            3     6
  \\  |                   |
  \\  4                   4
  my(vpar=[0, 1, 2, 3, 1, 5, 1, 0, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 1, 5, 0, 7, 7],
              "vpar_free_next_forest() G");
}
{
  \\         1    10          1
  \\       /  \              /  \
  \\      2    7      =>    2    7
  \\     /     |           /    /|\
  \\    3      8          3    8 9 10
  \\   /|\     |         /|\
  \\  4 5 6    9        4 5 6
  my(vpar=[0, 1, 2, 3, 3, 3, 1, 7, 8, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 3, 3, 1, 7, 7, 7],
              "vpar_free_next_forest() H");
}
{
  \\        1                 1
  \\      /  \  \            /  \ \
  \\     2    7   10   =>   2    7  9
  \\    / \   |\           / \   |  |
  \\   3   5  8 9         3   5  8  10
  \\   |   |              |   |
  \\   4   6              4   6
  my(vpar=[0, 1, 2, 3, 2, 5, 1, 7, 7, 1]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 3, 2, 5, 1, 7, 1, 9],
              "vpar_free_next_forest() I");
}
{
  my(vpar=[0, 1, 2, 2, 2, 2, 1, 7, 1, 0, 0, 0, 0, 0, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 2, 2, 2, 1, 7, 0, 9, 10, 10, 10, 9, 14],
              "vpar_free_next_forest() J");
}
{
  \\        1  10 11 12 13 14 15     1             9
  \\      /  \                      / \ \ \       /    \
  \\     2     7               =>  2   6 7 8     10     13
  \\  // \\    | \                /|\           / \     | \
  \\ 3 4 5 6   8  9              3 4 5         11  12   14 15
  \\
  \\ 9 raises to root
  \\ partial copy of new tree 1..8 chops off 8 making L2 too small
  \\ 13..15 copy of L1

  my(vpar=[0, 1, 2, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 2, 2, 1, 1, 1, 0, 9, 10, 10, 9, 13, 13],
              "vpar_free_next_forest() K");
}
{
  \\      1     7             1
  \\    /  \                 / \
  \\   2    5               2   6
  \\   |    |        =>    /|\   \
  \\   3    6             3 4 5   7
  \\   |
  \\   4
  \\
  my(vpar=[0, 1, 2, 3, 1, 5, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 2, 2, 2, 1, 6],
              "vpar_free_next_forest() L7");
}
{
  \\      1    5 6            1
  \\    /  \                / |\ \ \
  \\   2    4      =>      2  3 4 5 6
  \\   |
  \\   3
  \\
  my(vpar=[0, 1, 2, 1, 0, 0]);
  vpar_validate(vpar);
  check_equal(vpar_free_next_forest(vpar, 1),
              [0, 1, 1, 1, 1, 1],
              "vpar_free_next_forest() M6");
}

{
  \\ This example L1_height=3 == L2_height=3 and L1_size=5 < L2_size=7 primary.
  \\ Raising 12 up one depth to new root is ok too, it gives L2_size=6.
  \\ The partial copy of the tree is chopped one more, leaving a copy of 1..10.
  \\ That partial copy is L1_size=5 == L2_size=5.
  \\ Comparing L1,L2 contents then shows L1 > L2 which is bad, must advance
  \\ L1 in the partial.
  \\
  \\      1            14..21
  \\    /  \   \  \
  \\   2    7   11 12
  \\   |    |\
  \\   3    8 10
  \\   |    |
  \\   4    9
  \\   |\
  \\   5 6
  \\
  my(vpar=[0,1,2,3,4,4, 1,7,8,7,1,1, 0,0,0,0,0,0,0,0,0]);
  vpar_validate(vpar);
  check_equal(#vpar,21);
  check_equal(test_unlabelled_is_free_primary(vpar), 1,
             "test_unlabelled_is_free_primary() N");
  check_equal(test_free_L1_height(vpar), 3);
  check_equal(test_free_L2_height(vpar), 3);
  check_equal(test_free_L1_size(vpar), 5);
  check_equal(test_free_L2_size(vpar), 7);

  my(want=test_free_next_forest_by_search(vpar));
  check_equal(test_unlabelled_is_free_primary(want), 1,
             "test_unlabelled_is_free_primary() N want");

  my(got=vpar_free_next_forest(vpar, 1));
  check_equal(got, want, "vpar_free_next_forest() N");
  check_equal(test_unlabelled_is_free_primary(got), 1,
             "test_unlabelled_is_free_primary() N got");
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_free_next_forest_by_search
             (test_sample_unlabelled_forests[i]));
     if(vpar=='none, next);
     check_equal(test_unlabelled_is_free_primary(vpar), 1,
                 "test_unlabelled_is_free_primary()");

     for(extra=0,4,
        my(vpar=Vec(vpar, #vpar+extra),
           want=test_free_next_forest_by_search(vpar));
        check_equal(want=='none || test_unlabelled_is_free_primary(want)==1,  1,
                    "test_unlabelled_is_free_primary() want");

        my(got=vpar_free_next_forest(vpar, 1));
        check_equal(got, want, "vpar_free_next_forest() N")));
}


\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_str_join()");

check_equal(vpar_INTERNAL_str_join([],","), "");
check_equal(vpar_INTERNAL_str_join([1],","), "1");
check_equal(vpar_INTERNAL_str_join([1,2],","), "1,2");
check_equal(vpar_INTERNAL_str_join([1,2,3],","), "1,2,3");

check_equal(vpar_INTERNAL_str_join(['x],","), "x");


\\-----------------------------------------------------------------------------
progress("vpar_num_free_automorphism_generators()");

{
  \\         4
  \\         |
  \\         7         rooted
  \\        / \        10,3,13 subtrees
  \\       5   6
  \\      / \          free also
  \\    11   8         4,6 leaves
  \\     |
  \\     9
  \\   / | \
  \\ 10  3 13
  \\  |  |  |
  \\  2 12  1
  my(vpar=[13, 10, 9, 0, 7, 7, 4, 5, 11, 9, 5, 3, 9]);

  check_equal(vpar_num_automorphisms(vpar), 6);       \\ 3!
  check_equal(vpar_num_free_automorphisms(vpar), 12,  \\ 2*3!
             "vpar_num_free_automorphisms() 13");

  \\ print(vpar_automorphism_generators(vpar));
  check_equal(vpar_num_automorphism_generators(vpar), 2,
              "vpar_num_automorphism_generators() 13");
  check_equal(vpar_num_free_automorphism_generators(vpar), 3,
              "vpar_num_free_automorphism_generators() 13");
}


\\-----------------------------------------------------------------------------
progress("vpar_bipartite_sizes()");

check_equal(vpar_bipartite_sizes([]), [0,0]);
check_equal(vpar_bipartite_sizes([0]), [0,1]);
check_equal(vpar_bipartite_sizes([4,0,4,0]), [2,2]);
check_equal(vpar_bipartite_sizes([0,1]), [1,1]);
check_equal(vpar_bipartite_sizes([0,1,1]), [1,2]);
check_equal(vpar_bipartite_sizes([0,0,1,2]), [2,2]);

test_bipartite_sizes_by_depths(vpar) =
{
  my(depths=vpar_depths_vector(vpar),
     odds=sum(v=1,#vpar, depths[v]%2));
  vecsort([odds,#vpar-odds]);
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_bipartite_sizes(vpar),
                 test_bipartite_sizes_by_depths(vpar)));
}

\\-----------------------------------------------------------------------------
progress("vpar_minmax_height_descendants_vector()");

check_equal(vpar_minmax_height_descendants_vector([],min), [], "vpar_minmax_height_descendants_vector() empty");
check_equal(vpar_minmax_height_descendants_vector([],max), []);
check_equal(vpar_minmax_height_descendants_vector([0],min), [1]);
check_equal(vpar_minmax_height_descendants_vector([0],max), [1]);
check_equal(vpar_minmax_height_descendants_vector([0,1,1],min), [2,2,3]);
check_equal(vpar_minmax_height_descendants_vector([0,1,1],max), [3,2,3]);
check_equal(vpar_minmax_height_descendants_vector([0,1],min), [2,2]);
check_equal(vpar_minmax_height_descendants_vector([0,1],max), [2,2]);

\\     1
\\    / \
\\   2   4
\\  /   / \
\\ 3   5   6
\\          \
\\           7
check_equal(vpar_minmax_height_descendants_vector([0,1,2,1,4,4,6],min), \
            [7,3,3,7,5,7,7], \
            "vpar_minmax_height_descendants_vector() min");
check_equal(vpar_minmax_height_descendants_vector([0,1,2,1,4,4,6],max), \
            [7,3,3,7,5,7,7], \
            "vpar_minmax_height_descendants_vector() max");


\\-----------------------------------------------------------------------------
progress("vpar_unlabelled_all()");

{
  for(forest=0,1,
     for(n=0,10,
        my(got=vpar_unlabelled_all(n,forest));
        check_equal(#got,vpar_count_unlabelled(n,forest));
        check_equal(got,vecsort(got,,4));
        check_equal(got,Vecrev(Set(got)))));
}


\\-----------------------------------------------------------------------------
progress("vpar_free_all()");

{
  for(n=0,10,
     my(got=vpar_free_all(n));
     check_equal(#got,vpar_count_free(n));
     check_equal(got,Vecrev(Set(got))));
}


\\-----------------------------------------------------------------------------

\\ sans root n+1
test_perm_to_vpar(perm) =
{
  my(vpar=vector(#perm));
  for(v=1,#perm,
     my(m=#perm+1);
     for(p=v+1,#perm,
        if(perm[p]>perm[v] && perm[p]<m,
           m=min(m,perm[p]);
           vpar[v]=p)));
  vpar;
}

\\ how many ancestors > v
test_num_inversions_of_vertex(vpar,v) =
{
  my(ret=0, p=v);
  while(p=vpar[v], ret += (p>v));
  ret;
}


\\-----------------------------------------------------------------------------
progress("vpar_premax_ids_vector()");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        ids=vpar_premax_ids_vector(vpar),
        seq=vpar_seq_premax(vpar));

     \\ among siblings, seq should visit in descending order of id, and when
     \\ equal id by descending vertex number
     my(prev_child=vectorsmall(#vpar+1));
     for(i=1,#seq,
        my(v=seq[i],
           p=vpar[v],
           prev=prev_child[p+1]);
        check_equal(!prev
                    || ids[prev] > ids[v]
                    || (ids[prev] == ids[v] && prev < v),  1,
                   "vpar_seq_premax() sibling descending id");
        prev_child[p+1]=v);
  );
}

\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_premax()");

{
  \\      4
  \\     / \
  \\    1   2
  \\        |
  \\        3
  my(vpar=[4,4,2,0]);
  vpar_validate(vpar);
  check_equal(vpar_INTERNAL_premax(vpar,5), Vecsmall([2,1,3, 4]),
              "vpar_INTERNAL_premax() 4, free seq");
}


\\-----------------------------------------------------------------------------
progress("vpar_preorder_depths_is_free_primary()");

check_equal(vpar_preorder_depths_is_free_primary([]), 1, \
           "vpar_preorder_depths_is_free_primary() empty");

{
  for(n=0,16,
     my(vpar=vpar_free_first(n),
        depths=vpar_depths_vector(vpar));
     check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              Str("vpar_preorder_depths_is_free_primary() free first n="n)));
}

{
  \\ Michael Dineen, "Constant Time Generation of Free Trees"
  \\ example of condition (b) fails

  \\         1
  \\       / | \  \  \
  \\     2   6  8  9  10
  \\     |   |
  \\     3   7
  \\    / \
  \\   4   5
  \\
  my(vpar=[0,1,2,3,3, 1,6, 1,1,1]);
  check_equal(#vpar, 10);
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1,2]);

  my(depths=vpar_depths_vector(vpar));
  check_equal(depths, [0,1,2,3,3,1,2,1,1,1]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(b)");

  \\ Unlabelled iteration gives height of rest 7..11 too small.
  \\ q=6,p=7 consecutive so height not preserved.
  \\         1
  \\       / | \  \  \   \
  \\     2   6  7  8  9  10
  \\     |
  \\     3
  \\    / \
  \\   4   5
  \\
  vpar=vpar_unlabelled_next(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar, [0,1,2,3,3, 1,1,1,1,1]);
  check_equal(vpar_centres(vpar), [2]);  \\ no longer centre at root

  depths=vpar_depths_vector(vpar);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 0,
              "vpar_preorder_depths_is_free_primary() 10(b) successor");

  \\          1
  \\        /   \
  \\      2      8
  \\    / | \    |
  \\   3  5  7   9
  \\   |  |      |
  \\   4  6     10
  \\
  vpar=[0,1,2,3, 2,5, 2, 1,8,9];
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1]);

  depths=vpar_depths_vector(vpar);
  check_equal(depths, [0,1,2,3,2,3,2,1,2,3]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(b) wanted");
}

{
  \\ Michael Dineen, "Constant Time Generation of Free Trees"
  \\ example condition (c) fails

  \\         1
  \\       /    \  \
  \\     2       8  10
  \\   /|||\     |
  \\ 3 4 5 6 7   9
  \\
  my(vpar=[0,1,2,2,2,2,2, 1,8, 1]);
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1]);

  my(depths=vpar_depths_vector(vpar));
  check_equal(depths, [0,1,2,2,2,2,2, 1,2,1]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(c)");

  \\         1
  \\       /   \ \ \
  \\     2      8 9 10
  \\   /|||\
  \\ 3 4 5 6 7
  \\
  vpar=vpar_unlabelled_next(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar, [0,1,2,2,2,2,2, 1,1,1]);
  check_equal(vpar_centres(vpar), [1,2]); \\ centres ok

  depths=vpar_depths_vector(vpar);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 0,
              "vpar_preorder_depths_is_free_primary() 10(c) successor");

  \\         1
  \\       /   \
  \\     2      7
  \\   /||\    /|\
  \\ 3 4 5 6  8 9 10
  \\
  vpar=[0,1,2,2,2,2, 1,7,7,7];
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1]);

  depths=vpar_depths_vector(vpar);
  check_equal(depths, [0,1,2,2,2,2, 1,2,2,2]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(c) wanted");
}

{
  \\ Michael Dineen, "Constant Time Generation of Free Trees"
  \\ example condition (d) fails

  \\          1
  \\        /  \  \
  \\      2     7   9
  \\    / |     |   |
  \\   3  5     8   10
  \\   |  |
  \\   4  6
  \\
  my(vpar=[0,1,2,3,2,5, 1,7, 1,9]);
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1,2]);

  my(depths=vpar_depths_vector(vpar));
  check_equal(depths, [0,1,2,3,2,3, 1,2, 1,2]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(d)");

  \\          1
  \\        /  \  \  \
  \\      2     7  9  10
  \\    / |     |
  \\   3  5     8
  \\   |  |
  \\   4  6
  \\
  vpar=vpar_unlabelled_next(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar, [0,1,2,3,2,5, 1,7, 1,1]);
  check_equal(vpar_centres(vpar), [1,2]); \\ centres ok

  depths=vpar_depths_vector(vpar);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 0,
              "vpar_preorder_depths_is_free_primary() 10(d) successor");

  \\         1
  \\       /   \
  \\     2      8
  \\   /||\     |
  \\ 3 5 6 7    9
  \\ |          |
  \\ 4         10
  \\
  vpar=[0,1,2,3,2,2,2, 1,8,9];
  vpar_validate(vpar);
  check_equal(vpar_is_preorder(vpar), 1);
  check_equal(vpar_centres(vpar), [1]);

  depths=vpar_depths_vector(vpar);
  check_equal(depths, [0,1,2,3,2,2,2, 1,2,3]);
  check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
              "vpar_preorder_depths_is_free_primary() 10(d) wanted");
}

\\ WRONG
\\ test_premax_is_primary(vpar) =
\\ {
\\   print("test_premax_is_primary() "vpar);
\\   my(centres=vecsort(vpar_centres(vpar)));
\\   print(" centres "centres);
\\   if(#centres==0, 1,
\\      #centres==1, vpar[centres[1]]==0,
\\      #centres==2,
\\      centres==[1,2] || return(0);
\\      my(L1=vpar_subtree(vpar,2),
\\         L2=vpar_delete_subtree(vpar,2));
\\      print(" L1 "L1);
\\      print(" L2 "L2);
\\      lex(L2,L1)>=0);
\\ }

{
  for(n=3,12,
     my(vpar=vpar_unlabelled_first(n),
        count=0,
        count_primary=0);
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           my(depths=vpar_depths_vector(vpar));
           check_equal(vpar_is_preorder_depths_vector(depths),1);
           count++;

           my(is_primary=vpar_preorder_depths_is_free_primary(depths));

           \\ print(vpar" depths "depths);
           check_equal(is_primary,
                       test_depths_is_free_primary_by_vectors(depths),
                       "vpar_preorder_depths_is_free_primary() vs test_depths_is_free_primary_by_vectors()");

           \\ check_equal(is_primary,
           \\             test_premax_is_primary(vpar),
           \\             "vpar_preorder_depths_is_free_primary() vs test_premax_is_primary()");

           count_primary += is_primary;
           );

     \\ print("n="n"  "count_primary);

     check_equal(count, vpar_count_unlabelled(n));
     check_equal(count_primary, vpar_count_free(n),
                 Str("vpar_preorder_depths_is_free_primary() count n="n));
     );
}


\\-----------------------------------------------------------------------------
progress("vpar_nest()");

check_equal(lex([],     [[]]), -1);
check_equal(lex([[],[]],[[]]),  1);

check_equal(vpar_nest([]),    [],      "vpar_nest() empty");
check_equal(vpar_nest([0]),   [ [] ],  "vpar_nest() singleton");
check_equal(vpar_nest([0,0]), [[],[]], "vpar_nest() isolated 2");
check_equal(vpar_nest([0,1]), [[[]]],  "vpar_nest() path-2");
check_equal(vpar_nest([2,0]), [[[]]],  "vpar_nest() path-2 reverse");

\\   1   3
\\       |
\\       2
check_equal(vpar_nest([0,3,0]), [ [], [[]] ], "vpar_nest() 1,2");

\\   2  5  6
\\   |
\\   3
\\  / \
\\ 4   7
\\     |
\\     1
check_equal(vpar_nest([7,0,2,3,0,0,3]), [ [[ [], [[]] ]], [], [] ], "vpar_nest() 7");


test_subtree_nest_by_recurse(vpar,v,sort=0) =
{
  my(nest=apply(c->test_subtree_nest_by_recurse(vpar,c,sort),
               vpar_children_of_vertex(vpar,v)));
  if(sort,vecsort(nest),nest);
}
check_equal(test_subtree_nest_by_recurse([],1), [], "test_subtree_nest_by_recurse() empty");
check_equal(test_subtree_nest_by_recurse([0,1,2],1), [[[]]], "test_subtree_nest_by_recurse() path-2");
check_equal(test_subtree_nest_by_recurse([0,1,2,1,4],1), [ [[]], [[]] ]);
check_equal(test_subtree_nest_by_recurse([0,1,2,1],1), [ [[]], [] ]);
check_equal(test_subtree_nest_by_recurse([0,1,2,1],1,1), [ [], [[]] ]); \\ sorted

test_nest_by_recurse(vpar,sort) = test_subtree_nest_by_recurse(vpar,0,sort);
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(sort=0,1,
        check_equal(vpar_nest(vpar,sort),
                    test_nest_by_recurse(vpar,sort)));

     \\ sorted nest equal means isomorphic
     my(vpar2=vpar_relabel_random(vpar));
     check_equal(vpar_nest(vpar,1), vpar_nest(vpar2,1),
                 "vpar_nest() unchanged by random relabel"));
}

{
  \\        2
  \\      / | \
  \\    1   4   3
  \\    |
  \\    5
  \\    |
  \\    6
  my(vpar = [2, 0, 2, 2, 1, 5],
     nests=vector(#vpar,v, test_subtree_nest_by_recurse(vpar,v,1)),
     nests_m_cmp=matrix(#vpar,#vpar,u,v, lex(nests[u],nests[v])));
  check_equal(nests_m_cmp,
              [0, 1, 1, 1, 1, 1;
               -1, 0, 1, 1, 1, 1;
               -1, -1, 0, 0, -1, 0;
               -1, -1, 0, 0, -1, 0;
               -1, -1, 1, 1, 0, 1;
               -1, -1, 0, 0, -1, 0]);
}


\\-----------------------------------------------------------------------------
progress("vpar_spanfreq_poly()");

check_equal(vpar_spanfreq_poly([],,'y), Pol(1,'y), \
            "vpar_spanfreq_poly() empty");
check_equal(type(vpar_spanfreq_poly([])), "t_POL");
check_equal(variable(vpar_spanfreq_poly([],,'y)), 'y);

check_equal(vpar_spanfreq_poly([0]), 1 + x);
check_equal(vpar_spanfreq_poly([0,0]), 1 + 2*x);

check_equal(vpar_spanfreq_poly([0,1]), 1 + 2*x + x^2, \
            "vpar_spanfreq_poly() path-2");

check_equal(vpar_spanfreq_poly([0,1,2]), 1 + 3*x + 2*x^2 + x^3, \
            "vpar_spanfreq_poly() path-3");

check_equal(vpar_spanfreq_poly([0,1,2,3]), \
            1 + 4*x + 3*x^2 + 2*x^3 + x^4, \
            "vpar_spanfreq_poly() path-4");

\\      1                x + x^2  with         3:     x  with
\\     / .         1 +   x        without          1     without
\\    2   3        1 + 2*x + x^2  any
\\            new with  nothing of 3
\\                   or with 3 joined product
\\            new without  existing 1s and 3 with or without  = 1 + x
\\
check_equal((x + x^2) * (1 + x),   x + 2*x^2 + x^3,  "product with");
check_equal(vpar_spanfreq_poly([0,1,1]), 1 + 3*x + 2*x^2 + x^3, \
            "vpar_spanfreq_poly() star-3");

\\ Traverse ancestors of v while each flags[p] is true, and return the
\\ top-most of that chain of flagged ancestors.
test_flagged_root(vpar,flags,v) = \
  while(vpar[v] && flags[vpar[v]], v=vpar[v]); v;

\\ Return 1 if flags is vertices of a spanning subtree in vpar.
\\ So the subgraph induced by flags is a connected tree.
\\ This means all of flags followed upwards through flagged ancestors lead to
\\ a single common ancestor.
test_is_spanning_flags(vpar,flags) =
{
  my(want_ancestor);
  for(v=1,#vpar,
     if(flags[v],
        my(ancestor=test_flagged_root(vpar,flags,v));
        if(want_ancestor && ancestor!=want_ancestor, return(0));
        want_ancestor = ancestor));
  1;
}

test_spanning_freqpoly_by_search(vpar) =
{
  my(ret=0*x);
  forvec(flags=vector(#vpar,i,[0,1]),
        if(test_is_spanning_flags(vpar,flags),
           ret += x^hammingweight(flags)));
  ret;
}
check_equal(test_spanning_freqpoly_by_search([]), 1 + 0*x);
check_equal(test_spanning_freqpoly_by_search([0]), 1 + x);
check_equal(test_spanning_freqpoly_by_search([0,1]), 1 + 2*x + x^2);
check_equal(test_spanning_freqpoly_by_search([0,1,1]), \
            1 + 3*x + 2*x^2 + x^3);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     if(#vpar <= test_limit_flags_search,
        my(got=vpar_spanfreq_poly(vpar));
        \\ print(vpar);

        \\ Initial and final coeffs per
        \\ Denes Bartha, Peter Burcsi, "Reconstructibility of Trees From
        \\ Subtree Size Frequencies", Studia Universitatis Babe-Bolyai
        \\ Mathematica, volume 59, number 4, 2014, pages 435-442
        \\ http://www.cs.ubbcluj.ro/~studia-m/2014-4/Cuprins2014_4.htm
        \\ http://www.cs.ubbcluj.ro/~studia-m/2014-4/03-bartha-burcsi-final.pdf
        \\
        check_equal(polcoeff(got,0), 1,
                    "vpar_spanfreq_poly() size=0, 1 empty");
        check_equal(polcoeff(got,1), #vpar,
                    "vpar_spanfreq_poly() size=1, n");
        check_equal(polcoeff(got,2), vpar_num_edges(vpar),
                    "vpar_spanfreq_poly() size=2, num edges");
        my(degrees=vpar_degrees_vector(vpar));
        check_equal(polcoeff(got,3),
                    sum(v=1,#vpar, binomial(degrees[v],2)),
                    "vpar_spanfreq_poly() size=3, binomial 2 neighbours");
        check_equal(polcoeff(got,#vpar),
                    vpar_is_tree(vpar),
                    "vpar_spanfreq_poly() size=n, 1 whole tree");
        if(vpar_is_tree(vpar),
           check_equal(polcoeff(got,#vpar-1),
                       vpar_num_leaves(vpar),
                       "vpar_spanfreq_poly() size=n-1, num leaves"));

        check_equal(got, test_spanning_freqpoly_by_search(vpar)));
  );
}

test_make_dogbone(a,b,c,d,e) =
{
  my(vpar=vpar_make_path(c+1));
  vpar=vpar_join(vpar,1,vpar_make_path(a));
  vpar=vpar_join(vpar,1,vpar_make_path(b));
  vpar=vpar_join(vpar,c+1,vpar_make_path(d));
  vpar=vpar_join(vpar,c+1,vpar_make_path(e));
}

\\    1               9
\\     \             /
\\      2--5--6--7--8
\\     /             \
\\    3               10
\\   /                  \
\\  4                    11--12
\\
{
  \\             4        8
  my(vpar=[0,1,2,3, 2,5,6,7, 8, 8,10,11]);
  check_equal(#vpar,12);
  vpar_validate(vpar);

  my(got=test_make_dogbone(1,2,4,1,3));
  \\ vpar_view(got);
  check_equal(vpar_is_free_isomorphic(vpar,got), 1);
}

test_dogbone_abcde1(k) = [k, 2*k+1, 1,   k, 2*k+1];
test_dogbone_abcde2(k) = [k, k,     1, 2*k, 2*k+2];
check_equal(test_dogbone_abcde1(1), [1,3, 1, 1,3]);
check_equal(test_dogbone_abcde2(1), [1,1, 1, 2,4]);

\\ This poly per Bartha and Burcsi, low term size=1.
test_dogbone_freqpoly(k) =
{
  x* (x^(k+1) - 1)^2 * (x^(2*k+2) - 1)^2 / (x-1)^4
  + 2*( x*(x^(3*k+2) - 1)/(x-1) - 3*k - 2) / (x-1);
}
{
  for(k=1,10,
     my(T1 = test_make_dogbone(k, 2*k+1, 1,   k, 2*k+1),
        T2 = test_make_dogbone(k, k,     1, 2*k, 2*k+2));
     if(k==1,
        check_equal(#T1, 10, "T1 size");
        check_equal(#T2, 10);
        \\ vpar_view(T1);vpar_view(T2);
        \\ print(T1);print(T2);
       );
     check_equal(vpar_is_free_isomorphic(T1,T2), 0);

     my(want_freqpoly = test_dogbone_freqpoly(k),
        got_T1 = vpar_spanfreq_poly(T1),
        got_T2 = vpar_spanfreq_poly(T2));
     check_equal(got_T1, got_T2);
     check_equal(got_T1, want_freqpoly*x+1);
  );
}

\\ k=1
\\          T1                          T2       *
\\                                              /
\\   k   *        *  k            k   *        *  2k
\\        \      /                     \      /
\\         * -- *                       * -- *
\\        /      \                     /      \
\\ 2k+1  *        *  2k+1         k   *        *  2k+1
\\      /          \                            \
\\     *            *                            *
\\    /              \                            \
\\   *                *                            *
\\                                                  \
\\                                                   *


\\-----------------------------------------------------------------------------
progress("vpar_add_edge()");

check_equal(vpar_add_edge([0,0,0],2,3), [0,3,0]);
check_equal(vpar_add_edge([0,1,0],2,3), [2,3,0]);


\\------------------------------------------------------------------------------
progress("vpar_reroot_ith()");

{
  my(vpar=[0,1,0,3]);
  check_equal(vpar_reroot_ith(vpar,1), [0,1,0,3]);
  check_equal(vpar_reroot_ith(vpar,2), [0,1,4,0]);
  check_equal(vpar_reroot_ith(vpar,3), [2,0,0,3]);
  check_equal(vpar_reroot_ith(vpar,4), [2,0,4,0]);
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        num_rootings=vpar_num_rootings(vpar));

     check_equal(vpar_reroot_ith(vpar,1),
                 vpar_reroot_minmax(vpar,min));
     check_equal(vpar_reroot_ith(vpar,num_rootings),
                 vpar_reroot_minmax(vpar,max));
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_graph6_parse_size()");

check_equal(vpar_INTERNAL_graph6_parse_size(Vecsmall(93),1), \
            [30, 2], \
            "vpar_INTERNAL_graph6_parse_size() 30");
check_equal(vpar_INTERNAL_graph6_parse_size(Vecsmall([126, 66,63,120]),1), \
            [12345, 5], \
            "vpar_INTERNAL_graph6_parse_size() 12345");
check_equal(vpar_INTERNAL_graph6_parse_size \
            (Vecsmall([126,126, 126,126,126,126,126,126]), 1), \
            [2^36-1, 9], \
            "vpar_INTERNAL_graph6_parse_size() 12345");

{
  my(ranges=[63,258047,2^36]);
  for(rep=1,10,
     my(range=random(#ranges)+1,
        n=random(ranges[range]),
        str=vpar_INTERNAL_graph6_size_str(n),
        vec=Vecsmall(str));
     \\ print(vec);
     check_equal(vpar_INTERNAL_graph6_parse_size(vec, 1),
            [n,#vec+1],
            "vpar_INTERNAL_graph6_parse_size()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_num_ordered_isomorphic()");

check_equal(vpar_num_ordered_isomorphic([]), 1, \
            "vpar_num_ordered_isomorphic() empty");
check_equal(vpar_num_ordered_isomorphic([0]), 1);
check_equal(vpar_num_ordered_isomorphic([0,1]), 2, \
            "vpar_num_ordered_isomorphic() path-2");
{
  \\ star-n has n ordered isomorphics (any choice of root)
  for(n=1,10,
     check_equal(vpar_num_ordered_isomorphic(vpar_make_star(n)), n,
            "vpar_num_ordered_isomorphic() star"));
}
{
  \\ path-n has n! ordered isomorphics (any permutation)
  for(n=1,10,
     check_equal(vpar_num_ordered_isomorphic(vpar_make_path(n)), n!,
            "vpar_num_ordered_isomorphic() path"));
}

{
  for(forest=0,1,
     for(n=0, test_limit_trees_search - forest,
        my(lo=0,
           lo_count=0,
           hi=0,
           hi_count=0);
        forvec(vec=vpar_forvec_from(n,forest), \\ forests
              my(vpar=vpar_from_upascend(vec),
                 got=vpar_num_ordered_isomorphic(vpar));
              if(lo_count==0, lo=got);
              if(hi_count==0, hi=got);
              if(got==lo, lo_count++,
                 got<lo, lo=got;lo_count=1);
              if(got==hi, hi_count++,
                 got>hi, hi=got;hi_count=1);
              );

        \\ print("n="n" forest="forest"  lo "lo"["lo_count"]  hi "hi"["hi_count"]");

        if(forest,
           check_equal(lo, 1);
           check_equal(lo_count, 1);
           check_equal(hi, n!);
           check_equal(hi_count, n!);
           ,
           check_equal(lo, max(1,n));
           check_equal(lo_count, max(1,n));
           check_equal(hi, n!);
           check_equal(hi_count, n!));
  ));
}

test_num_ordered_isomorphic_by_search_perm(vpar) =
{
  \\ loop over ordered perms, l = distinct vpar vectors from relabelling
  my(l=List([]));
  for(i=1,(#vpar)!,
     my(perm=numtoperm(#vpar,i));
     if(vpar_perm_is_ordered(vpar,perm),
        my(new_vpar = vpar_relabel_perm(vpar, perm));
        my(pos=setsearch(l,new_vpar,1));
        if(pos, listinsert(l,new_vpar,pos))));
  #l;
}
test_num_ordered_isomorphic_by_search_vpar(vpar) =
{
  \\ loop over vpars
  my(ret=0);
  forvec(vec=vpar_forvec_from(#vpar, !vpar_is_tree(vpar)),
        my(try_vpar=vpar_from_upascend(vec));
        ret += vpar_is_ordered_isomorphic(vpar,try_vpar));
  ret;
}
check_equal(test_num_ordered_isomorphic_by_search_vpar([0,1]), 2);
check_equal(vpar_is_ordered_isomorphic([0,1], [2,0]), 1);

test_num_ordered_perm_by_search(vpar) = \
  sum(i=1,(#vpar)!, vpar_perm_is_ordered(vpar,numtoperm(#vpar,i)));

check_equal(test_num_ordered_isomorphic_by_search_vpar([0,1,1,3]), 12);
check_equal(test_num_ordered_perm_by_search([0,1,1,3]), 12);

check_equal(test_num_ordered_isomorphic_by_search_vpar([0,1,1,1]), 4);
check_equal(test_num_ordered_perm_by_search([0,1,1,1]), 4);


{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_num_ordered_isomorphic(vpar));

     check_equal(1 <= got && got <= (#vpar)!, 1,
                 "vpar_num_ordered_isomorphic() range");

     if(vpar_is_tree(vpar),
        check_equal(#vpar <= got && got <= (#vpar)!, 1,
                    "vpar_num_ordered_isomorphic() range, tree");
        );

     if(#vpar <= test_limit_trees_search,
        check_equal(got, test_num_ordered_isomorphic_by_search_vpar(vpar),
                    "vpar_num_ordered_isomorphic() vs test_num_ordered_isomorphic_by_search_vpar()");
       );
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_ordered_automorphism()");

check_equal(vpar_is_ordered_automorphism([],[]), 1, \
            "vpar_is_ordered_automorphism() empty");
check_equal(vpar_is_ordered_automorphism([0],[1]), 1, \
            "vpar_is_ordered_automorphism() singleton");

check_equal(vpar_perm_is_ordered([0,1],[2,1]), 1, \
            "vpar_perm_is_ordered() path-2, swap");

check_equal(vpar_is_ordered_automorphism([0,1],[1,2]), 1, \
            "vpar_is_ordered_automorphism() path-2, identity");
check_equal(vpar_is_ordered_automorphism([0,1],[2,1]), 1, \
            "vpar_is_ordered_automorphism() path-2, swap");

\\ WRONG
\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i]);
\\      for(rep=1,3,
\\         my(perm=vpar_INTERNAL_random_perm(#vpar));
\\         check_equal(vpar_is_ordered_automorphism(vpar,perm),
\\                     vpar_perm_is_ordered(vpar,perm)
\\                       && vpar_is_automorphism(vpar,perm),
\\           "vpar_is_ordered_automorphism() vs ordered and automorophism");
\\      ));
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_num_ordered_automorphisms()");

\\ FIXME
\\ {
\\   for(n=1,10,
\\      check_equal(vpar_num_ordered_automorphisms(vpar_make_path(n)), 1, \
\\             "vpar_num_ordered_automorphisms() path"));
\\ }

test_num_automorphisms_by_search(vpar) = \
  sum(i=1,(#vpar)!, vpar_is_ordered_automorphism(vpar,numtoperm(#vpar,i)));

\\ FIXME: vpar_num_ordered_automorphisms() is wrong
\\
\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i],
\\         got=vpar_num_ordered_automorphisms(vpar));
\\
\\      print(vpar" got "got);
\\
\\      check_equal(1 <= got && got <= (#vpar)!, 1,
\\                  "vpar_num_ordered_automorphisms() range");
\\
\\      if(#vpar <= test_limit_perms_search,
\\         check_equal(got, test_num_automorphisms_by_search(vpar),
\\         "vpar_num_ordered_automorphisms() vs test_num_automorphisms_by_search()");
\\        );
\\   );
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_seq_uplexqueue()");

check_equal(vpar_seq_uplexqueue([]), Vecsmall([]), \
           "vpar_seq_uplexqueue() empty");
{
  \\        10
  \\       / | \
  \\     3   4   9
  \\    / \     / \
  \\   1   2   5   8
  \\              / \
  \\             6   7
  \\
  my(vpar=[3,3,10,10,9,8,8,9,10, 0]);     \\ Knuth 2.3.4.4
  check_equal(vpar_seq_uplexqueue(vpar),
              Vecsmall([1,2,4,5,6,7,     \\ childless
                        3,8,             \\ two childless under
                        9,
                        10]),
             "vpar_seq_uplexqueue() 10");
}
{
  \\   1  2
  \\   |  |
  \\   4  3
  my(vpar=[0, 0, 2, 1]);
  \\ WRONG
  \\ check_equal(vpar_seq_uplexqueue(vpar), Vecsmall([3,4, 1,2]));
}

test_seq_uplex_by_sort(vpar) =
{
  my(ids=vpar_automorphism_ids_vector(vpar),
     queue=vpar_seq_upqueue(vpar)^-1);
  vecsort(vector(#vpar,v,[ids[v],queue[v]]),,1);
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        seq=vpar_seq_uplexqueue(vpar));

     check_equal(seq, test_seq_uplex_by_sort(vpar),
                 "vpar_seq_uplexqueue() vs sort");

     \\ print(vpar" "ids);
     my(ids=vpar_automorphism_ids_vector(vpar),
        ids_in_seq=vecextract(ids,seq));
     check_equal(test_vector_is_increasing(ids_in_seq),1,
               "vpar_seq_uplexqueue() makes increasing vpar_automorphism_ids_vector()");

     \\ WRONG.
     \\ Same subtrees of different parents can be jumbled.
     \\ my(seq_queue=vpar_seq_upqueue(vpar),
     \\    vec_queue=vecextract(vpar,seq_queue),
     \\    vec=vecextract(vpar,seq));
     \\ check_equal(vec, vec_queue,
     \\             "vpar_seq_uplexqueue() vecextract same as vpar_seq_queue()");
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_degrees_le()");

check_equal(vpar_is_degrees_le([0,1],  1), 1, "vpar_is_degrees_le() path-2");
check_equal(vpar_is_degrees_le([0,1,1],1), 0);
check_equal(vpar_is_degrees_le([0,1,1],  2), 1);
check_equal(vpar_is_degrees_le([0,1,1,1],2), 0);
check_equal(vpar_is_degrees_le([0,1,1,1],  3), 1);
check_equal(vpar_is_degrees_le([0,1,1,1,1],3), 0);


\\-----------------------------------------------------------------------------
progress("vpar_seq_childless_paths()");
\\ PENDING vpar_minmax_childless_descendants_vector()

\\ same as sort, as described in the help string
test_seq_childless_paths_by_sort(vpar,down) =
{
  my(min_childless = vpar_minmax_childless_descendants_vector(vpar,min),
     depths             = vpar_depths_vector(vpar));
  if(down,, depths=-depths); \\ decreasing depths when up
  my(keys = vector(#vpar,v, [min_childless[v],depths[v]]));
  vecsort(keys,,1);
}
{
  my(vpar=[0,1,0,0]);
  check_equal(vpar_seq_childless_paths(vpar,1), Vecsmall([1,2, 3,4]));
  check_equal(test_seq_childless_paths_by_sort(vpar,1), Vecsmall([1,2, 3,4]));
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(down=0,1,
        check_equal(vpar_seq_childless_paths(vpar,down),
                    test_seq_childless_paths_by_sort(vpar,down),
                    "vpar_seq_childless_paths() vs sort");
     ));
}


\\-----------------------------------------------------------------------------
progress("vpar_perm_is_ordered()");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     #vpar >= 1 || next();
     #vpar <= test_limit_perms_search || next();

     my(count=0);
     for(i=1,(#vpar)!,
        my(perm=Vecsmall(numtoperm(#vpar,i)));
        vpar_perm_is_ordered(vpar,perm) || next();

        my(vpar2=vpar_relabel_perm(vpar,perm));
        check_equal(vpar_is_ordered_isomorphic(vpar,vpar2), 1,
                    "vpar_perm_is_ordered() relabelling is vpar_is_ordered_isomorphic()");
        count++;
     );

     check_equal(count, vpar_num_ordered_isomorphic(vpar),
                 "vpar_perm_is_ordered() count vs vpar_num_ordered_isomorphic()");
  );
}

\\ WRONG
\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i]);
\\      #vpar >= 1 || next();
\\      #vpar <= test_limit_perms_search || next();
\\
\\      my(canonical_vpar=vpar_relabel_preorder(vpar),
\\         count=0);
\\      for(i=1,(#vpar)!,
\\         my(perm=Vecsmall(numtoperm(#vpar,i)),
\\            relabelled_vpar=vpar_relabel_perm(vpar,perm),
\\            canonical_relabelled_vpar=vpar_relabel_preorder(vpar),
\\            got = vpar_perm_is_ordered(vpar,perm));
\\         \\ print(vpar" perm "perm" seq "perm^-1" got "got);
\\         count += got;
\\         check_equal(got, canonical_vpar==canonical_relabelled_vpar);
\\      );
\\
\\      check_equal(count, vpar_num_ordered_isomorphic(vpar),
\\                 "vpar_perm_is_ordered() count vs vpar_num_ordered_isomorphic()");
\\   );
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_bandwidth()");
\\ vpar_bandwidth_and_perm()

check_equal(vpar_bandwidth([]), 0);
check_equal(vpar_bandwidth([0]), 0);
check_equal(vpar_bandwidth([0,1]), 1, "vpar_bandwidth_and_perm() path-2");
check_equal(vpar_bandwidth([0,0]), 0, "vpar_bandwidth_and_perm() isolated 2");
check_equal(vpar_bandwidth([0,1,2,3]), 1, "vpar_bandwidth_and_perm() path-4");
check_equal(vpar_bandwidth([0,1,1,1]), 2, "vpar_bandwidth_and_perm() star-4");

test_bandwidth_by_search(vpar) =
{
  my(b=#vpar);
  for(p=1,(#vpar)!,
     b=min(b,vpar_dilation(vpar_relabel_perm(vpar,numtoperm(#vpar,p)))));
  b;
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     if(#vpar>8, next());
     my(by_search=test_bandwidth_by_search(vpar),
        bp=vpar_bandwidth_and_perm(vpar));
     if(bp[1]!=by_search,
        vpar_view(vpar));
     check_equal(bp[1],by_search, "vpar_bandwidth_and_perm() vs search");
     check_equal(#bp[2],#vpar, "vpar_bandwidth_and_perm() perm length");
     check_equal(test_is_perm(bp[2]),1, "vpar_bandwidth_and_perm() is perm");

     my(vpar_relabelled=vpar_relabel_perm(vpar,bp[2]));
     if(vpar_dilation(vpar_relabelled)!=by_search,
        vpar_view(vpar_relabelled));
     check_equal(vpar_dilation(vpar_relabelled),by_search,
                 "vpar_bandwidth_and_perm() dilation after perm");
  );
}

{
  \\ reverse permutation does not commute, in general
  my(perm=Vecsmall([1, 3, 2]),
     rev=test_perm_reversal(3));
  check_equal(rev*perm, Vecsmall([3, 1, 2]));
  check_equal(perm*rev, Vecsmall([2, 3, 1]));
  check_equal(perm*rev != rev*perm, 1);
}


\\-----------------------------------------------------------------------------
progress("vpar_is_equimatchable()");

\\ PENDING vpar_lower_matchnum()

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got      = vpar_is_equimatchable(vpar));
     check_equal(got, vpar_matchnum(vpar)==vpar_lower_matchnum(vpar),
                 "vpar_is_equimatchable() vs matchnum==lower_matchnum");
  );
}

\\-----------------------------------------------------------------------------
progress("vpar_critical_inddiff()");

\\ 1-2 = -1 or 2-1 = 1
check_equal(vpar_critical_inddiff([]), 0, "vpar_critical_inddiff() empty");
check_equal(vpar_critical_inddiff([0]),1, "vpar_critical_inddiff() singleton");
check_equal(vpar_critical_inddiff(vpar_make_path(2)), 1-1, \
            "vpar_critical_inddiff() path-2");
check_equal(vpar_critical_inddiff(vpar_make_path(3)), 2-1, \
            "vpar_critical_inddiff() path-3"); \\ 1
check_equal(vpar_critical_inddiff(vpar_make_path(4)), 2-2);
check_equal(vpar_critical_inddiff(vpar_make_path(5)), 3-2);
check_equal(vpar_critical_inddiff(vpar_make_path(6)), 3-3);

test_critical_inddiff_by_sets(vpar) =
{
  \\ print("test_critical_inddiff_by_sets() "vpar);
  my(sets=vpar_indsets(vpar));
  \\ print("sets  "sets);

  my(diffs=apply(set->#set - #vpar_neighbours_of_set(vpar,set), sets));
  \\ print("diffs "diffs); if(#diffs, my(pos);vecmax(diffs,&pos);print(" smallest set "sets[pos]));

  if(#diffs, vecmax(diffs));
}
test_critical_inddiff_by_max(vpar) =
{
  \\ print("test_critical_inddiff_by_max() "vpar);

  my(seq=vpar_seq_upascend(vpar),
     with           = vectorsmall(#vpar,v,1),
     none           = -999,
     without_seen   = vectorsmall(#vpar,v,none),
     without_unseen = vectorsmall(#vpar),
     ret = 0);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v],
        v_without = max(without_seen[v], without_unseen[v]),
        v_max     = max(with[v], v_without));
     \\ print(" v="v" with "with[v]" without_seen "without_seen[v]" without_unseen "without_unseen[v]);

     if(p,
        with[p] += max(without_unseen[v]-1, without_seen[v]);
        without_seen[p] = max(without_seen[p] + v_max,           \\ extend
                              without_unseen[p] + with[v] - 1);  \\ build
        without_unseen[p] += v_without;

        \\ print("  set p="p" with "with[p]" without_seen "without_seen[p]" without_unseen "without_unseen[p]);
        ,
        ret += v_max));

  ret;
}

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i],
        got=vpar_critical_inddiff(vpar));

     my(vpar=test_sample_forests[i],
        got=vpar_critical_inddiff(vpar));
     check_equal(got, test_critical_inddiff_by_max(vpar),
                 "vpar_critical_inddiff() vs 2*indnum-n");

     if(#vpar <= test_limit_sets,
        \\ print(vpar);
        check_equal(vpar_critical_inddiff(vpar),
                    test_critical_inddiff_by_sets(vpar),
          "vpar_critical_inddiff() vs test_critical_inddiff_by_sets()"));
  );
}

{
  my(forest=1);
  for(n=1, test_limit_trees_search - forest,
     forvec(vec=vpar_forvec_from(n,forest), \\ forests
           my(vpar=vpar_from_upascend(vec),
              got=vpar_critical_inddiff(vpar));
           check_equal(vpar_critical_inddiff(vpar),
                       test_critical_inddiff_by_sets(vpar),
                       "vpar_critical_inddiff() vs test_critical_inddiff_by_sets()");
           check_equal(got, 2*vpar_indnum(vpar)-#vpar,
                       "vpar_critical_inddiff() vs 2*indnum-n");
     ));
}


\\-----------------------------------------------------------------------------
progress("vpar_count_ordered()");

check_equal(vpar_count_ordered(2), 1);
check_equal(vpar_count_ordered(3), 2);
check_equal(binomial(2*3,3), 20);
check_equal(binomial(2*3,3)/4, 5);
check_equal(binomial(2*3,2)/3, 5);


\\ binomial(2*n,n)/(n+1) = (2*n * ... * n+1) / n/.../1 / (n+1)
\\                       = (2*n * ... * n+2) / n/.../1
check_equal(vector(10,n, vpar_INTERNAL_Catalan_number(n)), \
            vector(10,n, prod(i=n+2,2*n, i) / n!));


\\ @ * * * * *
\\   |   | |      runs 1,1,0,1 = 3 total
\\                     r 1 2 3
\\   1
\\   |
\\   2
\\

\\ binomial(2*n,n)   = (2*n)!/n!/n!
\\ binomial(2*n,n-1) = (2*n)!/(n-1)!/(n+1)!
\\
\\ binomial(2*n-1,n) = (2*n-1)!/n!/(n-1)!
\\    = 2*n*(2*n-1)!/n!/(n-1)!/2/n
\\    = (2*n)!/n!/(n-1)!/2/n
\\    = binomial(2*n,n) / 2
\\ my(n=13); 2*binomial(2*n-1,n) == binomial(2*n,n)
\\
check_equal(vector(10,n, binomial(2*n,n)), \
            vector(10,n, 2*binomial(2*n-1,n)));


\\-----------------------------------------------------------------------------
progress("vpar_span_vertices()");

check_equal(vpar_span_vertices([0,1,2,3,4],[3]), [3]);
check_equal(vpar_span_vertices([0,1,2,3,4],[2,4]), [2,3,4]);
check_equal(vpar_span_vertices([2,3,4,5,0],[2,4]), [2,3,4]);

{
  \\         2
  \\        / \
  \\      *5   1
  \\           |
  \\           7
  \\          / \
  \\         8   3*
  \\        / \
  \\      *6   4*
  \\
  my(vpar=[2, 0, 7, 8, 2, 8, 1, 7],
     vec=[3, 4, 5, 6]);
  check_equal(vecsort(vpar_span_vertices(vpar,vec)),
              [1,2,3,4,5,6,7,8]);
}

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     if(#vpar==0,next());
     my(u=vpar_random_vertex(vpar),
        v=vpar_random_vertex(vpar));
     check_equal(vecsort(vpar_span_vertices(vpar,[u,v])),
                 vecsort(vpar_path_vertices(vpar,u,v)),
                 "vpar_span_vertices() vs vpar_path_vertices()"));
}

test_spanning_vertices_by_paths(vpar,vec) =
{
  my(ret=Set(vec));
  for(i=1,#vec,
     for(j=i,#vec,
        ret=setunion(ret, Set(vpar_path_vertices(vpar,vec[i],vec[j])))));
  ret;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        set=vpar_random_set(vpar));
     check_equal(vecsort(vpar_span_vertices(vpar,set)),
                 vecsort(test_spanning_vertices_by_paths(vpar,set)),
       "vpar_span_vertices() vs test_spanning_vertices_by_paths()"));
}


\\-----------------------------------------------------------------------------
progress("vpar_spanfreqs()");

check_equal(vpar_spanfreqs([]), []);
check_equal(vpar_spanfreqs([0]), [1]);
check_equal(vpar_spanfreqs([0,1]), [2,1]);
check_equal(vpar_spanfreqs([0,1,1]), [3,2,1]);
{
  for(n=1,10,
     my(vpar=vpar_make_path(n));
     vpar=vpar_reroot_random(vpar);
     check_equal(vpar_spanfreqs(vpar), vector(#vpar,s,n-s+1)));
}


\\-----------------------------------------------------------------------------
progress("vpar_distance_distribution()");

check_equal(vpar_distance_distribution([]), []);
check_equal(vpar_distance_distribution([0]), []);
check_equal(vpar_distance_distribution([0,1]), [1]);

test_distance_distribution_by_matrix(vpar) =
{
  my(m=vpar_path_lengths_matrix(vpar),
     ret=vector(if(#vpar,vecmax(m),0)));
  for(u=1,#vpar,
     for(v=u+1,#vpar,
        if(m[u,v],
           ret[m[u,v]]++)));
  ret;
}
{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i],
       vec=vpar_distance_distribution(vpar));
     check_equal(vec, test_distance_distribution_by_matrix(vpar));

     check_equal(#vec, if(#vpar==0,0, vpar_diameter(vpar)),
       "vpar_distance_distribution() addhelp on vpar_diameter()");

     check_equal(vecsum(vec), #vpar*(#vpar-1)/2,
       "vpar_distance_distribution() addhelp on vecsum");

     check_equal(sum(d=1,#vec, d*vec[d]), vpar_Wiener_index(vpar),
       "vpar_distance_distribution() addhelp on vpar_Wiener_index()");
  );
}

{
  \\ distance palindromic tree 21
  \\
  \\ Caporossi, Dobrynin, Gutman, Hansen, "Trees with Palindromic Hosoya
  \\ Polynomials", Graph Theory Notes of New York, volume 37, 1999, pages
  \\ 10-16.
  \\ Shown in Dobrynin, Entringer and Gutman, "Wiener Index of Trees: Theory
  \\ and Applications", Acta Applicandae Mathematicae, volume 66, issue 3,
  \\ May 2001, pages 211-249, figure 7 page 233.

  \\       1 2 3 4 5 6 7 8 9    11       14          18    21
  my(vpar=[0,1,2,3,4,5,6,7,8, 2, 3,11, 4, 6,14,15,15, 8,8,8,8]);
  vpar_validate(vpar);
  check_equal(#vpar,21);
  check_equal(vpar_is_distance_palindromic(vpar), 1);
}


\\-----------------------------------------------------------------------------
progress("vpar_reroot_degree1()");

check_equal(vpar_reroot_degree1([2,0]), [0,1]);
check_equal(vpar_reroot_degree1([2,3,4,0]), [0,1,2,3]);

test_roots_are_degree1(vpar) =
{
  for(v=1,#vpar,
     if(vpar[v]==0 && vpar_degree_of_vertex(vpar,v) > 1, return(0)));
  1;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_reroot_degree1(vpar));
     vpar_validate(got);
     check_equal(vpar_reroot_forest(got,vpar_roots(vpar)), vpar);
     check_equal(test_roots_are_degree1(got), 1));
}


\\-----------------------------------------------------------------------------
progress("vpar_is_subdivision()");

check_equal(vpar_is_subdivision([]), 1, "vpar_is_subdivision() empty");
check_equal(vpar_is_subdivision([0]), 1, "vpar_is_subdivision() singleton");
check_equal(vpar_is_subdivision([0,1]), 0);
check_equal(vpar_is_subdivision([0,1,2]), 1, "vpar_is_subdivision() path-3");
check_equal(vpar_is_subdivision([2,3,0]), 1, "vpar_is_subdivision() path-3");
check_equal(vpar_is_subdivision([0,1,2,3]), 0);
check_equal(vpar_is_subdivision([0,1,2,3],2), 1);

\\   1    2
\\        |
\\        3
\\       / \
\\      4   5
\\          |
\\          6
check_equal(vpar_is_subdivision([0,0,2,3,3,5]), 0);

test_is_subdivision_by_depths(vpar,n=1) =
{
  \\ print("test_is_subdivision_by_depths() n="n" "vpar);
  my(m=n+1,
     depths      =vpar_depths_vector(vpar),
     num_children=vpar_num_children_vector(vpar));
  for(v=1,#vpar,
     if(depths[v]%m && num_children[v]!=1,
        \\ print(" v="v" depth "depths[v]" num_children "num_children[v]);
        return(0)));
  1;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_is_subdivision(vpar),
                 test_is_subdivision_by_depths(vpar),
                 "vpar_is_subdivision() vs test_is_subdivision_by_depths()");

     my(n=random(5),
        vpar_sub=vpar_subdivide(vpar,n));
     \\ print("n="n);
     \\ print(vpar);
     \\ print(vpar_sub);
     check_equal(vpar_is_subdivision(vpar_sub,n), 1,
                 Str("vpar_is_subdivision() of subdivide n="n));
  );
}

\\    n = 0 1   3     5       7
\\        1,1,0,6,0,180,0,13440,0,1890000
\\ not in OEIS: 6,0,180,0,13440,0,1890000
\\
\\ n=3  1--2--3  permute 3!==6 from tree n=2
\\ n=5  1--2--3--4--5  permute 5!==120 from trees n=3
\\      and reroot to
\\ vpar_count_unlabelled(3) == 2     *     *
\\                                   |    / \
\\                                   *   *   *
\\                                   |
\\                                   *
\\ vpar_count(2) * binomial(3,1) == 6
\\ vpar_count(3) * binomial(5,2)*2! == 90
\\ vector(6,n,n++; vpar_count(n) * (n-1)!*binomial(2*n-1,n-1))
\\ vector(6,n,n++; vpar_count(n) * (2*n-1)!/n!)

\\ orig has orig-1 edges
\\ orig+s*(orig-1) == (orig-1)*(s+1) + 1
\\ n = (orig-1)*(s+1) + 1
\\ orig = (n-1)/(s+1) + 1
test_count_subdivisions(n,s=1) =
{
  my(N);
  N = (n-1)/(s+1) + 1;
  if(n==0,1, n%(s+1)!=1,0, vpar_count(N) * n!/N!);
}
{
  my(forest=0);
  for(n=0, test_limit_trees_search - forest,
     my(count=vector(4));
     forvec(vec=vpar_forvec_from(n,forest), \\ trees
           my(vpar=vpar_from_upascend(vec));
           for(s=1,#count,
              count[s] += vpar_is_subdivision(vpar,s)));
     for(s=1,#count,
        check_equal(count[s], test_count_subdivisions(n,s)));
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_corona_singletons()");

check_equal(vpar_corona_singletons([]), []);
check_equal(vpar_corona_singletons([0]), [0,1]);
check_equal(vpar_corona_singletons([0],2), [0,1,1]);
check_equal(vpar_corona_singletons([0,1],3), [0,1, 1,1,1, 2,2,2]);
check_equal(vpar_corona_singletons([0,1,2],4), \
            [0,1,2, 1,1,1,1, 2,2,2,2, 3,3,3,3]);
check_equal(vpar_corona_singletons([0,1,2],2), [0,1,2, 1,1, 2,2, 3,3], \
            "vpar_corona_singletons() example in addhelp");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_minimal_domsets_count(vpar_corona_singletons(vpar)),
                 2^#vpar));
}


\\-----------------------------------------------------------------------------
progress("vpar_distance_distribution_polynomial()");

check_equal(vpar_distance_distribution_polynomial([]), Pol(0));
check_equal(vpar_distance_distribution_polynomial([0]), Pol(0));
check_equal(vpar_distance_distribution_polynomial([0,1]), x);
check_equal(vpar_distance_distribution_polynomial([0,1],'y), 'y);

test_distance_distribution_polynomial_by_matrix(vpar) =
{
  my(m=vpar_path_lengths_matrix(vpar),
     ret=Pol(0));
  for(u=1,#vpar,
     for(v=u+1,#vpar,
        ret += x^m[u,v]));
  ret;
}
{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     check_equal(vpar_distance_distribution_polynomial(vpar),
                 test_distance_distribution_polynomial_by_matrix(vpar));

     \\ check_equal(vpar_distance_distribution_polynomial(vpar),
     \\             charpoly(vpar_path_lengths_matrix(vpar)));
     \\ print(vpar_distance_distribution_polynomial(vpar));
     \\ print(charpoly(vpar_path_lengths_matrix(vpar)));
     \\ print();
  );
}

\\-----------------------------------------------------------------------------
progress("vpar_seq_dandelion()");
\\ Dandelion = ER, Joyal

test_seq_dandelion_by_v_to_root(vpar) =
{
  if(#vpar==0,Vecsmall([]),
     my(above=vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max));
     above[1]=1;
     vpar_INTERNAL_seq_rotate_down(above));
}

{
  \\ *
  \\ 1 <- 2 <- 3 <- 4 <- 5 <- 6
  my(vpar=[0,1,2,3,4,5]);
  vpar_validate(vpar);
  check_equal(test_seq_dandelion_by_v_to_root(vpar),
              Vecsmall([1,2,3,4,5,6]), "dandelion path-6");
}

{
  \\      *                   *
  \\ 6 <- 5 <- 4 <- 3 <- 2 <- 1
  my(vpar=[2,3,4,5,6,0]);
  vpar_validate(vpar);
  check_equal(test_seq_dandelion_by_v_to_root(vpar),
              Vecsmall([5,2,3,4,1,6]));
  check_equal(vpar_seq_dandelion(vpar),
              Vecsmall([5,2,3,4,1,6]));
}

check_equal(vpar_seq_dandelion([]), Vecsmall([]));
check_equal(vpar_from_dandelion([]), []);

check_equal(vpar_seq_dandelion([0]), Vecsmall([1]));
check_equal(vpar_seq_dandelion([0,1]), Vecsmall([1,2]));

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_seq_dandelion(vpar),
                 test_seq_dandelion_by_v_to_root(vpar)));
}

\\ Picciotto thesis section 6.4
test_code_dandelion_by_surgery(vpar) =
{
  my(n=#vpar,
     weight=vector(n));
  for(i=1,n-1,
     my(m=vpar[n-i+1]);
     my(k=vpar[1]);
     vpar[n-i+1] = 1;
     weight[n-i+1] = m;     \\ B_m
     if(vpar_is_cyclic(vpar),
        vpar[1] = m;
        vpar[n-i+1] = 1;
        weight[n-i+1] = k)); \\ B_k
  weight[2..n];
}

test_code_dandelion_by_loop(vpar) =
{
  if(#vpar==0,return(Vecsmall([])));
  my(label1=vector(#vpar));
  forstep(v=#vpar,2,-1,
     my(h=vpar[v]);
     my(k=vpar[1]);
     vpar[v]=1;
     label1[v]=h;
     if(vpar_is_cyclic(vpar),
        my(k=vpar[1]);
        vpar[1]=h;
        label1[v]=k));
  label1[2..#vpar];
}

{
  \\ Evan Thompson, Tim Paulden, and David K. Smith, "The Dandelion Code: A
  \\ New Coding of Spanning Trees for Genetic Algorithms", IEEE Transactions
  \\ ON Evolutionary Computation, volume 11, number 1, February 2007.

  \\              *     *           *
  \\  1 --> 6 --> 9 --> 5 --> 2 --> 4 --> 12 root
  \\  |           |           |            |
  \\  7          11     3 -> 10            8

  my(vpar=[6,4,10,12,2,9,1,12,5,2,9,0]);
  vpar_validate(vpar);

  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max),
              \\        1     4 5       9    12
              Vecsmall([0,4,0,4,5,9,0,0,9,0,0,4]),
              "vpar_INTERNAL_minmax_v_to_root_vector root=12");

  \\                   *    *            *
  \\    1          2 = 4    5 loop   6 = 9     12
  \\    |          |                     |      |
  \\    7   3 --> 10                    11      8
  \\ cycles (2,4) (6,9) (5)
  \\             2           7  8
  my(want_code=[ 4,10,2,5,9, 1,12,6,2,9]);
  check_equal(#want_code,10);

  \\ parent of 1 = root 12
  my(want_vec=concat([12,want_code,[0]]));
  \\ vpar_view(want_vec);
  check_equal(vpar_INTERNAL_minmax_upto_root_or_cycle_vector(want_vec,1,max),
           \\  1         4  5             9
              [1, 4, -3, 4, 5, 9, -7, -8, 9, -3, -11, -8]);

  check_equal(vpar_INTERNAL_seqfrom_dandelion(want_vec),
              \\               4 5         9
              Vecsmall([4,2,3, 5,9, 6,7,8, 1, 10,11,12]),
              "dandelion 12 seqfrom code");

  check_equal(vecextract(vpar,vpar_seq_dandelion(vpar)), want_vec,
              "dandelion 12 vec");

  check_equal(vpar_from_dandelion(want_vec), vpar,
              "dandelion 12 from code");

  \\--------
  \\ with root=0

  \\              4       8
  my(vpar=[6,4,10,0,2,9,1,0,5,2,9]);
  vpar_validate(vpar);
  want_code=apply(p->if(p==12,0,p),want_code);

  check_equal(test_code_dandelion_by_loop(vpar), want_code,
              "dandelion 12 by_loop");
  check_equal(test_code_dandelion_by_surgery(vpar), want_code,
              "dandelion 12 by_surgery");
}

{
  \\ example in Picciotto thesis section 9.1 page 115 (first)
  \\        0
  \\        |
  \\        4
  \\      / | \
  \\    5   2  7
  \\       / \
  \\      6   3
  \\      |
  \\      1
  my(vpar=[6,4,2,0,4,2,4]);  \\ root=0
  vpar_validate(vpar);
  check_equal(vecextract(vpar,vpar_seq_upascend(vpar)), \\ Prufer
              [6,2,4,2,4,4, 0], "Prufer first");

  check_equal(test_code_dandelion_by_loop(vpar),
              [4,2,2,4,6, 4], "dandelion first loop");
  check_equal(test_code_dandelion_by_surgery(vpar),
              [4,2,2,4,6, 4], "dandelion first surgery");

  \\ check_equal(test_code_blob_by_surgery(vpar),
  \\             [6,4,2,4,2,4],
  \\             "test_code_blob_by_surgery");

  \\---------
  \\ with root=8

  my(vpar=[6,4,2,8,4,2,4,0]);  \\ root=8
  vpar_validate(vpar);
  check_equal(vecextract(vpar,vpar_seq_upascend(vpar)), \\ Prufer
              [6,2,4,2,4,4, 8,0], "Prufer first");

  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max),
              \\          2   4*  6*  8
              Vecsmall([0,4,0,4,0,6,0,4]));

  my(want_code=[4,2,2,4,6,4]);
  check_equal(vecextract(vpar,vpar_seq_dandelion(vpar)),
              concat([8,want_code,0]),
              "dandelion first vec");
}
{
  \\ example in Picciotto thesis section 9.1 page 116 (second)
  \\       0
  \\       |
  \\       6
  \\       |
  \\       2
  \\      / \
  \\     3   4
  \\       / | \
  \\      1  5  7
  \\
  \\       1 2 3 4     7
  my(vpar=[4,6,2,2,4,0,4]);
  vpar_validate(vpar);

  check_equal(test_code_dandelion_by_loop(vpar),
              [6,2,2,4,4, 4], "dandelion second loop");
  check_equal(test_code_dandelion_by_surgery(vpar),
              [6,2,2,4,4, 4], "dandelion second surgery");

  \\-----
  \\ with root=8
  my(vpar=[4,6,2,2,4,8,4,0]);
  vpar_validate(vpar);

  my(want_code=[6,2,2,4,4,4]);
  check_equal(vecextract(vpar,vpar_seq_dandelion(vpar)),
              concat([8,want_code,0]),
              "dandelion root=8");
}
{
  \\ Caminiti thesis section 5.2 figure 5.4 dandelion
  \\
  \\              0
  \\            /   \
  \\          5*      9
  \\          |        \
  \\          2         11
  \\        /   \
  \\       7    10*
  \\              \
  \\               4 -- 6 -- 3 -- 1 -- 8
  \\

  \\           1 2 3  4  5 6 7 8 9 10 11
  my(vpar=    [3,5,6,10, 0,4,2,1,0, 2, 9]);
  vpar_validate(vpar);
  check_equal(vpar_num_roots(vpar),2);

  \\              2       5        9 10 11
  my(want_code=[  5,6,10, 2,4,2,1, 0, 3, 9 ]);
  check_equal(test_code_dandelion_by_loop(vpar), want_code,
              "test_code_dandelion_by_loop()");
  check_equal(test_code_dandelion_by_surgery(vpar), want_code,
              "test_code_dandelion_by_surgery()");

  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max),
              \\          2       5          10
              Vecsmall([0,2,10,10,2,10,0,0,0,10,0]));

  \\----------
  \\ with 12 as root
  \\             12
  \\            /   \
  \\          5*      9
  \\          |        \
  \\          2         11
  \\        /   \
  \\       7    10*
  \\              \
  \\               4 -- 6 -- 3 -- 1 -- 8
  \\
  \\           1 2 3  4  5 6 7 8  9 10 11 12
  my(vpar=    [3,5,6,10,12,4,2,1,12, 2, 9, 0]);
  vpar_validate(vpar);
  check_equal(vpar_is_tree(vpar),1);

  check_equal(vpar_minmax_upto_root_vector(vpar,1,max),
              [12, 12, 12, 12, 12, 12, 0, 0, 0, 12, 0, 12]);
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max),
              \\                      5               10    12
              Vecsmall([0, 5, 10, 10, 5, 10, 0, 0, 0, 10, 0, 5]),
              "vpar_INTERNAL_minmax_v_to_root_vector 12");

  check_equal(vpar[1],3);   \\ new parent for 10
  check_equal(vpar[10],2);  \\ new parent for 5
  check_equal(vpar[5],12);   \\ root 12 new parent for 1
  my(seq=vpar_seq_dandelion(vpar));
  check_equal(seq,
              \\         1          5          10
              Vecsmall([ 5, 2,3,4, 10, 6,7,8,9, 1, 11,12]), "dandelion seq 12");
              \\      p=root

  my(want_vec=want_code);
  want_vec=apply(p->if(p==0,12,p), want_vec);  \\ root is 12
  want_vec=concat([12,want_vec,0]);  \\ init 1 parent root 12, 12 parent 0
  my(vec=vecextract(vpar,seq));
  check_equal(vec,want_vec, "dandelion vec 12");

  check_equal(vpar_INTERNAL_seqfrom_dandelion(vec),seq, "dandelion seqfrom 12");
  check_equal(vpar_from_dandelion(vec),vpar, "dandelion from 12");
}


\\ Is this rotate vertices on 1 to root which are v < non-root ancestors,
\\ rather than dandelion looking v > non-root ancestors ?
\\
\\ {
\\   \\ example slides 0529_From Dandelion Code to Rainbow Code.pdf
\\
\\   \\    11   14    20 -- 6 -- 24
\\   \\     |    |          |
\\   \\     1 -- 4 -- 15   18 -- 16    12       path 1 to n=25=root
\\   \\     |               |           |    1  7,3,23,9,19,8,10   25
\\   \\     7 -- 3 -- 23 -- 9 -- 19 --- 2      |   |         |  |
\\   \\     |    |                |               *         *  *   min of following
\\   \\    17    5         21 --- 8    13    map   |         |  |
\\   \\          |                |     |       3,7 9,19,8,23 10
\\   \\         22               10 -- 25    is new parent, rest parents unchanged
\\
\\   \\       1     3   5       8  9 10         15         20        25=root
\\   my(vpar=[7,19,23,1,3,18,3,10,19,25,1,2,25,4,4,18,7,9,8,6,8,5,9,6,0]);
\\   check_equal(vpar_roots(vpar),[25]);
\\
\\   check_equal(vpar_path_vertices(vpar,1,vpar_roots(vpar)[1]),
\\               [1, 7,3,23,9,19,8,10, 25], "pi path to root");
\\
\\   my(flags=vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max));
\\   check_equal(flags,
\\           \\             3        7  8  9 10                 19       23   25
\\           Vecsmall([0,0,23,0,0,0,23,10,19,10,0,0,0,0,0,0,0,0,19,0,0,0,23,0,10]),
\\           "vpar_INTERNAL_minmax_v_to_root_vector 12");
\\
\\   check_equal(select(i->flags[i]==i, [1..#flags]), [10,19,23]);
\\
\\   my(got_seq=vpar_seq_dandelion(vpar));
\\   check_equal(test_is_perm(got_seq),1, "is perm");
\\   check_equal(got_seq,
\\   Vecsmall(
\\   [10, 2,3,4,5,6,7,8,9, 19, 11,12,13,14,15,16,17,18, 23, 20,21,22,  1, 24,25]),
\\   \\ 1                  10                           19            23    root
\\     "dandelion seq");
\\   my(got_seq_diff_flags=vector(#got_seq,v,v!=got_seq[v]),
\\      got_seq_diff_vertices=Vec(select(f->f, got_seq_diff_flags, 1)));
\\   \\ print("flags ",got_seq_diff_flags);
\\   \\ print("got seq ",Vec(got_seq));
\\   check_equal(got_seq_diff_vertices, Set([1, 10, 19, 23]));
\\
\\   \\ print(min_vertices);
\\   \\ print(min_upto_root);
\\   \\ print(vector(#min_upto_root,v,v==min_upto_root[v]));
\\   \\ print([1..#min_upto_root]);
\\   \\ v = 1, 3, 8,10,25
\\   \\ p = 7,23,10,25, 0
\\
\\   \\             was 1       was 3 was 8
\\   \\                 *           *     *
\\   \\            1  2 3   5       8    10         15         20      24
\\   my(want_code=[   19,7,1,3,18,3,23,19,10,1,2,25,4,4,18,7,9,8,6,8,5,9,6],
\\      want_vec =concat([0,want_code,25]));
\\   \\ vpar_view(want_vec);
\\   my(got_vec=vecextract(vpar,got_seq));
\\   check_equal(got_vec,want_vec, "dandelion vec");
\\
\\   my(vec_upto_root=vpar_INTERNAL_minmax_upto_root_or_cycle_vector(want_vec,1,min),
\\      vec_upto_root_flags=vector(#vec_upto_root,v,v==vec_upto_root[v]),
\\      vec_vertices=Vec(select(f->f, vec_upto_root_flags, 1)));
\\   \\ print(vec_upto_root);
\\   check_equal(vec_vertices, [1, 3, 8, 10, 25]);
\\
\\   my(vpar_again=vpar_from_dandelion(want_vec));
\\   check_equal(vpar_again,vpar, "dandelion vpar_again");
\\
\\ \\  check_equal(test_vec_dandelion_by_loop(vpar), got_vec[^1]);
\\ }
\\ {
\\   \\ example slides 0529_From Dandelion Code to Rainbow Code.pdf
\\   \\ locality bound 5
\\
\\   my(vpar=[3,6,6,7,4,5,0]);
\\
\\   my(min_upto_root=vpar_minmax_upto_root_vector(vpar,1,min),
\\      min_upto_root_flags=vector(#min_upto_root,v,v==min_upto_root[v]),
\\      min_vertices=Vec(select(f->f, min_upto_root_flags, 1)));
\\   check_equal(min_vertices, [1,3,4,7]);
\\
\\   my(got_seq=vpar_seq_dandelion(vpar),
\\      got_seq_diff_flags=vector(#got_seq,v,v!=got_seq[v]),
\\      got_seq_diff_vertices=Vec(select(f->f, got_seq_diff_flags, 1)));
\\   check_equal(got_seq_diff_vertices, [1,3,4,7]);
\\
\\   \\             2     5            10         15         20      24
\\   my(want_code=[6,3,6,4,5],
\\      want_vec=concat([0,want_code,7]));
\\   my(got_vec=vecextract(vpar,got_seq));
\\   check_equal(got_vec,want_vec, "dandelion vec");
\\
\\   my(vec_upto_root=vpar_INTERNAL_minmax_upto_root_or_cycle_vector(want_vec,1,min),
\\      vec_upto_root_flags=vector(#vec_upto_root,v,v==vec_upto_root[v]),
\\      vec_vertices=Vec(select(f->f, vec_upto_root_flags, 1)));
\\   check_equal(vec_vertices, [1,3,4,7]);
\\
\\   my(vpar_again=vpar_from_dandelion(want_vec));
\\   check_equal(vpar_again,vpar, "dandelion vpar_again");
\\ }


\\ {
\\   \\ example in Egecioglu and Remmel
\\
\\   \\ minimum of each cycle
\\   \\
\\   \\              /------------\            /----\
\\   \\       *                    *     *           *     *
\\   \\       1  ->  4   ->  5  -> 3  -> 7 -> 20 -> 12 -> 21
\\   \\      / \    / \      |           |           |     |
\\   \\     9  16  10  11    2          19           8     6
\\   \\         |                       / \               / \
\\   \\        17                     13   14           15   18
\\
\\   \\       1   3   5     7     10           15          20 21
\\   my(vpar=[4,5,7,5,3,21,20,12,1,4,4,21,19,19,6,1,16,6,7,12,0]);
\\
\\   my(min_upto_root=vpar_minmax_upto_root_vector(vpar,1,min));
\\   check_equal(min_upto_root,
\\   \\           1   3 4 5   7         12               20
\\               [1,0,3,3,3,0,7,0,0,0,0,12,0,0,0,0,0,0,0,12,21]);
\\
\\   \\  1, 3,4,5, 7, 12, 20
\\   my(got_seq=vpar_seq_dandelion(vpar));
\\   check_equal(got_seq,
\\   \\                 1    3       7           12                         21
\\            Vecsmall([21,2,1,4,5,6,3,8,9,10,11, 7,13,14,15,16,17,18,19,20,12]));
\\
\\   my(got_vec=vecextract(vpar,got_seq));
\\   my(vpar_again=vpar_from_dandelion(got_vec));
\\   check_equal(vpar_again,vpar);
\\
\\   \\ \\----------
\\   \\ \\ v=18 upwards
\\   \\
\\   \\ min_upto_root=vpar_minmax_upto_root_vector(vpar,18,min);
\\   \\ check_equal(min_upto_root,
\\   \\ \\           1   3 4 5  6  7          12           18      21
\\   \\             [0,0,0,0,0, 6, 0, 0,0,0,0, 0, 0,0,0,0,0,6,0, 0,21]);
\\   \\
\\   \\ \\ 6, 21
\\   \\ got_seq=vpar_seq_dandelion(vpar,18);
\\   \\ check_equal(got_seq,
\\   \\ \\                            6                                       21
\\   \\          Vecsmall([1,2,3,4,5,21,7,8,9,10,11,12,13,14,15,16,17,18,19,20,6]));
\\   \\
\\   \\ got_vec=vecextract(vpar,got_seq);
\\   \\ check_equal(got_vec,
\\   \\ \\                      6                                    21
\\   \\             [4,5,7,5,3, 0,20,12,1,4,4,21,19,19,6,1,16,6,7,12,21],
\\   \\             "dandelion ER 18 got_vec");
\\   \\ check_equal(vpar_is_cyclic(got_vec),1);
\\   \\ check_equal(got_vec[18],6);
\\   \\ check_equal(got_vec[6],0);
\\   \\ check_equal(got_vec[21],21);
\\
\\   \\ NO
\\   \\ vpar_again=vpar_from_dandelion(got_vec,18);
\\   \\ check_equal(vpar_again,vpar, "dandelion ER 18 vpar_again");
\\ }


\\ {
\\   \\ example in Egecioglu and Remmel up to root 7
\\
\\   \\ minimum of each cycle
\\   \\
\\   \\             /-----\          /----------\          /----\
\\   \\       *           *     *               *     *         *    *
\\   \\       1 -> 14 -> 21 -> 12 -> 8 -> 5  -> 9  -> 2 -> 6 -> 4 -> 7
\\   \\       |     |         /  \        |               / \       / \
\\   \\      11     3        18   19     16              13  10    15  17
\\   \\             |
\\   \\            20
\\
\\   \\        1    3   5   7    10  12     15          20 21
\\   my(vpar=[14,6,14,7,9,4,0,5,2,6,1,8,6,21,7,5,7,12,12,3,12]);
\\
\\   my(min_upto_root=vpar_minmax_upto_root_vector(vpar,1,min));
\\   check_equal(min_upto_root,
\\   \\           1 2 3 4 5 6 7 8 9    12  14            21
\\               [1,2,0,4,2,4,7,2,2,0,0,2,0,2,0,0,0,0,0,0,2],
\\             \\   *   *     *
\\               "min_upto_root");
\\
\\   \\  1, 2, 4, 7, 9, 12, 21
\\   my(got_seq=vpar_seq_dandelion(vpar));
\\   check_equal(got_seq,
\\   \\                 1 2   4     7           12                         21
\\            Vecsmall([7,1,3,2,5,6,4,8,9,10,11,12,13,14,15,16,17,18,19,20,21]));
\\
\\   my(got_vec=vecextract(vpar,got_seq));
\\   my(vpar_again=vpar_from_dandelion(got_vec));
\\   check_equal(vpar_again,vpar);
\\ }

  \\
  \\                /-----\             /----------\
  \\        *             *      *                  *        *
  \\       21  ->  12 -> 20  ->  7   -> 3 -> 4  ->  5  ->    1
  \\        |       |            |          / \     |       / \
  \\        6       6           19         10  11   2      9  16
  \\       / \                  / \                            |
  \\     15   18              13   14                         17



  \\ Micikevicius, Caminiti, Deo example
  \\         1                              n      root
  \\         |                              6*      1
  \\         6*         =>      after                \
  \\         |                minancestor      cycle    2*
  \\         8*                                 8*     \
  \\       /   \                              //  \     3
  \\      7     9                             7    9
  \\      |     |                             |    |
  \\      2*    5     1,6,8,9                 2    5
  \\      |     |                             |    |
  \\      3     4                             3    4



\\-----------------------------------------------------------------------------
progress("vpar_minmax_childless_descendants_vector()");

check_equal(vpar_minmax_childless_descendants_vector([2,3,0,1],min), [4,4,4,4]);
check_equal(vpar_minmax_childless_descendants_vector([0,1,0,0],min), [2,2,3,4]);


\\-----------------------------------------------------------------------------
progress("vpar_length_containing_vertex()");

check_equal(vpar_length_containing_vertex([0],1), 0);
check_equal(vpar_length_containing_vertex([0,1],1), 1, "path-2 v=1");
check_equal(vpar_length_containing_vertex([0,1],2), 1, "path-2 v=2");
check_equal(vpar_length_containing_vertex([0,1,2],1), 2, "path-3 v=1");
check_equal(vpar_length_containing_vertex([0,1,2],2), 2, "path-3 v=2");
check_equal(vpar_length_containing_vertex([0,1,2],3), 2, "path-3 v=3");
check_equal(vpar_length_containing_vertex([2, 0, 2],1),2, "path-3 relabelled");

test_longest_containing_vertex_by_neighbours(vpar,v) =
{
  \\ print("vpar_length_containing_vertex() "v" in "vpar);
  my(neighbours=vpar_children_of_vertex(vpar,v));
  for(i=1,#neighbours,
     vpar[neighbours[i]]=0);  \\ disconnect children
  if(vpar[v],
     neighbours=concat([vpar[v]],neighbours);
     vpar[v]=0);              \\ disconnect parent
  \\ print(" neighbours "neighbours);
  for(i=1,#neighbours,vpar=vpar_reroot(vpar,neighbours[i]));
  \\ vpar_view(vpar);
  neighbours=apply(v->vpar_subtree_height(vpar,v),neighbours);
  neighbours=vecsort(neighbours);
  \\ print(" heights "neighbours);
  my(m=min(#neighbours,2));
  \\ print(" result ",vecsum(neighbours[1..m])+m);
  vecsum(neighbours[1..m]) + m;
}
test_longest_containing_vertex_by_reroot(vpar,v) =
{
  vpar=vpar_reroot(vpar,v);
  my(vec=vpar_children_of_vertex(vpar,v));
  vec=apply(v->1+vpar_subtree_height(vpar,v), vec);
  vec=concat(vec,[0,0]);
  vec=vecsort(vec);
  vec[#vec] + vec[#vec-1];
}
{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     if(#vpar==0,next());
     for(i=1,3,
        my(v=vpar_random_vertex(vpar));
        \\ check_equal(test_longest_containing_vertex_by_distances(vpar,v),
        \\             test_longest_containing_vertex_by_neighbours(vpar,v),
        \\             "longest tests");
        \\ print("vpar "vpar);
        \\ print("v="v);
        my(got =vpar_length_containing_vertex(Vecsmall(vpar),v));
        my(want=test_longest_containing_vertex_by_reroot(vpar,v));
        check_equal(got,want,"vpar_length_containing_vertex() random")));
}

\\-----------------------------------------------------------------------------
progress("vpar_star_replacement()");

check_equal(vpar_star_replacement([],5), []);

\\      2
\\      |                 1
\\      1               / | \
\\    /   \            2  3  4
\\  3       4
\\check_equal(vpar_star_replacement([0],4), [2,0,1,1]);
check_equal(vpar_star_replacement([0],4), [0,1,1,1]);


\\ check_equal(vpar_star_replacement([0,1],5), [3,7, 0,1,1,1, 4,2,2,2], \
\\            "vpar_star_replacement() addhelp example");
check_equal(vpar_star_replacement([0,1],5), [0,7, 1,1,1,1, 3,2,2,2], \
           "vpar_star_replacement() addhelp example");

test_star_replacement_by_subdivide(vpar,n) =
{
  my(new_vpar=vpar_subdivide(vpar,2),
     degrees=vpar_degrees_vector(new_vpar));
  \\ print(new_vpar);
  new_vpar=Vec(new_vpar,#vpar*n);
  \\ print(#vpar*n);
  \\ print(new_vpar);
  my(upto=#new_vpar);
  for(v=1,#vpar,
     for(i=degrees[v]+1,n-1,
        \\ print("v="v" upto="upto" child");
        new_vpar[upto]=v; upto--));
  new_vpar;
}
check_equal(test_star_replacement_by_subdivide([0],5), [0,1,1,1,1]);

\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i]);
\\      #vpar>=1 || next();
\\      my(n=max(3,vecmax(vpar_degrees_vector(vpar))) + random(4),
\\         vpar1=vpar_star_replacement(vpar,n),
\\         vpar2=test_star_replacement_by_subdivide(vpar,n));
\\      if(vpar_is_cyclic(vpar1),
\\         print("n="n);
\\         vpar_view(vpar);
\\         vpar_view(vpar1));
\\      check_equal(vpar_is_cyclic(vpar1),0, "vpar_star_replacement() not cyclic");
\\      vpar_validate(vpar1);
\\      vpar_validate(vpar2);
\\      my(is_isomorphic = vpar_is_free_isomorphic(vpar1,vpar2));
\\      if(!is_isomorphic,
\\         print("n="n);
\\         vpar_view(vpar);
\\         vpar_view(vpar1);
\\         vpar_view(vpar2);
\\        );
\\      check_equal(is_isomorphic, 1,
\\      "vpar_star_replacement() vs test_star_replacement_by_subdivide()"));
\\ }



\\-----------------------------------------------------------------------------
progress("vpar_limb_height()");

check_equal(vpar_limb_height([]),0);
check_equal(vpar_limb_height([0]),0);
check_equal(vpar_limb_height([0,0]),0);
check_equal(vpar_limb_height_and_count([0,1,1,1]), [1,3]);

{
  for(n=1,10,
     my(vpar=vpar_make_path(n));
     check_equal(vpar_limb_height(vpar), n-1,
                "vpar_limb_height() of path");
     check_equal(vpar_limb_height(vpar), vpar_height(vpar),
                "vpar_limb_height() of path same as vpar_height()");
  );
}

\\  1 <- 2 <- 3 <- 4 <- 5 <- 6 <- 7 <- 8
\\                            \
\\                             -- 9 <- 10
\\                            1 2 3 4 5 6 7 8 9 10
check_equal(vpar_limb_height_and_count([0,1,2,3,4,5,6,7,6,9]), [2,2], \
            "vpar_limb_height() 10");

test_limb_height_and_count(vpar) =
{
  my(num_children=vpar_num_children_vector(vpar),
     exclude=vector(#vpar));

  \\ exclude any num_children[v]==2 and all above it
  for(i=1,#vpar,
     if(num_children[i]>=2,
        my(v=i);
        exclude[v]=1;
        while((v=vpar[v]), exclude[v]=1)));

  my(max_height=0, count=(#vpar==0));
  for(i=1,#vpar,
     my(v=i,h=0);
     while(!exclude[v] && (v=vpar[v]), h++);
     if(h==max_height, count++,
        h>max_height, max_height=h; count=1));
  [max_height,count];
}
check_equal(test_limb_height_and_count([0,1,2,3]), [3,1], \
            "test_limb_height_and_count()");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_limb_height_and_count(vpar),
                 test_limb_height_and_count(vpar)));
}


test_limb_height_and_count_by_downwards(vpar) =
{
  \\ limb_height[v] is established as length of the path up from v passing
  \\ only through num_children[]==1 vertices.  This is established working
  \\ downwards in the tree starting from limb_height[r]==0 for roots.
  \\ On reaching a num_children[v]==0 can consider its limb_height[v] for
  \\ the maximum.

  if(#vpar==0,return([0,1]));
  my(num_children = vpar_INTERNAL_num_children_vecsmall(vpar),
     seq          = vpar_seq_upascend(vpar,num_children),
     limb_height  = vectorsmall(#vpar),
     max_height   = -1,
     count        = 0);
  forstep(i=#seq,1,-1,  \\ downwards in the tree
         my(v=seq[i],p=vpar[v]);
         if(num_children[v]>=2,
            limb_height[v]=0);
         if(p,
            if(num_children[p]==1,
               limb_height[v]=limb_height[p]+1,
               limb_height[v]=1));
         if(num_children[v]==0,
            if(limb_height[v]>max_height,
               max_height = limb_height[v]; count=1,
               count += (limb_height[v]==max_height))));
  [max_height,count];
}
check_equal(test_limb_height_and_count_by_downwards([0,0]), [0,2]);
check_equal(test_limb_height_and_count_by_downwards([3,0,0,3]), [1,2]);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_limb_height_and_count(vpar),
                 test_limb_height_and_count_by_downwards(vpar),
  "vpar_limb_height_and_count() vs test_limb_height_and_count_by_downwards()"));
}

\\-----------------------------------------------------------------------------
progress("vpar_limb_length()");

check_equal(vpar_limb_length_and_count([]), [0,1], \
            "vpar_limb_length_and_count() empty");
check_equal(vpar_limb_length_and_count([0]), [0,1], \
            "vpar_limb_length_and_count() singleton");
check_equal(vpar_limb_length_and_count([0,0]), [0,2], \
            "vpar_limb_length_and_count() 2 singletons");
check_equal(vpar_limb_length_and_count([0,1,1,1]), [1,3]);

{
  for(n=1,10,
     my(vpar=vpar_make_path(n));
     check_equal(vpar_limb_length(vpar), n-1,
                "vpar_limb_length() of path");
     check_equal(vpar_limb_length(vpar), vpar_height(vpar),
                "vpar_limb_length() of path same as vpar_height()");
  );
}
\\  1 <- 2 <- 3 <- 4 <- 5 <- 6 <- 7 <- 8
\\                            \
\\                             -- 9 <- 10
\\                            1 2 3 4 5 6 7 8 9 10
check_equal(vpar_limb_length_and_count([0,1,2,3,4,5,6,7,6,9]), [5,1], \
            "vpar_limb_length_and_count() 10");

\\ test_limb_length_reroot(vpar) returns vpar rerooted so
\\ vpar_limb_length(vpar) == vpar_limb_height(vpar).  This means ensuring
\\ there is no limb path going up to the root and down again.
\\
\\ Any root which is degree<3 switched for a vertex of degree!=2.  So root
\\ as a degree>=3, or if none of those exist then vpar is a path and root as
\\ any degree<=1 which is the path ends.
\\
test_limb_length_reroot(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar));
  forstep(v=#vpar,1,-1,
         my(r=vpar_root_of_vertex(vpar,v));
         if(degrees[r]<3 && degrees[v]!=2,
            vpar=vpar_reroot(vpar,v)));
  vpar;
}
test_limb_length_and_count(vpar) = \
  vpar_limb_height_and_count(test_limb_length_reroot(vpar));

\\     3
\\    / \
\\   1   2
check_equal(test_limb_length_reroot([3,3,0]), [0,3,1]);
check_equal(vpar_limb_height_and_count([3,3,0]), [1,2], \
            "vpar_limb_height_and_count() 3,3,0");
check_equal(vpar_limb_height_and_count([0,3,1]), [2,1], \
            "vpar_limb_height_and_count() 0,3,1");
\\
check_equal(test_limb_length_and_count([3,3,0]), [2,1], \
            "test_limb_length_and_count() 3,3,0");
check_equal(vpar_limb_length_and_count([3,3,0]), [2,1], \
            "vpar_limb_length_and_count() 3,3,0");

\\          1          7
\\        /  \        /
\\       3    5      2
\\            |
\\            4
\\            |
\\            6
check_equal(test_limb_length_and_count([0,7,1,5,1,4,0]), [4,1], \
            "test_limb_length_and_count() 7");
check_equal(vpar_limb_length_and_count([0,7,1,5,1,4,0]), [4,1], \
            "vpar_limb_length_and_count() 7");

\\
check_equal(test_limb_length_reroot([2,0,2,2]), [2,0,2,2], \
            "test_limb_length_reroot() star-4 root 2, no change");
check_equal(test_limb_length_and_count([2,0,2,2]), [1,3], \
            "test_limb_length_and_count() star-4 root 2");
check_equal(vpar_limb_length_and_count([2,0,2,2]), [1,3], \
            "vpar_limb_length_and_count() star-4 root 2");

check_equal(vpar_limb_length_and_count([10,9,8,11,6,9,10,7,1,0,8]), [2,2], \
            "vpar_limb_length_and_count() 10");

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_limb_length_and_count(vpar),
                 test_limb_length_and_count(vpar),
             "vpar_limb_length_and_count() vs test_limb_length_and_count()");

     check_equal(vpar_limb_length(vpar) >= vpar_limb_height(vpar), 1,
             "vpar_limb_length() >= vpar_limb_height()");
  );
}

test_limb_length_and_count_by_downwards(vpar) =
{
  \\ Think this code is inferior to the vpar_limb_length_and_count()
  \\ implementation.  Here there's just one pass over vertices, but an extra
  \\ 1.5 passes to establish seq and more space to store seq and root[].

  \\ limb_height[v] is established as length of the path up from v passing
  \\ only through num_children[]==1 vertices.  This is established working
  \\ downwards in the tree starting from limb_height[r]==0 for roots.
  \\
  \\ At num_children[v]==0, the limb_height[v] is the length of limb ending
  \\ at v, but perhaps longer if it goes up to the root and down again when
  \\ the root has 2 children.
  \\
  \\ root[v] is the root vertex number of v, but only when that root has <=2
  \\ children, and only for those vertices below the root with 1 child each
  \\ and thus part of a limb going up to the root and down again.
  \\
  \\ root_other[r] is the height of the "other" limb below r, when it has
  \\ <=2 children.  The first num_children[v]!=1 below r sets root_other[r]
  \\ to its distance up to the root.  If that v has 0 children then
  \\ root_other[r] = distance.  If that v has >=2 children then
  \\ root_other[r] = -distance.  When the second num_children[v]!=1 below r
  \\ is reached the negative root_other[r] indicates that it is not a leaf
  \\ vertex, so can only be used if v is a leaf, ie. num_children[v]==0.
  \\
  \\ "prospective" is the prospective limb length ending at v.  For
  \\ num_children[v]==0, prospective is limb_height[v] plus any
  \\ root_other[r] distance if v is directly below a root.  For
  \\ num_children[v]>=2, prospective is only a path up to the root and down
  \\ to a leaf, which is when root_other[r]>0 or num_children[r]==1.
  \\
  \\ Maybe another downwards sequence could avoid the root[] vector.
  \\ A sequence which worked down from root through its entire component
  \\ tree before the next root would only need a single most-recent root and
  \\ root_other.

  my(num_children = vpar_INTERNAL_num_children_vecsmall(vpar),
     seq          = vpar_seq_upascend(vpar,num_children),
     limb_height  = vectorsmall(#vpar),
     root         = vectorsmall(#vpar),
     root_other   = vectorsmall(#vpar),
     max_length   = 0,
     count        = (#vpar==0));
  forstep(i=#seq,1,-1,  \\ downwards in the tree
         my(v=seq[i],p=vpar[v]);
         \\ print("at v="v" "num_children[v]" children, p="p"  p root "if(p,root[p])" max "max_length" count "count);
         my(prospective=-1);

         if(p,
            if(num_children[v]<=1,
               limb_height[v]=limb_height[p]+1;
               \\ print(" set limb height "limb_height[v]);
              );
            if(num_children[v]==1,
               root[v]=root[p]);
            ,
            if(num_children[v]<=2,    \\ v is a root
               \\ print(" set root "v);
               root[v]=v));

         if(num_children[v]==0,
            prospective=limb_height[v]);

         if(p,
            if(num_children[v]==0,
               if(root[p],
                  prospective += abs(root_other[root[p]]);
                  \\ print(" set root_other "limb_height[p]+1);
                  root_other[root[p]] = limb_height[p]+1);
               ,
               num_children[v]>=2 && root[p],
               if(root_other[root[p]]>0 || num_children[root[p]]==1,
                  prospective = limb_height[p]+1+root_other[root[p]]);
               \\ print(" set root_other ",-(limb_height[p]+1));
               root_other[root[p]] = -(limb_height[p]+1));
           );

         \\ print(" prospective "prospective);
         if(prospective>max_length,
            max_length=prospective; count=1,
            count += (prospective==max_length)));

  [max_length,count];
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_limb_length_and_count(vpar),
                 test_limb_length_and_count_by_downwards(vpar),
  "vpar_limb_length_and_count() vs test_limb_length_and_count_by_downwards()");
  );
}

\\-----------------------------------------------------------------------------
progress("vpar_dilation()");

{
  \\ 1 <- 2 <- 3 <- 4 <- 5
  my(vpar_path=[0,1,2,3,4]);
  check_equal(vpar_dilation(vpar_path), 1);

  \\    2    4    1   2
  \\ 3 <- 1 <- 5 <- 4 <- 2
  my(perm=Vecsmall([3,1,5,4,2]),
     vpar_perm=vpar_relabel_perm(vpar_path,perm));
  \\                     +2 +2 -3 +1 -4
  \\                      1  2  3  4  5
  check_equal(vpar_perm, [3, 4, 0, 5, 1]);
  check_equal(vpar_dilation(vpar_perm), 4);

  \\    2    4    1   2
  \\ 3 <- 5 <- 1 <- 2 <- 4
  my(vpar_perm_rev=vpar_relabel_reverse(vpar_perm));
  \\                         +4 -1 -3 -2 -2
  \\                          1  2  3  4  5
  check_equal(vpar_perm_rev, [5, 1, 0, 2, 3]);
  check_equal(vpar_dilation(vpar_perm_rev), 4);

  my(perm_identity=test_identity_perm(5),
     perm_rev=test_perm_reversal(#vpar_path));
  check_equal(perm_rev, Vecsmall([5,4,3,2,1]));
  check_equal(perm_rev^-1, perm_rev, "perm_rev self-inverse");
  check_equal(perm_rev^2, perm_identity, "perm_rev squared");
  check_equal(vpar_relabel_perm(vpar_perm,perm_rev),
              vpar_perm_rev);

  check_equal(perm*perm_rev, Vecsmall([2, 4, 5, 1, 3]));
  check_equal(perm_rev*perm, Vecsmall([3, 5, 1, 2, 4]));

  check_equal(vpar_relabel_perm(vpar_path, perm_rev*perm),
              vpar_perm_rev);
}


\\-----------------------------------------------------------------------------
progress("vpar_perm_is_decomposable()");
\\ vpar_perm_is_indecomposable()

check_equal(vpar_perm_is_decomposable([]),   0);
check_equal(vpar_perm_is_indecomposable([]), 1);

check_equal(vpar_perm_is_decomposable([1]),   0);
check_equal(vpar_perm_is_indecomposable([1]), 1);

check_equal(vpar_perm_is_decomposable([1,2]), 1);
check_equal(vpar_perm_is_decomposable([2,1]), 0);
check_equal(vpar_perm_is_indecomposable([1,2]), 0);
check_equal(vpar_perm_is_indecomposable([2,1]), 1);

check_equal(vpar_perm_is_indecomposable([1,2,3]), 0, \
            "vpar_perm_is_indecomposable() three");
check_equal(vpar_perm_is_indecomposable([1,3,2]), 0);
check_equal(vpar_perm_is_indecomposable([2,1,3]), 0);
check_equal(vpar_perm_is_indecomposable([2,3,1]), 1);
check_equal(vpar_perm_is_indecomposable([3,1,2]), 1);
check_equal(vpar_perm_is_indecomposable([3,2,1]), 1);

check_equal(vpar_perm_is_decomposable([1,2,3]), 1, \
            "vpar_perm_is_decomposable() three");
check_equal(vpar_perm_is_decomposable([1,3,2]), 1);
check_equal(vpar_perm_is_decomposable([2,1,3]), 1);
check_equal(vpar_perm_is_decomposable([2,3,1]), 0);
check_equal(vpar_perm_is_decomposable([3,1,2]), 0);
check_equal(vpar_perm_is_decomposable([3,2,1]), 0);
check_equal(3!,6);

{
  for(n=1,7,
     my(perm=numtoperm(n,random(n!)));
     check_equal(vpar_perm_is_indecomposable(perm),
                 ! vpar_perm_is_decomposable(perm)));
}



\\-----------------------------------------------------------------------------
progress("vpar_perm_count_indecomposable()");

check_equal(vpar_perm_count_indecomposable(0),1, \
            "vpar_perm_count_indecomposable() n=0");
check_equal(vpar_perm_count_indecomposable(1),1);
check_equal(vpar_perm_count_indecomposable(2),1);
check_equal(vpar_perm_count_indecomposable(3),3, \
            "vpar_perm_count_indecomposable() n=3");

{
  check_equal(
    vector(15,n,n--; vpar_perm_count_decomposable(n)),
    vector(15,n,n--; sum(i=1,n-1, (i!-vpar_perm_count_decomposable(i))*(n-i)!)),
             "vpar_perm_count_decomposable() by sum");
}

\\ Michael Somos in A003319
test_A003319(n) =
{
  my(A);
  if(n<1,0,
     A=vector(n);
     A[1]=1;
     for(k=2,n, A[k] = (k-2)*A[k-1] + sum(j=1, k-1, A[j] * A[k-j]));
     A[n]);
}
check_equal(vector(12,n, vpar_perm_count_indecomposable(n)), \
            vector(12,n, test_A003319(n)));

{
  for(n=1,6,
     my(count_decomposable=0, count_indecomposable=0);
     for(i=1,n!,
        my(perm = numtoperm(n,i),
           is_decomposable = vpar_perm_is_decomposable(perm));
        count_decomposable   += is_decomposable;
        count_indecomposable += !is_decomposable);

     check_equal(count_decomposable + count_indecomposable, n!,
                 "count_decomposable + count_indecomposable");
     check_equal(vpar_perm_count_decomposable(n)
                 + vpar_perm_count_indecomposable(n), n!);
     check_equal(count_decomposable, vpar_perm_count_decomposable(n),
                Str("vpar_perm_is_decomposable() vs "
                    "vpar_perm_count_decomposable() n="n));
     check_equal(count_indecomposable, vpar_perm_count_indecomposable(n),
                Str("vpar_perm_is_indecomposable() vs "
                    "vpar_perm_count_indecomposable() n="n));
     check_equal(count_indecomposable,
                 vpar_count_min_descendant_increasing(n-1,1));
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_is_minmax_ancdec_vector()");

check_equal(vpar_is_minmax_ancdec_vector([],min), 1, \
            "vpar_is_minmax_ancdec_vector() empty");
check_equal(vpar_is_minmax_ancdec_vector([],max), 1);
check_equal(vpar_is_minmax_ancdec_vector([1],min), 1, \
            "vpar_is_minmax_ancdec_vector() singleton");
check_equal(vpar_is_minmax_ancdec_vector([1],max), 1);
check_equal(vpar_is_minmax_ancdec_vector([-1],max), 0, \
            "vpar_is_minmax_ancdec_vector() -1");
check_equal(vpar_is_minmax_ancdec_vector([9999],max), 0, \
            "vpar_is_minmax_ancdec_vector() 9999");
check_equal(vpar_is_minmax_ancdec_vector([1,1],min), 1);
check_equal(vpar_is_minmax_ancdec_vector([2,2],max), 1);

{
  for(m=0,1,
     my(func=if(m,max,min));
     for(i=1,#test_sample_trees,
        my(vpar=test_sample_trees[i],
           minmax_ancestors = vpar_minmax_ancestors_vector(vpar,func));
        check_equal(vpar_is_minmax_ancdec_vector(minmax_ancestors,func), 1,
                    "vpar_is_minmax_ancdec_vector()");
        my(minmax_descendants = vpar_minmax_descendants_vector(vpar,func));
        check_equal(vpar_is_minmax_ancdec_vector(minmax_descendants,func), 1,
                    "vpar_is_minmax_ancdec_vector()")));
}
{
  for(m=0,1,
     my(func=if(m,max,min));
     for(i=1,#test_sample_forests,
        my(vpar=test_sample_forests[i],
           minmax_ancestors = vpar_minmax_ancestors_vector(vpar,func));
        check_equal(vpar_is_minmax_ancdec_vector(minmax_ancestors,func), 1,
                    "vpar_is_minmax_ancdec_vector() forest");
        my(minmax_descendants = vpar_minmax_descendants_vector(vpar,func));
        check_equal(vpar_is_minmax_ancdec_vector(minmax_descendants,func), 1,
                    "vpar_is_minmax_ancdec_vector()")));
}

{
  for(forest=0,1,
     for(n=0, test_limit_trees_search - forest,
        for(m=0,1,
           my(func=if(m,max,min));
           my(seen=List([]));
           forvec(vec=vpar_forvec_from(n,forest),
                 my(vpar=vpar_from_upascend(vec),
                    minmax_ancestors = vpar_minmax_ancestors_vector(vpar,func));
                 listput(seen,minmax_ancestors));
           seen=Set(seen);
           check_equal(#seen,vpar_INTERNAL_Bell_number(n));

           \\ if(forest&&m==0,print(#seen","));
           \\ print("seen "seen);
           forvec(vec=vector(n,i,[1,n]),
                 \\ print(" vec "vec);
                 check_equal(vpar_is_minmax_ancdec_vector(vec,func),
                             setsearch(seen,vec)!=0,
                             Str("vpar_is_minmax_ancdec_vector() vs setsearch"
                                 " n="n" func="func)));
        )));
}

\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_minmax_cycle_vector()");

{
  \\       1 2  3 4 5 6 cycle
  my(vpar=[2,3, 4,5,6,3]);
  \\ vpar_view(vpar);
  check_equal(vpar_is_cyclic(vpar), 1);
  check_equal(vpar_INTERNAL_minmax_cycle_vector(vpar,max),
              Vecsmall([-1,-1, 6,6,6,6]));
  check_equal(vpar_INTERNAL_minmax_cycle_vector(vpar,min),
              Vecsmall([-1,-1, 3,3,3,3]));
}

\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_minmax_v_to_root_vector()");

{
  \\ 1 -> 2 -> 3 -> 4
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector([2,3,4,0],1,max),
              Vecsmall([0,3,3,3]));
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector([0],1,max),
              Vecsmall([0]));
}
{
  \\    6 -> 7 -> 1 -> 2 -> 5 -> 3 -> 4 root
  my(vpar=[2,5,4,0,3,7,1]);
  vpar_validate(vpar);
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,1,max),
              Vecsmall([0,5,3,3,5,0,0]), "1 up");
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,7,max),
              Vecsmall([5,5,3,3,5,0,0]), "7 up");
  check_equal(vpar_INTERNAL_minmax_v_to_root_vector(vpar,3,max),
              Vecsmall([0,0,0,0,0,0,0]), "3 up, none");
}


\\-----------------------------------------------------------------------------
progress("vpar_is_cordial()");

check_equal(vpar_cordial_num_diff([]), 0);
check_equal(vpar_cordial_num_diff([0,1]), 1);
check_equal(vpar_is_cordial([]), 1);
check_equal(vpar_is_cordial([0,1]), 1);
check_equal(vpar_is_cordial([0,1,2]), 0);
check_equal(vpar_cordial_num_same([0,1,2]), 0);
check_equal(vpar_cordial_num_diff([0,1,2]), 2);
check_equal(vpar_cordial_delta([0,1,2]), -2);

check_equal(vpar_cordial_num_diff([0,1,1]), 1);
check_equal(vpar_cordial_num_same([0,1,1]), 1);
check_equal(vpar_is_cordial([0,1,1]), 1, "vpar_is_cordial()");


\\-----------------------------------------------------------------------------
progress("vpar_INTERNAL_seq_rotate_up()");
\\ vpar_INTERNAL_seq_rotate_down()

check_equal(vpar_INTERNAL_seq_rotate_down([1,2,3]), Vecsmall([2,3,1]), \
            "vpar_INTERNAL_seq_rotate_down()");
check_equal(vpar_INTERNAL_seq_rotate_up([1,2,3]), Vecsmall([3,1,2]), \
            "vpar_INTERNAL_seq_rotate_up()");

check_equal(vpar_INTERNAL_seq_rotate_down([0,2,3,0]), Vecsmall([1,3,2,4]), \
            "vpar_INTERNAL_seq_rotate_down() middle 2");
check_equal(vpar_INTERNAL_seq_rotate_up([0,2,3,0]), Vecsmall([1,3,2,4]), \
            "vpar_INTERNAL_seq_rotate_up() middle 2");

{
  my(n=4,
     funcs=[vpar_INTERNAL_seq_rotate_up,
            vpar_INTERNAL_seq_rotate_down]);
  for(i=1,#funcs,
     my(func=funcs[i]);
     forvec(flags=vector(n,i, if(i==n,[1,1], [0,1])),
           my(above=vector(#flags,i, flags[i]*i),
              perm=func(above));
           check_equal(test_is_perm(perm),1, "is perm")));
}

\\-----------------------------------------------------------------------------
progress("vpar_seq_minancestor()");
\\ vpar_from_minancestor()

check_equal(vpar_from_minancestor([]),[], "vpar_from_minancestor() empty");
check_equal(vpar_from_minancestor([0,0,0,0]),[0,0,0,0]);
check_equal(vpar_seq_minancestor([0,1]), Vecsmall([1,2]));
check_equal(vpar_minmax_ancestors_vector([0,1],min), [1,1]);
check_equal(vpar_minmax_ancestors_vector([1,0],min), [1,2], \
            "vpar_minmax_ancestors_vector() min");

check_equal(vpar_from_minancestor([0,1]), [0,1]);
check_equal(vpar_from_minancestor([0,2]), [2,0]);


\\-----------------------------------------------------------------------------
progress("vpar_from_subtree_height()");

check_equal(vpar_from_subtree_height([]),[]);
check_equal(vpar_from_subtree_height([0]),[0]);
check_equal(vpar_seq_subtree_height([0,1]),Vecsmall([2,1]));
check_equal(vpar_from_subtree_height([1,0]),[0,1]);
check_equal(vpar_from_subtree_height([2,0]),[2,0]);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        seq=vpar_seq_subtree_height(vpar),
        got =vecextract(vpar,seq),
        want=Vec(Vecsmall(vpar)*Vecsmall(vpar_seq_subtree_height(vpar))));
     check_equal(got,want, "subtree_height vec by permute"));
}


\\-----------------------------------------------------------------------------
progress("vpar_seq_happy()");

\\ {
\\   \\ Caminiti root=12
\\   my(vpar    =[ 3, 5,6,10,12,4,2,1,12, 2,9,0]);
\\   \\ check_equal(vpar_seq_1_pre_root_reverse(vpar),
\\   \\             Vecsmall([5,3,2,4,1,10,7,8,9,6,11,12]));
\\ }

test_code_happy_by_loop(vpar) =
{
  my(J=vpar[1]);
  while(vpar[1],
        my(j=vpar[1]);
        vpar[1]=vpar[j];
        vpar[j]=0;
        if(j>J,
           J=j; vpar[J]=J,
           vpar[j]=vpar[J]; vpar[J]=j));
  vpar[2..#vpar];
}

{
  \\ Caminiti thesis section 5.2, Happy
  \\              0
  \\            /   \
  \\          5       9
  \\          |        \
  \\          2         11
  \\        /   \
  \\       7    10*
  \\              \     *    *    *
  \\               4 -- 6 -- 3 -- 1 -- 8

  my(vpar    =[ 3, 5,6,10,0,4,2,1,0, 2,9]);
  check_equal(#vpar,11);
  vpar_validate(vpar);

  \\             2 3  4  5         10
  my(want_code=[ 10,3, 6, 2,4,2,1,0, 5,9]);
  check_equal(test_code_happy_by_loop(vpar),want_code,
              "Caminiti happy vec by loop");

  \\----
  \\ with root=12
  \\            1          5        9
  my(vpar    =[ 3, 5,6,10,12,4,2,1,12, 2,9,0]);
  check_equal(#vpar,12);
  vpar_validate(vpar);

  \\                    1 2 3 4  5         10
  my(want_seq=Vecsmall([5,4,1,3,10,6,7,8,9, 2,11,12]));
  check_equal(test_is_perm(want_seq),1);

  my(want_vec=apply(p->if(p==0,12,p),want_code)); \\ root=12
  want_vec=concat([12,want_vec,0]);  \\ vpar[1]=root, vpar[12]=0 final
  my(vec=vecextract(vpar,want_seq));
  check_equal(vec,want_vec, "Caminiti happy root=12 vec of want_seq");

  my(seq=vpar_seq_happy(vpar));
  check_equal(seq,want_seq, "Caminiti happy root=12 seq");
}

{
  \\ Caminiti slides Happy
  \\
  \\      0
  \\    /  \
  \\  2      6*
  \\        / \
  \\      5    3   --+
  \\    /       \    |
  \\  7          4* -+
  \\              \
  \\               1*
  \\
  my(vpar=[4,0,6,3,6,0,5]);
  vpar_validate(vpar);

  my(want_code=[ 0,4,3,6,6,5]);
  check_equal(test_code_happy_by_loop(vpar),want_code,
              "Caminiti slides happy vec by loop");

  \\--------
  \\ with root=8
  my(vpar=[4,8,6,3,6,8,5,0]);

  my(seq=vpar_seq_happy(vpar));
  check_equal(test_is_perm(seq),1, "test_is_perm()");

  my(want_vec=apply(p->if(p==0,8,p),want_code)); \\ root=8
  want_vec=concat([8,want_vec,0]);  \\ vpar[1]=root, vpar[8]=0 final
  my(vec=vecextract(vpar,seq));
  check_equal(vec,want_vec, "Caminiti slides happy root=8 vec");
}

{
  \\ Micikevicius, Caminiti, Deo example Happy going 2 to 1
  \\
  \\                                             root
  \\         1 *                                  1
  \\         |                                      \
  \\         6          =>      after            6    2
  \\         |                  happy           | |   |
  \\         8 *                                 8    3
  \\       /   \                              n    \
  \\      7*    9                             7       9
  \\      |     |                                     |
  \\      2*    5     1,2,6,7                         5
  \\      |     |                                     |
  \\      3     4                                     4
  \\
  \\ Traverse 2 to 1.
  \\ 2 seek >2 which is 7.
  \\ mu(v) = max ancestor of v

  \\ Caminiti thesis
  \\ d(u,v) = distance
  \\ l(v) = height subtree of v
  \\ mu(v) = max descendant of v
  \\ lambda(v) = max leaf descendant of v

  \\           1 2 3 4 5 6 7 8 9    \\ vertex
  my(vpar=    [0,7,2,5,9,1,8,6,8]); \\ parent

  \\--------
  \\ root=0
  vpar=apply(p->p-1,vpar)[^1];

  \\ shown in section 3 codes as "Blob" but is Happy, per part (c) of
  \\ figure 1
  \\                          3 4 5 6 7 8 9
  my(want_code=apply(p->p-1,[ 2,5,9,8,7,6,8 ]));
  check_equal(want_code,    [ 1,4,8,7,6,5,7 ]);
  check_equal(test_code_happy_by_loop(vpar),want_code,
              "MCD happy code by loop");

  my(seq=vpar_seq_happy(vpar));
  my(want_vec=concat(0,want_code));

  \\ FIXME
  \\ my(vec=vecextract(vpar,seq));
  \\ check_equal(vec,want_vec, "MCD happy vec");
}

{
  \\ Picciotto thesis example section 5.2
  \\  1 -> 3 -> 2 -> 4 -> 0
  my(vpar = [3,4,2,0],
     seq = vpar_seq_minancestor(vpar),
     vec=vecextract(vpar,seq));
  check_equal(seq,Vecsmall([4,1,3,2]));
  check_equal(vec,[0,3,2,4]);
}
{
  \\ Picciotto thesis example 2.4
  \\ 1 --> 2 --> 0
  my(vpar=[2,0],
     seq = vpar_seq_happy(vpar));
  check_equal(seq,Vecsmall([2,1]));
  my(vec=vecextract(vpar,seq));
  check_equal(vec,[0,2]);
}


\\-----------------------------------------------------------------------------
progress("vpar_seq_mhappy()");
\\ vpar_from_mhappy()

check_equal(vpar_seq_mhappy([]), Vecsmall([]));
check_equal(vpar_seq_mhappy([0]), Vecsmall([1]));

check_equal(vpar_INTERNAL_seqfrom_mhappy([]), Vecsmall([]), \
            "vpar_INTERNAL_seqfrom_mhappy() empty");
check_equal(vpar_INTERNAL_seqfrom_mhappy([0]), Vecsmall([1]), \
            "vpar_INTERNAL_seqfrom_mhappy() singleton");

check_equal(vpar_from_mhappy([]), []);
check_equal(vpar_from_mhappy([0]), [0]);

{
  \\            4
  \\            |
  \\  +-        6
  \\  |       / | \
  \\  |      5  10  2
  \\  |      |
  \\  +-  * 11
  \\         |
  \\  +    * 8
  \\        / \
  \\  +  * 3    9
  \\       |
  \\       1
  \\       |
  \\       7
  \\
  my(vpar=[3, 6, 8, 0, 6, 4, 1, 11, 8, 6, 5]);
  \\vpar_view(vpar);
  my(seq=vpar_seq_mhappy(vpar));
  check_equal(test_is_perm(seq),1, "is perm");
  \\                         1     3        6     8
  check_equal(seq, Vecsmall([6, 2, 1, 4, 5, 8, 7, 3, 9, 10, 11]));

  \\   n  n       6 -+
  \\   3  8       |  |
  \\      |       5  |
  \\      9       |  v
  \\             11 /
  my(vec=vecextract(vpar,seq));
  \\ vpar_view(vec);
  check_equal(vpar_INTERNAL_minmax_cycleprev_vector(vec,max),
              \\                3          6      8
              Vecsmall([-1, -2, 3, -1, -2, 6, -7, 8, -9, -10, -2]),
              "vpar_INTERNAL_minmax_cycleprev_vector()");

  my(seq_again=vpar_INTERNAL_seqfrom_mhappy(vec));
  check_equal(test_is_perm(seq_again),1, "is perm");
  check_equal(seq_again,seq, "vpar_INTERNAL_seqfrom_mhappy()");

  check_equal(vpar_from_mhappy(vec),vpar,
              "vpar_from_mhappy()");

  \\                    A        C     B
\\                1     3        6     8
\\ got  Vecsmall([8, 2, 1, 4, 5, 3, 7, 6, 9, 10, 11])
\\ want Vecsmall([6, 2, 1, 4, 5, 8, 7, 3, 9, 10, 11])
\\                1     3        6     8
\\                      A        C     B
\\   A B C
\\ 1 3 8 6
\\ 6 1 3 8

\\              3        6     8   9
\\ got  [3, 6, 11, 0, 6, 8, 1, 8,  4, 6, 5]
\\ want [3, 6,  8, 0, 6, 4, 1, 11, 8, 6, 5]

}

{
  \\ Caminiti thesis section 5.2, MHappy
  \\              0                                     0
  \\            /   \                                  / \
  \\          5       9                             *1     9
  \\          |        \                            /        \
  \\          2         11                        8           11
  \\        /   \                                     5
  \\       7    10*                                   | \
  \\              \     *    *    *                   2   v
  \\               4 -- 6 -- 3 -- 1 -- 8            /   \  \    4 = 6   n
  \\                                               7      10            3
  \\ path 1 to root
  \\ v > max(pathset(1 to v))
  my(vpar    =[ 3, 5,6,10,0,4,2,1,0, 2,9]);
  check_equal(#vpar,11);
  my(want_code=[5,3,6,10,4,2,1,0,2,9]);

  \\------
  \\ with root=12

  my(vpar    =[ 3, 5,6,10,12,4,2,1,12, 2,9,0]);
  check_equal(#vpar,12);
  my(seq=vpar_seq_mhappy(vpar));
  check_equal(test_is_perm(seq),1);

  \\                    1   3 4 5   rotated up, last of cycle
  my(want_seq=Vecsmall([5,2,1,3,4,6,7,8,9,10,11,12]));
  check_equal(test_is_perm(want_seq),1);

  my(want_vec=apply(p->if(p==0,12,p),want_code)); \\ root=12
  want_vec=concat([12,want_vec,[0]]);  \\ vpar[1]=root, vpar[12]=0 final
  my(vec=vecextract(vpar,want_seq));
  check_equal(vec,want_vec, "Caminiti mhappy root=12 vec of want_seq");

  my(vec=vecextract(vpar,seq));
  check_equal(vec,want_vec, "Caminiti mhappy vec");

  my(vpar_again=vpar_from_mhappy(vec));
  check_equal(vpar_again,vpar, "Caminiti mhappy vpar_again");

  check_equal(vpar_INTERNAL_seqfrom_mhappy(vec),seq, "Caminiti mhappy seqfrom");
}


\\-----------------------------------------------------------------------------
progress("Sequences Reversibility and Uniqueness");

{
  my(forest=1,
     n_max= test_limit_trees_search - forest,
     debug=0,
     elems=
     [
      \\ FIXME: No, need own "from".
      \\ ["vpar_seq_uplexqueue",
      \\  vpar_seq_uplexqueue,
      \\  vpar_from_upqueue,
      \\  vpar_INTERNAL_seqfrom_upqueue,
      \\  'upwards, 'not_ordered, 0
      \\ ],

      \\ these in test.gp, but without their seqfrom
      ["vpar_seq_childless_paths",
       vpar_seq_childless_paths,
       vpar_from_childless_paths,
       vpar_INTERNAL_seqfrom_childless_paths,
       'bothwards, 'not_ordered, 0
      ],
      ["vpar_seq_childless_paths,down=1",
       vpar->vpar_seq_childless_paths(vpar,1),
       vec->vpar_from_childless_paths(vec,1),
       vec->vpar_INTERNAL_seqfrom_childless_paths(vec,1),
       'downwards, 'not_ordered, 0
      ],

      \\ these in test.gp, but without their seqfrom
      ["successive_paths",
       vpar_seq_successive_paths,
       vpar_from_successive_paths,
       vpar_INTERNAL_seqfrom_successive_paths,
       'bothwards, 'not_ordered, 0
      ],
      ["successive_paths,down=1",
       vpar->vpar_seq_successive_paths(vpar,1),
       vec->vpar_from_successive_paths(vec,1),
       vec->vpar_INTERNAL_seqfrom_successive_paths(vec,1),
       'downwards, 'not_ordered, 0
      ],


      ["mhappy",
       vpar_seq_mhappy,
       vpar_from_mhappy,
       vpar_INTERNAL_seqfrom_mhappy,
       'bothwards, 'not_ordered, 0
      ],

      \\ FIXME
      ["dandelion",
       vpar->vpar_seq_dandelion(vpar),
       vec->vpar_from_dandelion(vec),
       vec->vpar_INTERNAL_seqfrom_dandelion(vec),
       'bothwards, 'not_ordered,
       1    \\ min_size = 1 vertex to have v=1
      ],

      ["minancestor", vpar_seq_minancestor, vpar_from_minancestor,
       'no_seqfrom, 'bothwards, 'not_ordered, 0 ]
     ]);
  for(i=1,#elems,
     length(elems[i])==7 || error();
     my(func_name   =elems[i][1],
        seq_func    =elems[i][2],
        from_func   =elems[i][3],
        seqfrom_func=elems[i][4],
        direction   =elems[i][5],
        ordered     =elems[i][6],
        min_size    =elems[i][7]);
     if(debug, print(func_name));

     setsearch(Set(['upwards,'downwards,'bothwards]), direction)
       || error();
     setsearch(Set(['ordered,'ordered_except_roots,'not_ordered]), ordered)
       || error();

     for(i=1,#test_sample_forests,
        my(vpar=test_sample_forests[i]);
        if(#vpar<min_size,next());
        if(debug,print(func_name," vpar "vpar));

        my(seq=seq_func(vpar));
        if(debug,print("  seq "seq));
        check_equal(test_is_perm(seq),1, Str(func_name" seq is perm"));

        my(vec=vecextract(vpar,seq),
           vpar_again = from_func(vec));
        check_equal(vpar_again,vpar, Str(func_name" vpar again"));
        );

     for(n=min_size,n_max,
        if(debug, print("n="n" "func_name));

        my(count=0,
           count_downwards=0,
           count_upwards=0,
           count_not_downwards=0,
           count_not_upwards=0,
           count_ordered=0,
           count_not_ordered=0);
        forvec(vec=vpar_forvec_from(n,forest), \\ forests
              my(vpar=vpar_from_upascend(vec));
              count++;
              if(debug,print(func_name," vpar "vpar));

              my(seq=seq_func(vpar));
              if(debug,print("  seq "seq));

              my(is_upwards=vpar_seq_is_upwards(vpar,seq),
                 is_downwards=vpar_seq_is_downwards(vpar,seq));
              count_upwards   += is_upwards;
              count_downwards += is_downwards;
              count_not_upwards   += ! is_upwards;
              count_not_downwards += ! is_downwards;

              if(direction=='upwards,
                 check_equal(is_upwards,1,
                             Str(func_name" should be upwards")));
              if(direction=='downwards,
                 check_equal(is_downwards,1,
                             Str(func_name" should be downwards")));

              my(is_ordered = vpar_seq_is_ordered(vpar,seq));
              count_ordered     += is_ordered;
              count_not_ordered += ! is_ordered;
              if(ordered=='ordered,
                 check_equal(is_ordered, 1,
                             Str(func_name" should be sibling ordered")));

              my(is_ordered_except_roots
                 = test_seq_is_ordered_except_roots(vpar,seq));
              count_ordered     += is_ordered;
              count_not_ordered += ! is_ordered;
              if(ordered=='ordered_except_roots,
                 check_equal(is_ordered_except_roots, 1,
                    Str(func_name" should be sibling ordered except roots")));

              my(vec=vecextract(vpar,seq));

              my(vpar_again=from_func(vec));
              vpar_validate(vpar_again);
              check_equal(vpar_again,vpar,
                          Str(func_name" vpar_again by from_func"));

              if(seqfrom_func!='no_seqfrom,
                 my(seq_again=seqfrom_func(vec));
                 check_equal(seq_again,seq,
                             Str(func_name" seq_again by seqfrom_func")));

              if(n>=2 && func_name=="vpar_seq_blob",
                 \\ for tree vec[1]==1 iff vertex 1 is root
                 \\ for forest vec[1]==0or1 iff vertex 1 is root
                 my(roots=vpar_roots(vpar));
                 if(#roots==1,
                    check_equal(vec[1]==1, roots==[1],
                                "blob tree vec[1]==1 is root 1"),
                    check_equal(vec[1]<=1, vpar[1]==0,
                                "blob forest vec[1]==0,1 is root 1")));
              );

        check_equal(count,(n+1)^(n-1));
        if(direction=='bothwards,
           check_equal(count_upwards && count_downwards, 1,
                          Str(func_name" should be bothwards"));
           if(n>=3,
              check_equal(count_not_upwards && count_not_downwards, 1,
                          Str(func_name" n="n" bothwards should have not up and not down")));
          );

        if(ordered=='not_ordered && n>=4,
           check_equal(count_not_ordered >= 1, 1,
                       Str(func_name" n="n" not_ordered should have some unordered")));
     ));
}


\\==============================================================================
progress("No globals left over");
\\ test-symbols.txt is all symbols appearing in vpar.gp.

check_equal(vpar,'vpar, "variable \"vpar\" should be unset");
{
  my(test_symbols=readstr("test-symbols.txt"));
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_equal(eval(test_symbol),
                 eval(Str("'"test_symbol)),
                 Str("leftover global variable: "test_symbol)));
}
\\ {
\\   my(test_symbols=readstr("test-funcs.txt"));
\\   for(test_i=1,#test_symbols,
\\      my(test_symbol=test_symbols[test_i]);
\\      my(value=eval(test_symbol));
\\      check_equal(value == eval(Str("'"test_symbol))
\\                  || type(value) == "t_CLOSURE", 1,
\\                  Str("leftover non-function: "test_symbol)));
\\ }


\\-----------------------------------------------------------------------------
{
  printf("total %d tests in %.1f seconds\n",
         check_count,
         if(option_progress,progress_total_time,gettime()) /1000);
}

\\-----------------------------------------------------------------------------
